;/*****************************************************************************
; * @file:    startup_A31G32x.s
; * @purpose: CMSIS Cortex-M0+ Core Device Startup File 
; *           for the ARM 'Microcontroller Prototyping System' 
; * @version: V1.0
; * @date:    19. AUG. 2018
; *------- <<< Use Configuration Wizard in Context Menu >>> ------------------
; *
; * Copyright (C) 2008-2009 ARM Limited. All rights reserved.
; * ARM Limited (ARM) is supplying this software for use with Cortex-M0 
; * processor based microcontrollers.  This file can be freely distributed 
; * within development tools that are supporting such ARM based processors. 
; *
; * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
; * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
; * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
; * ARM SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
; * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
; *
; *****************************************************************************/


; <h> Stack Configuration
;   <o> Stack Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

Stack_Size      EQU     0x00000200

                AREA    STACK, NOINIT, READWRITE, ALIGN=3
Stack_Mem       SPACE   Stack_Size
__initial_sp


; <h> Heap Configuration
;   <o>  Heap Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

Heap_Size       EQU     0x00000000

                AREA    HEAP, NOINIT, READWRITE, ALIGN=3
__heap_base
Heap_Mem	SPACE   Heap_Size
__heap_limit


                PRESERVE8
                THUMB


; Vector Table Mapped to Address 0 at Reset

                AREA    RESET, DATA, READONLY
                EXPORT  __Vectors

__Vectors	DCD     __initial_sp				; Top of Stack
			DCD     Reset_Handler			; Reset Handler
			DCD     NMI_Handler			; NMI Handler
			DCD     HardFault_Handler		; Hard Fault Handler
			DCD     0						; Reserved
			DCD     0						; Reserved
			DCD     0						; Reserved
			DCD     0						; Reserved
			DCD     0						; Reserved
			DCD     0						; Reserved
			DCD     0						; Reserved
			DCD     SVC_Handler			; SVCall Handler
			DCD     0						; Reserved
			DCD     0						; Reserved
			DCD     PendSV_Handler			; PendSV Handler
			DCD     SysTick_Handler			; SysTick Handler

			; External Interrupts
			DCD     LVI_IRQHandler			; IRQ0
			DCD     SYSCLKFAIL_IRQHandler	; IRQ1
			DCD     WDT_IRQHandler			; IRQ2
			DCD     GPIOAB_IRQHandler		; IRQ3
			DCD     GPIOCF_IRQHandler		; IRQ4
			DCD     0						; IRQ5
			DCD     0						; IRQ6				
			DCD     TIMER10_IRQHandler		; IRQ7
			DCD     TIMER11_IRQHandler		; IRQ8
			DCD     TIMER12_IRQHandler		; IRQ9
			DCD     I2C0_IRQHandler			; IRQ10
			DCD     USART10_IRQHandler		; IRQ11
			DCD     WT_IRQHandler			; IRQ12
			DCD     TIMER30_IRQHandler		; IRQ13
			DCD     I2C1_IRQHandler			; IRQ14
			DCD     TIMER20_IRQHandler		; IRQ15
			DCD     TIMER21_IRQHandler		; IRQ16
			DCD     USART11_IRQHandler		; IRQ17
			DCD     ADC_IRQHandler			; IRQ18
			DCD     SPI20_IRQHandler		; IRQ19
			DCD     SPI21_IRQHandler		; IRQ20
			DCD     TIMER13_IRQHandler		; IRQ21
			DCD     TIMER40_IRQHandler		; IRQ22
			DCD     RTC_IRQHandler			; IRQ23
			DCD     0						; IRQ24
			DCD     0						; IRQ25		
			DCD     USART12_IRQHandler		; IRQ26
			DCD     USART13_IRQHandler		; IRQ27
			DCD     TEMPERATURE_IRQHandler	; IRQ28
			DCD     COMP_IRQHandler		; IRQ29
			DCD     USB_IRQHandler			; IRQ30
			DCD     CRC_IRQHandler			; IRQ31


                AREA    |.text|, CODE, READONLY


; Reset Handler

Reset_Handler   PROC
                EXPORT  Reset_Handler             	[WEAK]
                IMPORT  SystemInit
                IMPORT  __main
;				LDR		R0, =0x20000000 ; RAM bass address
;                LDR		R1, =0x20001000 ; 4KB
;                LDR		R2, =0x00000000 ; Fill Pattern
;				LDR		R3, =4			; increse offset
;				
;RAM_Init_Loop
;                STR		R2, [R0]
;				ADD     R0, R0, R3
;                CMP		R0, R1
;                BNE		RAM_Init_Loop 	
					
		LDR		R0, =SystemInit
                BLX		R0
                LDR		R0, =__main
                BX		R0
                ENDP


; Dummy Exception Handlers (infinite loops which can be modified)

NMI_Handler     PROC
                EXPORT  NMI_Handler			[WEAK]
                B       .
                ENDP
HardFault_Handler		PROC
                EXPORT  HardFault_Handler		[WEAK]
                B       .
                ENDP
SVC_Handler     PROC
                EXPORT  SVC_Handler			[WEAK]
                B       .
                ENDP
PendSV_Handler  PROC
                EXPORT  PendSV_Handler		[WEAK]
                B       .
                ENDP
SysTick_Handler PROC
                EXPORT  SysTick_Handler		[WEAK]
                B       .
                ENDP

NULL_IRQHandler PROC
                EXPORT  LVI_IRQHandler				[WEAK]
                EXPORT  SYSCLKFAIL_IRQHandler		[WEAK]
                EXPORT  WDT_IRQHandler			[WEAK]
                EXPORT  GPIOAB_IRQHandler			[WEAK]		
                EXPORT  GPIOCF_IRQHandler			[WEAK]
                EXPORT  TIMER10_IRQHandler		[WEAK]
                EXPORT  TIMER11_IRQHandler		[WEAK]                
                EXPORT  TIMER12_IRQHandler		[WEAK]
                EXPORT  I2C0_IRQHandler			[WEAK]
                EXPORT  USART10_IRQHandler		[WEAK]
                EXPORT  WT_IRQHandler			[WEAK]
                EXPORT  TIMER30_IRQHandler		[WEAK]
                EXPORT  I2C1_IRQHandler			[WEAK]
                EXPORT  TIMER20_IRQHandler		[WEAK]
                EXPORT  TIMER21_IRQHandler		[WEAK]	
                EXPORT  USART11_IRQHandler		[WEAK]					
                EXPORT  ADC_IRQHandler			[WEAK]		
                EXPORT  SPI20_IRQHandler			[WEAK]		
                EXPORT  SPI21_IRQHandler			[WEAK]	
                EXPORT  TIMER13_IRQHandler		[WEAK]		
                EXPORT  TIMER40_IRQHandler		[WEAK]
                EXPORT  RTC_IRQHandler			[WEAK]
                EXPORT  USART12_IRQHandler		[WEAK]
                EXPORT  USART13_IRQHandler		[WEAK]		
                EXPORT  TEMPERATURE_IRQHandler	[WEAK]		
                EXPORT  COMP_IRQHandler			[WEAK]
                EXPORT  USB_IRQHandler			[WEAK]
                EXPORT  CRC_IRQHandler			[WEAK]		

		
LVI_IRQHandler			; IRQ0
SYSCLKFAIL_IRQHandler		; IRQ1
WDT_IRQHandler			; IRQ2
GPIOAB_IRQHandler		; IRQ3
GPIOCF_IRQHandler		; IRQ4
TIMER10_IRQHandler		; IRQ7
TIMER11_IRQHandler		; IRQ8
TIMER12_IRQHandler		; IRQ9
I2C0_IRQHandler			; IRQ10
USART10_IRQHandler		; IRQ11
WT_IRQHandler			; IRQ12
TIMER30_IRQHandler		; IRQ13
I2C1_IRQHandler			; IRQ14
TIMER20_IRQHandler		; IRQ15
TIMER21_IRQHandler		; IRQ16
USART11_IRQHandler		; IRQ17				
ADC_IRQHandler			; IRQ18		
SPI20_IRQHandler			; IRQ19	
SPI21_IRQHandler			; IRQ20
TIMER13_IRQHandler		; IRQ21		
TIMER40_IRQHandler		; IRQ22
RTC_IRQHandler			; IRQ23
USART12_IRQHandler		; IRQ26
USART13_IRQHandler		; IRQ27		
TEMPERATURE_IRQHandler	; IRQ28	
COMP_IRQHandler			; IRQ29
USB_IRQHandler			; IRQ30
CRC_IRQHandler			; IRQ31
                
                B       .

                ENDP


                ALIGN


; User Initial Stack & Heap

                IF      :DEF:__MICROLIB
                
                EXPORT  __initial_sp
                EXPORT  __heap_base
                EXPORT  __heap_limit
                
                ELSE
                
                IMPORT  __use_two_region_memory
                EXPORT  __user_initial_stackheap

__user_initial_stackheap
                LDR     R0, =  Heap_Mem
                LDR     R1, =(Stack_Mem + Stack_Size)
                LDR     R2, = (Heap_Mem +  Heap_Size)
                LDR     R3, = Stack_Mem
                BX      LR

                ALIGN

                ENDIF


                END
