/**
* @file     A31G32x_hal_comp.h
* @brief    Contains all macro definitions and function prototypes support
*           for comparator driver on A31G32x
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
**/

#ifndef _COMP_H_
#define _COMP_H_

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x.h"
//#include "def.h"
#include "A31G32x_hal_aa_types.h"


#ifdef __cplusplus
extern "C"
{
#endif


/*******************************************************************************
* Public Macro
*******************************************************************************/
/**********************************************************************
 * Macro defines for Macro defines for Comparator control register
 **********************************************************************/
#define COMP_CR_PREF_DISEN    					((uint32_t)(0<<25))   	
#define COMP_CR_PREF_EN     						((uint32_t)(1<<25))   	

#define COMP_CR_WIND_DISEN    					((uint32_t)(0<<24))   	
#define COMP_CR_WIND_EN     						((uint32_t)(1<<24))   	
	
#define COMP_CR_MODSEL_ULPWR    				((uint32_t)(0<<20))   	
#define COMP_CR_MODSEL_LOWPWR   	 			((uint32_t)(1<<20))   	
#define COMP_CR_MODSEL_MEDIUMPWR    		((uint32_t)(2<<20))   	
#define COMP_CR_MODSEL_HIGHSPEED    		((uint32_t)(3<<20))   	
#define COMP_CR_MODSEL_BITMASK 					((uint32_t)(3<<20))   		

#define COMP_CR_HYSSEL_OFF     					((uint32_t)(0<<16))   	
#define COMP_CR_HYSSEL_LOW     					((uint32_t)(1<<16))   	
#define COMP_CR_HYSSEL_MEDIUM     			((uint32_t)(2<<16))   	
#define COMP_CR_HYSSEL_HIGH     				((uint32_t)(3<<16))   	
#define COMP_CR_HYSSEL_BITMASK    		 	((uint32_t)(3<<16))   

#define COMP_CR_DISEN    								((uint32_t)(0<<12))   	
#define COMP_CR_EN     									((uint32_t)(1<<12))   	

#define COMP_CR_INNSEL_CP0N2    			((uint32_t)(0<<4))
#define COMP_CR_INNSEL_CP1N2    			((uint32_t)(0<<4))   		
#define COMP_CR_INNSEL_CP0N0   				((uint32_t)(1<<4))   		
#define COMP_CR_INNSEL_CP1N0   				((uint32_t)(1<<4))   		
#define COMP_CR_INNSEL_CP0N1   				((uint32_t)(2<<4))   		
#define COMP_CR_INNSEL_CP1N1   				((uint32_t)(2<<4))   		
#define COMP_CR_INNSEL_VREF12MV 			((uint32_t)(3<<4)) 
#define COMP_CR_INNSEL_VREF9MV   			((uint32_t)(4<<4))   		
#define COMP_CR_INNSEL_VREF6MV 				((uint32_t)(5<<4))  
#define COMP_CR_INNSEL_VREF3MV   			((uint32_t)(6<<4))   		
#define COMP_CR_INNSEL_DACO3   				((uint32_t)(7<<4))   		
#define COMP_CR_INNSEL_BITMASK 				((uint32_t)(7<<4))   		

#define COMP_CR_BITMASK								((uint32_t)(0x03331070))		

/**********************************************************************
 * Macro defines for Macro defines for Comparator debounce register
 **********************************************************************/
#define COMP_DBNC_C0DBNC						((uint32_t)(0))	
#define COMP_DBNC_C0DBNC_BITMASK		((uint32_t)(0xFuL<<0))	

#define COMP_DBNC_C1DBNC						((uint32_t)(0<<4)) 	
#define COMP_DBNC_C1DBNC_BITMASK		((uint32_t)(0xFuL<<4)) 

#define COMP_DBNC_DBNCTB						((uint32_t)(0<<16)) 	
#define COMP_DBNC_DBNCTB_BITMASK		((uint32_t)(0xFFFFuL<<16)) 	

#define COMP_DBNC_BITMASK						((uint32_t)(0xFFFF00FF))	

/**********************************************************************
 * Macro defines for Macro defines for Comparator Interrupt control register
 **********************************************************************/
#define COMP_ICON_C0IMODE   					((uint32_t)(0<<0))   		
#define COMP_ICON_C0IMODE_BITMASK  		((uint32_t)(3<<0))   		

#define COMP_ICON_C1IMODE     				((uint32_t)(0<<2))   	
#define COMP_ICON_C1IMODE_BITMASK     ((uint32_t)(3<<2))   	

#define COMP_ICON_IPOL0     					((uint32_t)(0<<8))   	
#define COMP_ICON_IPOL0_BITMASK    		((uint32_t)(1<<8))   	

#define COMP_ICON_IPOL1    						((uint32_t)(0<<9))   	
#define COMP_ICON_IPOL1_BITMASK    		((uint32_t)(1<<9))   	

#define COMP_ICON_TPOL0								((uint32_t)(0<<12))   	
#define COMP_ICON_TPOL0_BITMASK				((uint32_t)(1<<12))   

#define COMP_ICON_TPOL1								((uint32_t)(0<<13))		
#define COMP_ICON_TPOL1_BITMASK				((uint32_t)(1<<13))		

#define COMP_ICON_BITMASK							((uint32_t)(0x330F))		

/**********************************************************************
 * Macro defines for Macro defines for Comparator interrupt enable register
 **********************************************************************/
#define COMP_IEN_C0_EN			((uint8_t)(1<<0)) 	
#define COMP_IEN_C1_EN			((uint8_t)(1<<1)) 	
#define COMP_IEN_BITMASK		((uint8_t)(0x03)) 

/**********************************************************************
 * Macro defines for Macro defines for Comparator Interrupt status register
 **********************************************************************/
#define COMP_IST_C0IRQ_ON			((uint8_t)(1<<0)) 	
#define COMP_IST_C1IRQ_ON			((uint8_t)(1<<1)) 	
#define COMP_IST_BITMASK			((uint8_t)(0x03)) 

/**********************************************************************
 * Macro defines for Macro defines for Comparator Interrupt clear register
 **********************************************************************/
#define COMP_ICLR_C0ICLR			((uint8_t)(1<<0)) 	
#define COMP_ICLR_C1ICLR			((uint8_t)(1<<1)) 	
#define COMP_ICLR_BITMASK			((uint8_t)(0x03)) 	



/*******************************************************************************
* Public Typedef
*******************************************************************************/

/**
* @brief Comparator enumeration
*/
 /* Comparator interrupt Flag definition */
enum {
	ICON_LEVEL		= 0,     		
	ICON_RISING_EDGE_INT,		     			
	ICON_FALLING_EDGE_INT,		     		
	ICON_BOTH_EDGE_INT		     			
};
 /* Comparator interrupt polarity(level mode) definition */
enum {
	COMP_INTOUT_HIGH		= 0,   					
	COMP_INTOUT_LOW		 							
};
 /* Comparator trigger output polarity(to trigger other IP) definition */
enum {
	COMP_TRIOUT_NORMAL		= 0,   					
	COMP_TRIOUT_INVERTED		 							
};

 /* Comparator Hysteresis select definition */
enum {
	COMP_HYSSEL_Off	= 0,   					
	COMP_HYSSEL_Low,
	COMP_HYSSEL_Medium,
	COMP_HYSSEL_High	
};

/* Comparator Debounce clock Selection */
enum {
	COMP_DBCLK_SYSTEMCLK = 0,
	COMP_DBCLK_LSI750
};

 /*Comparator input(input -) select definition */
enum {
	COMP_INPUT_CP0_N2	= 0,   					
	COMP_INPUT_CP0_N0,
	COMP_INPUT_CP0_N1,	
	COMP_INPUT_Vref12MV,
	COMP_INPUT_Vref9MV,
	COMP_INPUT_Vref6MV,
	COMP_INPUT_Vref3MV,
	COMP_INPUT_DAC_OUT3
};

/* Comparator mode Selection */
enum {
	COMP_MODE_UL = 0,
	COMP_MODE_Low,
	COMP_MODE_Medium,
	COMP_MODE_High
};

/* Comparator Select definition */
enum {
	COMP0 = 0,
	COMP1
};

/*******************************************************************************
* Exported Public Function
*******************************************************************************/
HAL_Status_Type HAL_COMP_Init(void);
HAL_Status_Type HAL_COMP_DeInit(void);

uint32_t HAL_COMP_GetStatus_IT(uint32_t comp_no);
//uint32_t HAL_COMP_GetIntStatusAll(void);
HAL_Status_Type HAL_COMP_ClearStatus_IT(uint32_t comp_no);
HAL_Status_Type HAL_COMP_ClearStatusAll_IT(void);

HAL_Status_Type HAL_COMP_ConfigHysterisis(uint32_t comp_no, uint32_t hyssel);
HAL_Status_Type HAL_COMP_ConfigInputLevel(uint32_t comp_no, uint32_t inputsel);
HAL_Status_Type HAL_COMP_ConfigDebounce(uint32_t comp_no, uint32_t dbncshift, uint32_t dbncclksel, uint32_t dbnccounter);
HAL_Status_Type HAL_COMP_ConfigInterrupt(uint32_t comp_no, uint32_t tpol, uint32_t ipol, uint32_t mode);

HAL_Status_Type HAL_COMP_InterruptCmd(uint32_t comp_no, FunctionalState NewState);
HAL_Status_Type HAL_COMP_Cmd(uint32_t comp_no,  FunctionalState NewState);

HAL_Status_Type HAL_COMP_SetVoltage(FunctionalState NewState);
HAL_Status_Type HAL_COMP_SetWindowMode(FunctionalState NewState);
HAL_Status_Type HAL_COMP_SetModeLevel(uint32_t comp_no, uint32_t modesel);

#ifdef __cplusplus
}
#endif


#endif /* _COMP_H_ */

/* --------------------------------- End Of File ------------------------------ */
