/**
* @file     A31G32x_hal_crc.h
* @brief    Contains all macro definitions and function prototypes support
*           for crc driver on A31G32x
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

#ifndef _CRCCHECK_H_
#define _CRCCHECK_H_

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x.h"
#include "A31G32x_hal_aa_types.h"


#ifdef __cplusplus
extern "C"
{
#endif

/*******************************************************************************
* Public Macro
*******************************************************************************/
#define CRC_INTR_DIS		(0x0uL<<9)
#define CRC_INTR_EN		(0x1uL<<9)

#define CRC_FLAG_OFF		(0x0uL<<8)
#define CRC_FLAG			(0x1uL<<8)

#define CRC_USER_M		(0x0uL<<7)
#define CRC_AUTO_M		(0x1uL<<7)

#define CRC_CRC			(0x0uL<<5)
#define CRC_CHECKSUM		(0x1uL<<5)

#define CRC_CCITT			(0x0uL<<4)
#define CRC_16			(0x1uL<<4)

#define CRC_MSB			(0x0uL<<1)
#define CRC_LSB			(0x1uL<<1)

#define CRC_InData(u32InData)               (CRC->IN = u32InData)
#define CRCRun()							{CRC->CR |= (1<<0); __NOP(); __NOP(); __NOP(); __NOP(); __NOP(); __NOP(); __NOP(); __NOP();}
#define CRCStop()							(CRC->CR &= ~(1<<0))
#define ChkCRCFinish()                         (CRC->CR&0x1)
/*******************************************************************************
* Public Typedef
*******************************************************************************/

/*******************************************************************************
* Exported Public Function
*******************************************************************************/
HAL_Status_Type HAL_CRC_Init(void);
uint32_t HAL_CRC_AutoGet(uint32_t u32SEL, uint32_t u32POLY, uint32_t u32FirstBit);
HAL_Status_Type HAL_CRC_ConfigUserMode(uint32_t u32SEL, uint32_t u32POLY, uint32_t u32FirstBit);
uint32_t HAL_CRC_UserInput(uint32_t u32Input);
uint32_t HAL_CRC_GetData(void);
uint32_t HAL_CRC_GetState_IT(void);
HAL_Status_Type HAL_CRC_ConfigInterrupt(uint32_t IntSel);
HAL_Status_Type HAL_CRC_ClearState_IT(void);
HAL_Status_Type HAL_CRC_SetUserInput(uint32_t InitVal);
#ifdef __cplusplus
}
#endif


#endif /* _CRCCHECK_H_ */

/* --------------------------------- End Of File ------------------------------ */
