/**
* @file     A31G32x_hal_dac.h
* @brief    Contains all macro definitions and function prototypes support
*           for dac driver on A31G32x
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

#ifndef _DAC_H_
#define _DAC_H_

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x.h"
#include "A31G32x_hal_aa_types.h"


#ifdef __cplusplus
extern "C"
{
#endif

/*******************************************************************************
* Public Macro
*******************************************************************************/
#define DAC_INTDIS				(0x0uL<<7)
#define DAC_INTEN				(0x1uL<<7)

#define DAC_AUTOCHANGEDIS	(0x0uL<<5)
#define DAC_AUTOCHANGEEN		(0x1uL<<5)

#define DAC_REFAVDD			(0x0uL)
#define DAC_REFAVREF			(0x1uL)

#define DAC_BUFFCLR			(0x1uL)

#define DAC_FETCHDIS			(0x0uL)
#define DAC_FETCHEN			(0x1uL)

#define DAC_RLDS_ALWAYS		(0x0uL)
#define DAC_RLDS_TIMER10		(0x2uL)
#define DAC_RLDS_TIMER11		(0x3uL)

#define DAC_STOP				(0x0uL)
#define DAC_START				(0x1uL)

#define DAC_PGAn30dB			(0x0uL)
#define DAC_PGAn24dB			(0x1uL)
#define DAC_PGAn18dB			(0x2uL)
#define DAC_PGAn12dB			(0x3uL)
#define DAC_PGAn6dB			(0x4uL)
#define DAC_PGAn0dB			(0x5uL)
#define DAC_PGAp6dB			(0x6uL)
#define DAC_PGAp12dB			(0x7uL)
#define DAC_PGAp18dB			(0x8uL)
#define DAC_PGAp24dB			(0x9uL)
#define DAC_PGAp30dB			(0xAuL)

#define DAC_OFFSETDIS			(0x0uL)
#define DAC_OFFSETEN			(0x1uL)
#define DAC_OFFSETDIRn		(0x0uL)
#define DAC_OFFSETDIRp		(0x1uL)
#define DAC_OFFSETVAL(n)		(((n)&0xF))

#define DAC_DR_RESULT(n)		((n)&0xFFFF)

/*******************************************************************************
* Public Typedef
*******************************************************************************/
typedef struct {
	// CR
	uint32_t	BufSel;
	uint32_t	RefSel;
	uint32_t	IntSel;
	uint32_t	DataChangeSel;
	uint32_t	ReloadSel;
	
	// PGSR
	uint32_t	PGASel;
	
	// DAOFSCR
	uint32_t	OffsetSel;
	uint32_t	OffsetDIRSel;
	uint32_t	OffsetVal;
}DAC_CFG_Type;

/*******************************************************************************
* Exported Public Function
*******************************************************************************/
HAL_Status_Type HAL_DAC_Init(DAC_Type *DACx, DAC_CFG_Type *DAC_ConfigStruct, uint32_t DAC_OutSel);
HAL_Status_Type HAL_DAC_DeInit(DAC_Type *DACx);
uint16_t	HAL_DAC_GetData(DAC_Type *DACx);
void	HAL_DAC_SetData(DAC_Type *DACx, uint16_t data_value);
HAL_Status_Type HAL_DAC_Start(DAC_Type *DACx, FunctionalState NewState);
HAL_Status_Type HAL_DAC_ConfigInterrup(DAC_Type *DACx, FunctionalState NewState);
uint32_t	HAL_DAC_GetState(DAC_Type *DACx);
HAL_Status_Type HAL_DAC_ClearState(DAC_Type *DACx);
uint16_t	HAL_DAC_GetBuffer(DAC_Type *DACx);
HAL_Status_Type HAL_DAC_SetOffset(uint32_t enable, uint32_t dir, uint32_t value);
HAL_Status_Type HAL_DAC_SetGain(uint32_t gain);

#ifdef __cplusplus
}
#endif

#endif /* _DAC_H_ */

/* --------------------------------- End Of File ------------------------------ */

