/**
* @file     A31G32x_hal_debug_frmwrk.h
* @brief    Contains all macro definitions and function prototypes support
*           for debug_frmwrk on A31G32x
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

#ifndef _DEBUG_FRMWRK_H_
#define _DEBUG_FRMWRK_H_

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x_hal_usartn.h"
#include "A31G32x_hal_libcfg.h"

/*******************************************************************************
* Public Macro
*******************************************************************************/
#ifdef _DEBUG_MSG
#define USED_USART_DEBUG_PORT	0

#if (USED_USART_DEBUG_PORT==0)
#define DEBUG_USART_PORT	USART10
#elif (USED_USART_DEBUG_PORT==1)
#define DEBUG_USART_PORT	USART11
#elif (USED_USART_DEBUG_PORT==2)
#define DEBUG_USART_PORT	USART12
#elif (USED_USART_DEBUG_PORT==3)
#define DEBUG_USART_PORT	USART13
#endif

#define _DBG(x)	 	_db_msg((USART_Type*)DEBUG_USART_PORT, x)
#define _DBG_(x)	_db_msg_((USART_Type*)DEBUG_USART_PORT, x)
#define _DBC(x)	 	_db_char((USART_Type*)DEBUG_USART_PORT, x)
#define _DBD(x)	 	_db_dec((USART_Type*)DEBUG_USART_PORT, x)
#define _DBD16(x)	 _db_dec_16((USART_Type*)DEBUG_USART_PORT, x)
#define _DBD32(x)	 _db_dec_32((USART_Type*)DEBUG_USART_PORT, x)
#define _DBH(x)	 	_db_hex((USART_Type*)DEBUG_USART_PORT, x)
#define _DBH16(x)	 _db_hex_16((USART_Type*)DEBUG_USART_PORT, x)
#define _DBH32(x)	 _db_hex_32((USART_Type*)DEBUG_USART_PORT, x)
#define _DG			_db_get_char((USART_Type*)DEBUG_USART_PORT)
#define _DG_(x)	_db_get_ch((USART_Type*)DEBUG_USART_PORT,x)

/*******************************************************************************
* Public Typedef
*******************************************************************************/

/*******************************************************************************
* Exported Public Function
*******************************************************************************/
void  cprintf(const  char *format, ...);


extern void (*_db_msg)(USART_Type *USARTx, const void *s);
extern void (*_db_msg_)(USART_Type *USARTx, const void *s);
extern void (*_db_char)(USART_Type *USARTx, uint8_t ch);
extern void (*_db_dec)(USART_Type *USARTx, uint8_t decn);
extern void (*_db_dec_16)(USART_Type *USARTx, uint16_t decn);
extern void (*_db_dec_32)(USART_Type *USARTx, uint32_t decn);
extern void (*_db_hex)(USART_Type *USARTx, uint8_t hexn);
extern void (*_db_hex_16)(USART_Type *USARTx, uint16_t hexn);
extern void (*_db_hex_32)(USART_Type *USARTx, uint32_t hexn);
extern uint8_t (*_db_get_char)(USART_Type *USARTx);
extern uint8_t (*_db_get_ch)(USART_Type *USARTx, uint8_t *ch);

void USARTPutChar (USART_Type *USARTx, uint8_t ch);
void USARTPuts(USART_Type *USARTx, const void *str);
void USARTPuts_(USART_Type *USARTx, const void *str);
void USARTPutDec(USART_Type *USARTx, uint8_t decnum);
void USARTPutDec16(USART_Type *USARTx, uint16_t decnum);
void USARTPutDec32(USART_Type *USARTx, uint32_t decnum);
void USARTPutHex (USART_Type *USARTx, uint8_t hexnum);
void USARTPutHex16 (USART_Type *USARTx, uint16_t hexnum);
void USARTPutHex32 (USART_Type *USARTx, uint32_t hexnum);
uint8_t USARTGetChar (USART_Type *USARTx);
uint8_t USARTGetCh(USART_Type *USARTx, uint8_t *ch);
void debug_frmwrk_init(void);

#endif

#endif /* _DEBUG_FRMWRK_H_ */

/* --------------------------------- End Of File ------------------------------ */
