/**
* @file     A31G32x_hal_dmac.h
* @brief    Contains all macro definitions and function prototypes support
*           for dma driver on A31G32x
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

#ifndef _A31G324_DMAC_H_
#define _A31G324_DMAC_H_

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x.h"
#include "A31G32x_hal_aa_types.h"

#ifdef __cplusplus
extern "C"
{
#endif

/*******************************************************************************
* Public Macro
*******************************************************************************/
#define DMAC_CR_DIR_Pos			1										/*!< DMAC CR: DIR Position		*/
#define DMAC_CR_DIR_Msk			(0x01UL << DMAC_CR_DIR_Pos)				/*!< DMAC CR: DIR Mask			*/
#define DMAC_CR_SIZE_Pos			2										/*!< DMAC CR: SIZE Position		*/
#define DMAC_CR_SIZE_Msk			(0x03UL << DMAC_CR_SIZE_Pos)				/*!< DMAC CR: SIZE Mask		*/
#define DMAC_CR_PERISEL_Pos		8										/*!< DMAC CR: PERISEL Position	*/
#define DMAC_CR_PERISEL_Msk		(0x0fUL << DMAC_CR_PERISEL_Pos)			/*!< DMAC CR: PERISEL Mask		*/
#define DMAC_CR_TRANSCNT_Pos		16										/*!< DMAC CR: TRANSCNT Position	*/
#define DMAC_CR_TRANSCNT_Msk		(0x00000fffUL << DMAC_CR_TRANSCNT_Pos)		/*!< DMAC CR: TRANSCNT Mask	*/

#define DMAC_SR_DMAEN_Pos		0										/*!< DMAC SR: DMAEN Position	*/
#define DMAC_SR_DMAEN_Msk		(0x01UL << DMAC_SR_DMAEN_Pos)			/*!< DMAC SR: DMAEN Mask		*/
#define DMAC_SR_EOT_Pos			7										/*!< DMAC SR: EOT Position		*/
#define DMAC_SR_EOT_Msk			(0x01UL << DMAC_SR_EOT_Pos)				/*!< DMAC SR: EOT Mask		*/

#define DMAC_PAR_PAR_Pos			0										/*!< DMAC PAR: PAR Position		*/
#define DMAC_PAR_PAR_Msk			(0xffffffffUL << DMAC_PAR_PAR_Pos)				/*!< DMAC PAR: PAR Mask		*/

#define DMAC_MAR_MAR_Pos			0										/*!< DMAC MAR: MAR Position		*/
#define DMAC_MAR_MAR_Msk			(0x0000ffffUL << DMAC_MAR_MAR_Pos)			/*!< DMAC MAR: MAR Mask		*/

#define DMAC_PERISEL_IDLE			((0UL))  	/**< CHANNEL IDLE */
#define DMAC_PERISEL_SPI20RX		((1UL))  	/**< SPI0 RX */
#define DMAC_PERISEL_SPI20TX		((2UL))  	/**< SPI0 TX */
#define DMAC_PERISEL_SPI21RX		((3UL))  	/**< SPI1 RX */
#define DMAC_PERISEL_SPI21TX		((4UL))  	/**< SPI1 TX */
#define DMAC_PERISEL_USART10RX	((5UL))  	/**< USART0 RX */
#define DMAC_PERISEL_USART10TX	((6UL))  	/**< USART0 TX */
#define DMAC_PERISEL_USART11RX	((7UL))  	/**< USART1 RX */
#define DMAC_PERISEL_USART11TX	((8UL))  	/**< USART1 TX */
#define DMAC_PERISEL_USART12RX	((9UL))  	/**< USART2 RX */
#define DMAC_PERISEL_USART12TX	((10UL)) 	/**< USART2 TX */
#define DMAC_PERISEL_USART13RX	((11UL)) 	/**< USART3 RX */
#define DMAC_PERISEL_USART13TX	((12UL)) 	/**< USART3 TX */ 
#define DMAC_PERISEL_CRC			((13UL)) 	/**< CRC */
#define DMAC_PERISEL_ADC			((14UL)) 	/**< ADC */

#define DMAC_WIDTH_BYTE			((0UL))	/**< Width = 1 byte */
#define DMAC_WIDTH_HALFWORD		((1UL))	/**< Width = 2 bytes */
#define DMAC_WIDTH_WORD			((2UL))	/**< Width = 4 bytes */

#define DMAC_DIR_TX				((0UL))	/**< TX */
#define DMAC_DIR_RX				((1UL))	/**< RX */

/*******************************************************************************
* Public Typedef
*******************************************************************************/
/**
 * @brief DMAC Channel configuration structure type definition
 */
typedef struct {
	uint32_t TransferCnt;		/** Length/Size of transfer */
	uint32_t PeriSel;			/** Peripheral selction */	
	uint32_t TransferWidth;		/** Transfer width */
	uint32_t Dir;				/** Select transfer direction */	
	uint32_t PeriAddr;			/** Peri address */
	uint32_t MemAddr;			/** Mem address */	
} DMAC_Ch_CFG_Type;

/*******************************************************************************
* Exported Public Function
*******************************************************************************/
HAL_Status_Type HAL_DMAC_Init(void);
HAL_Status_Type HAL_DMAC_Setup(DMAC_Type *DMACx, DMAC_Ch_CFG_Type *DMACChConfig);
FlagStatus HAL_DMAC_GetState(DMAC_Type *DMACx);
HAL_Status_Type HAL_DMAC_ChCmd(DMAC_Type *DMACx, FunctionalState NewState);

#ifdef __cplusplus
}
#endif

#endif /* _A31G324_DMAC_H_ */

/* --------------------------------- End Of File ------------------------------ */
