/**
* @file     A31G32x_hal_ebi.h
* @brief    Contains all macro definitions and function prototypes support
*           for ebi driver on A31G32x
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

#ifndef _EBI_H_
#define _EBI_H_

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x.h"
#include "A31G32x_hal_aa_types.h"


#ifdef __cplusplus
extern "C"
{
#endif

/*******************************************************************************
* Public Macro
*******************************************************************************/
#define	IDLE0		(0UL)			/** No additional cycle */
#define	IDLE1		(1UL)			/** Insert 1 IDLE cycle */
#define	IDLE2		(2UL)			/** Insert 2 IDLE cycle */
#define	IDLE3		(3UL)			/** Insert 3 IDLE cycle */

#define	EXTEND0		(0UL)			/** No additional cycle */
#define	EXTEND1		(1UL)			/** Extend 1 clock cycle */
#define	EXTEND2		(2UL)			/** Extend 2 clock cycle */
#define	EXTEND3		(3UL)			/** Extend 3 clock cycle */

#define	EWDIS		(0UL)			/** Ignore external input nWAIT */
#define	EWEN		(1UL)			/** Enable external input nWAIT request */

#define	BLDIS		(0UL)			/** Disable byte lane selection in case of 8bit memory organization */
#define	BLEN		(1UL)			/** Enable byte lane selection in case of 8bit memory organization */

#define	IODIS		(0UL)			/** Separated Address/Data access */
#define	IOE			(1UL)			/** Enable muxed Address/Data access */

#define	EWPLOW		(0UL)			/** External nWait Polarity Low */
#define	EWPHIGH		(1UL)			/** External nWait Polarity High */

#define	Clock1		(0UL)			/** Keep the address for 1 clock cycle */
#define	Clock2		(1UL)			/** Keep the address for 2 clock cycle */
#define	Clock3		(2UL)			/** Keep the address for 3 clock cycle */
#define	Clock4		(3UL)			/** Keep the address for 4 clock cycle */

#define	MWIDTH8		(0UL)			/** External memory bus is 8bit width */
#define	MWIDTH16	(1UL)			/** External memory bus is 16bit width */

/*******************************************************************************
* Public Typedef
*******************************************************************************/
typedef struct {
	uint32_t InsertIDLE;					/** Insert IDLE cycle at memory access time */
	uint32_t FallingTiming;				/** Changing falling timing */
	uint32_t ExternalWaitEnable;			/** Peripheral selction */	
	uint32_t WaitSignalPolarity;			/** External nWait Polarity */
	uint32_t AddressMuxed;				/** Enable muxed Address/Data access */
	uint32_t LaneSelection;				/** Enable byte lane selection in case of 8bit memory organization */
	uint32_t KeepAddress;				/** Address keeps the value for AH + 1 clock (HCLK) cycle. */
	uint32_t MemoryWidth;				/** Select transfer direction */	
	uint32_t NORMWAIT;					/** It represents wait value to be used for memory read/write operation. */	
} EBI_CFG_Type;	

/*******************************************************************************
* Exported Public Function
*******************************************************************************/
HAL_Status_Type HAL_EBI_Init(void);
HAL_Status_Type HAL_EBI_DeInit(void);
HAL_Status_Type HAL_EBI_Start(void);
HAL_Status_Type HAL_EBI_Stop(void);
HAL_Status_Type HAL_EBI_Setup(uint32_t BANKn, EBI_CFG_Type *EBIConfig);

#ifdef __cplusplus
}
#endif


#endif /* _EBI_H_ */

/* --------------------------------- End Of File ------------------------------ */
