/**
* @file     A31G32x_hal_pcu.h
* @brief    Contains all macro definitions and function prototypes support
*           for pcu driver on A31G32x
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

#ifndef _PCU_H_
#define _PCU_H_

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x.h"
#include "A31G32x_hal_aa_types.h"

#ifdef __cplusplus
extern "C"
{
#endif

/*******************************************************************************
* Public Macro
*******************************************************************************/
#define PORT_ACCESS_EN()    do { PCU->PORTEN=0x15; PCU->PORTEN=0x51; } while(0)
#define PORT_ACCESS_DIS()  do { PCU->PORTEN=0x00; } while(0) 

#define PCU_FUNC_Msk                        (0x0FUL)                            
#define PCU_MODE_Msk                        (0x03UL)     

/* Pin function */
#define FUNC0 			0x0				/** Function 0 	*/
#define FUNC1 			0x1				/** Function 1 	*/
#define FUNC2 			0x2				/** Function 2	*/
#define FUNC3 			0x3				/** Function 3	*/
#define FUNC4 			0x4				/** Function 4	*/
#define FUNC5 			0x5				/** Function 5 	*/
#define FUNC6 			0x6				/** Function 6 	*/
#define FUNC7 			0x7				/** Function 7	*/
#define FUNC8 			0x8				/** Function 8	*/
#define FUNC9 			0x9				/** Function 9	*/
#define FUNC10 			0xA			/** Function 10	*/

/*******************************************************************************
* Public Typedef
*******************************************************************************/
/*
 * @brief 	PCU port mode enumerate definition
 */
typedef enum {
	INPUT=0,
	PUSH_PULL_OUTPUT,
	ALTERN_FUNC,	
	OPEN_DRAIN_OUTPUT
}PCU_PORT_MODE;


#define DISPUPD			0x0				/** disable pu_pd */
#define ENPU   			0x1				/** enable pull up */
#define ENPD   	 		0x2				/** enable pull down */


enum {
	IER_DISABLE = 0,
	IER_LEVEL,
	IER_EDGE = 3
};

enum {
	ICR_PROHIBIT_INT = 0,
	ICR_LOW_LEVEL_INT = 1,
	ICR_FALLING_EDGE_INT = ICR_LOW_LEVEL_INT,
	ICR_HIGH_LEVEL_INT =2,
	ICR_RISING_EDGE_INT = ICR_HIGH_LEVEL_INT,
	ICR_BOTH_EDGE_INT =3	
};


/* 
 * @brief   GPIO Init structure definition  
 */
typedef struct
{
  uint32_t Pin;       /*!< Specifies the GPIO pins to be configured.
                           This parameter can be any value of @ref PCU_pins */

  uint32_t Output;      /*!< Specifies the operating mode for the selected pins.
                           This parameter can be a value of @ref PCU_ Output */

  uint32_t Pull;      /*!< Specifies the Pull-up or Pull-Down activation for the selected pins.
                           This parameter can be a value of @ref PCU_pull */

  uint32_t Function;  /*!< Peripheral to be connected to the selected pins 
                            This parameter can be a value of @ref function_selection */
}GPIO_CFG_Type;



/*******************************************************************************
* Exported Public Function
*******************************************************************************/
void HAL_GPIO_Init(PORT_Type *PCx, GPIO_CFG_Type *GPIO_Init);
void HAL_GPIO_SetPin(PORT_Type *Px, uint16_t bitValue);
void HAL_GPIO_ClearPin(PORT_Type *Pn, uint16_t bitValue);
uint16_t HAL_GPIO_ReadPin(PORT_Type *Pn);
void HAL_GPIO_WritePin(PORT_Type *Pn, uint16_t Value);

void HAL_GPIO_ConfigFunction(PORT_Type *PCx, uint8_t pin_no, uint32_t func);
void HAL_GPIO_ConfigOutput(PORT_Type *PCx, uint8_t pin_no, PCU_PORT_MODE dir_type);
void HAL_GPIO_ConfigPullup (PORT_Type *PCx, uint8_t pin_no, uint8_t pullupdown);
void HAL_GPIO_ConfigureOutDataMask (PORT_Type *PCx, uint8_t pin_no, FunctionalState maskctrl);

void HAL_GPIO_EXTI_Config(PORT_Type *PCx, uint8_t pin_no, uint8_t pin_en, uint8_t int_mode);
uint32_t HAL_GPIO_EXTI_GetState(PORT_Type *PCx);
void HAL_GPIO_EXTI_ClearPin(PORT_Type *PCx, uint32_t value);

void HAL_GPIO_SetDebouncePin(PORT_Type *Pn, uint32_t u32Pins);
void HAL_PC13_RTC_OUT(uint8_t u8Pins);
#ifdef __cplusplus
}
#endif

#endif /* end _PCU_H_ */

/* --------------------------------- End Of File ------------------------------ */
