/**
* @file     A31G32x_hal_rtc.h
* @brief    Contains all macro definitions and function prototypes support
*           for rtc driver on A31G32x
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

#ifndef _RTC_H_
#define _RTC_H_

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x.h"
#include "A31G32x_hal_aa_types.h"

#ifdef __cplusplus
extern "C"
{
#endif

/*******************************************************************************
* Public Macro
*******************************************************************************/
#define AMPM_24HOUR	1
#define AMPM_12HOUR	0
	
#define TIMESTAMP_FALLING_EDGE	0
#define TIMESTAMP_RISING_EDGE	1
#define TIMESTAMP_BOTH_EDGE	2

#define	AMPM_24_hour_00	0x00
#define	AMPM_24_hour_01	0x01
#define	AMPM_24_hour_02	0x02
#define	AMPM_24_hour_03	0x03
#define	AMPM_24_hour_04	0x04
#define	AMPM_24_hour_05	0x05
#define	AMPM_24_hour_06	0x06
#define	AMPM_24_hour_07	0x07
#define	AMPM_24_hour_08	0x08
#define	AMPM_24_hour_09	0x09
#define	AMPM_24_hour_10	0x10
#define	AMPM_24_hour_11	0x11
#define	AMPM_24_hour_12	0x12
#define	AMPM_24_hour_13	0x13
#define	AMPM_24_hour_14	0x14
#define	AMPM_24_hour_15	0x15
#define	AMPM_24_hour_16	0x16
#define	AMPM_24_hour_17	0x17
#define	AMPM_24_hour_18	0x18
#define	AMPM_24_hour_19	0x19
#define	AMPM_24_hour_20	0x20
#define	AMPM_24_hour_21	0x21
#define	AMPM_24_hour_22	0x22
#define	AMPM_24_hour_23	0x23

#define	AMPM_12_hour_00	0x12
#define	AMPM_12_hour_01	0x01
#define	AMPM_12_hour_02	0x02
#define	AMPM_12_hour_03	0x03
#define	AMPM_12_hour_04	0x04
#define	AMPM_12_hour_05	0x05
#define	AMPM_12_hour_06	0x06
#define	AMPM_12_hour_07	0x07
#define	AMPM_12_hour_08	0x08
#define	AMPM_12_hour_09	0x09
#define	AMPM_12_hour_10	0x10
#define	AMPM_12_hour_11	0x11
#define	AMPM_12_hour_12	0x32
#define	AMPM_12_hour_13	0x21
#define	AMPM_12_hour_14	0x22
#define	AMPM_12_hour_15	0x23
#define	AMPM_12_hour_16	0x24
#define	AMPM_12_hour_17	0x25
#define	AMPM_12_hour_18	0x26
#define	AMPM_12_hour_19	0x27
#define	AMPM_12_hour_20	0x28
#define	AMPM_12_hour_21	0x29
#define	AMPM_12_hour_22	0x30
#define	AMPM_12_hour_23	0x31

#define	ALARMWEEK_0		(1<<0)
#define	ALARMWEEK_1		(1<<1)
#define	ALARMWEEK_2		(1<<2)
#define	ALARMWEEK_3		(1<<3)
#define	ALARMWEEK_4		(1<<4)
#define	ALARMWEEK_5		(1<<5)
#define	ALARMWEEK_6		(1<<6)


/*******************************************************************************
* Public Typedef
*******************************************************************************/
typedef enum {
	Period_disable=0,
	Period_500ms,
	Period_1s,
	Period_1minute,
	Period_1hour,
	Period_1day,
	Preiod_1month	
} RTC_PERIODE;
	
typedef struct{
	uint8_t RTC_Sec;
	uint8_t RTC_Min;
	uint8_t RTC_Day;
	uint8_t RTC_Week;
	uint8_t RTC_Month;
	uint8_t RTC_Hour;
	uint8_t RTC_Year;
	uint32_t RTC_SUBDR;
} RTC_CONFIGURE_Type;

typedef struct{
	uint8_t RTC_ALARMWW;
	uint8_t RTC_ALARMWH;
	uint8_t RTC_ALARMWM;	
} RTC_Alarm_Type;

typedef struct{
	uint8_t RTC_DEV;
	uint8_t RTC_DIR;
	uint8_t RTC_COREN;
	uint32_t RTC_CORDATA;	
}RTC_ERRCOR_Type;


/*******************************************************************************
* Exported Public Function
*******************************************************************************/
HAL_Status_Type HAL_RTC_init(void);	
HAL_Status_Type HAL_RTC_DeInit(void);
HAL_Status_Type HAL_RTC_Start(FunctionalState NewState);
HAL_Status_Type HAL_RTC_ConfigInterrupt(void *RTC_ConfigStruct);
uint32_t HAL_RTC_GetMatch_IT(void);
HAL_Status_Type HAL_RTC_ClearMatch_IT(void);
uint32_t HAL_RTC_GetTS_IT(void);
HAL_Status_Type HAL_RTC_ClearTS_IT(void);
uint32_t HAL_RTC_GetAlarm_IT(void);
void HAL_RTC_ClearAlarm_IT(void);
void HAL_RTC_SetRwalt(void);
HAL_Status_Type HAL_RTC_ClearRwalt(void);
HAL_Status_Type HAL_RTC_SetConstantPeriod(uint8_t period);
HAL_Status_Type HAL_RTC_SetAmpm(uint8_t state);
HAL_Status_Type HAL_RTC_SetAlarm(void *RTC_ALARM_Config, FunctionalState NewState);
HAL_Status_Type HAL_RTC_SetTimestamp(FunctionalState NewState, uint8_t edge);
HAL_Status_Type HAL_RTC_SetErrcor(void *RTC_ERRCOR_Config);
uint8_t HAL_RTC_Get_YEAR(void);
uint8_t HAL_RTC_Get_MONTH(void);
uint8_t HAL_RTC_Get_WEEK(void);
uint8_t HAL_RTC_Get_DAY(void);
uint8_t HAL_RTC_Get_HOUR(void);
uint8_t HAL_RTC_Get_MIN(void);
uint8_t HAL_RTC_Get_SEC(void);
uint32_t HAL_RTC_GetTS_Time(void);
uint32_t HAL_RTC_GetTS_Date(void);
uint8_t HAL_RTC_AMPM_Display(uint8_t hour);
void HAL_RTC_Print(void);
void HAL_RTC_Time_Print(void);

#ifdef __cplusplus
}
#endif

#endif /* end _RTC_H_ */

