/**
* @file     A31G32x_hal_scu.h
* @brief    Contains all macro definitions and function prototypes support
*           for scu driver on A31G32x
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

#ifndef _SCU_H_
#define _SCU_H_

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x.h"
#include "A31G32x_hal_aa_types.h"

#ifdef __cplusplus
extern "C"
{
#endif

/*******************************************************************************
* Public Macro
*******************************************************************************/
/* External sub crystal oscillator control */
#define EXSOSC_STOP				0x0
#define EXSOSC_EN				0x8
#define EXSOSC_EN_DIV2			0x9
#define EXSOSC_EN_DIV4			0xa
#define EXSOSC_EN_DEEPSLEEP		0xC
#define EXSOSC_EN_DEEPSLEEP_DIV2	0xD
#define EXSOSC_EN_DEEPSLEEP_DIV4	0xE

/* External sub crystal oscillator stable timeout */
#define SOSC_STARTUP_TIMEOUT 0xFFFFFFFFUL

/* low speed oscillator control */
#define LSIOSC_STOP		0x0				
#define LSIOSC_EN			0x8				
#define LSIOSC_EN_DIV2		0x9		
#define LSIOSC_EN_DIV4		0xa
	
/* Internal high speed oscillator control */
#define HSIOSC_STOP		0x0				
#define HSIOSC_EN			0x8				
#define HSIOSC_EN_DIV2		0x9		
#define HSIOSC_EN_DIV4		0xa					
#define HSIOSC_EN_DIV8		0xb		
#define HSIOSC_EN_DIV16	0xc
#define HSIOSC_EN_DIV32	0xd

/* External crystal oscillator control */
#define EXOSC_STOP		0x0				
#define EXOSC_EN			0x8				
#define EXOSC_EN_DIV2		0x9		
#define EXOSC_EN_DIV4		0xa

/* External crystal oscillator stable timeout */
#define MOSC_STARTUP_TIMEOUT	0xFFFFFFFFUL

/* PLL stable timeout */
#define PLL_STARTUP_TIMEOUT  8 /* Time out start up, in 1ms */   

/* PLLCON[BYPASS] */
#define PLLCON_BYPASS_FIN		0
#define PLLCON_BYPASS_PLL	1

/* PLLCON[PREDIV] */
#define PLLCON_FIN_DIV_1 0
#define PLLCON_FIN_DIV_2 1

/* PLLCON[FBCTRL] */
#define PLLCON_M_6	0
#define PLLCON_M_8	1
#define PLLCON_M_10	2
#define PLLCON_M_12	3
#define PLLCON_M_16	4
#define PLLCON_M_18	5
#define PLLCON_M_20	6
#define PLLCON_M_26	7
#define PLLCON_M_32	8
#define PLLCON_M_36	9
#define PLLCON_M_40	10
#define PLLCON_M_64	11

/* PLLCON[POSTDIV] */
#define PLLCON_N_1	0
#define PLLCON_N_2	1
#define PLLCON_N_3	2
#define PLLCON_N_4	3
#define PLLCON_N_6	4
#define PLLCON_N_8	5
#define PLLCON_N_16	7

/* system clock control setting */
#define SC_LSIOSC		0x0
#define SC_SOSC		0x1
#define SC_HSIOSC		0x2
#define SC_HSIOSCPLL	0x3 
#define SC_EXOSC		0x6
#define SC_EXOSCPLL	0x7

#define SC_FIN_HSI   0x0
#define SC_FIN_HSE   0x1

/* SCU Reset Source definition */
#define RST_PINRST     		(1UL<<6)
#define RST_CPURST    	 	(1UL<<5)
#define RST_SWRST     		(1UL<<4)
#define RST_WDTRST     	(1UL<<3)
#define RST_MCKFRST       	(1UL<<2)
#define RST_MOFRST        	(1UL<<1)
#define RST_LVDRST        	(1UL<<0)

#define WAKEUP_I2C1	 	(1UL<<15)
#define WAKEUP_I2C0	 	(1UL<<14)
#define WAKEUP_GPIOF	 	(1UL<<13)
#define WAKEUP_USART13	(1UL<<12)
#define WAKEUP_USART12	(1UL<<11)
#define WAKEUP_GPIOC     	(1UL<<10)
#define WAKEUP_GPIOB    	(1UL<<9)
#define WAKEUP_GPIOA     	(1UL<<8)
#define WAKEUP_COMP     	(1UL<<6)
#define WAKEUP_RTC	     	(1UL<<5)
#define WAKEUP_USART11	(1UL<<4)
#define WAKEUP_USART10	(1UL<<3)
#define WAKEUP_WT	        (1UL<<2)
#define WAKEUP_WDT		(1UL<<1)
#define WAKEUP_LVD		(1UL<<0)


/* T1N SCU Clock Constant Definition */
#define T1NCLK_MCCR1		0
#define T1NCLK_PCLK		1

/* T20 SCU Clock Constant Definition */
#define T20CLK_MCCR2		0
#define T20CLK_PCLK		1

/* T21 SCU Clock Constant Definition */
#define T21CLK_MCCR6		0
#define T21CLK_PCLK		1

/*  T30 SCU Clock Constant Definition */
#define T30CLK_MCCR2		0
#define T30CLK_PCLK		1

/*  T40 SCU Clock Constant Definition */
#define T40CLK_MCCR6		0
#define T40CLK_PCLK		1

/*  USB SCU Clock Constant Definition */
#define USBCLK_MCCR6		0
#define USBCLK_PCLK		1

/*  Watch Timer Clock Constant Definition */
#define WTCLK_MCCR3		0
#define WTCLK_SOSC		1
#define WTCLK_WDTRC		2

/*  Watch-Dog Timer Clock Constant Definition */
#define WDTCLK_LSI40kHz	0
#define WDTCLK_MCCR3		1

/*  RTC Clock Constant Definition */
#define RTCCLK_LSI40kHz	0
#define RTCCLK_MCCR5		1


/* LVI Definition */
#define LVI_DISABLE			(0x0uL<<7)
#define LVI_ENABLE				(0x1uL<<7)

#define LVI_INTDIS				(0x0uL<<5)
#define LVI_INTEN				(0x1uL<<5)

#define LVI_LVL_1_60V			(0x0uL<<0)
#define LVI_LVL_1_69V			(0x1uL<<0)
#define LVI_LVL_1_78V			(0x2uL<<0)
#define LVI_LVL_1_90V			(0x3uL<<0)
#define LVI_LVL_1_99V			(0x4uL<<0)
#define LVI_LVL_2_12V			(0x5uL<<0)
#define LVI_LVL_2_30V			(0x6uL<<0)
#define LVI_LVL_2_47V			(0x7uL<<0)
#define LVI_LVL_2_67V			(0x8uL<<0)
#define LVI_LVL_3_04V			(0x9uL<<0)
#define LVI_LVL_3_18V			(0xAuL<<0)
#define LVI_LVL_3_59V			(0xBuL<<0)
#define LVI_LVL_3_72V			(0xCuL<<0)
#define LVI_LVL_4_03V			(0xDuL<<0)
#define LVI_LVL_4_20V			(0xEuL<<0)
#define LVI_LVL_4_48V			(0xFuL<<0)

#define LVR_LVL_1_60V			(0xFuL<<0)
#define LVR_LVL_1_69V			(0xEuL<<0)
#define LVR_LVL_1_78V			(0xDuL<<0)
#define LVR_LVL_1_90V			(0xCuL<<0)
#define LVR_LVL_1_99V			(0xBuL<<0)
#define LVR_LVL_2_12V			(0xAuL<<0)
#define LVR_LVL_2_30V			(0x9uL<<0)
#define LVR_LVL_2_47V			(0x8uL<<0)
#define LVR_LVL_2_67V			(0x7uL<<0)
#define LVR_LVL_3_04V			(0x6uL<<0)
#define LVR_LVL_3_18V			(0x5uL<<0)
#define LVR_LVL_3_59V			(0x4uL<<0)
#define LVR_LVL_3_72V			(0x3uL<<0)
#define LVR_LVL_4_03V			(0x2uL<<0)
#define LVR_LVL_4_20V			(0x1uL<<0)
#define LVR_LVL_4_48V			(0x0uL<<0)


/*******************************************************************************
* Public Typedef
*******************************************************************************/  
enum {
	SYSTICK_TYPE=0,
	TIMER1N_TYPE,
	TIMER20_TYPE,
	TIMER21_TYPE,
	TIMER30_TYPE,
	TIMER40_TYPE	,
	WDT_TYPE,
	WT_TYPE,
	PD_TYPE,
	USB_TYPE,
	RTC_TYPE,
	USE_TYPE,
	TSREF_TYPE,
	TSSENSE_TYPE,
	LSI40K_TYPE
};

enum {
	LSI_MISC_CLOCK=0,
	LSE_MISC_CLOCK=3,	
	MCLK_MISC_CLOCK=4,
	HSI_MISC_CLOCK=5,	
	HSE_MISC_CLOCK=6,
	PLL_MISC_CLOCK=7,
  NONE_MISC_CLOCK,	
};

enum {
	CLKSRC_LSI=0,
	CLKSRC_SOSC=3,	
	CLKSRC_MCLK=4,
	CLKSRC_HSI=5,	
	CLKSRC_MOSC=6,
	CLKSRC_PLL=7	
};

enum {
	LVD_LEVEL_1_65 = 0,
	LVD_LEVEL_2_5,
	LVD_LEVEL_3_6,
	LVD_LEVEL_4_2
};

enum{
	TSSENSECLK_LSITS=0,
	TSSENSECLK_LSI750kHz,
	TSSENSECLK_LSI40kHz,
	TSSENSECLK_HSI_I
};

typedef struct {
	uint32_t	EnSel;
	uint32_t	IntSel;	
	uint32_t	LvlSel;
} LVI_CFG_Type;

enum{
	TSREFCLK_HSI_I=0,
	TSREFCLK_MCLK,
	TSREFCLK_MOSC,
	TSREFCLK_SOSC
};


/*******************************************************************************
* Exported Public Function
*******************************************************************************/
void HAL_SCU_LSE_ClockConfig(uint32_t esosccon);
Status HAL_SCU_LSE_ClockMonitoring(void);

void HAL_SCU_LSI_ClockConfig(uint32_t ringosc);	// in ring osc set 
void HAL_SCU_HSI_ClockConfig(uint32_t hsiosc);	// hsi osc set

void HAL_SCU_HSE_ClockConfig(uint32_t eosccon);	// exosc set
Status HAL_SCU_HSE_ClockMonitoring(void);	// check for MOSC stable 

void HAL_SCU_SystemClockChange(uint32_t sysclkcon); // set system clock ==> final MCLK
void HAL_SCU_SystemClockFinClock(uint32_t sysclkcon);
void HAL_SCU_ClockOutput(uint8_t divval, FunctionalState endis);
Status HAL_SCU_PLL_ClockConfig(FunctionalState pllsetstate, uint8_t selbypass, uint8_t selfin,  uint8_t PREDIV, uint8_t POSTDIV1, uint8_t POSTDIV2, uint8_t OUTDIV);

void HAL_SCU_WDTLSI40K_ClockConfig(uint32_t lsi40clk);
void HAL_SCU_TSLSI40K_ClockConfig(uint32_t lists);
void HAL_SCU_TempSensor_ClockConfig(uint32_t tempsensor,  uint32_t clksel);

void HAL_SCU_MiscClockConfig( uint8_t mccrnum, uint8_t type, uint8_t clksrc, uint8_t clkdiv);

void HAL_SCU_WakeUpSRCCmd(uint32_t WakeUpSrc, FunctionalState NewState);
uint32_t HAL_SCU_GetWakeUpSRCStatus(void);
void HAL_SCU_ClearWakeUpSRC(uint32_t WakeUpStatus);

void HAL_SCU_LVRCmd(FunctionalState NewState);
void HAL_SCU_SetLVRLevel(uint32_t lvdlevel);

void HAL_SCU_LVICmd(LVI_CFG_Type *LVI_ConfigStruct);
uint32_t HAL_SCULV_GetLviFlag(void);
void HAL_SCULV_ClrLviFlag(void);

uint32_t HAL_SCU_GetNMIStatus(void);
void HAL_SCU_SetNMI(uint32_t nmisrc);

	
void HAL_SCU_SetResetSrc(uint32_t RstSrc, FunctionalState NewState);
uint32_t HAL_SCU_GetResetSrc(void);
void HAL_SCU_ClearResetStatus(uint32_t rststatus);


void HAL_SCU_Timer1n_ClockConfig(uint32_t t1nclk);
void HAL_SCU_Timer20_ClockConfig(uint32_t t20clk);
void HAL_SCU_Timer21_ClockConfig(uint32_t t21clk);
void HAL_SCU_Timer30_ClockConfig(uint32_t t30clk);
void HAL_SCU_Timer40_ClockConfig(uint32_t t40clk);
void HAL_SCU_USB_ClockConfig(uint32_t usbclk);
void HAL_SCU_WT_ClockConfig(uint32_t wtclk);
void HAL_SCU_WDT_ClockConfig(uint32_t wdtclk);
void HAL_SCU_RTC_ClockConfig(uint32_t rtcclk);
	

#ifdef __cplusplus
}
#endif

#endif /* end _SCU_H_ */

/* --------------------------------- End Of File ------------------------------ */
