/**
* @file     A31G32x_hal_tempsensor.h
* @brief    Contains all macro definitions and function prototypes support
*           for spi driver on A31G32x
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

#ifndef _TEMPSENSOR_H_
#define _TEMPSENSOR_H_

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x.h"
#include "A31G32x_hal_aa_types.h"

#ifdef __cplusplus
extern "C"
{
#endif

/*******************************************************************************
* Public Macro
*******************************************************************************/

/** Temp sensor enable bit */
#define TSENSECON_TSSEN			    				((uint32_t)(1<<0))
#define TSENSECON_ENABLE_BITMASK  			0x0001

/** Temp sensor start bit */
#define TSENSECON_TSSTART			  				((uint32_t)(1<<1))
#define TSENSECON_START_BITMASK   			0x0002

/** Temp sensor match flag bit */
#define TSENSECON_TSSFLAG			  				((uint32_t)(1<<2))
#define TSENSECON_FLAG_BITMASK   				0x0004

/** Temp sensor interrupt enable bit */
#define TSENSECON_TSSINTEN			  			((uint32_t)(1<<3))
#define TSENSECON_INTERRUPT_BITMASK   	0x0008


/*******************************************************************************
* Public Typedef
*******************************************************************************/


/*******************************************************************************
* Exported Public Function
*******************************************************************************/
HAL_Status_Type HAL_TS_Init(void);
HAL_Status_Type HAL_TS_Deinit(void);
HAL_Status_Type HAL_TS_Start(FunctionalState NewState);
HAL_Status_Type HAL_TS_ClearStatus(void);
HAL_Status_Type HAL_TS_ConfigInterrupt(FunctionalState NewState);
uint8_t HAL_TS_GetStatus(void);
//void TSensor_SetReferencePeriod(uint32_t period);
HAL_Status_Type HAL_TS_SetReferencePeriod(void);
uint32_t HAL_TS_GetReferencePeroid(void);
uint16_t HAL_TS_GetSenseCount(void);
float HAL_TS_ConvertToCelsius(void);


#ifdef __cplusplus
}
#endif

#endif /*  */
