/**
* @file     A31G32x_hal_timer30.h
* @brief    Contains all macro definitions and function prototypes support
*           for timer30 driver on A31G32x
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

#ifndef _TIMER30_H_
#define _TIMER30_H_

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x.h"
#include "A31G32x_hal_aa_types.h"
#include "A31G32x_hal_timer_common.h"

#ifdef __cplusplus
extern "C"
{
#endif

/*******************************************************************************
* Public Macro
*******************************************************************************/
/* Timer 30 Enable/Disable Definition */
#define T30_DISABLE				(0x0uL<<15)
#define T30_ENABLE				(0x1uL<<15)

/* Timer 30 Clock Selection Definition  */
#define T30_CLKINT					(0x0uL<<14)
#define T30_CLKEXT				(0x1uL<<14)

/* Timer 30 Mode Selection Definition  */
#define T30_INVM					(0x0uL<<12)
#define T30_CAPM					(0x1uL<<12)
#define T30_BTOB					(0x2uL<<12)

/* Timer 30 External Clock Edge Selection Definition  */
#define T30_FEDGE					(0x0uL<<11)
#define T30_REDGE					(0x1uL<<11)

/* Timer 30 Output Mode Selection Definition  */
#define T30_6CHMOD				(0x0uL<<10)
#define T30_FORAMOD				(0x1uL<<10)

/* Timer 30 Delay Time Insert En/Disable Definition  */
#define T30_DLYINSDIS				(0x0uL<<9)
#define T30_DLYINSEN				(0x1uL<<9)

/* Timer 30 Delay Timer Insertion Position Definition  */
#define T30_INSFRONT				(0x0uL<<8)
#define T30_INSBACK				(0x1uL<<8)

/* Timer30 Capture Polarity Selection Definition  */
#define T30_CAPFALL				(0x0uL<<6)
#define T30_CAPRISE				(0x1uL<<6)
#define T30_CAPBOTH				(0x2uL<<6)

/* Timer 30 Data Reload Time Selection Definition  */
#define T30_UPWRITE				(0x0uL<<4)
#define T30_UPMATCH				(0x1uL<<4)
#define T30_UPBOTTOM				(0x2uL<<4)

/* Timer 30 Period Match Interrupt Occurrence Selection Definition  */
#define T30_E1PERIOD				(0x00uL<<0)
#define T30_E2PERIOD				(0x01uL<<0)
#define T30_E3PERIOD				(0x02uL<<0)
#define T30_E4PERIOD				(0x03uL<<0)
#define T30_E5PERIOD				(0x04uL<<0)
#define T30_E6PERIOD				(0x05uL<<0)
#define T30_E7PERIOD				(0x06uL<<0)
#define T30_E8PERIOD				(0x07uL<<0)

/* Timer 30 T30OUTCR Write Key Definition  */
#define T30O_WRITEKEY				(0xE06CuL<<16)

/* Timer 30 PWM30xB Output Polarity Selection Definition  */
#define T30O_BPOLOW				(0x0uL<<15)
#define T30O_BPOHIGH				(0x1uL<<15)

/* Timer 30 PWM30xA Output Polarity Selection Definition  */
#define T30O_APOLOW				(0x0uL<<14)
#define T30O_APOHIGH				(0x1uL<<14)

/* Timer 30 PWM30AB Output En/Disable Definition  */
#define T30O_PWMABDIS			(0x0uL<<13)
#define T30O_PWMABEN				(0x1uL<<13)

/* Timer 30 PWM30BB Output En/Disable Definition  */
#define T30O_PWMBBDIS			(0x0uL<<12)
#define T30O_PWMBBEN				(0x1uL<<12)

/* Timer 30 PWM30CB Output En/Disable Definition  */
#define T30O_PWMCBDIS			(0x0uL<<11)
#define T30O_PWMCBEN				(0x1uL<<11)

/* Timer 30 PWM30AA Output En/Disable Definition  */
#define T30O_PWMAADIS			(0x0uL<<10)
#define T30O_PWMAAEN				(0x1uL<<10)

/* Timer 30 PWM30BA Output En/Disable Definition  */
#define T30O_PWMBADIS			(0x0uL<<9)
#define T30O_PWMBAEN				(0x1uL<<9)

/* Timer 30 PWM30CA Output En/Disable Definition  */
#define T30O_PWMCADIS			(0x0uL<<8)
#define T30O_PWMCAEN				(0x1uL<<8)

/* Timer 30 PWM30AB Output When Disable  */
#define T30O_ABLOW				(0x0uL<<6)
#define T30O_ABHIGH				(0x1uL<<6)

/* Timer 30 PWM30BB Output When Disable  */
#define T30O_BBLOW				(0x0uL<<5)
#define T30O_BBHIGH				(0x1uL<<5)

/* Timer 30 PWM30CB Output When Disable  */
#define T30O_CBLOW				(0x0uL<<4)
#define T30O_CBHIGH				(0x1uL<<4)

/* Timer 30 PWM30AA Output When Disable  */
#define T30O_AALOW				(0x0uL<<2)
#define T30O_AAHIGH				(0x1uL<<2)

/* Timer 30 PWM30BA Output When Disable  */
#define T30O_BALOW				(0x0uL<<1)
#define T30O_BAHIGH				(0x1uL<<1)

/* Timer 30 PWM30CA Output When Disable  */
#define T30O_CALOW				(0x0uL<<0)
#define T30O_CAHIGH				(0x1uL<<0)

/* Timer 30 High-Impedance Interrupt EN/Disable Definition */
#define T30INT_HIZDIS				(0x0uL<<6)
#define T30INT_HIZEN				(0x1uL<<6)

/* Timer 30 Capture Interrupt EN/Disable Definition */
#define T30INT_CAPDIS				(0x0uL<<5)
#define T30INT_CAPEN				(0x1uL<<5)

/* Timer 30 Bottom Interrupt EN/Disable Definition */
#define T30INT_BOTDIS				(0x0uL<<4)
#define T30INT_BOTEN				(0x1uL<<4)

/* Timer 30 Period Match Interrupt EN/Disable Definition */
#define T30INT_PMATDIS				(0x0uL<<3)
#define T30INT_PMATEN				(0x1uL<<3)

/* Timer 30 A Match Interrupt EN/Disable Definition */
#define T30INT_AMATDIS				(0x0uL<<2)
#define T30INT_AMATEN				(0x1uL<<2)

/* Timer 30 B Match Interrupt EN/Disable Definition */
#define T30INT_BMATDIS				(0x0uL<<1)
#define T30INT_BMATEN				(0x1uL<<1)

/* Timer 30 C Match Interrupt EN/Disable Definition */
#define T30INT_CMATDIS				(0x0uL<<0)
#define T30INT_CMATEN				(0x1uL<<0)

/* Timer 30 PWM Output High-Impedance En/Disable Definition */
#define T30HIZ_DISABLE				(0x0uL<<7)
#define T30HIZ_ENABLE				(0x1uL<<7)

/* Timer 30 High-Impedance(BLNK) Edge Definition */
#define T30HIZ_BLNKFALL			(0x0uL<<2)
#define T30HIZ_BLNKRISE			(0x1uL<<2)

/* Timer 30 Bottom for A/DC Trigger Signal Generator EN/Disable Definition */
#define T30ADT_BTTGDIS			(0x0uL<<4)
#define T30ADT_BTTGEN				(0x1uL<<4)

/* Timer 30 Period Match for A/DC Trigger Signal Generator EN/Disable Definition */
#define T30ADT_PMTGDIS			(0x0uL<<3)
#define T30ADT_PMTGEN			(0x1uL<<3)

/* Timer 30 A-ch Match for A/DC Trigger Signal Generator EN/Disable Definition */
#define T30ADT_AMTGDIS			(0x0uL<<2)
#define T30ADT_AMTGEN			(0x1uL<<2)

/* Timer 30 B-ch Match for A/DC Trigger Signal Generator EN/Disable Definition */
#define T30ADT_BMTGDIS			(0x0uL<<1)
#define T30ADT_BMTGEN			(0x1uL<<1)

/* Timer 30 C-ch Match for A/DC Trigger Signal Generator EN/Disable Definition */
#define T30ADT_CMTGDIS			(0x0uL<<0)
#define T30ADT_CMTGEN			(0x1uL<<0)

#define T30_GetCnt()				(TIMER30->CNT)
#define T30_GetCapData()			(TIMER30->CAPDR)

/*******************************************************************************
* Public Typedef
*******************************************************************************/
typedef struct
{
	uint16_t CLK;	
	uint16_t MS;	
	uint16_t FORCA;		/**< set initial output value, should be:	START_LOW or START_HIGH */		
	uint16_t UPDT;
	uint16_t PMOC;
	uint16_t ECE;	
	uint16_t Prescaler;
	uint16_t CPOL;
} TIMER3n_CFG_Type;

/*******************************************************************************
* Exported Public Function
*******************************************************************************/
HAL_Status_Type HAL_TIMER3n_SetPeriod(TIMER3n_Type* TIMER3n, uint32_t prescale);
HAL_Status_Type HAL_TIMER3n_SetADuty(TIMER3n_Type* TIMER3n, uint32_t period);
HAL_Status_Type HAL_TIMER3n_SetBDuty(TIMER3n_Type* TIMER3n, uint32_t period);
HAL_Status_Type HAL_TIMER3n_SetCDuty(TIMER3n_Type* TIMER3n, uint32_t period);
HAL_Status_Type HAL_TIMER3n_SetDelayTime(TIMER3n_Type* TIMER3n, uint32_t dten, uint32_t dtpos, uint32_t clkdata);
HAL_Status_Type HAL_TIMER3n_Start(TIMER3n_Type* TIMER3n, uint32_t NewState);
HAL_Status_Type HAL_TIMER3n_OutputCtrl(TIMER3n_Type* TIMER3n,  uint32_t NewState, uint32_t pwmApol, uint32_t pwmBpol);
HAL_Status_Type HAL_TIMER3n_ConfigInterrupt(TIMER3n_Type* TIMER3n,uint32_t NewState,uint32_t T30IntCfg);
HAL_Status_Type HAL_TIMER3n_ClearStatus_IT(TIMER3n_Type* TIMER3n, uint32_t T30IntCfg);
uint32_t HAL_TIMER3n_GetStatus_IT(TIMER3n_Type* TIMER3n);
HAL_Status_Type HAL_TIMER3n_MPWMCmd(TIMER3n_Type* TIMER3n, uint32_t updatedata, uint32_t intcount);
HAL_Status_Type HAL_TIMER3n_SetHizReg(TIMER3n_Type* TIMER3n, uint32_t u32T30HizSet);
HAL_Status_Type HAL_TIMER3n_SetADCTrigger(TIMER3n_Type* TIMER3n, uint32_t u32triggerpoint, uint32_t u32triggertime);
HAL_Status_Type HAL_TIMER3n_Init(TIMER3n_Type* TIMER3n, void *TIMER3n_CFG_Type);
#ifdef __cplusplus
}
#endif

#endif /* _T30_H_ */

