/**
* @file     A31G32x_hal_wdt.h
* @brief    Contains all macro definitions and function prototypes support
*           for wdt driver on A31G32x
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

#ifndef _WDT_H_
#define _WDT_H_

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x.h"
#include "A31G32x_hal_aa_types.h"

#ifdef __cplusplus
extern "C"
{
#endif


/*******************************************************************************
* Public Macro
*******************************************************************************/
/** WDTCR interrupt enable bit */
#define WDTCR_UNFIEN				((uint32_t)(1<<2))	// Watch-dog Timer Underflow Interrupt Enable bit	
#define WDTCR_WINMIEN			((uint32_t)(1<<3))	// Watch-dog Timer Window Match Interrupt Enable bit
	
#define WDT_INTERRUPT_BITMASK   0x000c

/** WDTSR interrupt enable bit */
#define WDTSR_UNFIFLAG			((uint32_t)(1<<0))	// Watch-dog Timer Underflow Interrupt Flag bit	
#define WDTSR_WINMIFLAG			((uint32_t)(1<<1))	// Watch-dog Timer Window Match Interrupt Flag bit

#define WDT_STATUS_BITMASK		0x0003

	
/*******************************************************************************
* Public Typedef
*******************************************************************************/
/** @brief WDT structure definitions */
enum {
		WDT_DIV_4=0,
		WDT_DIV_16,
		WDT_DIV_64,
		WDT_DIV_256	
};

typedef enum {
	WDT_INTCFG_UNFIEN = 0,		/*!< UNFIEN Interrupt enable*/
	WDT_INTCFG_WINMIEN,			/*!< WINMIEN Interrupt enable*/
} WDT_INT_Type;

typedef struct Wdt_Config
{
	uint8_t wdtResetEn;				/*!< if ENABLE -> the Reset bit is enabled*/
	uint16_t wdtClkDiv;				/*!< wdtClkDiv */
	uint32_t wdtTmrConst;			/*!< Set Watch-dog Timer Data Register	*/
	uint32_t wdtWTmrConst;			/*!< Set Watch-dog Timer Window Data Register	*/	
} WDT_CFG_Type;

/*******************************************************************************
* Exported Public Function
*******************************************************************************/
HAL_Status_Type HAL_WDT_Init(WDT_CFG_Type wdtCfg);
HAL_Status_Type HAL_WDT_ReloadTimeCounter(void);
HAL_Status_Type HAL_WDT_Start(FunctionalState ctrl);
uint32_t HAL_WDT_GetCurrentCount(void);
uint32_t HAL_WDT_GetStatus(void);
HAL_Status_Type HAL_WDT_ClearStatus(uint32_t clrbit);
HAL_Status_Type HAL_WDT_ConfigInterrupt(WDT_INT_Type WDTIntCfg, FunctionalState NewState);

#ifdef __cplusplus
}
#endif

#endif /* _WDT_H_ */

/* --------------------------------- End Of File ------------------------------ */
