/**
* @file     A31G32x_hal_wt.h
* @brief    Contains all macro definitions and function prototypes support
*           for wt driver on A31G32x
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

#ifndef _WT_H_
#define _WT_H_

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x.h"
#include "A31G32x_hal_aa_types.h"

#ifdef __cplusplus
extern "C"
{
#endif


/*******************************************************************************
* Public Macro
*******************************************************************************/
/** WTCR interrupt enable bit */
#define WTCR_WTIEN			((uint32_t)(1<<3))	// Watch Timer Interrupt Enable bit	
#define WT_INTERRUPT_BITMASK	0x0008

/** WTCR interrupt status flag bit */
#define WTCR_WTIFLAG			((uint32_t)(1<<1))	// Watch Timer Interrupt Flag bit	
#define WT_STATUS_BITMASK	0x0002

#define WTCR_WTCLR			((uint32_t)(1<<0))	// Watch Timer counter and divider clear bit	


/*******************************************************************************
* Public Typedef
*******************************************************************************/
/** @brief WDT structure definitions */
enum {
		WT_DIV_2_7=0,
		WT_DIV_2_13,
		WT_DIV_2_14,
		WT_DIV_2_14_MUL_WTDR	
};

typedef struct Wt_Config
{
	uint32_t wtClkDiv;			/* wtClkDiv */
	uint32_t wtTmrConst;		/**< Set Watch Timer Data Register	*/
} WT_CFG_Type;


/*******************************************************************************
* Exported Public Function
*******************************************************************************/
HAL_Status_Type HAL_WT_Init(WT_CFG_Type wtCfg);
HAL_Status_Type HAL_WT_DeInit(void);
HAL_Status_Type HAL_WT_Start(FunctionalState ctrl);
uint32_t HAL_WT_GetCurrentCount(void);
uint32_t HAL_WT_GetStatus(void);
HAL_Status_Type HAL_WT_ClearStatus(void);
HAL_Status_Type HAL_WT_ConfigInterrupt(FunctionalState NewState);

#ifdef __cplusplus
}
#endif

#endif /* _WT_H_ */

/* --------------------------------- End Of File ------------------------------ */
