/**
* @file		A31G32x_hal_dmac.c
* @brief	Contains all functions support for dma firmware library on A31G324
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x_hal_ebi.h"

/*******************************************************************************
* Private Pre-processor Definition & Macro
*******************************************************************************/

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/**
* @brief		Initialize EBI peripheral
* @param		None
* @return		HAL_Status
*/
HAL_Status_Type HAL_EBI_Init(void)
{
	SCU->PER1 &=~(1UL<<5);
	SCU->PCER1&=~(1UL<<5);
	/* Set up peripheral clock for EBI module */
	SCU->PER1 |= (1UL<<5);
	SCU->PCER1|= (1UL<<5);
	return HAL_OK;
}

/**
* @brief		De-Initialize EBI peripheral
* @param		None
* @return		HAL_Status
*/
HAL_Status_Type HAL_EBI_DeInit(void)
{
	SCU->PER1 &=~(1UL<<5);
	SCU->PCER1&=~(1UL<<5);
	return HAL_OK;
}


/**
* @brief		Start EBI peripheral
* @param		Not Support
* @return		HAL_Status
*/
HAL_Status_Type HAL_EBI_Start(void)
{
	return HAL_OK;
}


/**
* @brief		Stop EBI peripheral
* @param		Not Support
* @return		HAL_Status
*/
HAL_Status_Type HAL_EBI_Stop(void)
{
	return HAL_OK;
}

/**
* @brief		Configure EBI setup
* @param		BANKn : BANK(0 ~ 3)
* @param      	EBIConfig : Pointer contains configuration of EBI
* @return		HAL_Status
*/
HAL_Status_Type HAL_EBI_Setup(uint32_t BANKn, EBI_CFG_Type *EBIConfig)
{
	if(BANKn == 0)
	{
		EBI->SMIBCR0 = 0
		|((EBIConfig->InsertIDLE & 0x03) << 30)
		|((EBIConfig->FallingTiming & 0x03) << 26)
		|((EBIConfig->ExternalWaitEnable & 0x1) << 23)
		|((EBIConfig->WaitSignalPolarity & 0x1) << 22)
		|((EBIConfig->AddressMuxed & 0x1) << 21)
		|((EBIConfig->LaneSelection & 0x1) << 20)
		|((EBIConfig->KeepAddress & 0x03) << 18)
		|((EBIConfig->MemoryWidth & 0x01) << 16)
		|((EBIConfig->NORMWAIT & 0xfffff) << 0)
		;
	}
	else if(BANKn == 1)
	{
		EBI->SMIBCR1 = 0
		|((EBIConfig->InsertIDLE & 0x03) << 30)
		|((EBIConfig->FallingTiming & 0x03) << 26)
		|((EBIConfig->ExternalWaitEnable & 0x1) << 23)
		|((EBIConfig->WaitSignalPolarity & 0x1) << 22)
		|((EBIConfig->AddressMuxed & 0x1) << 21)
		|((EBIConfig->LaneSelection & 0x1) << 20)
		|((EBIConfig->KeepAddress & 0x03) << 18)
		|((EBIConfig->MemoryWidth & 0x01) << 16)
		|((EBIConfig->NORMWAIT & 0xfffff) << 0)
		;
	}
	else if(BANKn == 2)
	{
		EBI->SMIBCR2 = 0
		|((EBIConfig->InsertIDLE & 0x03) << 30)
		|((EBIConfig->FallingTiming & 0x03) << 26)
		|((EBIConfig->ExternalWaitEnable & 0x1) << 23)
		|((EBIConfig->WaitSignalPolarity & 0x1) << 22)
		|((EBIConfig->AddressMuxed & 0x1) << 21)
		|((EBIConfig->LaneSelection & 0x1) << 20)
		|((EBIConfig->KeepAddress & 0x03) << 18)
		|((EBIConfig->MemoryWidth & 0x01) << 16)
		|((EBIConfig->NORMWAIT & 0xfffff) << 0)
		;
	}
	else if(BANKn == 3)
	{
		EBI->SMIBCR3 = 0
		|((EBIConfig->InsertIDLE & 0x03) << 30)
		|((EBIConfig->FallingTiming & 0x03) << 26)
		|((EBIConfig->ExternalWaitEnable & 0x1) << 23)
		|((EBIConfig->WaitSignalPolarity & 0x1) << 22)
		|((EBIConfig->AddressMuxed & 0x1) << 21)
		|((EBIConfig->LaneSelection & 0x1) << 20)
		|((EBIConfig->KeepAddress & 0x03) << 18)
		|((EBIConfig->MemoryWidth & 0x01) << 16)
		|((EBIConfig->NORMWAIT & 0xfffff) << 0)
		;
	}
	return HAL_OK;
}	



/* --------------------------------- End Of File ------------------------------ */

