/**
* @file		A31G32x_hal_pcu.c
* @brief	Contains all functions support for pcu firmware library on A31G324
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
********************************************************************************
* DISCLAIMER 
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*/

/*******************************************************************************
* Included File
*******************************************************************************/
#include "A31G32x_hal_pcu.h"

/*******************************************************************************
* Private Pre-processor Definition & Macro
*******************************************************************************/

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/**
* @brief		Initialize GPIO(PCU) peripheral
* @param      	PCx : Pointer to selected PCU peripheral, should be: A,B,C,F
* @param      	GPIO_Init : Pointer contains configuration of GPIO
* @return		None
*/
void HAL_GPIO_Init(PORT_Type *PCx, GPIO_CFG_Type *GPIO_Init)
{
	uint8_t pin_offset; 
	uint32_t reg_val; 
	uint32_t dir_type_temp;
		
	PORT_ACCESS_EN();	
      
	//Function
      if (GPIO_Init->Pin<8){ // 0~7
		//------------------------------------------------------------------------------
		// pin_offset = pin_no * 4
		//------------------------------------------------------------------------------
		pin_offset = (GPIO_Init->Pin << 2); 

		//------------------------------------------------------------------------------
		// MR
		//------------------------------------------------------------------------------
		reg_val = PCx->AFSR1;
		reg_val &= ~(PCU_FUNC_Msk << pin_offset); 
		reg_val |= (GPIO_Init->Function << pin_offset); 

		PCx->AFSR1=reg_val;
	}
	else { // 8~15
		//------------------------------------------------------------------------------
		// pin_offset = pin_no * 4
		//------------------------------------------------------------------------------
		pin_offset = ((GPIO_Init->Pin-8) << 2); 

		//------------------------------------------------------------------------------
		// MR
		//------------------------------------------------------------------------------
		reg_val = PCx->AFSR2;
		reg_val &= ~(PCU_FUNC_Msk << pin_offset); 
		reg_val |= (GPIO_Init->Function << pin_offset); 

		PCx->AFSR2=reg_val;		
	}
	
	/* Output */	
	dir_type_temp = GPIO_Init->Output;
	if (dir_type_temp==OPEN_DRAIN_OUTPUT)
		 GPIO_Init->Output = PUSH_PULL_OUTPUT;
	//------------------------------------------------------------------------------
	// pin_offset = pin_no * 2
	//------------------------------------------------------------------------------
	pin_offset = (GPIO_Init->Pin << 1); 
	
	PORT_ACCESS_EN();
	
	//------------------------------------------------------------------------------
	// PnMOD
	//------------------------------------------------------------------------------
	reg_val = PCx->MOD; 
	reg_val &= ~(PCU_MODE_Msk << pin_offset); 
	reg_val |= ( GPIO_Init->Output << pin_offset); 
	PCx->MOD=reg_val; 


	//------------------------------------------------------------------------------
	// PnTYP
	//------------------------------------------------------------------------------
	reg_val = PCx->TYP; 
	reg_val &= ~(1 << GPIO_Init->Pin); 	
	if (dir_type_temp==OPEN_DRAIN_OUTPUT){
		reg_val |= (1 << GPIO_Init->Pin); 
	}
	PCx->TYP=reg_val; 
	
	/*pullup*/
	pin_offset = (GPIO_Init->Pin << 1); 
	
	
	reg_val = PCx->PUPD; 
	reg_val &= ~(3<<pin_offset); 
	reg_val |= (GPIO_Init->Pull<<pin_offset); 
	PCx->PUPD=reg_val; 
	
	PORT_ACCESS_DIS();
	
	if((PCx == PC) && (GPIO_Init->Output == PUSH_PULL_OUTPUT))
	{
		SCU->PER2 |= (1<<29);	// RTC Function Enable
		SCU->PCER2 |= (1<<29);	// RTC Function Enable
		if(GPIO_Init->Pin == 13)
		{
			RTC->RTCPFCR |= (2<<2);	// PC13 PUSH_PULL_OUTPUT 
		}
		if(GPIO_Init->Pin == 14)
		{
			RTC->RTCPFCR |= (1<<5);	// PC14 PUSH_PULL_OUTPUT 
		}
		if(GPIO_Init->Pin == 15)
		{
			RTC->RTCPFCR |= (1<<7);	// PC15 PUSH_PULL_OUTPUT 
		}
	}
	
	if((PCx == PC) && (GPIO_Init->Output == INPUT))
	{
		SCU->PER2 |= (1<<29);	// RTC Function Enable
		SCU->PCER2 |= (1<<29);	// RTC Function Enable
		if(GPIO_Init->Pin == 13)
		{
			RTC->RTCPFCR &= ~(3<<2);	// PC13 INPUT 
		}
		if(GPIO_Init->Pin == 14)
		{
			RTC->RTCPFCR &= ~(1<<5);	// PC14 INPUT 
		}
		if(GPIO_Init->Pin == 15)
		{
			RTC->RTCPFCR &= ~(1<<7);	// PC15 INPUT 
		}
	}
}

/**
* @brief 		Configure pin function
* @param		PCx		Pointer to selected PCU peripheral, should be: A,B,C,F
* @param		pin_no	Pin number, shouldS be: 0..15
* @param		func 	Function mode, should be:
* 					- FUNC0		:Function 0
* 					- FUNC1		:Function 1
* 					- FUNC2		:Function 2
* 					- FUNC3		:Function 3
* 					- FUNC4		:Function 4 
* 					- FUNC5		:Function 5
* 					- FUNC6		:Function 6
* 					- FUNC7		:Function 7
* 					- FUNC8		:Function 8
* 					- FUNC9		:Function 9
* 					- FUNC10	:Function 10 
* @return		None
*/
void HAL_GPIO_ConfigFunction(PORT_Type *PCx, uint8_t pin_no, uint32_t func)
{
	uint8_t pin_offset; 
	uint32_t reg_val; 

	
	PORT_ACCESS_EN();	
	
	if (pin_no<8){ // 0~7
		//------------------------------------------------------------------------------
		// pin_offset = pin_no * 4
		//------------------------------------------------------------------------------
		pin_offset = (pin_no << 2); 

		//------------------------------------------------------------------------------
		// MR
		//------------------------------------------------------------------------------
		reg_val = PCx->AFSR1;
		reg_val &= ~(PCU_FUNC_Msk << pin_offset); 
		reg_val |= (func << pin_offset); 

		PCx->AFSR1=reg_val;
	}
	else { // 8~15
		//------------------------------------------------------------------------------
		// pin_offset = pin_no * 4
		//------------------------------------------------------------------------------
		pin_offset = ((pin_no-8) << 2); 

		//------------------------------------------------------------------------------
		// MR
		//------------------------------------------------------------------------------
		reg_val = PCx->AFSR2;
		reg_val &= ~(PCU_FUNC_Msk << pin_offset); 
		reg_val |= (func << pin_offset); 

		PCx->AFSR2=reg_val;		
	}
	
	PORT_ACCESS_DIS();
	
	if(PCx == PC && pin_no==13 && func==FUNC7)
	{
		RTC->RTCPFCR &= ~(3<<2);
		RTC->RTCPFCR |= (1<<2);
	}
}

/**
* @brief 		Configure pin mode
* @param		PCx			Pointer to selected PCU peripheral, should be: A,B,C,F
* @param		pin_no		Pin number, should be: 0..15
* @param		dir_type	Pin mode, should be:
* 					- INPUT              :0
* 					- PUSH_PULL_OUTPUT   :1
* 					- ALTERN_FUNC        :2
* 					- OPEN_DRAIN_OUTPUT  :3
* @return		None
*/
void HAL_GPIO_ConfigOutput(PORT_Type *PCx, uint8_t pin_no, PCU_PORT_MODE dir_type)
{
	uint8_t pin_offset; 
	uint32_t reg_val; 
	uint32_t dir_type_temp;
	
	dir_type_temp = dir_type;
	if (dir_type_temp==OPEN_DRAIN_OUTPUT)
		dir_type = PUSH_PULL_OUTPUT;
	//------------------------------------------------------------------------------
	// pin_offset = pin_no * 2
	//------------------------------------------------------------------------------
	pin_offset = (pin_no << 1); 
	
	PORT_ACCESS_EN();
	
	//------------------------------------------------------------------------------
	// PnMOD
	//------------------------------------------------------------------------------
	reg_val = PCx->MOD; 
	reg_val &= ~(PCU_MODE_Msk << pin_offset); 
	reg_val |= (dir_type << pin_offset); 
	PCx->MOD=reg_val; 


	//------------------------------------------------------------------------------
	// PnTYP
	//------------------------------------------------------------------------------
	reg_val = PCx->TYP; 
	reg_val &= ~(1 << pin_no); 	
	if (dir_type_temp==OPEN_DRAIN_OUTPUT){
		reg_val |= (1 << pin_no); 
	}
	PCx->TYP=reg_val; 
	
	PORT_ACCESS_DIS();
	
	if((PCx == PC) && (dir_type == PUSH_PULL_OUTPUT))
	{
		SCU->PER2 |= (1<<29);	// RTC Function Enable
		SCU->PCER2 |= (1<<29);	// RTC Function Enable
		if(pin_no == 13)
		{
			RTC->RTCPFCR |= (2<<2);	// PC13 PUSH_PULL_OUTPUT 
		}
		if(pin_no == 14)
		{
			RTC->RTCPFCR |= (1<<5);	// PC14 PUSH_PULL_OUTPUT 
		}
		if(pin_no == 15)
		{
			RTC->RTCPFCR |= (1<<7);	// PC15 PUSH_PULL_OUTPUT 
		}
	}
	
	if((PCx == PC) && (dir_type == INPUT))
	{
		SCU->PER2 |= (1<<29);	// RTC Function Enable
		SCU->PCER2 |= (1<<29);	// RTC Function Enable
		if(pin_no == 13)
		{
			RTC->RTCPFCR &= ~(3<<2);	// PC13 INPUT 
		}
		if(pin_no == 14)
		{
			RTC->RTCPFCR &= ~(1<<5);	// PC14 INPUT 
		}
		if(pin_no == 15)
		{
			RTC->RTCPFCR &= ~(1<<7);	// PC15 INPUT 
		}
	}
}

/**
* @brief 		Configure pin pullup
* @param		PCx			Pointer to selected PCU peripheral, should be: A,B,C,F
* @param		pin_no		Pin number, should be: 0..15
* @param		pullupdown 	state, should be:
* 					- 0: DISABLE
* 					- 1: Enable pull-up resistor 		
* 					- 2: Enable pull-down resistor 
* 					- 3: reserved 
* @return		None
*/
void HAL_GPIO_ConfigPullup (PORT_Type *PCx, uint8_t pin_no, uint8_t pullupdown)
{
	uint32_t	reg_val; 
	uint8_t pin_offset; 
	
	pin_offset = (pin_no << 1); 
	
	PORT_ACCESS_EN();
	
	reg_val = PCx->PUPD; 
	reg_val &= ~(3<<pin_offset); 
	reg_val |= (pullupdown<<pin_offset); 
	PCx->PUPD=reg_val; 
	
	PORT_ACCESS_DIS();
}

/**
* @brief 		Configure out data Mask
* @param		PCx	Pointer to selected PCU peripheral, should be: A,B,C,F
* @param		pin_no		Pin number, should be: 0..15
* @param		maskctrl state, should be:
* 					- 0: DISABLE(Unmask)-changed 
* 					- 1: ENABLE(Mask)-protected
* @return		None
*/
void HAL_GPIO_ConfigOutDataMask (PORT_Type *PCx, uint8_t pin_no, FunctionalState maskctrl)
{
	uint32_t	reg_val; 
	
	PORT_ACCESS_EN();
	
	reg_val = PCx->OUTDMSK; 
	reg_val &= ~(1<<pin_no); 
	reg_val |= (maskctrl<<pin_no); 
	PCx->OUTDMSK=reg_val; 
	
	PORT_ACCESS_DIS();
}

/**
* @brief		Enable GPIO interrupt
* @param		PCx		Pointer to selected PCU peripheral, should be: A,B,C,F
* @param		pin_no	Pin number, should be: 0..15
* @param		pin_en 	Value that contains all bits on GPIO to enable,
* 					- IER_DISABLE = 0
*					- IER_LEVEL = 1
*					- IER_EDGE = 3				
* @param		int_mode		interrupt mode, should be:
* 					- ICR_PROHIBIT_INT = 0
*                     if level
* 					- ICR_LOW_LEVEL_INT = 1
* 					- ICR_HIGH_LEVEL_INT = 2
*                     if EDGE
* 					- ICR_FALLING_EDGE_INT = 1
* 					- ICR_RISING_EDGE_INT = 2
* 					- ICR_BOTH_EDGE_INT = 3
* @return		None
*/
void HAL_GPIO_EXTI_Config(PORT_Type *PCx, uint8_t pin_no, uint8_t pin_en, uint8_t int_mode)
{
	uint32_t temp_reg;
	
	PORT_ACCESS_EN();
	// port n interrupt enable: disable or level or edge
	temp_reg = PCx->IER & (uint32_t)(~(3 << (pin_no<<1))); // PCx Pin_no Register Init
	temp_reg |= (uint32_t)(pin_en <<  (pin_no<<1));			// PCx Pin_no Register Enable (Level or Edge)
	PCx->IER = temp_reg;

	switch (pin_en){
		case 0: // disable 
			PCx->ICR = PCx->ICR & (uint32_t)(~(3 <<  (pin_no<<1)));
			break;			
		case 1: // level High Level or Low Level??
		case 3: // edge			
			temp_reg = PCx->ICR & (uint32_t)(~(3 <<  (pin_no<<1)));
			temp_reg |= (uint32_t)(int_mode <<  (pin_no<<1));		
			PCx->ICR = temp_reg;
			break;			
		default:
			break;
	}
	
	PORT_ACCESS_DIS();
}

/**
* @brief		Get GPIO Interrupt Status
* @param		PCx	Pointer to selected PCU peripheral, should be: A,B,C,F
* @return		ISR register value 
*/
uint32_t HAL_GPIO_EXTI_GetState(PORT_Type *PCx)
{
	return (PCx->ISR);
}

/**
* @brief		Clear GPIO interrupt status
* @param		PCx	Pointer to selected PCU peripheral, should be: A,B,C,F
* @return		None
*/
void HAL_GPIO_EXTI_ClearPin(PORT_Type *PCx, uint32_t value)
{
	PORT_ACCESS_EN();
	
	PCx->ISR = value;
	
	PORT_ACCESS_DIS();
}

/**
* @brief  	Toggle the specified GPIO pin.
* @param  	PCx where x can be (A..F) to select the GPIO peripheral 
* @param  	GPIO_Pin specifies the pin to be toggled.
* @retval 	None
*/
void HAL_GPIO_TogglePin(PORT_Type* PCx, uint16_t GPIO_Pin)
{
	PORT_ACCESS_EN();
	
	PCx->OUTDR ^= GPIO_Pin;
	
	PORT_ACCESS_DIS();
	
}
/**
* @brief		Set Value for bits that have output direction on GPIO port.
* @param		PCx	Pointer to selected PCU peripheral, should be: A,B,C,F
* @param		bitValue Value that contains all bits on GPIO to set, should
* 				be in range from 0 to 0xFFFF.
* 				example: value 0x3 to set bit 0 and bit 1.
* @return		None
*/
void HAL_GPIO_SetPin(PORT_Type *Px, uint16_t bitValue)
{
	PORT_ACCESS_EN();
	
	Px->BSR = bitValue;
	if(Px== PC)
	{
		if(bitValue == (1<<13))	RTC->RTCPFCR |= (1<<1);
		if(bitValue == (1<<14))	RTC->RTCPFCR |= (1<<4);
		if(bitValue == (1<<15))	RTC->RTCPFCR |= (1<<6);
	}
	
	PORT_ACCESS_DIS();
}


/**
* @brief		Clear Value for bits that have output direction on GPIO port.
* @param		PCx	Pointer to selected PCU peripheral, should be: A,B,C,F
* @param		bitValue Value that contains all bits on GPIO to clear, should
* 				be in range from 0 to 0xFFFF.
* 				example: value 0x3 to clear bit 0 and bit 1.
* @return		None
*/
void HAL_GPIO_ClearPin(PORT_Type *Pn, uint16_t bitValue)
{
	PORT_ACCESS_EN();
	
	Pn->BCR = bitValue;
	if(Pn== PC)
	{
		if(bitValue == (1<<13))	RTC->RTCPFCR &= ~(1<<1);
		if(bitValue == (1<<14))	RTC->RTCPFCR &= ~(1<<4);
		if(bitValue == (1<<15))	RTC->RTCPFCR &= ~(1<<6);
	}
	PORT_ACCESS_DIS();
}


/**
* @brief		Read Current state on port pin that have input direction of GPIO
* 				Note: Return value contain state of each port pin (bit) on that GPIO regardless
* 						its direction is input or output.
* @param		PCx	Pointer to selected PCU peripheral, should be: A,B,C,F
* @return		Current value of GPIO port.
*/
uint16_t HAL_GPIO_ReadPin(PORT_Type *Pn)
{
	return Pn->INDR;
}

/**
* @brief		Write Value on port that have output direction of GPIO
* @param		PCx	Pointer to selected PCU peripheral, should be: A,B,C,F
* @param		Value	should be in range from 0 to 0xFFFF.
* @return		None
*/
void HAL_GPIO_WritePin(PORT_Type *Pn, uint16_t Value)
{		
	PORT_ACCESS_EN();
	
	Pn->OUTDR = Value;
	
	PORT_ACCESS_DIS();
}

/**
* @brief       	Set PCU Debounce. 
*				This function set the debounce filter sampling clock and pins for PA, PB, PC, PF
* @param    	PCx		Pointer to selected PCU peripheral, should be: A,B,C,F
* @param   		u32Pins (0 ~ 0xFFFF) Pins for Debounce Filter
* @return       None
*/
void HAL_GPIO_SetDebouncePin(PORT_Type *Pn, uint32_t u32Pins)
{
	uint32_t reg_val; 
	
	PORT_ACCESS_EN();
	
	reg_val = (0x00ff & Pn->DBCR);
	reg_val |= (0x01 << u32Pins);
	Pn->DBCR = reg_val;
	
	PORT_ACCESS_DIS();
}

/**
* @brief       	Set PC13 RTC OUT. 
* @param   		uint8_t PC13 RTC OUT Enable/Disable
* @return       None
*/
void HAL_PC13_RTC_OUT(uint8_t u8Pins)
{
	if(u8Pins==ENABLE)	RTC->RTCPFCR |= (3<<2);
	else				RTC->RTCPFCR &= ~(3<<2);
}

/* --------------------------------- End Of File ------------------------------ */
