/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void ADC_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void UART_DebugMessage(void);
void GPIO_Configure(void);
void TIMER_Configure(void);
void ADC_Configure(void);
void DMA_Configure(void);
void mainloop(void);



/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" Using DMA in ADC \n\r"
"************************************************\n\r";
__IO	uint32_t fflag;
__IO uint32_t adcirq_sequence;
__IO uint32_t adcirq_trigger;
__IO uint32_t adcirq_single;
__IO uint32_t match;


/**********************************************************************
 * @brief		ADC_IRQHandler
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void ADC_IRQHandler_IT(void)
{
	uint16_t status;
	status = HAL_ADC_GetStatus(ADC);
	
	if(status & ADC_STAT_TRIGGER)	//	TRIGGER
	{
		HAL_ADC_ClearStatus(ADC,ADC_STAT_TRIGGER);
		adcirq_trigger++;		
	}
	
	if(status & ADC_STAT_SEQ)		//	EOS
	{
		HAL_ADC_ClearStatus(ADC,ADC_STAT_SEQ);
		adcirq_sequence++;
	}
	
	if(status & ADC_STAT_SINGLE)		//	EOC
	{
		HAL_ADC_ClearStatus(ADC,ADC_STAT_SINGLE);
		adcirq_single++;
		fflag=1;
	}
}

/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}

/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	HAL_GPIO_ConfigOutput(PA, 0, ALTERN_FUNC);			// ADC Input Channel AN0
	HAL_GPIO_ConfigFunction(PA, 0, FUNC10);
	HAL_GPIO_ConfigPullup(PA, 0, DISPUPD);
}

/**********************************************************************
 * @brief		ADC_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_Configure(void)
{
	ADC_CFG_Type ADC_config;
	
	ADC_config.Mode = ADC_SINGLE_MODE;
	ADC_config.SamplingTime=0x1f;
	ADC_config.SeqCnt = 1;
	ADC_config.RestartEn = 0;
	ADC_config.TrgSel = ADC_TRIGGER_TIMER;
	
	ADC_config.InClkDiv = 0x7f;//(((SystemPeriClock/1000000UL) * 2)/ (3*(15+AD0_config.SampingTime))) + 1; 
	if (HAL_ADC_Init(ADC, &ADC_config) != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();		
	}

	/**         ADC Timer Trigger         **
	value	|	TIMER(TRGSEL[1:0])|
	ѤѤѤѤѤѤѤѤѤѤѤѤѤѤѤѤѤ
	0		|	TIMER10			 |
	1		|	TIMER11			 |
	2		|	TIMER12			 |
	3		|	TIMER13			 |
	4		|	TIMER20			 |
	5		|	TIMER21			 |
	6		|	TIMER30			 |
	7		|	TIMER40			 |
	*********************************/

	HAL_ADC_TriggerlSel(ADC, 0x0000000F);		// if ADC_config.TrgSel = 0, Not use.
	HAL_ADC_ChannelSel_1(ADC, 0x00000000);
	
	HAL_ADC_ClearStatus(ADC, 0x0f);
	HAL_ADC_ConfigInterrupt(ADC, ADC_INTEN_SINGLE, ENABLE);
	
	NVIC_SetPriority(ADC_IRQn, 3);
	/* Enable Interrupt for ADC channel */
	NVIC_EnableIRQ(ADC_IRQn);
}


/**********************************************************************
 * @brief		DMA_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void DMA_Configure(void)
{
	DMAC_Ch_CFG_Type DMA_config;
				
	if(HAL_DMAC_Init() != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();		
	}
	
	DMA_config.TransferCnt=1;
	DMA_config.PeriSel=DMAC_PERISEL_ADC;
	DMA_config.TransferWidth=DMAC_WIDTH_HALFWORD;
	DMA_config.Dir=DMAC_DIR_RX;				// From peripheral to memory
	DMA_config.PeriAddr = ADC_BASE+0x2C;		// 0x4000302C
	DMA_config.MemAddr = 0x20003000;
	HAL_DMAC_Setup(DMAC0, &DMA_config);

}

/**********************************************************************
 * @brief		ADC_DMA_Loop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_DMA_Loop(void)
{
	uint32_t i, j;
	
	// ADC TEST
	for(i=0;i<3;i++)
	{
		HAL_ADC_Start(ADC);
		
		for(j=0;j<100000;j++);
		HAL_DMAC_ChCmd(DMAC0, ENABLE);
		for(j=0;j<100000;j++);		

		while (HAL_DMAC_GetState(DMAC0) == RESET) ;		// Transfer...
		while ((HAL_ADC_GetStatus(ADC) & (ADC_STAT_DMA)) == RESET);			// ADC DMA Flag
		HAL_DMAC_ChCmd(DMAC0, DISABLE);	
			
		HAL_ADC_Stop(ADC);
		
#ifdef _DEBUG_MSG
		_DBG("\n\rAN0 Value :\t");	_DBH16(HAL_ADC_GetData(ADC)>>4);
#endif
	}
}

/**********************************************************************
 * @brief		Main 1oop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{

	/*Initialize menu prinf*/	
	DEBUG_Print_menu();	
	
	/*Initialize port peripheral*/
	GPIO_Configure();
	
	/*Initialize ADC peripheral*/
	ADC_Configure(); 

	/*Initialize DMA peripheral*/
	DMA_Configure();

	 /*  Enable IRQ Interrupts */
	 __enable_irq();   
	
	ADC_DMA_Loop();
	 
	 /* Infinite Loop */
	while(1)
	{
	}
}


/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{
	/* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

