/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"



/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void ADC_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void UART_DebugMessage(void);
void GPIO_Configure(void);
void ADC_Configure(void);
uint16_t adcaverage(uint16_t *val);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" Using ADC Interrupt \n\r"
"************************************************\n\r";
__IO	uint32_t count;
ADC_CFG_Type ADC_config;
uint16_t adcval[8];

/**********************************************************************
 * @brief		ADC Interrupt
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_IRQHandler_IT(void)
{
	uint16_t status;
	status = HAL_ADC_GetStatus(ADC);
		
	if(status & ADC_STAT_SEQ)		//	EOS
	{
		HAL_ADC_ClearStatus(ADC,ADC_STAT_SEQ);
		count=1;
	}
	
	if(status & ADC_STAT_SINGLE)		//	EOC
	{		
		HAL_ADC_ClearStatus(ADC,ADC_STAT_SINGLE);
		count=1;
	}
	
}


/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	HAL_GPIO_ConfigOutput(PA, 0, ALTERN_FUNC);		// ADC Channel AN0
	HAL_GPIO_ConfigFunction(PA, 0, FUNC10);
	HAL_GPIO_ConfigPullup(PA, 0, DISPUPD);
}



/**********************************************************************
 * @brief		UART_DebugMessage
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_DebugMessage(void)
{
	
}


/**********************************************************************
 * @brief		ADC_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_Configure(void)
{

	ADC_config.Mode = ADC_SINGLE_MODE;
	ADC_config.SamplingTime=0x1f;
	ADC_config.SeqCnt = 1;
	ADC_config.RestartEn = 0;

	ADC_config.TrgSel = ADC_TRIGGER_DISABLE;
	
	ADC_config.InClkDiv = 0x7f; 
	if (HAL_ADC_Init(ADC, &ADC_config) != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();		
	}
	
	HAL_ADC_TriggerlSel(ADC, 0x0000000F);	// SW Trigger
	HAL_ADC_ChannelSel_1(ADC, 0x00000000);		
	HAL_ADC_ChannelSel_2(ADC, 0x00000000);
	
	HAL_ADC_ClearStatus(ADC, 0x0f); // before adc interrupt enable, you must clear adc status!! 
	HAL_ADC_ConfigInterrupt(ADC, ADC_INTEN_SINGLE, ENABLE);
	
	NVIC_SetPriority(ADC_IRQn, 3);
	/* Enable Interrupt for ADC channel */
	NVIC_EnableIRQ(ADC_IRQn);

	
}

/**********************************************************************
 * @brief		UART_DebugMessage
 * @param[in]	None
 * @return 	None
 **********************************************************************/
uint16_t adcaverage(uint16_t *val)
{
  uint32_t i;
	uint32_t adcavg;
	
	adcavg=0;			
	for (i=0;i<8;i++){	
		adcavg+=val[i];
	}
	adcavg>>=3;
	
	return (adcavg);
}

/**********************************************************************
 * @brief		ADC_Start
 * @param[in]	None
 * @return	None
 **********************************************************************/
void ADC_Start(void)
{
  uint32_t j;
	uint32_t adcavg;
	
	for (j=0;j<(8);j++)
	{
		count=0;
		HAL_ADC_Start(ADC); // start 	
		while(count==0){}
		adcval[j]=HAL_ADC_GetData(ADC);
		#ifdef _DEBUG_MSG
		_DBG("ADC value Number "); _DBD(j); _DBG(" = "); _DBH16(adcval[j]>>4); _DBG_(" ");	
		#endif
	}
	adcavg=adcaverage(adcval);
	#ifdef _DEBUG_MSG
	_DBG("ADC average :\t");  _DBH16(adcavg>>4);
	#endif
	
	HAL_ADC_Stop(ADC);
}
/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{	

	/*Initialize menu prinf*/
	DEBUG_Print_menu();
	
	/*Initialize port peripheral*/
	GPIO_Configure();
	
	/*Initialize ADC peripheral*/
	ADC_Configure(); 
  
	/*  Enable IRQ Interrupts */
	__enable_irq();  
	
	/*ADC Start */
	ADC_Start();
	
	 /* Infinite Loop */
	while(1)
	{
		
	}
}


/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{
 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif



