/**********************************************************************
* @file		A31G32x_PortInit.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void Port_Init(void);
/* Private variables ---------------------------------------------------------*/


/**********************************************************************
 * @brief		This function handles NMI exception.
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Port_Init(void)
{
//Peripheral Enable Register 1  0:Disable, 1:Enable	
	SCU->PER1=SCU->PER1  
	| (1<<13)   // GPIOF
	| (1<<10)   // GPIOC
	| (1<<9) 	// GPIOB
	| (1<<8)  	// GPIOA
	; 		
//Peripheral Clock Enable Register 1 0:Disable, 1:Enable	
	SCU->PCER1=SCU->PCER1
	| (1<<13)   // GPIOF
	| (1<<10)   // GPIOC
	| (1<<9)   	// GPIOB
	| (1<<8)   	// GPIOA
	; 	

 	PORT_ACCESS_EN();  // enable writing permittion of ALL PCU register

	//--------------------------------------------------------------
	//	PORT INIT
	//		PA, PB, PC, PF
	//--------------------------------------------------------------
	// PORT - A
	PA->MOD = 0             // 0 : Input Mode,	1 : Output Mode,	2 : Alternative function mode
	| (0x01<<30)              // P15
	| (0x02<<28)              // P14	SWCLK
	| (0x02<<26)              // P13	SWDIO
	| (0x01<<24)              // P12
	| (0x01<<22)              // P11
	| (0x01<<20)              // P10
	| (0x01<<18)              // P9
	| (0x01<<16)              // P8
	| (0x01<<14)              // P7
	| (0x01<<12)              // P6
	| (0x01<<10)              // P5
	| (0x01<<8)            	  // P4
	| (0x01<<6)            	  // P3
	| (0x01<<4)            	  // P2
	| (0x01<<2)            	  // P1
	| (0x01<<0)            	  // P0
	;
	
	PA->TYP = 0             // 0 : Push-pull Output,	1 : Open-drain Output
	| (0x00<<15)              // P15
	| (0x00<<14)              // P14
	| (0x00<<13)              // P13
	| (0x00<<12)  						// P12
	| (0x00<<11)              // P11
	| (0x00<<10)              // P10
	| (0x00<<9)              	// P9
	| (0x00<<8)            	 // P8
	| (0x00<<7)            	 // P7
	| (0x00<<6)            	 // P6
	| (0x00<<5)              // P5
	| (0x00<<4)              // P4
	| (0x00<<3)              // P3
	| (0x00<<2)              // P2
	| (0x00<<1)              // P1
	| (0x00<<0)              // P0
	;
	
	PA->AFSR1 = 0
	| (0x00<<28)             // P7
	| (0x00<<24)             // P6
	| (0x00<<20)             // P5
	| (0x00<<16)             // P4
	| (0x00<<12)             // P3
	| (0x00<<8)              // P2
	| (0x00<<4)              // P1
	| (0x00<<0)              // P0
	;

	PA->AFSR2 = 0
	| (0x00<<28)             // P15
	| (0x01<<24)             // P14
	| (0x01<<20)             // P13
	| (0x00<<16)             // P12
	| (0x00<<12)             // P11
	| (0x00<<8)              // P10
	| (0x00<<4)              // P9
	| (0x00<<0)              // P8 
	;
	
	PA->PUPD = 0           // 0 : Disable Pull-up/down,	1 : Enable Pull-up,	2 : Enable Pull-down
	| (0x0<<30)              // P15
	| (0x0<<28)              // P14
	| (0x0<<26)              // P13
	| (0x0<<24)              // P12
	| (0x0<<22)              // P11
	| (0x0<<20)              // P10
	| (0x0<<18)              // P9
	| (0x0<<16)              // P8
	| (0x0<<14)              // P7
	| (0x0<<12)              // P6
	| (0x0<<10)              // P5
	| (0x0<<8)              // P4
	| (0x0<<6)              // P3
	| (0x0<<4)              // P2
	| (0x0<<2)              // P1
	| (0x0<<0)              // P0
	;
	
	PA->OUTDR = 0          // 0 : Output Low,	1 : Output High	
	| (0x00<<15)             // P15
	| (0x00<<14)             // P14
	| (0x00<<13)             // P13
	| (0x00<<12)             // P12
	| (0x00<<11)             // P11
	| (0x00<<10)             // P10
	| (0x00<<9)              // P9
	| (0x00<<8)              // P8
	| (0x00<<7)              // P7
	| (0x00<<6)              // P6
	| (0x00<<5)              // P5
	| (0x00<<4)              // P4
	| (0x00<<3)              // P3
	| (0x00<<2)              // P2
	| (0x00<<1)              // P1
	| (0x00<<0)              // P0
	;

//////////////////////////////////////////////////////////////////////////////////////////////////////
	// PORT - B
	PB->MOD = 0              // 0 : Input Mode,	1 : Output Mode,	2 : Alternative function mode
	| (0x01<<30)              // P15
	| (0x01<<28)              // P14
	| (0x01<<26)              // P13
	| (0x01<<24)              // P12
	| (0x01<<22)              // P11
	| (0x01<<20)              // P10
	| (0x01<<18)              // P9
	| (0x01<<16)              // P8
	| (0x01<<14)              // P7
	| (0x01<<12)              // P6
	| (0x01<<10)              // P5   
	| (0x01<<8)              // P4   
	| (0x01<<6)              // P3
	| (0x01<<4)              // P2
	| (0x01<<2)              // P1
	| (0x01<<0)              // P0
	;
	
	PB->TYP = 0              // 0 : Push-pull Output,	1 : Open-drain Output
	| (0x00<<15)              // P15
	| (0x00<<14)              // P14
	| (0x00<<13)              // P13
	| (0x00<<12)              // P12
	| (0x00<<11)              // P11
	| (0x00<<10)              // P10
	| (0x00<<9)              // P9
	| (0x00<<8)              // P8
	| (0x00<<7)              // P7
	| (0x00<<6)              // P6
	| (0x00<<5)              // P5
	| (0x00<<4)              // P4
	| (0x00<<3)              // P3
	| (0x00<<2)              // P2
	| (0x00<<1)              // P1
	| (0x00<<0)              // P0
	;
	
	PB->AFSR1 = 0
	| (0x00<<28)             // P7
	| (0x00<<24)             // P6
	| (0x00<<20)             // P5
	| (0x00<<16)             // P4
	| (0x00<<12)             // P3
	| (0x00<<8)              // P2
	| (0x00<<4)              // P1
	| (0x00<<0)              // P0
	;

	PB->AFSR2 = 0
	| (0x00<<28)             // P15
	| (0x00<<24)             // P14
	| (0x00<<20)             // P13
	| (0x00<<16)             // P12
	| (0x00<<12)             // P11
	| (0x00<<8)              // P10
	| (0x00<<4)              // P9
	| (0x00<<0)              // P8 
	;
	
	PB->PUPD = 0           // 0 : Disable Pull-up/down,	1 : Enable Pull-up,	2 : Enable Pull-down
	| (0x0<<30)              // P15
	| (0x0<<28)              // P14
	| (0x0<<26)              // P13
	| (0x0<<24)              // P12
	| (0x0<<22)              // P11
	| (0x0<<20)              // P10
	| (0x0<<18)              // P9
	| (0x0<<16)              // P8
	| (0x0<<14)              // P7
	| (0x0<<12)              // P6
	| (0x0<<10)              // P5
	| (0x0<<8)              // P4
	| (0x0<<6)              // P3
	| (0x0<<4)              // P2
	| (0x0<<2)              // P1
	| (0x0<<0)              // P0
	;
	
	PB->OUTDR = 0          // 0 : Output Low,	1 : Output High	
	| (0x00<<15)             // P15
	| (0x00<<14)             // P14
	| (0x00<<13)             // P13
	| (0x00<<12)             // P12
	| (0x00<<11)             // P11
	| (0x00<<10)             // P10
	| (0x00<<9)              // P9
	| (0x00<<8)              // P8
	| (0x00<<7)              // P7
	| (0x00<<6)              // P6
	| (0x00<<5)              // P5
	| (0x00<<4)              // P4
	| (0x00<<3)              // P3
	| (0x00<<2)              // P2
	| (0x00<<1)              // P1
	| (0x00<<0)              // P0
	;

//////////////////////////////////////////////////////////////////////////////////////////////////////
	// PORT - C
	PC->MOD = 0              // 0 : Input Mode,	1 : Output Mode,	2 : Alternative function mode
	| (0x01<<30)              // P15
	| (0x01<<28)              // P14
	| (0x01<<26)              // P13
	| (0x01<<24)              // P12
	| (0x01<<22)              // P11
	| (0x01<<20)              // P10
	| (0x01<<18)              // P9
	| (0x01<<16)              // P8
	| (0x01<<14)              // P7
	| (0x01<<12)              // P6
	| (0x01<<10)              // P5
	| (0x01<<8)              // P4
	| (0x01<<6)              // P3
	| (0x01<<4)              // P2
	| (0x01<<2)              // P1
	| (0x01<<0)              // P0
	;
	
	PC->TYP = 0              // 0 : Push-pull Output,	1 : Open-drain Output
	| (0x00<<15)             // P15
	| (0x00<<14)             // P14
	| (0x00<<13)             // P13
	| (0x00<<12)             // P12
	| (0x00<<11)             // P11
	| (0x00<<10)             // P10
	| (0x00<<9)              // P9
	| (0x00<<8)              // P8
	| (0x00<<7)              // P7
	| (0x00<<6)              // P6
	| (0x00<<5)              // P5
	| (0x00<<4)              // P4
	| (0x00<<3)              // P3
	| (0x00<<2)              // P2
	| (0x00<<1)              // P1
	| (0x00<<0)              // P0
	;
	
	PC->AFSR1 = 0
	| (0x00<<28)             // P7
	| (0x00<<24)             // P6
	| (0x00<<20)             // P5
	| (0x00<<16)             // P4
	| (0x00<<12)             // P3
	| (0x00<<8)              // P2
	| (0x00<<4)              // P1
	| (0x00<<0)              // P0
	;

	PC->AFSR2 = 0
	| (0x00<<28)             // P15
	| (0x00<<24)             // P14
	| (0x00<<20)             // P13
	| (0x00<<16)             // P12
	| (0x00<<12)             // P11
	| (0x00<<8)              // P10
	| (0x00<<4)              // P9
	| (0x00<<0)              // P8 
	;
	
	PC->PUPD = 0           // 0 : Disable Pull-up/down,	1 : Enable Pull-up,	2 : Enable Pull-down
	| (0x0<<30)              // P15
	| (0x0<<28)              // P14
	| (0x0<<26)              // P13
	| (0x0<<24)              // P12
	| (0x0<<22)              // P11
	| (0x0<<20)              // P10
	| (0x0<<18)              // P9
	| (0x0<<16)              // P8
	| (0x0<<14)              // P7
	| (0x0<<12)              // P6
	| (0x0<<10)              // P5
	| (0x0<<8)              // P4
	| (0x0<<6)              // P3
	| (0x0<<4)              // P2
	| (0x0<<2)              // P1
	| (0x0<<0)              // P0
	;
	
	PC->OUTDR = 0          // 0 : Output Low,	1 : Output High	
	| (0x00<<15)             // P15
	| (0x00<<14)             // P14
	| (0x00<<13)             // P13
	| (0x00<<12)             // P12
	| (0x00<<11)             // P11
	| (0x00<<10)             // P10
	| (0x00<<9)              // P9
	| (0x00<<8)              // P8
	| (0x00<<7)              // P7
	| (0x00<<6)              // P6
	| (0x00<<5)              // P5
	| (0x00<<4)              // P4
	| (0x00<<3)              // P3
	| (0x00<<2)              // P2
	| (0x00<<1)              // P1
	| (0x00<<0)              // P0
	;
	
//////////////////////////////////////////////////////////////////////////////////////////////////////	
	// PORT - F
	PF->MOD = 0            // 0 : Input Mode,	1 : Output Mode,	2 : Alternative function mode
	| (0x01<<4)              // P2
	| (0x01<<2)              // P1
	| (0x01<<0)              // P0
	;
	
	PF->TYP = 0						// 0 : Push-pull Output,	1 : Open-drain Output
	| (0x0<<2)              // P2
	| (0x0<<1)              // P1
	| (0x0<<0)              // P0
	;
	
	PF->AFSR1 = 0		 
	| (0x0<<8)              // P2					
	| (0x0<<4)              // P1	
	| (0x0<<0)              // P0		
	;
	
	PF->PUPD = 0          // 0 : Disable Pull-up/down,	1 : Enable Pull-up,	2 : Enable Pull-down
	| (0x0<<4)              // P2
	| (0x0<<2)              // P1
	| (0x0<<0)              // P0
	;
	
	PF->OUTDR = 0          // 0 : Output Low,	1 : Output High
	| (0x00<<2)              // P2
	| (0x00<<1)              // P1
	| (0x00<<0)              // P0
	;
	
	PORT_ACCESS_DIS();  // disable writing permittion of ALL PCU register 	
}

