/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"



/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void ADC_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void UART_DebugMessage(void);
void GPIO_Configure(void);
void TIMER_Configure(void);
void ADC_Configure(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
TIMER1n_PERIODICCFG_Type TIMER1n_Config;
ADC_CFG_Type ADC_config;

__IO	uint32_t count;
__IO	uint32_t flag;
__IO uint32_t adcirq_sequence;
__IO uint32_t adcirq_trigger;
__IO uint32_t adcirq_single;
__IO uint32_t match;


const uint8_t menu[] =
"\n\r************************************************\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" Using Multiple in ADC \n\r"
"************************************************\n\r";


/**********************************************************************
 * @brief		ADC_IRQHandler
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void ADC_IRQHandler_IT(void)
{
	uint16_t status;
	status = HAL_ADC_GetStatus(ADC);
	
	if(status & ADC_STAT_SEQ)		//EOS
	{
		HAL_ADC_ClearStatus(ADC,ADC_STAT_SEQ);
		adcirq_sequence++;
	}
	
	if(status & ADC_STAT_SINGLE)		// EOC
	{		
		HAL_ADC_ClearStatus(ADC,ADC_STAT_SINGLE);
		adcirq_single++;
	}
	if(status & ADC_STAT_TRIGGER)
	{
		HAL_ADC_ClearStatus(ADC,ADC_STAT_TRIGGER);
		adcirq_trigger++;		
	}	
}

/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	HAL_GPIO_ConfigOutput(PA, 0, ALTERN_FUNC);			// ADC Channel AN0
	HAL_GPIO_ConfigFunction(PA, 0, FUNC10);
	HAL_GPIO_ConfigPullup(PA, 0, DISPUPD);
}

/**********************************************************************
 * @brief		UART_DebugMessage
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_DebugMessage(void)
{
	#ifdef _DEBUG_MSG
	_DBG("DR0 = "); _DBH16(ADC->DR0>>4); _DBG(" ");
	_DBG("DR1 = "); _DBH16(ADC->DR1>>4); _DBG(" ");
	_DBG("DR2 = "); _DBH16(ADC->DR2>>4); _DBG(" ");
	_DBG("DR3 = "); _DBH16(ADC->DR3>>4); _DBG_(" ");			
	_DBG("DR4 = "); _DBH16(ADC->DR4>>4); _DBG(" ");
	_DBG("DR5 = "); _DBH16(ADC->DR5>>4); _DBG(" ");
	_DBG("DR6 = "); _DBH16(ADC->DR6>>4); _DBG(" ");
	_DBG("DR7 = "); _DBH16(ADC->DR7>>4); _DBG(" ");	
   #endif	
}


/**********************************************************************
 * @brief		TIMER_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_Configure( void)
{
	#if 1	//Timer1n clock source from PCLK
	//HAL_SCU_SetTimer1nClk(T1NCLK_MCCR1);
	HAL_SCU_MiscClockConfig(1, TIMER1N_TYPE, LSI_MISC_CLOCK, 1);
	TIMER1n_Config.CkSel = MCCR1PCLK;    
	TIMER1n_Config.Prescaler = 750;    // 1kHz
	TIMER1n_Config.StartLevel=START_HIGH;  //START_LOW, START_HIGH
  #endif
	
#if 1	//Timer10	

	TIMER1n_Config.TnADR = (500); // 500msec	
	if(HAL_TIMER1n_Init(TIMER10, PERIODIC_MODE, &TIMER1n_Config) != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();				
	}
	HAL_TIMER1n_Cmd(TIMER10, ENABLE); // timer start & clear
	
#endif

#if 1	//Timer11	

	TIMER1n_Config.TnADR = (260); // 260msec
	if(HAL_TIMER1n_Init(TIMER11, PERIODIC_MODE, &TIMER1n_Config) != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();				
	}
	HAL_TIMER1n_Cmd(TIMER11, ENABLE); // timer start & clear
	
#endif

#if 1	//Timer12	

	TIMER1n_Config.TnADR = (420); // 420msec
	if(HAL_TIMER1n_Init(TIMER12, PERIODIC_MODE, &TIMER1n_Config) != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();				
	}
	HAL_TIMER1n_Cmd(TIMER12, ENABLE); // timer start & clear
#endif

}


/**********************************************************************
 * @brief		ADC_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void ADC_Configure(void)
{
	ADC_config.Mode = ADC_MULTI_MODE;
	ADC_config.SamplingTime=0x1f;
	ADC_config.SeqCnt = 8;	// Ignored in Multiple mode
	ADC_config.RestartEn = 0;
	
	ADC_config.TrgSel = ADC_TRIGGER_TIMER;
	
	ADC_config.InClkDiv = 0x7f;//(((SystemPeriClock/1000000UL) * 2)/ (3*(15+AD0_config.SampingTime))) + 1; 
	if(HAL_ADC_Init(ADC, &ADC_config) != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();				
	}

	/**         ADC Timer Trigger         **
	value	|	TIMER(TRGSEL[1:0])|
	ѤѤѤѤѤѤѤѤѤѤѤѤѤѤѤѤѤ
	0		|	TIMER10			 |
	1		|	TIMER11			 |
	2		|	TIMER12			 |
	3		|	TIMER13			 |
	4		|	TIMER20			 |
	5		|	TIMER21			 |
	6		|	TIMER30			 |
	7		|	TIMER40			 |
	*********************************/

	HAL_ADC_TriggerlSel(ADC, 0x10210210);		
	HAL_ADC_ChannelSel_1(ADC, 0x00000000);	// ADC Channel AN0
	HAL_ADC_ChannelSel_2(ADC, 0x00000000);
	
	HAL_ADC_ClearStatus(ADC, 0x0f); 					// before adc interrupt enable, you must clear adc status!! 
	HAL_ADC_ConfigInterrupt(ADC, ADC_INTEN_SINGLE, ENABLE);	// ADC EOS flag isn't occur in Mulitple mode
	
	NVIC_SetPriority(ADC_IRQn, 3);
	/* Enable Interrupt for ADC channel */
	NVIC_EnableIRQ(ADC_IRQn);
	
}

/**********************************************************************
 * @brief		ADC_Multiple_Loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void ADC_Multiple_Loop(void)
{
	HAL_ADC_Start(ADC);
	
	while(!(adcirq_single>8));
	
	HAL_ADC_Stop(ADC);
	
	UART_DebugMessage();
}
/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{

	/*Initialize menu prinf*/	
	DEBUG_Print_menu();	
	
	/*Initialize port peripheral*/
	GPIO_Configure();

	/*Initialize Timer peripheral*/
	TIMER_Configure();
	
	/*Initialize ADC peripheral*/
	ADC_Configure(); 

   /*  Enable IRQ Interrupts */
	 __enable_irq(); 
	
	ADC_Multiple_Loop();

  /* Infinite Loop */
	while(1)
	{		
	}
}


/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{
/* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif


