#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define	COMP_Number	0		// 0: CPM0 , 1: COMP1
/* Private macro -------------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void COMP_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void delay (void);
void UART_DebugMessage(void);
void GPIO_Configure(void);
void COMP_Configure( void);
void LED_Set_loop( void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint32_t status;
uint32_t wtif;
const uint8_t menu[] =
"************************************************\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" COMP0 wakeup test \n\r"
"************************************************\n\r";


/**********************************************************************
 * @brief		COMP_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void COMP_IRQHandler_IT(void)
{
	if (HAL_COMP_GetStatus_IT(COMP_Number)){	//COMP0 Interrupt
		HAL_COMP_ClearStatus_IT(COMP_Number);
	}
	
	if (HAL_COMP_GetStatus_IT(COMP_Number)){	//COMP1 Interrupt
		HAL_COMP_ClearStatus_IT(COMP_Number);
	}	
}


/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/*********************************************************************//**
 * @brief		Delay function
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void delay (void) 
{
	uint32_t i;
	
	for (i = 0; i < 1000000; i++) ;
}


/**********************************************************************
 * @brief		UART_DebugMessage
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_DebugMessage(void)
{
	_DBG("\r\nWake Up from comp interrupt !! \r\n");
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
#if COMP_Number		// COMP1 Port Set
	/*
	// PA2 -> CP1_N2
	HAL_GPIO_ConfigOutput(PA, 2, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 2, FUNC10);	
	HAL_GPIO_ConfigPullup(PA, 2, DISPUPD);

	// PA4 -> CP1_N0
	HAL_GPIO_ConfigOutput(PA, 4, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 4, FUNC10);		
	HAL_GPIO_ConfigPullup(PA, 4, DISPUPD);

	// PA5 ->CP1_N1
	HAL_GPIO_ConfigOutput(PA, 5, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 5, FUNC10);		
	HAL_GPIO_ConfigPullup(PA, 5, DISPUPD);
	*/
	// PA3 -> CP1_P0
	HAL_GPIO_ConfigOutput(PA, 3, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 3, FUNC10);		
	HAL_GPIO_ConfigPullup(PA, 3, DISPUPD);
	
	// PA7 -> CP1_OUT
	HAL_GPIO_ConfigOutput(PA, 7, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 7, FUNC1);		
	HAL_GPIO_ConfigPullup(PA, 7, DISPUPD);
#else		// COMP0 Port Set
	/*
	// PA0 -> CP0_N2
	HAL_GPIO_ConfigOutput(PA, 0, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 0, FUNC10);	
	HAL_GPIO_ConfigPullup(PA, 0, DISPUPD);

	// PA4 -> CP0_N0
	HAL_GPIO_ConfigOutput(PA, 4, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 4, FUNC10);		
	HAL_GPIO_ConfigPullup(PA, 4, DISPUPD);

	// PA5 -> CP0_N1
	HAL_GPIO_ConfigOutput(PA, 5, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 5, FUNC10);		
	HAL_GPIO_ConfigPullup(PA, 5, DISPUPD);
	*/
	// PA1 -> CP0_P0
	HAL_GPIO_ConfigOutput(PA, 1, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 1, FUNC10);		
	HAL_GPIO_ConfigPullup(PA, 1, DISPUPD);
	
	// PA6 -> CP0_OUT
	HAL_GPIO_ConfigOutput(PA, 6, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 6, FUNC1);		
	HAL_GPIO_ConfigPullup(PA, 6, DISPUPD);
#endif
}


/**********************************************************************
 * @brief		COMP_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void COMP_Configure(void)
{
	
	if(HAL_COMP_Init() != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();				
	}
	
	HAL_COMP_SetVoltage(ENABLE);
	HAL_COMP_SetWindowMode(DISABLE);

#if COMP_Number		// COMP1 interrupt mode
	
//	COMP_HysterisisConfigure(COMP1, COMP_HYSSEL_High);
																											  
	HAL_COMP_ConfigInputLevel(COMP1, COMP_INPUT_Vref9MV);	// CP0 == CP1
	
	HAL_COMP_Cmd(COMP1, ENABLE); //Comparator1 enable
	
	HAL_COMP_ConfigDebounce(COMP1, 10, COMP_DBCLK_LSI750, 48); //systemclk / (DBNCTB * 2) = 48MHz / 48 = 1MH 
	
	HAL_COMP_ConfigInterrupt(COMP1, COMP_TRIOUT_NORMAL, COMP_INTOUT_HIGH, ICON_BOTH_EDGE_INT);	
	
	HAL_COMP_InterruptCmd(COMP1, ENABLE);
	
#else		// COMP0 interrupt mode
	
//	HAL_COMP_ConfigHysterisis(COMP0, COMP_HYSSEL_High);
	
	HAL_COMP_ConfigInputLevel(COMP0, COMP_INPUT_Vref9MV);
														 	
	HAL_COMP_Cmd(COMP0, ENABLE); //Comparator0 enable
	
	HAL_COMP_ConfigDebounce(COMP0, 1, COMP_DBCLK_LSI750, 48); //systemclk / (DBNCTB * 2) = 48MHz / 48 = 1MH 
	
	HAL_COMP_ConfigInterrupt(COMP0, COMP_TRIOUT_NORMAL, COMP_INTOUT_HIGH, ICON_BOTH_EDGE_INT);	
	
	HAL_COMP_InterruptCmd(COMP0, ENABLE);	
#endif

	NVIC_SetPriority(COMP_IRQn, 0);
	NVIC_EnableIRQ(COMP_IRQn);
	
	HAL_SCU_WakeUpSRCCmd(WAKEUP_COMP, ENABLE);

}


/**********************************************************************
 * @brief		Comp_Wakeup_Loop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void Comp_Wakeup_Loop( void)
{
		delay();delay();delay();delay();delay();
	
		_DBG("\r\n ENTER POWER DOWN MODE!! (Need to trigger comp interrupt) \r\n");
		HAL_PWR_EnterPowerDownMode();

	
		//Wake-UP
		HAL_COMP_ClearStatusAll_IT();
	
		SystemClock_Config();

		delay();delay(); 	

		UART_DebugMessage();
}
/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void mainloop(void)
{	

	/*Initialize menu prinf*/	
	DEBUG_Print_menu();	
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*COMP Configure*/
 COMP_Configure(); 

  	/*Enable IRQ Interrupts */
	__enable_irq(); 
	
	 /* Infinite Loop */
	while(1)
	{
		Comp_Wakeup_Loop();
	}
}


/**********************************************************************
 * @brief		main
 * @param[in]	None
 * @return 		None
 **********************************************************************/
int main(void)
{

 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif


