/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"



/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void CRC_Configure(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;
uint32_t check1, check2;

const uint8_t menu[] =
"\n\r************************************************\n\r"
"CRC_Checksum \n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
"************************************************\n\r";


/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		CRC_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void CRC_Configure(void)
{
	uint16_t	chk_sum_init;		
	
	if(HAL_CRC_Init() != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();				
	}
	
	chk_sum_init = 0;			// CRC_INIT Value
	
	// Checksum calculation by CRC block
	if(HAL_CRC_SetUserInput(chk_sum_init) != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();				
	}
	
	HAL_CRC_ConfigUserMode(CRC_CHECKSUM, CRC_CCITT, CRC_MSB);

}

/**********************************************************************
 * @brief		CRC_Checksum_Loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void CRC_Checksum_Loop(void)
{
	uint32_t	chk_in;
	uint16_t	chk_sum;
	uint16_t	chk_sum_init;		
	uint32_t	i;
	
	// Checksum calculation by Core
	chk_sum_init = 0;			// CRC_INIT Value
	chk_sum = chk_sum_init;
	chk_in = 0x12345678;	// CRC_IN Value
	for (i=0; i<4; i++) 
	{
		chk_sum += (uint16_t) (chk_in >> (24-i*8)) & 0x00FF;
	}
	
	check1 = chk_sum;

	CRCRun();
	HAL_CRC_UserInput(chk_in);
	chk_sum = HAL_CRC_GetData();
	CRCStop();
	
	check2 = chk_sum;
  
	_DBG("\r\n sw crc value = "); _DBH16(check1);
	_DBG("\r\n hw crc value = "); _DBH16(check2);
	
	if(check1 == check2) {
		_DBG("\r\n correct CRC!!\r\n");		
	} else {
		_DBG("\r\n incorrect CRC!!\r\n");		
	}
	
}

/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void mainloop(void)
{

	/*Initialize menu prinf*/	
	DEBUG_Print_menu();	
  
	/*Initialize CRC peripheral*/
	CRC_Configure(); 

	CRC_Checksum_Loop();
	/* Infinite Loop */
	while(1)
	{
	}
		
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{
	
	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
  /* Initialize all port */
	Port_Init(); 

	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
static void Error_Handler(void)
{
  while (1)
  {
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif


