/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* @author	PTS Team, ABOV Semiconductor Co., Ltd.
* @date		15. OCT. 2019
* @version	1.0
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void SysTick_Handler_IT (void);
void TIMER_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void DAC_Configure(void);
void UART_Configure(void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;
const uint8_t menu[] =
"************************************************\n\r"
" DAC Polling \n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
"************************************************\n\r";
TIMER1n_PERIODICCFG_Type TIMER1n_Config;
DAC_CFG_Type DA_config;
uint32_t i=0;
uint8_t flag=0;

/**********************************************************************
 * @brief		SysTick handler sub-routine (1ms)
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler_IT (void)			// SysTick Interrupt Handler @ 1000Hz
{
	if(msec)msec--;
}


/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TIMER_IRQHandler_IT(void)
{
	uint32_t status=0;
	
	status = HAL_TIMER1n_GetMatchInterrupt(TIMER11);
	if (status == 1){
		HAL_TIMER1n_ClearMatchInterrupt(TIMER11);
		flag=1;
		PB->OUTDR ^= (1<<12);
	}
}

/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void GPIO_Configure(void)
{
	//DAC_OUT1 setting
	HAL_GPIO_ConfigOutput(PA, 4, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 4, FUNC10);
	HAL_GPIO_ConfigPullup(PA, 4, DISPUPD);	
}


/**********************************************************************
 * @brief		DAC Configure
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DAC_Configure(void)
{
	uint32_t DAC_OSEL = 1;//0;	// 0 = DISABLE, 1 = DACOUT1, 2 = DACOUT2, 3 = DACOUT3

	//DAC Setting
	DA_config.BufSel = ENABLE;
	DA_config.RefSel = DAC_REFAVDD;
	DA_config.IntSel = DISABLE;
	DA_config.DataChangeSel = ENABLE;
	DA_config.ReloadSel = DAC_RLDS_TIMER11;
	
	DA_config.PGASel = DAC_PGAn0dB;
	
	DA_config.OffsetSel = DISABLE;
	DA_config.OffsetDIRSel = DAC_OFFSETDIRn;
	DA_config.OffsetVal = 0;
	
	if(HAL_DAC_Init(DAC, &DA_config, DAC_OSEL) != HAL_OK)
	{
		/* Initialization Error */
           Error_Handler();				
	}
	
	SysTick_Config(SystemCoreClock/1000);  //1msec interrupt 
	
	HAL_DAC_Start(DAC, ENABLE);
}

/**********************************************************************
 * @brief		TIMER Configure
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TIMER_Configure(void)
{
  //Timer1n clock source from PCLK
	HAL_SCU_Timer1n_ClockConfig(T1NCLK_MCCR1);
	HAL_SCU_MiscClockConfig(1, TIMER1N_TYPE, HSI_MISC_CLOCK, 1);
	TIMER1n_Config.CkSel = MCCR1PCLK;    
	TIMER1n_Config.Prescaler = 48;    // 48Mhz / 48 = 1Mhz ->1us
		
	TIMER1n_Config.TnADR = (1000); // 100us
	TIMER1n_Config.StartLevel=START_LOW;
	
	if(HAL_TIMER1n_Init(TIMER11, PERIODIC_MODE, &TIMER1n_Config) != HAL_OK)
	{
		/* Initialization Error */
		Error_Handler();
	}
	HAL_TIMER1n_ConfigInterrupt(TIMER11, TIMER1n_INTCFG_MIE, ENABLE);
	
	NVIC_EnableIRQ(TIMER11_IRQn);
	HAL_TIMER1n_Cmd(TIMER11, ENABLE); // timer start & clear
}

/**********************************************************************
 * @brief		DAC_SetData_loop
 * @param[in]	None
 * @return	None
 **********************************************************************/

void DAC_SetData_loop(void)
{
	if(flag)
	{
		HAL_DAC_SetData(DAC, i++);
		if(i==0x400)
		{
			i=0;
		}
		flag=0;
	}
}
/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void mainloop(void)
{

	/*Initialize menu prinf*/	
	DEBUG_Print_menu();	
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*Configure DAC*/
	DAC_Configure();
	
	/*Configure TIMER*/
	TIMER_Configure();
	
      /*Enable IRQ Interrupts */
	__enable_irq(); 
	 
	/* Infinite Loop */
	while(1)
	{	
		DAC_SetData_loop();
	}
}


/**********************************************************************
 * @brief		main
 * @param[in]	None
 * @return 		None
 **********************************************************************/
int main(void)
{
 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();  
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif


