/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define TEST2	(*((volatile unsigned int *) 0x20000000))
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void SysTick_Handler_IT (void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void flash_iap_pgm(void);
void flash_iap_erase(void);
void flash_crc_read (void);
void GPIO_Configure(void);
void SysTick_Configure(void);
void FMC_Program_loop(void);
void mainloop(void);
int main(void);



/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" FMC_CRC_CCITT \n\r"
"************************************************\n\r";
uint32_t msec;
uint32_t t500msec;
uint32_t tflag;



/**********************************************************************
 * @brief		SysTick handler sub-routine (1ms)
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler_IT (void) 					// SysTick Interrupt Handler @ 1000Hz
{
	if(msec)msec--;
	
	t500msec++;
	if (t500msec>=500){
		if (tflag==0){
			tflag=1;
		}
		else {
			tflag=0;
		}
	}
}


/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		Flash Program Write
 * @param[in]	None
 * @return	None
 **********************************************************************/
void  flash_iap_pgm(void)
{
	int	i;
	unsigned char	fbuffer[512];
	
	for(i=0;i<512;i++) {
		fbuffer[i]= 0x4F;
	}
	__disable_irq();

	ProgramPage(0,0xFF00,512, fbuffer);

	__enable_irq();
}

/**********************************************************************
 * @brief		Flash Program Erase
 * @param[in]	None
 * @return	None
 **********************************************************************/
void flash_iap_erase(void)
{
	__disable_irq();
	
	EraseSector(0, 0xFF00);

	__enable_irq();
}

/**********************************************************************
 * @brief		Flash CRC-CCITT Check
 * @param[in]	None
 * @return	None
 **********************************************************************/
void flash_crc_read (void)
{
	
	unsigned long *dst;
	unsigned long *str;
	unsigned long i;
	volatile unsigned long crc_ccitt_in;
		
	FMC->MR = 0x66;
	FMC->MR = 0x99;
	FMC->WPROT = 0x00000000;	// Remove FLASH write protection
	FMC->MR = 0x00;
	FMC->MR = 0x00;
	
	FMC->MR = 0x81;FMC->MR = 0x28;
	FMC->CFG= 0x78580300 | (1<<7) | (1<<6);		//	Init CRC
	FMC->CFG&= ~(1<<7);
	FMC->MR = 0x00;FMC->MR = 0x00;
		
	dst=(unsigned long *) 0x0000FF00;
	str=(unsigned long *) 0x0000FE00;
		
	for(i=0;i<32;i++)
	{
		str[i]=dst[i];
		
		if((* (unsigned long *) 0x40000120) != 0x00000000)
		{	
			 crc_ccitt_in= *(unsigned long *) 0x40000120;
			break;
		}
	}

	__NOP();__NOP();__NOP();
	
	FMC->CR = 0x00000000;
		
	FMC->MR = 0x81;FMC->MR = 0x28;	
	FMC->CFG = 0x78580380; 
	FMC->MR = 0x00;FMC->MR = 0x00;
	
	__NOP();__NOP();__NOP();
	
	return;
}
/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void GPIO_Configure(void)
{
	t500msec=0;tflag=0;
}

/**********************************************************************
 * @brief		SysTick_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void SysTick_Configure(void)
{
	SysTick_Config(SystemCoreClock/1000);  //1msec interrupt 
}


/**********************************************************************
 * @brief		FMC_Program_loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void FMC_Program_loop(void)
{
		msec = 1000;
		while(msec);
	  flash_iap_pgm();
		__NOP();
		
		msec = 1000;
		while(msec);
		flash_crc_read();
		__NOP();
		
		msec = 1000;
		while(msec);
		flash_iap_erase();
		__NOP();
}
/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{

	/*Initialize menu prinf*/	
	DEBUG_Print_menu();	
	
	/*Initialize port peripheral*/
	GPIO_Configure();
	
	/*Initialize RTC peripheral*/
	SysTick_Configure(); 

	 /*  Enable IRQ Interrupts */
	 __enable_irq(); 
	 
	/* Infinite Loop */
	while(1)
	{
		FMC_Program_loop();
	}	

}

/**********************************************************************
 * @brief		Main 
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{
	SCB->VTOR = 0x20000000;
	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
  /* Initialize all port */
	Port_Init(); 

	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();	
	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

