/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void flash_iap_pgm(void);
void flash_iap_erase(void);
void flash_crc_read (void);
void GPIO_Configure(void);
void FMC_Configure(void);
void FMC_pgm_loop(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" FMC_WriteErase_lock_in_RAM \n\r"
"************************************************\n\r";


/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		Flash Write
 * @param[in]	None
 * @return	None
 **********************************************************************/
void	flash_iap_pgm(void)
{
	int	i;
	unsigned char	fbuffer[512];
	
	for(i=0;i<512;i++) {
		fbuffer[i]= i&0xff;
	}
	__disable_irq();
	ProgramPage(0,0xFE00,512, fbuffer);
	__enable_irq();
}

/**********************************************************************
 * @brief		Flash Erase
 * @param[in]	None
 * @return	None
 **********************************************************************/
void	flash_iap_erase(void)
{
	__disable_irq();
	EraseSector(0, 0xFE00);
	__enable_irq();
}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Test Pin setting PE0
	HAL_GPIO_ConfigOutput(PB, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 0, DISPUPD);	
	HAL_GPIO_ClearPin(PB, _BIT(0));

	// Test Pin setting PE1
	HAL_GPIO_ConfigOutput(PB, 1, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 1, DISPUPD);
	HAL_GPIO_ClearPin(PB, _BIT(1));
}


/**********************************************************************
 * @brief		FMC_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void FMC_Configure(void)
{
	FMC->MR = 0x66;
	FMC->MR = 0x99;
	FMC->LOCK = 0x00000000;	// flash read protection on
	FMC->MR = 0x00;
	FMC->MR = 0x00;	
	
	LockEraseChip();

	FMC->MR = 0x66;
	FMC->MR = 0x99;
	FMC->LOCK = 0x000000FF;	// flash read protection off
	FMC->MR = 0x00;
	FMC->MR = 0x00;
}

void FMC_pgm_loop(void)
{
	flash_iap_erase();
	__NOP();

	flash_iap_pgm();
	__NOP();

	EraseChipBBlock ();
	__NOP();
}

/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{

	/*Initialize menu prinf*/	
	DEBUG_Print_menu();		
	
	/*Initialize port peripheral*/
	GPIO_Configure();
	
	/*Initialize FMC peripheral*/
	FMC_Configure(); 

	 /*  Enable IRQ Interrupts */
	 __enable_irq(); 
	  
	/* Infinite Loop */
	while(1)
	{
		FMC_pgm_loop();
	}	

}

/**********************************************************************
 * @brief		Main 
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{
	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
  /* Initialize all port */
	Port_Init(); 

	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();	
	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

