/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void SysTick_Handler_IT (void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void flash_iap_pgm(void);
void flash_iap_erase(void);
void flash_crc_read (void);
void GPIO_Configure(void);
void SysTick_Configure(void);
void FMC_pgm_loop(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;
uint32_t t500msec;
uint32_t tflag;
unsigned long flash_array[512/4];

const uint8_t menu[] =
"************************************************\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" FMC_WriteErase_selfPGM\n\r"
"************************************************\n\r";


/**********************************************************************
 * @brief		SysTick handler sub-routine (1ms)
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler_IT (void) 					// SysTick Interrupt Handler @ 1000Hz
{
	if(msec)msec--;
	
	t500msec++;
	if (t500msec>=500){
		if (tflag==0){
			tflag=1;
			HAL_GPIO_SetPin(PB,_BIT(1));
		}
		else {
			tflag=0;
			HAL_GPIO_ClearPin(PB,_BIT(1));
		}
	}
}

/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/**********************************************************************
 * @brief		Flash ProgramPage Write
 * @param[in]	None
 * @return	None
 **********************************************************************/
void	flash_iap_pgm(void)
{
	unsigned int	i;
	unsigned char	fbuffer[512];
	
	for(i=0;i<512;i++) {
		fbuffer[i]= i&0xff;
	}
	__disable_irq();
			
	HAL_GPIO_ClearPin(PB,_BIT(0));
	ProgramPage(0xFE00,512, fbuffer);
	HAL_GPIO_SetPin(PB,_BIT(0));
	ProgramPage(0x10000,512, fbuffer);
	HAL_GPIO_SetPin(PB,_BIT(0));
	ProgramPage(0x10200,512, fbuffer);
	HAL_GPIO_SetPin(PB,_BIT(0));
	ProgramPage(0x10400,512, fbuffer);
	HAL_GPIO_SetPin(PB,_BIT(0));
	ProgramPage(0x10600,512, fbuffer);
	HAL_GPIO_SetPin(PB,_BIT(0));
	ProgramPage(0x10800,512, fbuffer);
	HAL_GPIO_SetPin(PB,_BIT(0));
	ProgramPage(0x11000,512, fbuffer);
	HAL_GPIO_SetPin(PB,_BIT(0));
	ProgramPage(0x11200,512, fbuffer);
	HAL_GPIO_SetPin(PB,_BIT(0));
	ProgramPage(0x11600,512, fbuffer);
	HAL_GPIO_SetPin(PB,_BIT(0));
	ProgramPage(0x3F000000,512, fbuffer);
	HAL_GPIO_ClearPin(PB,_BIT(0));
	ProgramPage(0x3F000200,512, fbuffer);
	HAL_GPIO_SetPin(PB,_BIT(0));
	ProgramPage(0x3F000400,512, fbuffer);	
	HAL_GPIO_ClearPin(PB,_BIT(0));
	__enable_irq();
}

/**********************************************************************
 * @brief		Flash ProgramPage Erase
 * @param[in]	None
 * @return	None
 **********************************************************************/
void	flash_iap_erase(void)
{
	__disable_irq();
	HAL_GPIO_SetPin(PB,_BIT(0));

#if	1	// Erase 1-Page
	EraseSector(0x10000);	
#endif
	
#if	0	// Erase 1k (2-Page)
	EraseSector1k(0x10000);
#endif

#if	0	// Erase 4k (8-Page)
	EraseSector4k(0x10F00);
#endif

#if	0	// Erase User OTP Mirror Area
	EraseSector(0x3F000200);
	EraseSector(0x3F000400);	
#endif

#if	0	// Bulk erase & BBLOCK DISABLE		
	EraseSector_Bulk (0x400, DISABLE);
#endif

#if	0	// Bulk erase & BBLOCK ENABLE
	EraseSector_Bulk(0x400, ENABLE);
#endif
		
	HAL_GPIO_ClearPin(PB,_BIT(0));	
	__enable_irq();
}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void GPIO_Configure(void)
{
		// Test Pin setting PB0	
	HAL_GPIO_ConfigOutput(PB, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 0, DISPUPD);	
	HAL_GPIO_ClearPin(PB, _BIT(0));

	// Test Pin setting PB1
	HAL_GPIO_ConfigOutput(PB, 1, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 1, DISPUPD);
	HAL_GPIO_ClearPin(PB, _BIT(1));

	// Test Pin setting PB2
	HAL_GPIO_ConfigOutput(PB, 2, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 2, DISPUPD);
	HAL_GPIO_ClearPin(PB, _BIT(2));
	t500msec=0;tflag=0;
}

/**********************************************************************
 * @brief		SysTick_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void SysTick_Configure(void)
{
	SysTick_Config(SystemCoreClock/1000);  //1msec interrupt 
}


/**********************************************************************
 * @brief		FMC_pgm_loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void FMC_pgm_loop(void)
{
	unsigned long *read_temp;
	unsigned int i;
	
	msec = 500;
	while(msec);
	HAL_GPIO_ClearPin(PB,_BIT(2));
	
	flash_iap_pgm();
	__NOP();
	
	read_temp=(unsigned long *) 0x10000;
	
	_DBG("flash wrtie data \r\n");
	for(i=0;i<(512/4);i++)
	{
		flash_array[i]=*read_temp++;
		_DBH(flash_array[i]);
	}
	_DBG("\r\n\r\n");
			
	msec = 500;
	while(msec);
	HAL_GPIO_SetPin(PB,_BIT(2));

	flash_iap_erase();
	__NOP();
	
	read_temp=(unsigned long *) 0x10000;
		
	_DBG("flash erase data \r\n");
	for(i=0;i<(512/4);i++)
	{
		flash_array[i]=*read_temp++;
		_DBH(flash_array[i]);
	}
	_DBG("\r\n\r\n");
}

/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	
	/*Initialize menu prinf*/	
	DEBUG_Print_menu();		
	
	/*Initialize port peripheral*/
	GPIO_Configure();
	
	/*Initialize RTC peripheral*/
	SysTick_Configure(); 

	 /*  Enable IRQ Interrupts */
	 __enable_irq(); 
	 
	/* Infinite Loop */
	while(1)
	{
		FMC_pgm_loop();
	}	
		
}

/**********************************************************************
 * @brief		Main 
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{
	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
  /* Initialize all port */
	Port_Init(); 

	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();	
	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

