/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void SysTick_Handler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void SysTick_Configure(void);
void GPIO_LED_Blinky(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;
const uint8_t menu[] =
"************************************************\n\r"
" GPIO demo \n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" Using GPIO test \n\r"
"************************************************\n\r";
static GPIO_CFG_Type  GPIO_InitStruct;

/**********************************************************************
 * @brief		SysTick handler sub-routine (1ms)
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler_IT(void) 					// SysTick Interrupt Handler @ 1000Hz
{
	if(msec)msec--;
}

/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	/*Configure IOs in output push-pull mode to drive external LEDs */
	GPIO_InitStruct.Output  = PUSH_PULL_OUTPUT;
	GPIO_InitStruct.Pull  = DISPUPD;
	GPIO_InitStruct.Pin = 0;
	HAL_GPIO_Init(PC, &GPIO_InitStruct);
	
	GPIO_InitStruct.Pin = 1;
	HAL_GPIO_Init(PC, &GPIO_InitStruct);
	
	GPIO_InitStruct.Pin = 2;
	HAL_GPIO_Init(PC, &GPIO_InitStruct);
	
	GPIO_InitStruct.Pin = 3;
	HAL_GPIO_Init(PC, &GPIO_InitStruct);
	
	GPIO_InitStruct.Pin = 4;
	HAL_GPIO_Init(PC, &GPIO_InitStruct);
	
	GPIO_InitStruct.Pin = 5;
	HAL_GPIO_Init(PC, &GPIO_InitStruct);
	
	GPIO_InitStruct.Pin = 6;
	HAL_GPIO_Init(PC, &GPIO_InitStruct);
}

/**********************************************************************
 * @brief		SysTick_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SysTick_Configure(void)
{
	SysTick_Config(SystemCoreClock/1000);  //1msec interrupt 	
}

/**********************************************************************
 * @brief		GPIO_LED_Toggle
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_LED_Blinky(void)
{
	 uint32_t i;
	
	  msec = 200;
		while(msec);
		
		for(i=0;i<6;i++){
			msec = 50;
			while(msec);
			HAL_GPIO_ClearPin(PC,_BIT(i));
		}
		
		for(i=0;i<6;i++){
			msec = 50;
			while(msec);
			HAL_GPIO_SetPin(PC,_BIT(i));
		}
}


/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void mainloop(void)
{

	/*Initialize menu prinf*/	
	DEBUG_Print_menu();	
	
	/*Initialize port peripheral*/
	GPIO_Configure();


	/*Initialize SysTick peripheral*/
	SysTick_Configure(); 	

	 /*  Enable IRQ Interrupts */
	 __enable_irq(); 
	  
	/* Infinite Loop */
	while(1)
	{
	  GPIO_LED_Blinky();
	}
}


/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{	
	
	/* Configure the system clock to HSE  48MHz */
	SystemClock_Config();

	 /* Initialize all port */
	Port_Init(); 

	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop(); 
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif


