/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"



/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void SysTick_Handler_IT(void);
void GPIOAB_IRQHandler_IT(void);
void GPIOCF_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void SysTick_Configure(void);
void delay (void);
void mainloop(void);
int main (void);



/* Private variables ---------------------------------------------------------*/
uint16_t status;

PORT_Type *Px;
uint8_t pin_number;
uint8_t count = 3;
uint8_t flag;
const uint8_t menu[] =
"************************************************\n\r"
" GPIO demo \n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" This example used to configure the\n\r "
" GPIO PB14,15 as source of external Interrupt \n\r"
"************************************************\n\r";
uint8_t mode_level[]=
	"\t < Check the MODE LEVEL >\n\r"
	"\t - 0: Prohibit external Interrupt \n\r"
	"\t - 1: low level Interrupt \n\r"
	"\t - 2: high level Interrupt \n\r";
uint8_t mode_edge[]=
	"\t < Check the MODE EDGE >\n\r"
	"\t - 0: Prohibit external Interrupt \n\r"
	"\t - 1: falling edge Interrupt \n\r"
	"\t - 2: rising edge Interrupt \n\r"
	"\t - 3: both edge Interrupt \n\r";
uint8_t mode_port[]=
	"\n\rSelect Port A, B, C, F : Port ";

uint8_t mode_pin[]=
	"\n\rSelect Pin 0~15 (10->A ...) : ";



/**********************************************************************
 * @brief		GPIOAB_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void GPIOAB_IRQHandler_IT(void)
{
	uint32_t j;
	uint32_t status;
	
	// GPIOAB interrupt 
	{	
		status = HAL_GPIO_EXTI_GetState(Px);
		
		if(status & (3UL<<(pin_number<<1)))
		{
			HAL_GPIO_EXTI_ClearPin(Px, status & (3UL<<(pin_number<<1)));
			for (j= 0; j<4; j++)
			{
				HAL_GPIO_SetPin(PC, _BIT(13));
				delay();
				HAL_GPIO_ClearPin(PC, _BIT(13));
				delay();
			}
		}			
	}
	count--;
}


/**********************************************************************
 * @brief		GPIOCF_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void GPIOCF_IRQHandler_IT(void)
{
	uint32_t j;
	uint32_t status;

	// GPIOCF interrupt 
	{	
		status = HAL_GPIO_EXTI_GetState(Px);
		
		if(status & (3UL<<(pin_number<<1)))
		{
			HAL_GPIO_EXTI_ClearPin(Px, status & (3UL<<(pin_number<<1)));
			for (j= 0; j<4; j++)
			{
				HAL_GPIO_SetPin(PB, _BIT(1));
				delay();
				HAL_GPIO_ClearPin(PB, _BIT(1));
				delay();
			}
		}			
	}
	count--;
}


/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/*********************************************************************//**
 * @brief		Delay function
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void delay (void) 
{
	uint32_t i;
	
	for (i = 0; i < 0x100000; i++) ;
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	uint8_t port;

	#ifdef _DEBUG_MSG	
	_DBG(mode_port);		// Select Port
	#endif
	port=0;
	flag=1;
	
	/** Select Port A, B, C, F **/
	while (flag)
	{
		port = _DG;
		switch (port)
		{
			case 'a': case 'A': _DBC(port); Px = PA; flag = 0; break;
			case 'b': case 'B': _DBC(port); Px = PB; flag = 0; break;
			case 'c': case 'C': _DBC(port); Px = PC; flag = 0; break;
			case 'f': case 'F': _DBC(port); Px = (PORT_Type*)PF; flag = 0; break;
			default:
				#ifdef _DEBUG_MSG
				_DBG("\n\r...Please input character from A,B,C,F only! \n\r");
				_DBG(mode_port);
			  #endif
				break;
		}
	}
	
	if(Px==PA | Px==PB)
	{
		// Test Pin setting PC13
		HAL_GPIO_ConfigOutput(PC, 13, PUSH_PULL_OUTPUT);
		HAL_GPIO_ConfigPullup(PC, 13, DISPUPD);	
		HAL_GPIO_ClearPin(PC, _BIT(13));
	}
	else if(Px==PC | Px==(PORT_Type *)PF)
	{
		HAL_GPIO_ConfigOutput(PB, 1, PUSH_PULL_OUTPUT);
		HAL_GPIO_ConfigPullup(PB, 1, DISPUPD);	
		HAL_GPIO_ClearPin(PB, _BIT(1));
	}
  #ifdef _DEBUG_MSG
	_DBG("\n\r");
	
	/** Select Pin Number **/
	_DBG(mode_pin);
	#endif 
		pin_number=_DG;
	
	#ifdef _DEBUG_MSG
	_DBC(pin_number);
	_DBG("\n\r");
	#endif
	if(pin_number=='A' | pin_number=='a')		{ pin_number=0x0A; }
	else if(pin_number=='B' | pin_number=='b')	{ pin_number=0x0B; }
	else if(pin_number=='C' | pin_number=='c')	{ pin_number=0x0C; }
	else if(pin_number=='D' | pin_number=='d')	{ pin_number=0x0D; }
	else if(pin_number=='E' | pin_number=='e')	{ pin_number=0x0E; }
	else if(pin_number=='F' | pin_number=='f')	{ pin_number=0x0F; }
	else
	{
		pin_number=pin_number-0x30;
	}
		
	// external interrupt test pin Px15
	HAL_GPIO_ConfigOutput(Px, pin_number, INPUT);
	HAL_GPIO_ConfigPullup(Px, pin_number, ENPU);
	HAL_GPIO_SetDebouncePin(Px, pin_number);
	
	if(Px==PA | Px==PB)
	{
		NVIC_SetPriority(GPIOAB_IRQn, 3);	
		NVIC_EnableIRQ(GPIOAB_IRQn);
	}
	else if(Px==PC | Px==(PORT_Type *)PF)
	{
		NVIC_SetPriority(GPIOCF_IRQn, 4);	
		NVIC_EnableIRQ(GPIOCF_IRQn);
	}

	HAL_SCU_MiscClockConfig(4,PD_TYPE,LSI_MISC_CLOCK,101);			//  PORT A,B,C Debounce set. ( Use Switch )
	
	/* Choose interrupt test ( Level or Edge ) */
	#ifdef _DEBUG_MSG
	_DBG("\n\rPlease input character 0:LEVEL or 1: EDGE! \n\r");	// First Massage
	#endif


}


/**********************************************************************
 * @brief		GPIO_EXINT_TEST
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_EXINT_TEST(void)
{
	
	uint8_t val_level_or_edge, mode;
	val_level_or_edge = 0;	
	flag = 1;
	while (flag)
	{
		val_level_or_edge = _DG;
		#ifdef _DEBUG_MSG
		_DBG("Select Number : ");
		#endif
		switch (val_level_or_edge)
		{
			case '0':
				#ifdef _DEBUG_MSG
				_DBC(val_level_or_edge);
				_DBG("\n\r");
			  #endif
				val_level_or_edge = IER_LEVEL;
				flag = 0; 
				break;			
			case '1': 
				#ifdef _DEBUG_MSG
				_DBC(val_level_or_edge);
				_DBG("\n\r");
			  #endif
				val_level_or_edge = IER_EDGE;
				flag = 0; 
				break;
			default:
				#ifdef _DEBUG_MSG
				_DBG("\n\r...Please input character 0:LEVEL or 1: EDGE! \n\r");
			  #endif
				break;
		}
	}
	/***********************************/
	/* Edge or Level Mode*/
	/***********************************/
	if (val_level_or_edge==IER_LEVEL){
		_DBG(mode_level);
	}
	else {
		_DBG(mode_edge);		
	}
	
	mode = 0;
	flag = 1;
	while (flag)
	{
		mode = _DG;
		if (val_level_or_edge==IER_LEVEL){
			switch (mode)
			{
				case '0':
				case '1': 
				case '2': 					
					_DBC(mode);
					mode = mode -0x30;	// ASCII CODE
					flag = 0; 
					break;
				default:
					#ifdef _DEBUG_MSG
					_DBG("\n\r...Please input digit from 0 to 2 only! \n\r");
				   #endif
					break;
			}			
		}
		else {
			switch (mode)
			{
				case '0':
				case '1': 
				case '2': 		
				case '3': 						
					_DBC(mode);
					mode = mode -0x30;
					flag = 0; 
					break;
				default:
					#ifdef _DEBUG_MSG
					_DBG("\n\r...Please input digit from 0 to 3 only! \n\r");
				  #endif
					break;
			}			
		}
	}
	
	_DBG("\n\rCheck GPIO extint !!(test pin aPB3 or PC13)\n\r");
	HAL_GPIO_EXTI_Config(Px, pin_number, val_level_or_edge, mode);
}


/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void mainloop(void)
{
	
	/*Initialize menu prinf*/	
	DEBUG_Print_menu();	
	
	/*Initialize port peripheral*/
	GPIO_Configure();

	
	GPIO_EXINT_TEST();

	 /*  Enable IRQ Interrupts */
	 __enable_irq(); 
	
	/* Infinite Loop */
	while(1)
	{
	}
}


/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{	
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop(); 
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

