/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void SysTick_Handler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void SysTick_Configure(void);
void GPIO_Input_Output_Test(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;
const uint8_t menu[] =
"************************************************\n\r"
"\t - GPIO demo \n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
"\t - GPIO Input Output test \n\r"
"************************************************\n\r";

uint8_t mode_port[]=
	"\n\rSelect Port A, B, C, F : Port ";

uint8_t mode_pin[]=
	"\n\rSelect Pin 0~15 (10->A ...) : ";
/**********************************************************************
 * @brief		SysTick handler sub-routine (1ms)
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler_IT (void) 					// SysTick Interrupt Handler @ 1000Hz
{
	if(msec)msec--;
}


/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	
}


/**********************************************************************
 * @brief		SysTick_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SysTick_Configure(void)
{
	SysTick_Config(SystemCoreClock/1000);  //1msec interrupt 	
}


/**********************************************************************
 * @brief		GPIO_Input_Output_Test
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Input_Output_Test(void)
{
	uint8_t flag;
	uint8_t port;
	uint8_t pin_number;
	PORT_Type *PCx;
	
		/* Get some inputs */
	_DBG("\n\rPlease enter PORT number (from A, B, C, F) \n\r");	

	port = 0;
	flag = 1;
	while (flag)
	{
		port = _DG;
		switch (port)
		{
			case 'A': _DBC(port); PCx = PA; flag = 0; break;
			case 'B': _DBC(port); PCx = PB; flag = 0; break;
			case 'C': _DBC(port); PCx = PC; flag = 0; break;
			case 'F': _DBC(port); PCx = PF; flag = 0; break;
			default:
				_DBG("\n\r...Please input character from A, B, C, F only! \n\r");
				break;
		}
	}
	
	if(PCx==PA | PCx==PB)
	{
		// Test Pin setting PC13
		HAL_GPIO_ConfigOutput(PC, 13, PUSH_PULL_OUTPUT);
		HAL_GPIO_ConfigPullup(PC, 13, DISPUPD);	
		HAL_GPIO_ClearPin(PC, _BIT(13));
	}
	else if(PCx==PC | PCx==(PORT_Type *)PF)
	{
		HAL_GPIO_ConfigOutput(PB, 1, PUSH_PULL_OUTPUT);
		HAL_GPIO_ConfigPullup(PB, 1, DISPUPD);	
		HAL_GPIO_ClearPin(PB, _BIT(1));
	}
	
	/** Select Pin Number **/
	_DBG(mode_pin);
	pin_number=_DG;
	
	_DBC(pin_number);
	_DBG("\n\r");
	if(pin_number=='A' | pin_number=='a')		{ pin_number=0x0A; }
	else if(pin_number=='B' | pin_number=='b')	{ pin_number=0x0B; }
	else if(pin_number=='C' | pin_number=='c')	{ pin_number=0x0C; }
	else if(pin_number=='D' | pin_number=='d')	{ pin_number=0x0D; }
	else if(pin_number=='E' | pin_number=='e')	{ pin_number=0x0E; }
	else if(pin_number=='F' | pin_number=='f')	{ pin_number=0x0F; }
	else
	{
		pin_number=pin_number-0x30;
	}
	
	HAL_GPIO_ConfigOutput(PCx, pin_number, INPUT);

	_DBG("PORT SETTING OK\r\n");
	
	while(1) {
			msec = 400;
			while(msec);
			
			if((HAL_GPIO_ReadPin(PCx)<<pin_number) & 0x1) {
				if(PCx==PA | PCx==PB)
					HAL_GPIO_SetPin(PC, _BIT(13));
				else if(PCx==PC | PCx==(PORT_Type *)PF)
					HAL_GPIO_SetPin(PB, _BIT(1));				
			}else{
				if(PCx==PA | PCx==PB)
					HAL_GPIO_ClearPin(PC, _BIT(13));
				else if(PCx==PC | PCx==(PORT_Type *)PF)
					HAL_GPIO_ClearPin(PB, _BIT(1));				
			}
	}

}


/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void mainloop(void)
{

	/*Initialize menu prinf*/
	DEBUG_Print_menu();	

	/*Initialize port peripheral*/
	GPIO_Configure();
	
	/*Initialize SysTick peripheral*/
	SysTick_Configure(); 
	
	 /*  Enable IRQ Interrupts */
	 __enable_irq(); 
	
	GPIO_Input_Output_Test();
	 
	while(1)
	{
	}
}


/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{	
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop(); 
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

