
/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define M24C02_ADDR    (0xAE)
/** Max buffer length */
#define BUFFER_SIZE			10

/* Private function prototypes -----------------------------------------------*/
void I2C0_IRQHandler_IT(void);
void I2C1_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void Buffer_Init(uint8_t* buffer, uint8_t type);
void App_I2C_Polling_Master(void);
void App_I2C_Interrupt_Master(void);
void App_I2C_Polling_Slave (void);
void App_I2C_Interrupt_Slave (void);
void GPIO_Configure(void);
void I2C_Configure(void);
void I2C_PollingInterrupt_loop(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;
//I2C_Type* I2Cx = I2C0;
I2C_Type* I2Cx = I2C1;

Bool complete;
Bool isMasterMode = TRUE;
/** These global variables below used in interrupt mode - Slave device ----------------*/
uint8_t Master_Buf[BUFFER_SIZE];

const uint8_t menu[] =
"\n\r************************************************\n\r"
"I2C demo \n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
"\n\rPress 1-4 to select I2C running mode:\n\r\n\r"
"\t 1 - I2C Polling   Master\n\r"
"\t 2 - I2C Interrupt Master\n\r"
"\t 3x - I2C Polling   Slave \n\r"
"\t 4x - I2C Interrupt Slave \n\r"
"************************************************\n\r";
const uint8_t  menu1[] = "\n\r \t - Press x to exit this mode!\n\r";
const uint8_t  menu3[] = "\t - Press c to continue...\n\r";
uint8_t dispSlave_Buf[BUFFER_SIZE];


/**********************************************************************
 * @brief		I2C0_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void I2C0_IRQHandler_IT(void)
{
	if (isMasterMode)// Master Mode
	{
		HAL_I2C_Master_IRQHandler_IT(I2C0);
		if (HAL_I2C_Master_GetState(I2C0)){
			complete = TRUE;
		}
	}
	else// Slave Mode
	{
		HAL_I2C_Slave_IRQHandler_IT(I2C0);
		if (HAL_I2C_Slave_GetState(I2C0))
		{
			complete = TRUE;
		}
	}	
}

/**********************************************************************
 * @brief		I2C1_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void I2C1_IRQHandler_IT(void)
{
	if (isMasterMode)// Master Mode
	{
		HAL_I2C_Master_IRQHandler_IT(I2C1);
		if (HAL_I2C_Master_GetState(I2C1)){
			complete = TRUE;
		}
	}
	else// Slave Mode
	{
		HAL_I2C_Slave_IRQHandler_IT(I2C1);
		if (HAL_I2C_Slave_GetState(I2C1))
		{
			complete = TRUE;
		}
	}	
}

/**********************************************************************
 * @brief		DEBUG_Print_menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/*********************************************************************//**
 * @brief		Initialize buffer
 * @param[in]	buffer	buffer to initialize
 * @param[in]	type:
 * 				- 0: Initialize buffer with 0
 * 				- 1: Initialize buffer with increment value from 0
 * @return 		None
 **********************************************************************/
void Buffer_Init(uint8_t* buffer, uint8_t type)
{
	uint8_t i;

	if (type) {
		for (i = 0; i < BUFFER_SIZE; i++) {
			buffer[i] = i;
		}
	}
	else	{
		for (i = 0; i < BUFFER_SIZE; i++) {
			buffer[i] = 0;
		}
	}
}



/*********************************************************************//**
 * @brief		I2C master application in polling mode
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void App_I2C_Polling_Master(void)
{
	uint8_t j=0;
	I2C_M_SETUP_Type transferMCfg;
	uint8_t buffer = 0xFF, regAddr[2] = {0x00, 0x00};
	uint8_t Master_Buf[BUFFER_SIZE];

	while (1) {
		_DBG_("\n\r\n\rI2C Polling Master is running...\n\r");
		_DBG_(menu1);
		_DBG_(menu3);

		while (1)
		{
			buffer = _DG;
			if (buffer == 'x' || buffer == 'X')
			{
				return;
			}
			else if (buffer == 'c' || buffer == 'C')
			{
				break;
			}
		}
		/* Transmit -------------------------------------------------------- */
		_DBG_("\n\rPress '1' to transmit");
		while (1)
		{
			buffer = _DG;
			if (buffer == '1')
				break;
		}
		_DBG_("\n\rTransmitting...");

		/* Initialize buffer */
		Buffer_Init(Master_Buf, 0);
		Master_Buf[0]=0x00; //address
		Master_Buf[1]=0x00; //data	
		Master_Buf[2]=0xa4; //data	
		Master_Buf[3]=0xa6; //data	
		Master_Buf[4]=0xa8; //data		
		Master_Buf[5]=0xaA; //data		
		transferMCfg.sl_addr7bit = M24C02_ADDR;
		transferMCfg.tx_data = Master_Buf;
		transferMCfg.tx_length = 6;
		HAL_I2C_Master_Transmit(I2Cx, &transferMCfg, I2C_TRANSFER_POLLING);
		
		for (j=0;j<BUFFER_SIZE;j++)
		{
			_DBG("\n\rMaster_write_Buf[");_DBD(j);_DBG("]");_DBH(Master_Buf[j]);	
		}
		_DBG("\n\r");
		__NOP();
		__NOP();
		
		/* Transmit and receive -------------------------------------------------------- */
		_DBG_("\n\rPress '2' to Transmit, then repeat start and receive...");
		while (1)
		{
			buffer = _DG;
			if (buffer == '2')
				break;
		}

		/* Initialize buffer */
		Buffer_Init(Master_Buf, 0);

		transferMCfg.sl_addr7bit = M24C02_ADDR;
		transferMCfg.tx_data = regAddr;
		transferMCfg.tx_length = 2;
		transferMCfg.rx_data = Master_Buf;
		transferMCfg.rx_length = 4;
		transferMCfg.tx_count = 0;
		transferMCfg.rx_count = 0;
		HAL_I2C_MasterTransferData(I2Cx, &transferMCfg, I2C_TRANSFER_POLLING);
		
		for (j=0;j<BUFFER_SIZE;j++)
		{
			_DBG("\n\rMaster_read_Buf[");_DBD(j);_DBG("]");_DBH(Master_Buf[j]);	//_DBG("\n\r");
		}
		__NOP();
		__NOP();
	}
}

/***********************************************************************
 * @brief		I2C master application using interrupt
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void App_I2C_Interrupt_Master(void)
{
	uint8_t j=0;
	I2C_M_SETUP_Type transferMCfg;
	uint8_t buffer = 0xFF, regAddr[2] = {0x00, 0x00};
	uint8_t Master_Buf[BUFFER_SIZE];

	while (1) {
		_DBG_("\n\r\n\rI2C Interrupt Master is running...\n\r");
		_DBG_(menu1);
		_DBG_(menu3);

		while (1)
		{
			buffer = _DG;
			if (buffer == 'x' || buffer == 'X')
			{
				return;
			}
			else if (buffer == 'c' || buffer == 'C')
			{
				break;
			}
		}
		/* Transmit -------------------------------------------------------- */
		_DBG_("\n\rPress '1' to transmit");
		while (1)
		{
			buffer = _DG;
			if (buffer == '1')
				break;
		}
		_DBG_("\n\rTransmitting...");

		/* Initialize buffer */
		complete = FALSE;
		Buffer_Init(Master_Buf, 0);
		Master_Buf[0]=0x00; //address
		Master_Buf[1]=0x00; //data	
		Master_Buf[2]=0x4A; //data	
		Master_Buf[3]=0x6A; //data	
		Master_Buf[4]=0x8A; //data		
		Master_Buf[5]=0xA5; //data		
		transferMCfg.sl_addr7bit = M24C02_ADDR;
		transferMCfg.tx_data = Master_Buf;
		transferMCfg.tx_length = 6;
		HAL_I2C_Master_Transmit(I2Cx, &transferMCfg, I2C_TRANSFER_INTERRUPT);
		while(!complete);

		for (j=0;j<BUFFER_SIZE;j++)
		{
			_DBG("\n\rMaster_write_Buf[");_DBD(j);_DBG("]");_DBH(Master_Buf[j]);	
		}
		_DBG("\n\r");
		__NOP();
		__NOP();
		
		/* Transmit and receive -------------------------------------------------------- */
		_DBG_("\n\rPress '2' to Transmit, then repeat start and receive...");
		while (1)
		{
			buffer = _DG;
			if (buffer == '2')
				break;
		}

		/* Initialize buffer */
		complete = FALSE;
		Buffer_Init(Master_Buf, 0);
		transferMCfg.sl_addr7bit = M24C02_ADDR;
		transferMCfg.tx_data = regAddr;
		transferMCfg.tx_length = 2;
		transferMCfg.rx_data = Master_Buf;
		transferMCfg.rx_length = 4;
		transferMCfg.tx_count = 0;
		transferMCfg.rx_count = 0;
		
		HAL_I2C_MasterTransferData(I2Cx, &transferMCfg, I2C_TRANSFER_INTERRUPT);
		while(!complete);
		
		for (j=0;j<BUFFER_SIZE;j++)
		{
			_DBG("\n\rMaster_read_Buf[");_DBD(j);_DBG("]");_DBH(Master_Buf[j]);	//_DBG("\n\r");
		}
		__NOP();
	}
}

/**********************************************************************
 * @brief		I2C slave application in polling mode
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void App_I2C_Polling_Slave (void)
{
	I2C_S_SETUP_Type transferSCfg;
	uint8_t buffer = 0xFF,i=0,j;
	uint8_t Slave_Buf[BUFFER_SIZE], S_buff[2];

	while (1) {
		_DBG_("\n\r\n\rI2C Polling Slave is running...\n\r");
		_DBG_(menu1);
		_DBG_(menu3);

		while (1)
		{
			buffer = _DG;
			if (buffer == 'x' || buffer == 'X')
			{
				return;
			}
			else if (buffer == 'c' || buffer == 'C')
			{
				break;
			}
		}
		/* Receive -------------------------------------------------------- */
		_DBG_("\n\rPress '1' to Start");

		while (1)
		{
			buffer = _DG;
			if (buffer == '1')
				break;
		}
		_DBG_("\n\rReceiving...");

		/* Initialize buffer */
		Buffer_Init(Slave_Buf, 0);

		transferSCfg.rx_data = Slave_Buf;
//		transferSCfg.rx_length = 3;
		transferSCfg.rx_length = 7;
		HAL_I2C_Slave_Receive(I2Cx, &transferSCfg, I2C_TRANSFER_POLLING);
		for (j=0;j<BUFFER_SIZE;j++)
		{
			dispSlave_Buf[j]=Slave_Buf[j];
			_DBG("\n\rSlave_buf[");_DBD(j);_DBG("]");_DBH(Slave_Buf[j]);	//_DBG("\n\r");
		}
		__NOP();

		/* Receive and transmit -------------------------------------------------------- */
		_DBG_("\n\rStart Receive, wait for repeat start and transmit...");

		/* Initialize buffer */
		Buffer_Init(Slave_Buf, 1);

		transferSCfg.rx_data = S_buff;
		transferSCfg.rx_length = 2;
		Slave_Buf[0]=((i++)&0xff);//0x8A; //data	
		Slave_Buf[1]=0xAA; //data		
		Slave_Buf[2]=0xA5; //data	
		transferSCfg.tx_data = Slave_Buf;
		transferSCfg.tx_length = BUFFER_SIZE;
		HAL_I2C_SlaveTransferData(I2Cx, &transferSCfg, I2C_TRANSFER_POLLING);
		__NOP();		
		
		for (j=0;j<BUFFER_SIZE;j++)
		{
			_DBG("\n\rSlave_buf[");_DBD(j);_DBG("]");_DBH(Slave_Buf[j]);	//_DBG("\n\r");
		}
	}
}

/*********************************************************************//**
 * @brief		I2C slave application using interrupt
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void App_I2C_Interrupt_Slave (void)
{
	I2C_S_SETUP_Type transferSCfg;
	uint8_t buffer = 0xFF;
	uint8_t Slave_Buf[BUFFER_SIZE], S_buff[2];
	uint8_t j;

	while (1) {
		_DBG_("\n\r\n\rI2C Interrupt Slave is running...\n\r");
		_DBG_(menu1);
		_DBG_(menu3);

		while (1)
		{
			buffer = _DG;
			if (buffer == 'x' || buffer == 'X')
			{
				return;
			}
			else if (buffer == 'c' || buffer == 'C')
			{
				break;
			}
		}
		/* Receive -------------------------------------------------------- */
		_DBG_("\n\rPress '1' to Start");

		while (1)
		{
			buffer = _DG;
			if (buffer == '1')
				break;
		}
		_DBG_("\n\rReceiving...");

		/* Initialize buffer */
		Buffer_Init(Slave_Buf, 0);

		complete = FALSE;
		transferSCfg.rx_data = Slave_Buf;
		transferSCfg.rx_length = 7;
		HAL_I2C_Slave_Receive(I2Cx, &transferSCfg, I2C_TRANSFER_INTERRUPT);
		 while(!complete);
		__NOP();
		
		for (j=0;j<BUFFER_SIZE;j++)
		{
			_DBG("\n\rSlave_buf[");_DBD(j);_DBG("]");_DBH(Slave_Buf[j]);	//_DBG("\n\r");
		}
		
		/* Receive and transmit -------------------------------------------------------- */
		_DBG_("\n\rStart Receive, wait for repeat start and transmit...");

		/* Initialize buffer */
		Buffer_Init(Slave_Buf, 1);

		complete = FALSE;
		
		transferSCfg.rx_data = S_buff;
		transferSCfg.rx_length = 2;
		Slave_Buf[0]=0x6A; //data	
		Slave_Buf[1]=0x8A; //data		
		Slave_Buf[2]=0xA5; //data		
		transferSCfg.tx_data = Slave_Buf;
		transferSCfg.tx_length = BUFFER_SIZE;
		HAL_I2C_SlaveTransferData(I2Cx, &transferSCfg, I2C_TRANSFER_INTERRUPT);
		 while(!complete);
		__NOP();
		
		for (j=0;j<BUFFER_SIZE;j++)
		{
			_DBG("\n\rSlave_buf[");_DBD(j);_DBG("]");_DBH(Slave_Buf[j]);	//_DBG("\n\r");
		}
	}
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Test Pin setting PA0
	HAL_GPIO_ConfigOutput(PA, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PA, 0, DISPUPD);	
	HAL_GPIO_ClearPin(PA, _BIT(0));
	
	PA->OUTDR = 0x01;
	

	if(I2Cx == I2C0)
	{
		#if 0	// I2C0 PB6 : SCL0, PB7 : SDA0  x not yet used
			HAL_GPIO_ConfigOutput(PB, 6, ALTERN_FUNC);
			HAL_GPIO_ConfigFunction(PB, 6, FUNC2);
			
			HAL_GPIO_ConfigOutput(PB, 7, ALTERN_FUNC);
			HAL_GPIO_ConfigFunction(PB, 7, FUNC2);
			
	//		PCU_SetDebounce(PB, 6);
	//		PCU_SetDebounce(PB, 7);	

			
		#else	// I2C0 PB8 : SCL0, PB9 : SDA0
			HAL_GPIO_ConfigOutput(PB, 8, ALTERN_FUNC);
			HAL_GPIO_ConfigFunction(PB, 8, FUNC1);
			
			HAL_GPIO_ConfigOutput(PB, 9, ALTERN_FUNC);
			HAL_GPIO_ConfigFunction(PB, 9, FUNC1);
			
	//		PCU_SetDebounce(PB, 8);
	//		PCU_SetDebounce(PB, 9);
		#endif
	}

	else if (I2Cx == I2C1)
	{

		#if 1	// I2C1 PB10 : SCL1, PB11 : SDA1
			HAL_GPIO_ConfigOutput(PB, 10, ALTERN_FUNC);
			HAL_GPIO_ConfigFunction(PB, 10, FUNC2);
			
			HAL_GPIO_ConfigOutput(PB, 11, ALTERN_FUNC);
			HAL_GPIO_ConfigFunction(PB, 11, FUNC2);
			
	//		PCU_SetDebounce(PB, 10);
	//		PCU_SetDebounce(PB, 11);
			
		#else // I2C1 PB13 : SCL1, PB14 : SDA1
			HAL_GPIO_ConfigOutput(PB, 13, ALTERN_FUNC);
			HAL_GPIO_ConfigFunction(PB, 13, FUNC2);
			
			HAL_GPIO_ConfigOutput(PB, 14, ALTERN_FUNC);
			HAL_GPIO_ConfigFunction(PB, 14, FUNC2);
			
	//		PCU_SetDebounce(PB, 13);
	//		PCU_SetDebounce(PB, 14);		
		#endif
	}
}

/**********************************************************************
 * @brief		I2C_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void I2C_Configure(void)
{

	if ( HAL_I2C_Init(I2Cx, 100000) != HAL_OK)
	{
		Error_Handler();
	}
	//HAL_I2C_Slave_SetAddress0(I2Cx, I2CDEV_S_ADDR, ENABLE);

	__enable_irq();	
		
	
	if(I2Cx == I2C0)		{	_DBG("\n\r***I2C0_MODE***\n\r");	}
	else if(I2Cx == I2C1)	{	_DBG("\n\r***I2C1_MODE***\n\r");	}
}

/**********************************************************************
 * @brief		I2C_PollingInterrupt_loop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void I2C_PollingInterrupt_loop(void)
{
	 uint8_t buffer = 0xFF;
	
	 buffer = _DG;
		
		switch (buffer)
		{
			case '1': //Master Polling Mode
				App_I2C_Polling_Master();
				DEBUG_Print_menu();
				break;
			case '2': //Master Interrupt Mode
				isMasterMode = TRUE;
				App_I2C_Interrupt_Master();
				DEBUG_Print_menu();
				break;
//			case '3': //Slave Polling Mode
//				App_I2C_Polling_Slave();
//				DEBUG_Print_menu();
//				break;
//			case '4': //Slave Interrupt Mode
//				isMasterMode = FALSE;
//				App_I2C_Interrupt_Slave();
//				DEBUG_Print_menu();
//				break;
			default:
				break;
		}
}

/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void mainloop(void)
{
	
   /*Initialize menu prinf*/
	DEBUG_Print_menu();
	
	/*Initialize port peripheral*/
	GPIO_Configure();

	/*I2C Init*/
	I2C_Configure();

	 /*  Enable IRQ Interrupts */
	 __enable_irq(); 
	  
	 /* Infinite Loop */	
	while(1)
	{
     I2C_PollingInterrupt_loop();		
	}
}

/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 		None
 **********************************************************************/
int main(void)
{
   /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	 
	/* Initialize Debug frame work through initializing USART port  */
  DEBUG_Init();

	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
