
/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"



/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define M24C02_ADDR    (0xA0)
/** Own Slave address in Slave I2C device */
#define I2CDEV_S_ADDR	(0xA0)//(0xC0>>1)
/** Max buffer length */
#define BUFFER_SIZE			10

/* Private function prototypes -----------------------------------------------*/
void I2C0_IRQHandler_IT(void);
void I2C1_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void Buffer_Init(uint8_t* buffer, uint8_t type);
void Stop_mode(void);
void App_I2C_Interrupt_Slave (void);
void GPIO_Configure(void);
void TIMER_Configure(void);
void I2C_Configure(void);
void I2C_I2C_InterruptSlave_loop(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;
//I2C_Type* I2Cx = I2C0;
I2C_Type* I2Cx = I2C1;
Bool complete;
Bool isMasterMode = TRUE;
/** These global variables below used in interrupt mode - Slave device ----------------*/
uint8_t Master_Buf[BUFFER_SIZE];
const uint8_t menu[] =
"\n\r************************************************\n\r"
"I2C demo \n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
"\n\rPress 1 to select I2C running mode:\n\r\n\r"
"\t 1 - I2C Interrupt Slave \n\r"
"************************************************\n\r";
const uint8_t  menu1[] = "\n\r \t - Press x to exit this mode!\n\r";
const uint8_t  menu3[] = "\t - Press c to continue...\n\r";
TIMER1n_PWMCFG_Type TIMER1n_Config;

/**********************************************************************
 * @brief		I2C0_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void I2C0_IRQHandler_IT(void)
{

//	PA->OUTDR ^= (1<<0);
	if (isMasterMode)// Master Mode
	{
		HAL_I2C_Master_IRQHandler_IT(I2C0);
		if (HAL_I2C_Master_GetState(I2C0)){
			complete = TRUE;
		}
	}
	else// Slave Mode
	{
		HAL_I2C_Slave_IRQHandler_IT(I2C0);
		if (HAL_I2C_Slave_GetState(I2C0))
		{
			complete = TRUE;
		}
	}	
}

/**********************************************************************
 * @brief		I2C1_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void I2C1_IRQHandler_IT(void)
{	
	PA->OUTDR ^= (1<<0);
	if (isMasterMode)// Master Mode
	{
		HAL_I2C_Master_IRQHandler_IT(I2C1);
		if (HAL_I2C_Master_GetState(I2C1)){
			complete = TRUE;
		}
	}
	else// Slave Mode
	{
		HAL_I2C_Slave_IRQHandler_IT(I2C1);
		if (HAL_I2C_Slave_GetState(I2C1))
		{
			complete = TRUE;
		}
	}	
}

/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/*********************************************************************//**
 * @brief		Initialize buffer
 * @param[in]	buffer	buffer to initialize
 * @param[in]	type:
 * 				- 0: Initialize buffer with 0
 * 				- 1: Initialize buffer with increment value from 0
 * @return 		None
 **********************************************************************/
void Buffer_Init(uint8_t* buffer, uint8_t type)
{
	uint8_t i;

	if (type) {
		for (i = 0; i < BUFFER_SIZE; i++) {
			buffer[i] = i;
		}
	}
	else	{
		for (i = 0; i < BUFFER_SIZE; i++) {
			buffer[i] = 0;
		}
	}
}

/**********************************************************************
 * @brief		Stop_mode
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Stop_mode(void)
{
	if(I2Cx==I2C1)
	{
		HAL_SCU_WakeUpSRCCmd(WAKEUP_I2C1, ENABLE);
	}
	else if(I2Cx==I2C0)
	{
		HAL_SCU_WakeUpSRCCmd(WAKEUP_I2C0, ENABLE);
	}
	
	HAL_PWR_EnterPowerDownMode();	
	
	if(I2Cx==I2C1)
	{
		HAL_SCU_ClearWakeUpSRC(WAKEUP_I2C1);
		NVIC_ClearPendingIRQ(I2C1_IRQn);
	}
	else if(I2Cx==I2C0)
	{
		HAL_SCU_ClearWakeUpSRC(WAKEUP_I2C0);
		NVIC_ClearPendingIRQ(I2C0_IRQn);
	}
	
	SystemClock_Config();
	
	_DBG("\n\rWaked Up from Deep Sleep");
	
}

/*********************************************************************//**
 * @brief		I2C slave application using interrupt
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void App_I2C_Interrupt_Slave (void)
{
	I2C_S_SETUP_Type transferSCfg;
	uint8_t buffer = 0xFF;
	uint8_t Slave_Buf[BUFFER_SIZE], S_buff[2];
	uint8_t j;

	while (1) {
		_DBG_("\n\r\n\rI2C Interrupt Slave is running...\n\r");
		_DBG_(menu1);
		_DBG_(menu3);

		while (1)
		{
			buffer = _DG;
			if (buffer == 'x' || buffer == 'X')
			{
				return;
			}
			else if (buffer == 'c' || buffer == 'C')
			{
				break;
			}
		}
		/* Receive -------------------------------------------------------- */
		_DBG_("\n\rPress '1' to Start");

		while (1)
		{
			buffer = _DG;
			if (buffer == '1')
				break;
		}
		_DBG_("\n\rReceiving...");

		/* Initialize buffer */
		Buffer_Init(Slave_Buf, 0);

		complete = FALSE;
		transferSCfg.rx_data = Slave_Buf;
		transferSCfg.rx_length = 7;
		HAL_I2C_Slave_Receive(I2Cx, &transferSCfg, I2C_TRANSFER_INTERRUPT);
		HAL_GPIO_ClearPin(PA, _BIT(0));
//***********STOP CODE**************
		I2Cx->MR |= (1<<0);
		Stop_mode();
		I2Cx->MR &= ~(1<<0);
//************************************
		HAL_GPIO_SetPin(PA, _BIT(0));
		 while(!complete);
		__NOP();
		
		
		for (j=0;j<BUFFER_SIZE;j++)
		{
			_DBG("\n\rSlave_buf[");_DBD(j);_DBG("]");_DBH(Slave_Buf[j]);	//_DBG("\n\r");
		}
		
		/* Receive and transmit -------------------------------------------------------- */
		_DBG_("\n\rStart Receive, wait for repeat start and transmit...");

		/* Initialize buffer */
		Buffer_Init(Slave_Buf, 1);

		complete = FALSE;
		
		transferSCfg.rx_data = S_buff;
		transferSCfg.rx_length = 2;
		Slave_Buf[0]=0x6A; //data	
		Slave_Buf[1]=0x8A; //data		
		Slave_Buf[2]=0xA5; //data		
		transferSCfg.tx_data = Slave_Buf;
		transferSCfg.tx_length = BUFFER_SIZE;
		HAL_I2C_SlaveTransferData(I2Cx, &transferSCfg, I2C_TRANSFER_INTERRUPT);
		 while(!complete);
		__NOP();
		
		for (j=0;j<BUFFER_SIZE;j++)
		{
			_DBG("\n\rSlave_buf[");_DBD(j);_DBG("]");_DBH(Slave_Buf[j]);	//_DBG("\n\r");
		}
	}
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Test Pin setting PA0
		HAL_GPIO_ConfigOutput(PA, 0, PUSH_PULL_OUTPUT);
		HAL_GPIO_ConfigPullup(PA, 0, DISPUPD);	
		HAL_GPIO_ClearPin(PA, _BIT(0));
		
		HAL_GPIO_ConfigFunction(PA, 2, FUNC6);
	       HAL_GPIO_ConfigOutput(PA, 2, ALTERN_FUNC);
	if(I2Cx == I2C0)
	{
#if 0	// I2C0 PB6 : SCL0, PB7 : SDA0  x not yet used
			HAL_GPIO_ConfigOutput(PB, 6, ALTERN_FUNC);
			HAL_GPIO_ConfigFunction(PB, 6, FUNC2);
			
			HAL_GPIO_ConfigOutput(PB, 7, ALTERN_FUNC);
			HAL_GPIO_ConfigFunction(PB, 7, FUNC2);
			
	//		PCU_SetDebounce(PB, 6);
	//		PCU_SetDebounce(PB, 7);	

			
#else	// I2C0 PB8 : SCL0, PB9 : SDA0
			HAL_GPIO_ConfigOutput(PB, 8, ALTERN_FUNC);
			HAL_GPIO_ConfigFunction(PB, 8, FUNC1);
			
			HAL_GPIO_ConfigOutput(PB, 9, ALTERN_FUNC);
			HAL_GPIO_ConfigFunction(PB, 9, FUNC1);
			
	//		PCU_SetDebounce(PB, 8);
	//		PCU_SetDebounce(PB, 9);
#endif
	}

	else if (I2Cx == I2C1)
	{

#if 1 // I2C1 PB10 : SCL1, PB11 : SDA1
			HAL_GPIO_ConfigOutput(PB, 10, ALTERN_FUNC);
			HAL_GPIO_ConfigFunction(PB, 10, FUNC2);
			
			HAL_GPIO_ConfigOutput(PB, 11, ALTERN_FUNC);
			HAL_GPIO_ConfigFunction(PB, 11, FUNC2);
#else // I2C1 PB13 : SCL1, PB14 : SDA1
			HAL_GPIO_ConfigOutput(PB, 13, ALTERN_FUNC);
			HAL_GPIO_ConfigFunction(PB, 13, FUNC2);
			
			HAL_GPIO_ConfigOutput(PB, 14, ALTERN_FUNC);
			HAL_GPIO_ConfigFunction(PB, 14, FUNC2);
#endif
	}
}

/**********************************************************************
 * @brief		TIMER_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_Configure(void)
{
#if 1
	//Timer1n clock source from PCLK
		HAL_SCU_Timer1n_ClockConfig(T1NCLK_MCCR1);
		HAL_SCU_MiscClockConfig(1, TIMER1N_TYPE,LSI_MISC_CLOCK,1);
		
		TIMER1n_Config.CkSel = MCCR1PCLK;	 
		TIMER1n_Config.Prescaler = 1;	 // 48Mhz / 48 = 1Mhz ->1us
		TIMER1n_Config.TnADR = 1; // 1msec	
		TIMER1n_Config.StartLevel=START_HIGH;  //START_LOW, START_HIGH
#endif
		HAL_TIMER1n_Init(TIMER10, PERIODIC_MODE, &TIMER1n_Config);
		HAL_TIMER1n_Cmd(TIMER10, ENABLE); // timer start & clear



}

/**********************************************************************
 * @brief		I2C_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void I2C_Configure(void)
{
	if ( HAL_I2C_Init(I2Cx, 100000) != HAL_OK )
	{
		Error_Handler();
	}
	HAL_I2C_Slave_SetAddress0(I2Cx, I2CDEV_S_ADDR, DISABLE);
	if(I2Cx == I2C0)		{	_DBG("\n\r***I2C0_MODE***\n\r");	}
	else if(I2Cx == I2C1)	{	_DBG("\n\r***I2C1_MODE***\n\r");	}

}

/**********************************************************************
 * @brief		I2C_slave_wakeup_loop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void I2C_slave_wakeup_loop(void)
{
     App_I2C_Interrupt_Slave();
}

/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{

	
  /*Initialize menu prinf*/
	DEBUG_Print_menu();
	
	/*Initialize port peripheral*/
	GPIO_Configure();
	
	/*Initialize TIMER peripheral*/
	TIMER_Configure(); 
	

	/*I2C Init*/
	I2C_Configure();

	 /*  Enable IRQ Interrupts */
	 __enable_irq(); 
	  
	 /* Infinite Loop */	
	while(1)
	{	
		I2C_slave_wakeup_loop();
	}
}


/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{
     /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	 
	/* Initialize Debug frame work through initializing USART port  */
  DEBUG_Init();

	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

