/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void LVI_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void LVI_Configure(void);
void mainloop(void);
int main(void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"\n\r************************************************\n\r"
"LVI Demo\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
"************************************************\n\r";

/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/**********************************************************************
 * @brief		LVI Handler IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void LVI_IRQHandler_IT(void)
{
	if (HAL_SCULV_GetLviFlag()){
		HAL_SCULV_ClrLviFlag();
	}
		
	HAL_GPIO_SetPin(PA, _BIT(0));
	__NOP();__NOP();__NOP();__NOP();__NOP();
	HAL_GPIO_ClearPin(PA, _BIT(0));			
	_DBG("LVI occur!!\r\n");
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Test Pin setting PA0
	HAL_GPIO_ConfigOutput(PA, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PA, 0, DISPUPD);	
	HAL_GPIO_ClearPin(PA, _BIT(0));
}

/**********************************************************************
 * @brief		LVI_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void LVI_Configure(void)
{
	LVI_CFG_Type	LVI_config;
	
	LVI_config.EnSel = ENABLE;
	LVI_config.IntSel = ENABLE;
	LVI_config.LvlSel = LVI_LVL_3_18V;
	
	HAL_SCU_LVICmd(&LVI_config);
	
	NVIC_SetPriority(LVI_IRQn, 3);
	NVIC_EnableIRQ(LVI_IRQn);
}


/**********************************************************************
 * @brief		LVI Handler IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void mainloop(void)
{

	/*Initialize menu prinf*/
	DEBUG_Print_menu();
  
	/*Initialize port peripheral*/
	GPIO_Configure();
	
	/*Initialize LVI peripheral*/
	LVI_Configure(); 

	 /*  Enable IRQ Interrupts */
	 __enable_irq(); 
	 
	/* Infinite Loop */
	while(1)
	{
	}

}


/**********************************************************************
 * @brief		main
 * @param[in]	None
 * @return 		None
 **********************************************************************/
int main(void)
{
	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
  /* Initialize all port */
	Port_Init(); 

	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}
