/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro ------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -------------------------------------------------*/

void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void PWR_Configure(void);
void GPIO_Set_loop(void);


/* Private variables ---------------------------------------------------------*/
uint32_t msec;
const uint8_t menu[] =
"\n\r************************************************\n\r"
"STANDBY demo\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
"************************************************\n\r";

/**********************************************************************
 * @brief		DEBUG_Print_menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}
/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/**********************************************************************
 * @brief		PWR_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void PWR_Configure(void)
{
 _DBG("Press Enter to STANDBY Mode !");
	while(_DG-0x0D != 0x00);
	
	_DBG("\n\rSTANDBY MODE... PC13 Rising edge to wake-up");
	HAL_PWR_StandbyMode_PC13_Wakeup(ENABLE);
	HAL_GPIO_ConfigOutput(PC, 14, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigOutput(PC, 15, PUSH_PULL_OUTPUT);
	
	HAL_GPIO_SetPin(PC, _BIT(14));
	HAL_GPIO_SetPin(PC, _BIT(15));
	
	HAL_PWR_EnterStandbyMode();
}

/**********************************************************************
 * @brief		Standby_test
 * @param[in]	None
 * @return	None
 **********************************************************************/
void Standby_test(void)
{
	if(HAL_PC13_Wakeup_State())
	{
		HAL_PWR_StandbyMode_PC13_Wakeup(DISABLE);    // After PC13 Stadnby wake-up, clear this register
		HAL_GPIO_ClearPin(PC, _BIT(15));
		 _DBG("\n\rWaked Up from Standby");
		 while(1);
	}
	else
	{
		
	}
}

/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void mainloop(void)
{
	Standby_test();	
	
	/*Initialize menu prinf*/
	DEBUG_Print_menu();
	
	/*Initialize PWR peripheral*/
	PWR_Configure(); 

	 /*  Enable IRQ Interrupts */
	 __enable_irq(); 
	/* Infinite loop */
	while(1)
	{
	}

}


/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{
 	/* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

