/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro ------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -------------------------------------------------*/
void SysTick_Handler_IT(void);
void GPIOAB_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void PWR_Configure(void);
void GPIO_Set_loop(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;
const uint8_t menu[] =
"\n\r************************************************\n\r"
"Sleep demo\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
"************************************************\n\r";

/**********************************************************************
 * @brief		SysTick handler sub-routine (1ms)
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler_IT(void) 					// SysTick Interrupt Handler @ 1000Hz
{
	if(msec)msec--;
}


/**********************************************************************
 * @brief		GPIOE Handler
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void GPIOAB_IRQHandler_IT(void)
{
	uint32_t	j, status;

	status = HAL_GPIO_EXTI_GetState(PA);
	if (status & (3UL<<(0)))
	{
		HAL_GPIO_EXTI_ClearPin(PA, status&(3UL<<(0)));
		
		for (j=0; j<10000; j++);
		HAL_GPIO_SetPin(PC, _BIT(3));
		for (j=0; j<10000; j++);
		HAL_GPIO_ClearPin(PC, _BIT(3));
		
	}
}


/**********************************************************************
 * @brief		DEBUG_Print_menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}
/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}



/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// External Interrupt wake up Test pin PA0
	HAL_GPIO_ConfigOutput(PA, 0, INPUT);
	HAL_GPIO_ConfigPullup(PA, 0, ENPU);
	HAL_GPIO_SetDebouncePin(PA, 0);

	HAL_GPIO_ConfigOutput(PB, 3, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 3, DISPUPD);	
	HAL_GPIO_ClearPin(PB, _BIT(3));

	HAL_GPIO_ConfigOutput(PC, 3, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PC, 3, DISPUPD);	
	HAL_GPIO_ClearPin(PC, _BIT(3));


}

/**********************************************************************
 * @brief		PWR_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void PWR_Configure(void)
{
	HAL_SCU_MiscClockConfig(4,PD_TYPE,LSI_MISC_CLOCK,0xff);
	
	HAL_GPIO_EXTI_Config(PA, 0, IER_LEVEL, ICR_LOW_LEVEL_INT);		// Low level Interrupt
	NVIC_EnableIRQ(GPIOAB_IRQn);
	HAL_SCU_WakeUpSRCCmd(WAKEUP_GPIOA, ENABLE);
}

/**********************************************************************
 * @brief		PWR_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void PWR_Sleep_mode(void)
{
	unsigned int i=0;

	_DBG("\n\rPress Enter to Sleep Mode");
	while(_DG-0x0D != 0x00);

	_DBG("\n\rSLEEP MODE... Connect PA0 pin to GND to exit sleep mode");
	HAL_PWR_EnterSleepMode();

	for(i=0;i<1000;i++);
	HAL_SCU_HSI_ClockConfig(HSIOSC_EN);
	HAL_SCU_SystemClockChange(SC_HSIOSC);
	_DBG("\n\rWaked Up from Sleep\n\r");

	SysTick_Config(SystemCoreClock/1000);  //1msec interrupt 
	for(i=0;i<1000;i++);
}
/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void GPIO_Set_loop(void)
{
  msec = 100;
  while(msec);
  HAL_GPIO_SetPin(PB, _BIT(3));

  msec = 100;
  while(msec);
  HAL_GPIO_ClearPin(PB, _BIT(3));
}


/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void mainloop(void)
{
		
	/*Initialize menu prinf*/
	DEBUG_Print_menu();
	
	/*Initialize port peripheral*/
	GPIO_Configure();
	
	/*Initialize PWR peripheral*/
	PWR_Configure(); 

	PWR_Sleep_mode();

	 /*  Enable IRQ Interrupts */
	 __enable_irq(); 
	  
	/* Infinite loop */
	while(1)
	{
	   GPIO_Set_loop();	
	}

}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{
 	/* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}


