/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"



/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void RTC_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void mainloop(void);
int main(void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;
Bool complete;
Bool isMasterMode = TRUE;

const uint8_t menu[] =
"\n\r************************************************\n\r"
"Real Time Clock (RTC) Timestamp \n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
"************************************************\n\r";
uint32_t RTC_flag_test;
void RTC_Print(void);
RTC_CONFIGURE_Type RTC_Config;
RTC_Alarm_Type RTC_ALARM_Config;

/*********************************************************************
 * @brief		RTC_Interrupt
			- RTC_TimeStamp
			- RTC_Alaram
			- RTC_Constant_Period
 * @param[in]	None
 * @return	None
 **********************************************************************/
void RTC_IRQHandler_IT(void)
{	
	if(HAL_RTC_GetTS_IT())
	{
		HAL_RTC_ClearTS_IT();
		HAL_RTC_Time_Print();
	}
	
	if(HAL_RTC_GetMatch_IT())
	{
		HAL_RTC_Print();
	}
}


/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void GPIO_Configure(void)
{
	/********************************
*	RTC function
*	PC13	FUNC7 : RTC_TAMP1
*	PA0		FUNC7 : RTC_TAMP2
*	PC13	RTCPFCR[3:2]= 3 : RTC_OUT
********************************/
	// RTC_TAMP2
	HAL_GPIO_ConfigFunction(PA, 0, FUNC7);
	HAL_GPIO_ConfigOutput(PA, 0, ALTERN_FUNC);
}

/**********************************************************************
 * @brief		RTC_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void RTC_Configure(void)
{
	HAL_SCU_MiscClockConfig(5, RTC_TYPE, LSI_MISC_CLOCK, 12);	//	RTC clock select
	HAL_SCU_RTC_ClockConfig(RTCCLK_MCCR5); 				//	RTC PPCLK(MCLK or LSI40kHz)
	
	RTC_Config.RTC_SUBDR=0xF424;
	HAL_RTC_SetAmpm(AMPM_12HOUR);		// AMPM SETUP
	RTC_Config.RTC_Sec=0x40;
	RTC_Config.RTC_Min=0x01;
	RTC_Config.RTC_Hour=AMPM_12_hour_09;
	RTC_Config.RTC_Day=0x01;
	RTC_Config.RTC_Week=0x00;
	RTC_Config.RTC_Month=0x02;
	RTC_Config.RTC_Year=0x10;
	
	HAL_RTC_ConfigInterrupt(&RTC_Config);
		
	// RTC Period Select
	HAL_RTC_SetConstantPeriod(Period_1s);
	
	// RTC TimerStamp Enable or Disable
	HAL_RTC_SetTimestamp(ENABLE,TIMESTAMP_FALLING_EDGE);

	HAL_RTC_ClearMatch_IT();		// RTC Match flag clear
	NVIC_EnableIRQ(RTC_IRQn);
	HAL_RTC_Start(ENABLE);		// RTCE ENABLE (start)
}

/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{

	/*Initialize menu prinf*/
	DEBUG_Print_menu();
  
	/*Initialize port peripheral*/
	GPIO_Configure();
	
	/*Initialize RTC peripheral*/
	RTC_Configure(); 

	 /*Enable IRQ Interrupts */
	 __enable_irq(); 
	 
	/* Infinite Loop */
	while(1)
	{
	}
	
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{
	
	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
  /* Initialize all port */
	Port_Init(); 

	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}


