/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"



/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define DMAC_SPI
//#define TEST_SINGLE_POLLING // spi master polling : single data 
#define TEST_SINGLE_INTERRUPT // spi master interrupt : single data 
//#define TEST_MULTI_INTERRUPT // spi master interrupt : multi data 10 bytes

//-------------------------
// DMAC Selection
//-------------------------
//#define	DMACx	DMAC0
//#define	DMACx	DMAC1
//#define	DMACx	DMAC2
//#define	DMACx	DMAC3

//-------------------------
// SPI Selection
//-------------------------
//#define SPI_n		SPI20
//#define SPI_n		SPI21



/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void SPI20_IRQHandler_IT(void);
void SPI21_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void SPI_RegisterInitCHK(SPI_Type *SPIx);
void UART_DebugMessage(void);
void GPIO_Configure( void);
void SPI_Configure(void);
void SPI_GetData_loop(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
const uint8_t cmdm[] =
"A31G324> ";
const uint8_t menu[] =
"************************************************\n\r"
" A31G324_SPI_TEST_EXAMPLE\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
"\t SPI Master Demo...  \n\r"
"\t 1. SPI Init \n\r"
"\t 2. SPI Test \n\r"
"\t 3. Test screen \n\r"
"************************************************\n\r";
uint8_t data[10] = {0xA5, 0xA6, 0xA7, 0xA8, 0xA9, 0xAA, 0xAB, 0xAC, 0xAD, 0xAE};

uint32_t fflag;
volatile uint32_t rxdata, txdata;


/**********************************************************************
 * @brief		SPI20_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SPI20_IRQHandler_IT(void)
{
	uint32_t status;
	
	status = HAL_SPI_GetStatus(SPI20);
	if ( (status & SPI_STAT_TXBUF_EMPTY) && (HAL_SPI_GetControl(SPI20) & SPI_INTCFG_TXIE)){ // tx buffer empty
		HAL_SPI_ConfigInterrupt(SPI20, SPI_INTCFG_TXIE, DISABLE);
		
		HAL_SPI_Enable(SPI20, DISABLE);  // SPI disable 	
		HAL_SPI_TransmitData(SPI20, txdata);	
		HAL_SPI_Enable(SPI20, ENABLE);  // SPI enable	
		
	}
}


/**********************************************************************
 * @brief		SPI21_IRQHandler
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SPI21_IRQHandler_IT(void)
{
	uint32_t status;
	
	status = HAL_SPI_GetStatus(SPI21);
	
	if ( (status & SPI_STAT_RXBUF_READY) && (HAL_SPI_GetControl(SPI21) & SPI_INTCFG_RXIE)){ // rx buffer full		
		rxdata = HAL_SPI_ReceiveData(SPI21);
		_DBG(" rx = "); _DBH(rxdata);	_DBG("\n\r");
		fflag=1;
	}
				
}


/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}




/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void GPIO_Configure(void)
{

	#if 1
		HAL_GPIO_ConfigOutput(PA, 4, ALTERN_FUNC);				//SPI20_SS
		HAL_GPIO_ConfigFunction(PA, 4, FUNC0);
		HAL_GPIO_ConfigPullup(PA, 4, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PA, 5, ALTERN_FUNC);				//SPI20_SCK
		HAL_GPIO_ConfigFunction(PA, 5, FUNC0);
		HAL_GPIO_ConfigPullup(PA, 5, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PA, 7, ALTERN_FUNC);				//SPI20_MOSI
		HAL_GPIO_ConfigFunction(PA, 7, FUNC0);
		HAL_GPIO_ConfigPullup(PA, 7, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PA, 6, ALTERN_FUNC);				//SPI20_MISO
		HAL_GPIO_ConfigFunction(PA, 6, FUNC0);
		HAL_GPIO_ConfigPullup(PA, 6, ENPU);
				
#else			
		HAL_GPIO_ConfigOutput(PA, 15, ALTERN_FUNC);				//SPI20_SS
		HAL_GPIO_ConfigFunction(PA, 15, FUNC3);
		HAL_GPIO_ConfigPullup(PA, 15, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PB, 3, ALTERN_FUNC);				//SPI200_SCK
		HAL_GPIO_ConfigFunction(PB, 3, FUNC3);
		HAL_GPIO_ConfigPullup(PB, 3, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PB, 5, ALTERN_FUNC);				//SPI20_MOSI
		HAL_GPIO_ConfigFunction(PB, 5, FUNC3);
		HAL_GPIO_ConfigPullup(PB, 5, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PB, 4, ALTERN_FUNC);				//SPI20_MISO
		HAL_GPIO_ConfigFunction(PB, 4, FUNC3);
		HAL_GPIO_ConfigPullup(PB, 4, ENPU);
#endif
				
#if 0
		HAL_GPIO_ConfigOutput(PB, 12, ALTERN_FUNC);				//SPI21_SS
		HAL_GPIO_ConfigFunction(PB, 12, FUNC0);
		HAL_GPIO_ConfigPullup(PB, 12, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PB, 13, ALTERN_FUNC);				//SPI21_SCK
		HAL_GPIO_ConfigFunction(PB, 13, FUNC0);
		HAL_GPIO_ConfigPullup(PB, 13, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PB, 15, ALTERN_FUNC);				//SPI21_MOSI
		HAL_GPIO_ConfigFunction(PB, 15, FUNC0);
		HAL_GPIO_ConfigPullup(PB, 15, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PB, 14, ALTERN_FUNC);				//SPI21_MISO
		HAL_GPIO_ConfigFunction(PB, 14, FUNC0);
		HAL_GPIO_ConfigPullup(PB, 14, ENPU);
		
#else
		HAL_GPIO_ConfigOutput(PB, 9, ALTERN_FUNC);				//SPI21_SS
		HAL_GPIO_ConfigFunction(PB, 9, FUNC3);
		HAL_GPIO_ConfigPullup(PB, 9, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PB, 10, ALTERN_FUNC);				//SPI21_SCK
		HAL_GPIO_ConfigFunction(PB, 10, FUNC3);
		HAL_GPIO_ConfigPullup(PB, 10, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PC, 3, ALTERN_FUNC);				//SPI21_MOSI
		HAL_GPIO_ConfigFunction(PC, 3, FUNC0);
		HAL_GPIO_ConfigPullup(PC, 3, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PC, 2, ALTERN_FUNC);				//SPI21_MISO
		HAL_GPIO_ConfigFunction(PC, 2, FUNC0);
		HAL_GPIO_ConfigPullup(PC, 2, ENPU);
#endif
				
				
}

/**********************************************************************
 * @brief		SPI_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void SPI_Configure(void)
{
		SPI_CFG_Type SP_config;
		IRQn_Type	SPIx_IRQn;
	
		SPIx_IRQn = SPI20_IRQn;
				
				/* SPI enable */
		SP_config.Databit = SPI_DS_8BITS;
		SP_config.CPHA = SPI_CPHA_LO;
		SP_config.CPOL = SPI_CPOL_LO;
		SP_config.DataDir = SPI_MSB_FIRST;
		SP_config.Mode = SPI_MASTER_MODE;
		SP_config.BaudRate = 79; // PCLK / (79+1) 
		if ( HAL_SPI_Init(SPI20, &SP_config) != HAL_OK)
		{
			Error_Handler();
		}
		//SPI_IntConfig(SPI20, SPI_INTCFG_RXIE, ENABLE); // rx interrupt enable 
		HAL_SPI_Enable(SPI20, ENABLE);  // SPI enable 

		/* preemption = 1, sub-priority = 1 */
		NVIC_SetPriority(SPIx_IRQn, ((0x01<<1)|0x01));
		NVIC_EnableIRQ(SPIx_IRQn);
	
	
	  SPIx_IRQn = SPI21_IRQn;
				
		/* SPI enable */
		SP_config.Databit = SPI_DS_8BITS;
		SP_config.CPHA = SPI_CPHA_LO;
		SP_config.CPOL = SPI_CPOL_LO;
		SP_config.DataDir = SPI_MSB_FIRST;
		SP_config.Mode = SPI_SLAVE_MODE;
		SP_config.BaudRate = 79; // PCLK / (79+1) 
	
		if ( HAL_SPI_Init(SPI21, &SP_config) != HAL_OK)
		{
			Error_Handler();
		}

		HAL_SPI_ConfigInterrupt(SPI21, SPI_INTCFG_RXIE, ENABLE); // rx interrupt enable 
		HAL_SPI_Enable(SPI21, ENABLE);  // SPI enable 

		/* preemption = 1, sub-priority = 2 */
		NVIC_SetPriority(SPIx_IRQn, ((0x01<<1)|0x02));
		NVIC_EnableIRQ(SPIx_IRQn);
}

/**********************************************************************
 * @brief		SPI_TxRx_Cmd
 * @param[in]	None
 * @return	None
 **********************************************************************/
void SPI_GetData_loop(void)
{
	uint32_t i, j;
	
	i=0x30;
	while(1){
		txdata = i;	
		_DBG(" tx = "); _DBH(txdata);
		rxdata = 0;
		fflag=0;
		HAL_SPI_ConfigInterrupt(SPI20, SPI_INTCFG_TXIE, ENABLE);
		
		while(fflag == 0);					// check rx buffer full 
								
		i++;
		i&=(0xff);
		
		for(j=0;j<10000;j++);
		
		if (i==0x39)
		{
			break;
		}
		
	}
}



/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{

	uint8_t		cmdn_cnt=0, ch_rtn, mod_check[4] = {0x30, 0x31, 0x32, 0x33};
	
	
	/*Initialize menu prinf*/
	DEBUG_Print_menu();
	
	 /*Enable IRQ Interrupts */
	 __enable_irq(); 
	 
	while(1)
	{
		if (cmdn_cnt == 0)
		{
			_DBG(cmdm);
			cmdn_cnt=1;
		}
		
	      ch_rtn = _DG;
		//ch_rtn = UARTGetChar(UART0);
		_DBC(ch_rtn);
		_DBG("\r\n");
	
		if (ch_rtn == mod_check[1])
		{
			/*PCU Configure*/
			GPIO_Configure();
			/*Initialize SPI peripheral*/
			SPI_Configure();
		  cmdn_cnt=0;
			
		}
		else if (ch_rtn == mod_check[2])
		{
			SPI_GetData_loop();
			cmdn_cnt=0;
		}
		else 
		{
				DEBUG_Print_menu();
		}
	}
}


/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{
	/* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	


	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
