/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"



/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/

//SPI with DMA test -> DMAC_SPI
//SPI polling test -> DMAC_SPIx
#define DMAC_SPI

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure( void);
void SPI_Configure(void);
void DMAC_Configure(void);
void SPI_GetData_loop(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
const uint8_t cmdm[] =
"A31G324> ";
const uint8_t menu[] =
"************************************************\n\r"
" A31G324_SPI_TEST_EXAMPLE\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
"\t SPI Poling/DMA Demo...  \n\r"
"\t 1. SPI Init \n\r"
"\t 2. SPI Test \n\r"
"\t 3. Test screen \n\r"
"************************************************\n\r";
uint8_t data[10] = {0xA5, 0xA6, 0xA7, 0xA8, 0xA9, 0xAA, 0xAB, 0xAC, 0xAD, 0xAE};
uint8_t dma_data[10] = {0xB5, 0xB6, 0xB7, 0xB8, 0xB9, 0xBA, 0xBB, 0xBC, 0xBD, 0xBE};

SPI_CFG_Type SP_config;
DMAC_Ch_CFG_Type	DMAC_Config;

/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void GPIO_Configure(void)
{
#if 1
		HAL_GPIO_ConfigOutput(PA, 4, ALTERN_FUNC);				//SPI20_SS
		HAL_GPIO_ConfigFunction(PA, 4, FUNC0);
		HAL_GPIO_ConfigPullup(PA, 4, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PA, 5, ALTERN_FUNC);				//SPI20_SCK
		HAL_GPIO_ConfigFunction(PA, 5, FUNC0);
		HAL_GPIO_ConfigPullup(PA, 5, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PA, 7, ALTERN_FUNC);				//SPI20_MOSI
		HAL_GPIO_ConfigFunction(PA, 7, FUNC0);
		HAL_GPIO_ConfigPullup(PA, 7, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PA, 6, ALTERN_FUNC);				//SPI20_MISO
		HAL_GPIO_ConfigFunction(PA, 6, FUNC0);
		HAL_GPIO_ConfigPullup(PA, 6, ENPU);
		
#else			
		HAL_GPIO_ConfigOutput(PA, 15, ALTERN_FUNC);				//SPI20_SS
		HAL_GPIO_ConfigFunction(PA, 15, FUNC3);
		HAL_GPIO_ConfigPullup(PA, 15, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PB, 3, ALTERN_FUNC);				//SPI200_SCK
		HAL_GPIO_ConfigFunction(PB, 3, FUNC3);
		HAL_GPIO_ConfigPullup(PB, 3, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PB, 5, ALTERN_FUNC);				//SPI20_MOSI
		HAL_GPIO_ConfigFunction(PB, 5, FUNC3);
		HAL_GPIO_ConfigPullup(PB, 5, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PB, 4, ALTERN_FUNC);				//SPI20_MISO
		HAL_GPIO_ConfigFunction(PB, 4, FUNC3);
		HAL_GPIO_ConfigPullup(PB, 4, ENPU);
#endif



#if 0
		HAL_GPIO_ConfigOutput(PB, 12, ALTERN_FUNC);				//SPI21_SS
		HAL_GPIO_ConfigFunction(PB, 12, FUNC0);
		HAL_GPIO_ConfigPullup(PB, 12, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PB, 13, ALTERN_FUNC);				//SPI21_SCK
		HAL_GPIO_ConfigFunction(PB, 13, FUNC0);
		HAL_GPIO_ConfigPullup(PB, 13, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PB, 15, ALTERN_FUNC);				//SPI21_MOSI
		HAL_GPIO_ConfigFunction(PB, 15, FUNC0);
		HAL_GPIO_ConfigPullup(PB, 15, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PB, 14, ALTERN_FUNC);				//SPI21_MISO
		HAL_GPIO_ConfigFunction(PB, 14, FUNC0);
		HAL_GPIO_ConfigPullup(PB, 14, ENPU);
		
#else
		HAL_GPIO_ConfigOutput(PB, 9, ALTERN_FUNC);				//SPI21_SS
		HAL_GPIO_ConfigFunction(PB, 9, FUNC3);
		HAL_GPIO_ConfigPullup(PB, 9, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PB, 10, ALTERN_FUNC);				//SPI21_SCK
		HAL_GPIO_ConfigFunction(PB, 10, FUNC3);
		HAL_GPIO_ConfigPullup(PB, 10, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PC, 3, ALTERN_FUNC);				//SPI21_MOSI
		HAL_GPIO_ConfigFunction(PC, 3, FUNC0);
		HAL_GPIO_ConfigPullup(PC, 3, DISPUPD);
		
		HAL_GPIO_ConfigOutput(PC, 2, ALTERN_FUNC);				//SPI21_MISO
		HAL_GPIO_ConfigFunction(PC, 2, FUNC0);
		HAL_GPIO_ConfigPullup(PC, 2, ENPU);
#endif
}

/**********************************************************************
 * @brief		SPI_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void SPI_Configure(void)
{
	SP_config.Databit = SPI_DS_8BITS;//SPI_DS_17BITS;
	SP_config.CPHA = SPI_CPHA_LO;//SPI_CPHA_HI;
	SP_config.CPOL = SPI_CPOL_LO;//SPI_CPOL_HI;
	SP_config.DataDir = SPI_MSB_FIRST;//SPI_LSB_FIRST;
	SP_config.Mode = SPI_MASTER_MODE;
	SP_config.BaudRate = 79; // PCLK / (79+1) 
	
	if (HAL_SPI_Init(SPI20, &SP_config) != HAL_OK)
	{
		Error_Handler();
	}
	
	HAL_SPI_ConfigDelay(SPI20, 0x02, 0x02, 0x02);
	
	HAL_SPI_Enable(SPI20, ENABLE);  // SPI enable 
	
	
	
	SP_config.Databit = SPI_DS_8BITS;//SPI_DS_17BITS;
	SP_config.CPHA = SPI_CPHA_LO;//SPI_CPHA_HI;
	SP_config.CPOL = SPI_CPOL_LO;//SPI_CPOL_HI;
	SP_config.DataDir = SPI_MSB_FIRST;//SPI_LSB_FIRST;
	SP_config.Mode = SPI_SLAVE_MODE;
	SP_config.BaudRate = 79; // PCLK / (79+1) 
	
	if (HAL_SPI_Init(SPI21, &SP_config) != HAL_OK)
	{
		Error_Handler();
	}
		
	HAL_SPI_ConfigDelay(SPI21, 0x02, 0x02, 0x02);
	
	HAL_SPI_Enable(SPI21, ENABLE);  // SPI enable 
}


/**********************************************************************
 * @brief		DMAC_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void DMAC_Configure(void)
{
#ifdef DMAC_SPI
			
	if(HAL_DMAC_Init() != HAL_OK)
	{
		Error_Handler();
  }
	
	DMAC_Config.TransferCnt = 10;
	DMAC_Config.PeriSel = DMAC_PERISEL_SPI20TX;
	DMAC_Config.TransferWidth = DMAC_WIDTH_BYTE;//DMAC_WIDTH_WORD;
	DMAC_Config.Dir = DMAC_DIR_TX;
	DMAC_Config.PeriAddr = SPI20_BASE;
	DMAC_Config.MemAddr = (uint32_t)(&dma_data[0]);//0x20002000;
					
	HAL_DMAC_Setup(DMAC0, &DMAC_Config);
	
	DMAC_Config.TransferCnt = 10;
	DMAC_Config.PeriSel = DMAC_PERISEL_SPI21RX;
	DMAC_Config.TransferWidth =DMAC_WIDTH_BYTE;//DMAC_WIDTH_WORD;
	DMAC_Config.Dir = DMAC_DIR_RX;
	DMAC_Config.PeriAddr = SPI21_BASE;
	DMAC_Config.MemAddr = 0x20003000;

	HAL_DMAC_Setup(DMAC1, &DMAC_Config);

#endif	
}
/**********************************************************************
 * @brief		SPI_GetData_loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void SPI_GetData_loop(void)
{
	uint32_t rxdata;
	uint32_t i, j;
	
_DBG_(" ");
	i=0;
	while(1){
	
#ifdef DMAC_SPI
	HAL_DMAC_ChCmd(DMAC0, ENABLE);
	HAL_DMAC_ChCmd(DMAC1, ENABLE);	
	for(j=0;j<10000;j++);
		
	while (HAL_DMAC_GetState(DMAC0) == RESET) ;		// Transfer...
	while ((SPI20->SR & (SPI20_SR_TXDMAF_Msk)) == RESET);			//DMA Flag
	
	for(i = 0; i < 10; i++)
	{
		_DBG(" tx = "); _DBH(dma_data[i]);//_DBG("\n\r");
	}
	_DBG("\n\r");
	
	while (HAL_DMAC_GetState(DMAC1) == RESET) ;		// Transfer...
	while ((SPI21->SR & (SPI21_SR_RXDMAF_Msk)) == RESET);			//DMA Flag
  
	// check rx dma buffer full	
	for(i = 0; i <10; i++)
	{
			rxdata = (*(volatile unsigned char *)(0x20003000+i));
			_DBG(" rx = "); _DBH(rxdata);	
			//_DBG("\n\r");
	}
	
	HAL_DMAC_ChCmd(DMAC0, DISABLE);		
	HAL_DMAC_ChCmd(DMAC1, DISABLE);		
  
	break;
#else
	
		while((HAL_SPI_GetStatus(SPI20) & SPI_STAT_TXBUF_EMPTY) != SPI_STAT_TXBUF_EMPTY); // check tx buffer empty 
		HAL_SPI_Enable(SPI20, DISABLE);
		HAL_SPI_TransmitData(SPI20, data[i]);
		HAL_SPI_TransmitData(SPI20, i);
		HAL_SPI_Enable(SPI20, ENABLE);

		_DBG(" tx = "); _DBH(data[i]);

		while((HAL_SPI_GetStatus(SPI21) & SPI_STAT_RXBUF_READY) != SPI_STAT_RXBUF_READY)
		{					
			j++;
			
			if(j > 1000000)
			{
				_DBG(" SPI communication error RXBUF_READY!! ");
				j = 0;	
				break;
			}
		}
		
		rxdata = HAL_SPI_ReceiveData(SPI21);
		
		_DBG(" rx = "); _DBH(rxdata);	
		_DBG("\n\r");
		
		i++; i&=(0x1ffff);
		
		for(j=0;j<10000;j++);
		
		if(i == 10)
		{
			break;
		}
			
#endif
	
	}
}
/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{	

	uint8_t		cmdn_cnt=0, ch_rtn, mod_check[4] = {0x30, 0x31, 0x32, 0x33};
	
	DEBUG_Print_menu();
	
	while(1)
	{
		if (cmdn_cnt == 0)
		{
			_DBG(cmdm);
			cmdn_cnt=1;
		}
		ch_rtn = _DG;
		_DBC(ch_rtn);
		_DBG("\r\n");
		
		if (ch_rtn == mod_check[1])
		{
			 GPIO_Configure();	
       SPI_Configure();
			 DMAC_Configure();
			 cmdn_cnt=0;
		}
		else if (ch_rtn == mod_check[2])
		{
			SPI_GetData_loop();
			cmdn_cnt=0;
		}
		else
		{
			DEBUG_Print_menu();
		}
	}
}

/**********************************************************************
 * @brief		Main 
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{
	/* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
