/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define Do	1915
#define Re	1700
#define Me	1519
#define Pa	1432
#define So	1275
#define Ra	1136
#define Si	1014
#define Do2	956
#define Re2	852

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void SysTick_Handler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void SysTick_Configure(void);
void GPIOAB_IRQHandler_IT(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
TIMER1n_PWMCFG_Type TIMER1n_Config;
uint32_t msec;
typedef enum {
	Melody1=1,
	Melody2=2,
	Buzzer_stop=3
	
} MODE;
uint8_t Mode_check = Buzzer_stop;
unsigned int Melody1_tones[] = 
{
	Do, Do, So, So, Ra, Ra, So,
	Pa, Pa, Me, Me, Re, Re, Do,
	So, So, Pa, Pa, Me, Me, Re,
	So, So, Pa, Pa, Me, Me, Re,
	Do, Do, So, So, Ra, Ra, So,
	Pa, Pa, Me, Me, Re, Re, Do
};
unsigned int Melody1_bit[] = { 1, 1, 1, 1, 1, 1, 2}; 

unsigned int Melody2_tones[] = 
{
	So, Ra, So, Me, Do2, Ra, So, So, Ra, So, Ra, So, Do2, Si,
	Pa, So, Pa, Re, Si, Ra, So, So, Ra, So, Ra, So, Ra, Me,
	So, Ra, So, Me, Do2, Ra, So, So, Ra, So, Ra, So, Do2, Si,
	Pa, So, Pa, Re, Si, Ra, So, So, Ra, So, Ra, So, Re2, Do2,
	Ra, Ra, Do2, Ra, So, Me, So, Pa, Ra, So, Pa, Me,
	Re, Me, So, Ra, Si, Si, Si, Do2, Do2, Si, Ra, So, Pa, Re,
	So, Ra, So, Me, Do2, Ra, So, So, Ra, So, Ra, So, Do2, Si,
	Pa, So, Pa, Re, Si, Ra, So, So, Ra, So, Ra, So, Re2, Do2
};
unsigned int Melody2_bit[] =
{ 
	1, 2, 1, 2, 2, 2, 4, 1, 1, 1, 1, 2, 2, 4,
	1, 2, 1, 2, 2, 2, 4, 1, 1, 1, 1, 2, 2, 4,
	1, 2, 1, 2, 2, 2, 4, 1, 1, 1, 1, 2, 2, 4,
	1, 2, 1, 2, 2, 2, 4, 1, 1, 1, 1, 2, 2, 4,
	2, 2, 2, 2, 2, 1, 4, 2, 2, 2, 2, 6,
	2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 4,
	1, 2, 1, 2, 2, 2, 4, 1, 1, 1, 1, 2, 2, 4,
	1, 2, 1, 2, 2, 2, 4, 1, 1, 1, 1, 2, 2, 4
}; 
	
const uint8_t menu[] =
"************************************************\n\r"
" GPIO demo \n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" Using Starter-Kit Buzzer \n\r"
"************************************************\n\r";

/**********************************************************************
 * @brief		SysTick handler sub-routine (1ms)
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler_IT(void) 					// SysTick Interrupt Handler @ 1000Hz
{
	if(msec)msec--;
}

/**********************************************************************
 * @brief		GPIOAB_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void GPIOAB_IRQHandler_IT(void)
{
	uint32_t status;
	
	// GPIOAB interrupt 
	{	
		status = HAL_GPIO_EXTI_GetState(PA);
		
		// Melody 1
		if(status & (3UL<<(0<<1)))
		{
			HAL_GPIO_EXTI_ClearPin(PA, status & (3UL<<(0<<1)));
			Mode_check=Melody1;
		}
		// Melody 2
		else if(status & (3UL<<(5<<1)))
		{
			HAL_GPIO_EXTI_ClearPin(PA, status & (3UL<<(5<<1)));
			Mode_check=Melody2;
		}
		
		status = HAL_GPIO_EXTI_GetState(PB);
		// Buzzer OFF
		if(status & (3UL<<(9<<1)))
		{
			HAL_GPIO_EXTI_ClearPin(PB, status & (3UL<<(9<<1)));
			Mode_check=Buzzer_stop;
		}
	}
}

/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// T12O Pin is Buzzer
	HAL_GPIO_ConfigFunction(PA, 7, FUNC6);		
	HAL_GPIO_ConfigOutput(PA, 7, ALTERN_FUNC);	
	
	// Switch PA0, PA5, PB9
	HAL_GPIO_ConfigOutput(PA, 0, INPUT);
	HAL_GPIO_ConfigPullup(PA, 0, DISPUPD);
	HAL_GPIO_SetDebouncePin(PA, 0);
	
	HAL_GPIO_ConfigOutput(PA, 5, INPUT);
	HAL_GPIO_ConfigPullup(PA, 5, DISPUPD);
	HAL_GPIO_SetDebouncePin(PA, 5);
	
	HAL_GPIO_ConfigOutput(PB, 9, INPUT);
	HAL_GPIO_ConfigPullup(PB, 9, DISPUPD);
	HAL_GPIO_SetDebouncePin(PB, 9);
	
	HAL_SCU_MiscClockConfig(4,PD_TYPE,CLKSRC_LSI,101); //  PORT A,B,C Debounce set. ( Use Switch )
		
	NVIC_SetPriority(GPIOAB_IRQn, 3);	
	NVIC_EnableIRQ(GPIOAB_IRQn);	
	
	HAL_GPIO_EXTI_Config(PA, 0, IER_EDGE, ICR_RISING_EDGE_INT);
	HAL_GPIO_EXTI_Config(PA, 5, IER_EDGE, ICR_RISING_EDGE_INT);
	HAL_GPIO_EXTI_Config(PB, 9, IER_EDGE, ICR_RISING_EDGE_INT);

}

/**********************************************************************
 * @brief		TIMER_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_Configure(void)
{
	//Timer1n clock source from PCLK
	HAL_SCU_Timer1n_ClockConfig(T1NCLK_PCLK);
	TIMER1n_Config.CkSel = MCCR1PCLK;    
	TIMER1n_Config.Prescaler = 48;    // 48Mhz / 48 = 1Mhz ->1us
	TIMER1n_Config.StartLevel=START_HIGH;  
	if (HAL_TIMER1n_Init(TIMER12, PERIODIC_MODE, &TIMER1n_Config) != HAL_OK)
	{
		Error_Handler();
	}
	HAL_TIMER1n_ConfigInterrupt(TIMER12, TIMER1n_INTCFG_MIE, ENABLE);
	
	HAL_TIMER1n_Cmd(TIMER12, ENABLE); // timer start & clear
}

/**********************************************************************
 * @brief		SysTick_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SysTick_Configure(void)
{
	SysTick_Config(SystemCoreClock/1000);  //1msec interrupt 	
}

/**********************************************************************
 * @brief		GPIO_LED_Toggle
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void StarterKit_Buzzer(void)
{
	 uint32_t count;
	
	msec=1000;
	while(msec);
	
	Mode_check = Buzzer_stop;
	
	while(1)
	{
		if(Mode_check==Melody1)
		{
			for(count=0;count<sizeof(Melody1_tones)/4;count++)
			{
				TIMER12->ADR = Melody1_tones[count];
				msec = 500*Melody1_bit[(count%7)];
				while(msec);
				
				TIMER12->ADR =Melody1_bit[(count%7)];
				msec = 100;
				while(msec);
				if(Mode_check!=Melody1) break;
			}
		}
		else if(Mode_check==Melody2)
		{
			for(count=0;count<sizeof(Melody2_tones)/4;count++)
			{
				TIMER12->ADR = Melody2_tones[count];
				msec = 200*Melody2_bit[(count)];
				while(msec);
				
				TIMER12->ADR =Melody2_bit[(count)];
				msec =10;
				while(msec);
				if(Mode_check!=Melody2) break;
			}
		}
		else
		{
			TIMER12->ADR=0;
		}

		msec=1000;
		while(msec);
	}
}


/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void mainloop(void)
{

	/*Initialize menu prinf*/	
	DEBUG_Print_menu();	
	
	/*Initialize port peripheral*/
	GPIO_Configure();

	/*TIMER10 Init*/
	TIMER_Configure();
	
	/*Initialize SysTick peripheral*/
	SysTick_Configure(); 	

	 /*  Enable IRQ Interrupts */
	 __enable_irq(); 
	 
	/* Infinite Loop */
	while(1)
	{
		StarterKit_Buzzer();
	}
}


/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{	
	
	/* Configure the system clock to HSE  48MHz */
	SystemClock_Config();

	 /* Initialize all port */
	Port_Init(); 
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop(); 
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif


