/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define Bank1_LCD    (0x60000000)   //Disp Reg ADDR

#define LCD_WR_REG(index)    ((*(__IO uint8_t *) (Bank1_LCD)) = ((uint8_t)index))
#define LCD_WR_Data(val)     ((*(__IO uint8_t *) (Bank1_LCD)) = ((uint8_t)(val)))

#define LCD_ST7789V_CMD(index)         LCD_WR_REG(index)
#define LCD_ST7789V_Parameter(val)    LCD_WR_Data(val)

#define size	230400  //240*320*3byte(24bit / 8,8,8 RGB)
//LCD IO Control
#define CLR_RS     PA->OUTDR &= ~((0x1<<15))
#define SET_RS     PA->OUTDR |= (0x1<<15)
#define SET_CS     (PB->OUTDR &= ~((0x1)<<7))
#define CLR_CS     (PB->OUTDR |=  (0x1)<<7)
#define CLR_RST   (PB->OUTDR &= ~((0x1)<<9))
#define SET_RST   (PB->OUTDR |=  (0x1)<<9)

// Defines for the ST7789 registers.
// ref: https://www.crystalfontz.com/controllers/Sitronix/ST7789V/
#define ST7789_00_NOP       (0x00)
#define ST7789_01_SWRESET   (0x01)
#define ST7789_04_RDDID     (0x04)
#define ST7789_09_RDDST     (0x09)
#define ST7789_0A_RDDPM     (0x0A)
#define ST7789_0B_RDDMADCTL (0x0B)
#define ST7789_0C_RDDCOLMOD (0x0C)
#define ST7789_0D_RDDIM     (0x0D)
#define ST7789_0E_RDDSM     (0x0E)
#define ST7789_0F_RDDSDR    (0x0F)
#define ST7789_10_SLPIN     (0x10)
#define ST7789_11_SLPOUT    (0x11)
#define ST7789_12_PTLON     (0x12)
#define ST7789_13_NORON     (0x13)
#define ST7789_20_INVOFF    (0x20)
#define ST7789_21_INVON     (0x21)
#define ST7789_26_GAMSET    (0x26)
#define ST7789_28_DISPOFF   (0x28)
#define ST7789_29_DISPON    (0x29)
#define ST7789_2A_CASET     (0x2A)
#define ST7789_2B_RASET     (0x2B)
#define ST7789_2C_RAMWR     (0x2C)
#define ST7789_2E_RAMRD     (0x2E)
#define ST7789_30_PTLAR     (0x30)
#define ST7789_33_VSCRDEF   (0x33)
#define ST7789_34_TEOFF     (0x34)
#define ST7789_35_TEON      (0x35)
#define ST7789_36_MADCTL    (0x36)
#define ST7789_37_VSCRSADD  (0x37)
#define ST7789_38_IDMOFF    (0x38)
#define ST7789_39_IDMON     (0x39)
#define ST7789_3A_COLMOD    (0x3A)
#define ST7789_3C_RAMWRC    (0x3C)
#define ST7789_3E_RAMRDC    (0x3E)
#define ST7789_44_TESCAN    (0x44)
#define ST7789_45_RDTESCAN  (0x45)
#define ST7789_51_WRDISBV   (0x51)
#define ST7789_52_RDDISBV   (0x52)
#define ST7789_53_WRCTRLD   (0x53)
#define ST7789_54_RDCTRLD   (0x54)
#define ST7789_55_WRCACE    (0x55)
#define ST7789_56_RDCABC    (0x56)
#define ST7789_5E_WRCABCMB  (0x5E)
#define ST7789_5F_RDCABCMB  (0x5F)
#define ST7789_68_RDABCSDR  (0x68)
#define ST7789_B0_RAMCTRL   (0xB0)
#define ST7789_B1_RGBCTRL   (0xB1)
#define ST7789_B2_PORCTRL   (0xB2)
#define ST7789_B3_FRCTRL1   (0xB3)
#define ST7789_B7_GCTRL     (0xB7)
#define ST7789_BA_DGMEN     (0xBA)
#define ST7789_BB_VCOMS     (0xBB)
#define ST7789_C0_LCMCTRL   (0xC0)
#define ST7789_C1_IDSET     (0xC1)
#define ST7789_C2_VDVVRHEN  (0xC2)
#define ST7789_C3_VRHS      (0xC3)
#define ST7789_C4_VDVSET    (0xC4)
#define ST7789_C5_VCMOFSET  (0xC5)
#define ST7789_C6_FRCTR2    (0xC6)
#define ST7789_C7_CABCCTRL  (0xC7)
#define ST7789_C8_REGSEL1   (0xC8)
#define ST7789_CA_REGSEL2   (0xCA)
#define ST7789_CC_PWMFRSEL  (0xCC)
#define ST7789_D0_PWCTRL1   (0xD0)
#define ST7789_D2_VAPVANEN  (0xD2)
#define ST7789_DA_RDID1     (0xDA)
#define ST7789_DB_RDID2     (0xDB)
#define ST7789_DC_RDID3     (0xDC)
#define ST7789_DF_CMD2EN    (0xDF)
#define ST7789_E0_PVGAMCTRL (0xE0)
#define ST7789_E1_NVGAMCTRL (0xE1)
#define ST7789_E2_DGMLUTR   (0xE2)
#define ST7789_E3_DGMLUTB   (0xE3)
#define ST7789_E4_GATECTRL  (0xE4)
#define ST7789_E8_PWCTRL2   (0xE8)
#define ST7789_E9_EQCTRL    (0xE9)
#define ST7789_EC_PROMCTRL  (0xEC)
#define ST7789_FA_PROMEN    (0xFA)
#define ST7789_FC_NVMSET    (0xFC)
#define ST7789_FE_PROMACT   (0xFE)
/* Private macro -------------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void SysTick_Handler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void sendCommand(uint8_t command);
void sendData(uint8_t data);
void Initialize_LCD(void);
void Set_LCD_for_write_at_X_Y(uint16_t x, uint16_t y);
void Fill_LCD(uint8_t R, uint8_t G, uint8_t B);
void GPIO_Configure(void);
void EBI_Configure(void);
void EBI_SendCommand_loop(void);
void GPIOAB_IRQHandler_IT(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
" EBI_LCD with Switch \n\r"
"************************************************\n\r";
volatile uint32_t msTicks = 0;                              /* Variable to store millisecond ticks */
uint8_t *memory0;
uint32_t CS0 = 0;
enum {
	LCD_MODE0=0,
	LCD_MODE1=1,
	LCD_MODE2=2,
	LCD_MODE3=3
	
};
uint8_t LCD_MODE = LCD_MODE0;



EBI_CFG_Type EBICFG;
/**********************************************************************
 * @brief		SysTick_Handler
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler_IT(void)  
{                               /* SysTick interrupt Handler. */
  msTicks--;                   /* See startup file startup_LPC17xx.s for SysTick vector */ 
}

/**********************************************************************
 * @brief		GPIOAB_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void GPIOAB_IRQHandler_IT(void)
{
	uint32_t status;
	
	// GPIOAB interrupt 
	{	
		status = HAL_GPIO_EXTI_GetState(PA);
		
		if(status & (3UL<<(0<<1)))
		{
			HAL_GPIO_EXTI_ClearPin(PA, status & (3UL<<(0<<1)));
			if(LCD_MODE==LCD_MODE0)
			{
				LCD_MODE=LCD_MODE1;
			}
			else
			{
				LCD_MODE=LCD_MODE0;
			}
		}
		
		else if(status & (3UL<<(5<<1)))
		{
			HAL_GPIO_EXTI_ClearPin(PA, status & (3UL<<(5<<1)));
			if(LCD_MODE==LCD_MODE2)	
			{
				LCD_MODE=LCD_MODE3;
			}
			else
			{
				LCD_MODE=LCD_MODE2;
			}
		}
	}
}

/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		sendCommand
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void sendCommand(uint8_t command)
{
  // Select the LCD's command register
	SET_CS;
	CLR_RS;
	__NOP();__NOP();__NOP();__NOP();__NOP();
  LCD_ST7789V_CMD(command);
	SET_RS;
	CLR_CS;
}


/**********************************************************************
 * @brief		sendData
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void sendData(uint8_t data)
{
  // Select the LCD's data register
	SET_CS;
	
  SET_RS;
	__NOP();__NOP();__NOP();__NOP();__NOP();
  LCD_ST7789V_Parameter(data);
	CLR_RS;
	
	CLR_CS;
}


/**********************************************************************
 * @brief		Initialize_LCD
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Initialize_LCD(void)
{
		msTicks = 20;
		while(msTicks){};
			
		SET_RST;
		msTicks = 1;
		while(msTicks){};
			
		CLR_RST;
		msTicks = 1;
		while(msTicks){};
			
		SET_RST;
			
		sendCommand(ST7789_00_NOP);
		sendCommand(ST7789_00_NOP);
		sendCommand(ST7789_00_NOP);


		msTicks = 120;
		while(msTicks){};

		sendCommand(0x11);     //Sleep out

		msTicks = 120;
		while(msTicks){};
			
		sendCommand(ST7789_00_NOP);
		sendCommand(ST7789_00_NOP);
		sendCommand(ST7789_00_NOP);

		sendCommand(0x36);     
		sendData(0x00);   

		sendCommand(0x21);     

		sendCommand(0xB2);     
		sendData(0x05);   
		sendData(0x05);   
		sendData(0x00);   
		sendData(0x33);   
		sendData(0x33);   

		sendCommand(0xB7);     
		sendData(0x75);   

		sendCommand(0xBB);     
		sendData(0x22);   

		sendCommand(0xC0);     
		sendData(0x2C);   

		sendCommand(0xC2);     
		sendData(0x01);   

		sendCommand(0xC3);     
		sendData(0x13);   

		sendCommand(0xC4);     
		sendData(0x20);   

		sendCommand(0xC6);     
		sendData(0x11);   

		sendCommand(0xD0);     
		sendData(0xA4);   
		sendData(0xA1);   

		sendCommand(0xD6);     
		sendData(0xA1);   

		sendCommand(0xE0);     
		sendData(0xD0);   
		sendData(0x05);   
		sendData(0x0A);   
		sendData(0x09);   
		sendData(0x08);   
		sendData(0x05);   
		sendData(0x2E);   
		sendData(0x44);   
		sendData(0x45);   
		sendData(0x0F);   
		sendData(0x17);   
		sendData(0x16);   
		sendData(0x2B);   
		sendData(0x33);   

		sendCommand(0xE1);     
		sendData(0xD0);   
		sendData(0x05);   
		sendData(0x0A);   
		sendData(0x09);   
		sendData(0x08);   
		sendData(0x05);   
		sendData(0x2E);   
		sendData(0x43);   
		sendData(0x45);   
		sendData(0x0F);   
		sendData(0x16);   
		sendData(0x16);   
		sendData(0x2B);   
		sendData(0x33); 
	
		sendCommand(0x29);     //Display on		
}
	

/**********************************************************************
 * @brief		Set_LCD_for_write_at_X_Y
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Set_LCD_for_write_at_X_Y(uint16_t x, uint16_t y)
{
  //CASET (2Ah): Column Address Set
  // * The value of XS [15:0] and XE [15:0] are referred when RAMWR
  //   command comes.
  // * Each value represents one column line in the Frame Memory.
  // * XS [15:0] always must be equal to or less than XE [15:0]
  sendCommand(ST7789_2A_CASET); //Column address set
  //Write the parameters for the "column address set" command
  sendData(x>>8);     //Start MSB = XS[15:8]
  sendData(x&0x00FF); //Start LSB = XS[ 7:0]
  sendData(0);        //End MSB   = XE[15:8] 240-1
  sendData(240);      //End LSB   = XE[ 7:0]
  //Write the "row address set" command to the LCD
  //RASET (2Bh): Row Address Set
  // * The value of YS [15:0] and YE [15:0] are referred when RAMWR
  //   command comes.
  // * Each value represents one row line in the Frame Memory.
  // * YS [15:0] always must be equal to or less than YE [15:0]
  sendCommand(ST7789_2B_RASET); //Row address set
  //Write the parameters for the "row address set" command
  //Use 1st quadrant coordinates: 0,0 is lower left, 239,319 is upper right.
  y=319-y;
  sendData(y>>8);     //Start MSB = YS[15:8]
  sendData(y&0x00FF); //Start LSB = YS[ 7:0]
  sendData(0x01);     //End MSB   = YE[15:8] 320-1
  sendData(0x3F);     //End LSB   = YE[ 7:0]
  //Write the "write data" command to the LCD
  //RAMWR (2Ch): Memory Write
  sendCommand(ST7789_2C_RAMWR); //write data
}

/**********************************************************************
 * @brief		Fill_LCD
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Fill_LCD(uint8_t R, uint8_t G, uint8_t B)
{
  uint32_t  i;
  Set_LCD_for_write_at_X_Y(0, 319);

  // Select the LCD controller
  SET_CS;  
  // Select the LCD's data register
  SET_RS;

  //Fill display with a given RGB value
  for (i = 0; i < (320UL * 240UL); i++)
	{
    //Send out one pixel
    LCD_ST7789V_Parameter(B); //Blue
    LCD_ST7789V_Parameter(G); //Green
    LCD_ST7789V_Parameter(R); //Red
   }
  // Deselect the LCD controller
	CLR_CS;
}


/**********************************************************************
 * @brief		PCU Configure
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void GPIO_Configure(void)
{
	// RD pin
	HAL_GPIO_ConfigOutput(PA, 6, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 6, FUNC9);
	
	// AD8
	HAL_GPIO_ConfigOutput(PB, 15, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 15, FUNC9);
	// AD9
	HAL_GPIO_ConfigOutput(PB, 14, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 14, FUNC9);
	// AD10
	HAL_GPIO_ConfigOutput(PB, 13, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 13, FUNC9);
	// AD11
	HAL_GPIO_ConfigOutput(PB, 12, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 12, FUNC9);
	// AD12
	HAL_GPIO_ConfigOutput(PB, 11, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 11, FUNC9);
	// AD13
	HAL_GPIO_ConfigOutput(PB, 10, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 10, FUNC9);
	// AD14
	HAL_GPIO_ConfigOutput(PB, 2, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 2, FUNC9);
	// AD15
	HAL_GPIO_ConfigOutput(PB, 1, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 1, FUNC9);
	
	//LWR
	HAL_GPIO_ConfigOutput(PB, 6, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 6, FUNC9);
	
#if 0
	//CS0
	HAL_GPIO_ConfigOutput(PB, 7, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 7, FUNC9);
#else
	HAL_GPIO_ConfigOutput(PB, 7, PUSH_PULL_OUTPUT);
	PB->OUTDR |= (0x01 << 7);
#endif

	//RESET pin
	HAL_GPIO_ConfigOutput(PB, 9, PUSH_PULL_OUTPUT);
	PB->OUTDR |= (0x01 << 9);
	
	//RS pin CMD, DATA signal
	HAL_GPIO_ConfigOutput(PA, 15, PUSH_PULL_OUTPUT);
	PA->OUTDR |= (0x01 << 15);
	
//	/* CLKO OUTPUT FUNCTION */
	HAL_GPIO_ConfigOutput((PORT_Type*)PF, 0, ALTERN_FUNC);
	HAL_GPIO_ConfigPullup((PORT_Type*)PF, 0, DISPUPD);
	HAL_GPIO_ConfigFunction((PORT_Type*)PF, 0, FUNC1);

	// Switch PA0, PA5
	HAL_GPIO_ConfigOutput(PA, 0, INPUT);
	HAL_GPIO_ConfigPullup(PA, 0, DISPUPD);
	HAL_GPIO_SetDebouncePin(PA, 0);
	
	HAL_GPIO_ConfigOutput(PA, 5, INPUT);
	HAL_GPIO_ConfigPullup(PA, 5, DISPUPD);
	HAL_GPIO_SetDebouncePin(PA, 5);
	
	HAL_SCU_MiscClockConfig(4,PD_TYPE,CLKSRC_LSI,101); //  PORT A,B,C Debounce set. ( Use Switch )
		
	NVIC_SetPriority(GPIOAB_IRQn, 1);	
	NVIC_EnableIRQ(GPIOAB_IRQn);	
	
	HAL_GPIO_EXTI_Config(PA, 0, IER_EDGE, ICR_RISING_EDGE_INT);
	HAL_GPIO_EXTI_Config(PA, 5, IER_EDGE, ICR_RISING_EDGE_INT);

}

/**********************************************************************
 * @brief		EBI Configure
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void EBI_Configure(void)
{
  
	if(HAL_EBI_Init() != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();				
	}

	EBICFG.InsertIDLE = IDLE0;
	EBICFG.FallingTiming = EXTEND0;
	EBICFG.ExternalWaitEnable = EWDIS;
	EBICFG.WaitSignalPolarity = EWPLOW;
	EBICFG.AddressMuxed = IODIS;
	EBICFG.LaneSelection = BLDIS;
	EBICFG.KeepAddress = Clock1;
	EBICFG.MemoryWidth = MWIDTH8;
	EBICFG.NORMWAIT = 10;									

	HAL_EBI_Setup(CS0, &EBICFG);

	msTicks = 10;
	while(msTicks){};

	Initialize_LCD();
		
	msTicks = 10;
	while(msTicks){};
}

/**********************************************************************
 * @brief		EBI_SendCommand_loop
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void EBI_SendCommand_loop(void)
{
	msTicks = 1000;
	while(msTicks){};
	LCD_MODE=LCD_MODE0;
		
	while(1)
	{
		if(LCD_MODE==LCD_MODE0)
		{
			msTicks = 500;
			while(msTicks){};

			sendCommand(ST7789_28_DISPOFF);     //Display off
			Fill_LCD(0x00, 0x00, 0xFF);
			sendCommand(ST7789_29_DISPON);     //Display on	
				
			msTicks = 500;
			while(msTicks){};

			sendCommand(ST7789_28_DISPOFF);     //Display off
			Fill_LCD(0x00, 0xFF, 0x00);
			sendCommand(ST7789_29_DISPON);     //Display on	
		}
		else if(LCD_MODE==LCD_MODE1)
		{
			msTicks = 500;
			while(msTicks){};

			sendCommand(ST7789_28_DISPOFF);     //Display off
			Fill_LCD(0xFF, 0x00, 0x00);
			sendCommand(ST7789_29_DISPON);     //Display on	
				
			msTicks = 500;
			while(msTicks){};

			sendCommand(ST7789_28_DISPOFF);     //Display off
			Fill_LCD(0xFF, 0x00, 0xFF);
			sendCommand(ST7789_29_DISPON);     //Display on	
		}
		else if(LCD_MODE==LCD_MODE2)
		{
			msTicks = 500;
			while(msTicks){};

			sendCommand(ST7789_28_DISPOFF);     //Display off
			Fill_LCD(0xFF, 0xFF, 0x00);
			sendCommand(ST7789_29_DISPON);     //Display on				
				
			msTicks = 500;
			while(msTicks){};

			sendCommand(ST7789_28_DISPOFF);     //Display off
			Fill_LCD(0x00, 0xFF, 0xFF);
			sendCommand(ST7789_29_DISPON);     //Display on	
		}
		else if(LCD_MODE==LCD_MODE3)
		{
			msTicks = 500;
			while(msTicks){};

			sendCommand(ST7789_28_DISPOFF);     //Display off
			Fill_LCD(0xFF, 0xFF, 0xFF);
			sendCommand(ST7789_29_DISPON);     //Display on	
				
			msTicks = 500;
			while(msTicks){};

			sendCommand(ST7789_28_DISPOFF);     //Display off
			Fill_LCD(0x00, 0x00, 0x00);
			sendCommand(ST7789_29_DISPON);     //Display on					
		}	
	}		
}



	
/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void mainloop(void)
{  

	/*Initialize menu prinf*/	
	//DEBUG_Print_menu();	// NO use Debug pin

	/*Configure port peripheral*/
	GPIO_Configure();
	
	      /*Enable IRQ Interrupts */
	__enable_irq(); 
	
	EBI_Configure();
	
      /* Infinite Loop */
	while(1)
	{
		EBI_SendCommand_loop();
	}
		
}

/**********************************************************************
 * @brief		main
 * @param[in]	None
 * @return 		None
 **********************************************************************/
int main(void)
{
 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	SysTick_Config(SystemCoreClock / 1000); 
	/* Initialize Debug frame work through initializing USART port  */
	//DEBUG_Init();
	 
	/* Infinite loop */
	mainloop();  
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

