/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"



/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void SysTick_Handler_IT (void);
void TEMPERATURE_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void TEMPERATURE_Configure(void);
void TEMPERATURE_Startloop(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;
uint32_t status;
uint32_t i;
const uint8_t menu[] =
"************************************************\n\r"
" Temp Sensor Test \n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" Using Temp Sensor test \n\r"
"************************************************\n\r";
uint8_t tsintflag;  // flag
uint8_t printflag;  // flag
uint16_t count;  // flag

/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/**********************************************************************
 * @brief		SysTick handler sub-routine (1ms)
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler_IT (void) 					// SysTick Interrupt Handler @ 1000Hz
{
	if(msec)msec--;
}



/**********************************************************************
 * @brief		TEMPERATURE_IRQHandler
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TEMPERATURE_IRQHandler_IT(void)
{
	uint8_t status;
	
	status = HAL_TS_GetStatus();
	if (status & 0x04){
		if (tsintflag == 1){
			tsintflag=0;
			HAL_GPIO_ClearPin(PB, _BIT(2));
		}
		else {
			tsintflag=1;
			HAL_GPIO_SetPin(PB, _BIT(2));
		}
		HAL_TS_ClearStatus();
		
		printflag = 1;
	}

}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void GPIO_Configure(void)
{
// Test Pin setting PB2
		HAL_GPIO_ConfigOutput(PB, 2, PUSH_PULL_OUTPUT);
		HAL_GPIO_ConfigPullup(PB, 2, DISPUPD);	
		HAL_GPIO_ClearPin(PB, _BIT(2));
    
		HAL_GPIO_ConfigOutput(PF, 0, ALTERN_FUNC);	
		HAL_GPIO_ConfigFunction(PF, 0, FUNC1);
	
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return	None
 **********************************************************************/
void TEMPERATURE_Configure(void)
{
	  HAL_SCU_ClockOutput(4,ENABLE); //    /10

		//temp sense clock select test  
		//LSIT_EN
		HAL_SCU_TSLSI40K_ClockConfig(ENABLE);
		
		//TSSENSECLK_SEL 0 : LSITS, 1 : LSI750kHz, 2 : LSI40kHz, 3 : HSI_I(CSCR)
		HAL_SCU_TempSensor_ClockConfig(TSSENSE_TYPE, TSSENSECLK_LSITS);

		//TSREFCLK_SEL 0 : HSI_I(CSCR), 1 : MCLK, 2 : MOSC, 3 : SOSC
		HAL_SCU_TempSensor_ClockConfig(TSREF_TYPE, TSREFCLK_MCLK);

		if (HAL_TS_Init() != HAL_OK )
		{
			Error_Handler();
		}
		
		HAL_TS_SetReferencePeriod();  //1kHz REFPeriod
		
		HAL_TS_Start(ENABLE);
		HAL_TS_ConfigInterrupt(ENABLE);
	
		NVIC_SetPriority(TEMPERATURE_IRQn, 3);
		NVIC_EnableIRQ(TEMPERATURE_IRQn);
}

/**********************************************************************
 * @brief		TEMPSENSOR_Startloop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void TEMPERATURE_Startloop(void)
{

	int8_t tmp;
	
  if(printflag)
	{
			count++;
		
			if(count >= 500)
			{
				tmp = (int8_t)HAL_TS_ConvertToCelsius();
				
				if(tmp < 0)
				{
						tmp = 0xFF - tmp;
						_DBG("-");
				}
				
				_DBD16(tmp);
				_DBG("\r\n");
					
				count = 0;
			}
		
			HAL_TS_Start(ENABLE);
			printflag = 0;
	}
}

/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void mainloop(void)
{

  /*Initialize menu prinf*/
	DEBUG_Print_menu();
  
	/*Initialize port peripheral*/
	GPIO_Configure();
	
	/*Initialize RTC peripheral*/
	TEMPERATURE_Configure(); 

	 
	 /*Enable IRQ Interrupts */
	 __enable_irq(); 
	 
	/* Infinite Loop */
	while(1)
	{
		TEMPERATURE_Startloop();
	}
	
}


/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{
	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
  /* Initialize all port */
	Port_Init(); 

	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
