/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void TIMER_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void TIMER_Configure( void);
void TIMER_GetData(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
TIMER1n_CAPTURECFG_Type TIMER1n_Config;
__IO	uint32_t count;
__IO	uint32_t fflag;

const uint8_t menu[] =
"************************************************\n\r"
"TIMER1n Capture Mode \n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" Using Timer10 in Capture mode \n\r"
"************************************************\n\r";


/**********************************************************************
 * @brief		TIMER_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TIMER_IRQHandler_IT (void)
{
	uint32_t status;
	
	status = HAL_TIMER1n_GetCaptureInterrupt(TIMER10);
	if (status == 1){
		HAL_TIMER1n_ClearCaptureInterrupt(TIMER10);
		if (fflag == 0){
			fflag=1;
		}
		else if (fflag ==1){
			fflag = 2;
		}
	}		
}


/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Test Pin Setting PA5
	HAL_GPIO_ConfigPullup(PA, 5, DISPUPD);	
	HAL_GPIO_ConfigOutput(PA, 5, PUSH_PULL_OUTPUT);
	HAL_GPIO_ClearPin(PA, _BIT(5));
	
	HAL_GPIO_SetPin(PA, _BIT(5));
		
	// T10Capture pin config set if need
	HAL_GPIO_ConfigFunction(PB, 15, FUNC5);		// T10 Capture Alternative Function 5
	HAL_GPIO_ConfigOutput(PB, 15, ALTERN_FUNC);	// PA3 Capture Mode
}


/**********************************************************************
 * @brief		TIMER_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_Configure(void)
{
	uint32_t i;
	
	HAL_SCU_MiscClockConfig(1, TIMER1N_TYPE, MCLK_MISC_CLOCK, 1);
	HAL_SCU_Timer1n_ClockConfig(T1NCLK_PCLK); 
	TIMER1n_Config.CkSel = MCCR1PCLK;    
	TIMER1n_Config.Prescaler = 48;
	TIMER1n_Config.TnADR = (0xFFFF); 	// capture mode
	TIMER1n_Config.ClrMode=FALLING_EGDE;   //	FALLING_EGDE,RISING_EGDE,BOTH_EGDE,
	
	
	/* TIMER10*/
	if( HAL_TIMER1n_Init(TIMER10, CAPTURE_MODE, &TIMER1n_Config) != HAL_OK)
	{
		Error_Handler();
	}
	HAL_TIMER1n_ConfigInterrupt(TIMER10, TIMER1n_INTCFG_CIE, ENABLE);
	HAL_TIMER1n_ClearCaptureInterrupt(TIMER10);

	NVIC_SetPriority(TIMER10_IRQn, 5);
	/* Enable Interrupt for TIMERx channel */
	NVIC_EnableIRQ(TIMER10_IRQn);

	fflag=0;
	
	HAL_TIMER1n_ClearCounter(TIMER10);
	HAL_TIMER1n_Cmd(TIMER10, ENABLE); // timer start & clear
	
	__NOP();
	__enable_irq();
	__NOP();

	_DBG("\n\r Wait Pulse! \n\r");	
	HAL_GPIO_ClearPin(PA, _BIT(5));	
	for (i=0;i<10000;i++);
	HAL_GPIO_SetPin(PA, _BIT(5));
	for (i=0;i<10000;i++);	
	for (i=0;i<10000;i++);	
	for (i=0;i<10000;i++);	
	for (i=0;i<10000;i++);	
	for (i=0;i<10000;i++);	
	for (i=0;i<10000;i++);		
	for (i=0;i<10000;i++);		
	HAL_GPIO_ClearPin(PA, _BIT(5));
	for (i=0;i<1000;i++);
	HAL_GPIO_SetPin(PA, _BIT(5));
}

/**********************************************************************
 * @brief		TIMER_GetData
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_GetData(void)
{
  switch(fflag){
			case 0:
				break;
			case 1:
				break;
			case 2:
				_DBG("\n\rcapture value! \n\r");
				count = HAL_TIMER1n_GetCaptureData(TIMER10); 
				_DBG("Capture CNT = "); _DBD32(count); _DBG_(" ");
				fflag=0;
				break;
		}
}

/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void mainloop(void)
{
	/*Initialize menu prinf*/
	DEBUG_Print_menu();
	
	/*Initialize port peripheral*/
	GPIO_Configure();
	

	/*TIMER10 Init*/
	TIMER_Configure();

	
	 /*Enable IRQ Interrupts */
	 __enable_irq(); 
	 
       /* Infinite Loop */
	while(1)
	{
		TIMER_GetData();
	}
}

/**********************************************************************
 * @brief		main
 * @param[in]	None
 * @return 	None
 **********************************************************************/
int main(void)
{
	/* Configure the system clock to 48 MHz */
	SystemClock_Config();

	 /* Initialize all port */
	Port_Init(); 
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
