/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void TIMER_Configure( void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
TIMER1n_ONESHOTCFG_Type TIMER1n_Config;
const uint8_t menu[] =
"************************************************\n\r"
"Timer demo \n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" Using Timer10 in One Shot mode \n\r"
"************************************************\n\r";



/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Test Pin setting PB11
	HAL_GPIO_ConfigOutput(PB, 11, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 11, DISPUPD);	
	HAL_GPIO_ClearPin(PB, _BIT(11));	
	
	// TIMER10OUT pin config set if need 
	HAL_GPIO_ConfigFunction(PB, 14, FUNC6);		// T10OUT
	HAL_GPIO_ConfigOutput(PB, 14, ALTERN_FUNC);	
}


/**********************************************************************
 * @brief		TIMER_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_Configure(void)
{
	// Timer1n clock source select : MCCR1 / PCLK / ECn
#if 1  //Timer1n clock source from MCCR1
	HAL_SCU_MiscClockConfig(1, TIMER1N_TYPE, MCLK_MISC_CLOCK, 1);
	HAL_SCU_Timer1n_ClockConfig(T1NCLK_PCLK); 
	TIMER1n_Config.CkSel = MCCR1PCLK;
	TIMER1n_Config.Prescaler = 48;    
#endif
#if 0	//Timer1n clock source from PCLK
	HAL_SCU_Timer1n_ClockConfig(T1NCLK_PCLK);
	TIMER1n_Config.CkSel = MCCR1PCLK;    
	TIMER1n_Config.Prescaler = 48;	// 48Mhz / 48 = 1Mhz ->1us
#endif
#if 0   //Timer1n clock source from ECn (PA2, PA3, PB14, PB15)
		// EC10 pin config set
		HAL_GPIO_ConfigFunction(PA, 3, FUNC4);
		HAL_GPIO_ConfigOutput(PA, 3, ALTERN_FUNC);
		
		TIMER1n_Config.CkSel = ECn;  
		TIMER1n_Config.ECE = 1; // 0:falling edge, 1:rising edge	
		TIMER1n_Config.Prescaler = 1;  // no used in ECn mode  
#endif 
	TIMER1n_Config.TnADR = (1000);	// 1msec	
	TIMER1n_Config.TnBDR = (400);	// 300usec		
	TIMER1n_Config.StartLevel=START_HIGH;  //START_LOW, START_HIGH

	if(HAL_TIMER1n_Init(TIMER10, ONESHOT_MODE, &TIMER1n_Config) != HAL_OK)
	{
		Error_Handler();
	}
	HAL_GPIO_SetPin(PB, _BIT(11));
	HAL_TIMER1n_Cmd(TIMER10, ENABLE); // timer start & clear	
	
	while(HAL_TIMER1n_GetMatchInterrupt(TIMER10) != 1);
	HAL_TIMER1n_ClearMatchInterrupt(TIMER10);	
	HAL_GPIO_ClearPin(PB, _BIT(11));
}


/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void mainloop(void)
{
	/*Initialize menu prinf*/
	DEBUG_Print_menu();
	
	/*Initialize port peripheral*/
	GPIO_Configure();	

	/*TIMER10 Init*/
	TIMER_Configure();

	
	 /*Enable IRQ Interrupts */
	 __enable_irq(); 
	 
      /* Infinite Loop */
	while(1)
	{		
	}
}


/**********************************************************************
 * @brief		main
 * @param[in]	None
 * @return 	None
 **********************************************************************/
int main(void)
{
	/* Configure the system clock to 48 MHz */
	SystemClock_Config();

	 /* Initialize all port */
	Port_Init(); 
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	return (0);
}
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

