/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void TIMER_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void TIMER_Configure( void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;
TIMER2n_CAPTURECFG_Type TIMER2n_Config;
__IO	uint32_t count1, count2;
__IO	uint32_t fflag;
__IO uint32_t i;
const uint8_t menu[] =
"************************************************\n\r"
"Timer Capture demo \n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" Using Timer 2n_capture \n\r"
"************************************************\n\r";


/**********************************************************************
 * @brief		TIMER_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TIMER_IRQHandler_IT (void)
{
	if (HAL_TIMER2n_GetCaptureInterrupt(TIMER20) ==  1){
		HAL_TIMER2n_ClearCaptureInterrupt(TIMER20);

		if (fflag == 0){
			fflag = 1;
		}
		else if (fflag ==1){
			fflag = 2;
		}		
	}	
}


/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	HAL_GPIO_ConfigOutput(PB, 1, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 1, DISPUPD);	
	HAL_GPIO_SetPin(PB, _BIT(1));

	// Timer20 -- Use External Clock to Capture clock
	// T20 Capture Pin	T20CAP	: PA1, PA2, PA3, PB3, PB10, PB11
	HAL_GPIO_ConfigFunction(PA, 1, FUNC7);
	HAL_GPIO_ConfigOutput(PA, 1, ALTERN_FUNC);	
}


/**********************************************************************
 * @brief		TIMER_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_Configure(void)
{
	#if 1   //Timer20 clock source from MCCR2
	HAL_SCU_MiscClockConfig(2, TIMER20_TYPE, MCLK_MISC_CLOCK, 2);
	HAL_SCU_Timer20_ClockConfig(T20CLK_MCCR2); 
	TIMER2n_Config.CkSel = MCCR2PCLK;
	TIMER2n_Config.Prescaler = 24;
	#endif
	
	#if 0  //Timer20 clock source from PCLK
	HAL_SCU_Timer20_ClockConfig(T20CLK_PCLK); 
	TIMER2n_Config.CkSel = MCCR2PCLK;
	TIMER2n_Config.Prescaler = 24;
	#endif
	
	#if 0   //Timer20 clock source from EC2n (PA0, PA5)
	// EC20 pin config set
	HAL_GPIO_ConfigFunction(PA, 5, FUNC4);
	HAL_GPIO_ConfigOutput(PA, 5, ALTERN_FUNC);

	TIMER2n_Config.CkSel = EC2n;  
	TIMER2n_Config.ECE = 0; // 0:falling edge, 1:rising edge	
	TIMER2n_Config.Prescaler = 1;  // not used in ECn mode  
	#endif 

	TIMER2n_Config.T2nADR = (0xFFFF);
	TIMER2n_Config.ClrMode=RISING_EGDE;
	TIMER2n_Config.CAPCkSel=CAP_EXTERNAL_CLK;     
	TIMER2n_Config.CNCLR=ENABLE;     
		
	if (HAL_TIMER2n_Init(TIMER20, CAPTURE_MODE, &TIMER2n_Config)!= HAL_OK)
	{
		Error_Handler();
	}

	HAL_TIMER2n_ConfigInterrupt(TIMER20, TIMER2n_CR_CAPTURE_INTR, ENABLE);
	HAL_TIMER2n_ClearCaptureInterrupt(TIMER20);

	NVIC_SetPriority(TIMER20_IRQn, 3);
	/* Enable Interrupt for TIMER20 channel */
	NVIC_EnableIRQ(TIMER20_IRQn);
	
	HAL_TIMER2n_ClearCounter(TIMER20);
	HAL_TIMER2n_Cmd(TIMER20, ENABLE); // timer start & clear
	
	fflag=0;	
	_DBG("\n\r Wait Pulse! \n\r");
	
	HAL_GPIO_ClearPin(PB, _BIT(1));
	for (i=0;i<40000;i++);
	for (i=0;i<40000;i++);
	HAL_GPIO_SetPin(PB, _BIT(1));
	for (i=0;i<8000;i++);	
	for (i=0;i<8000;i++);	
	
	HAL_GPIO_ClearPin(PB, _BIT(1));
	for (i=0;i<1000;i++);
	HAL_GPIO_SetPin(PB, _BIT(1));
	
	while(1){
		switch(fflag){
			case 0:
				break;
			case 1:
				break;
			case 2:
				count1 = HAL_TIMER2n_GetCaptureData(TIMER20); 	// Capture count
				_DBG("capture value! \n\r");
				_DBG("Capture CNT = "); _DBD32(count1); _DBG_(" ");
				fflag++;
				break;
		}
	}	
}


/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void mainloop(void)
{		
	/*Enable IRQ Interrupts */
	 __enable_irq();  
	
	/*Initialize menu prinf*/
	DEBUG_Print_menu();
	
	/*Initialize port peripheral*/
	GPIO_Configure();
	
	/*Initialize timer peripheral*/
	TIMER_Configure();
	 
      /* Infinite Loop */
	while(1)
	{		
	}
}


/**********************************************************************
 * @brief		main
 * @param[in]	None
 * @return 	None
 **********************************************************************/
int main(void)
{
	/* Configure the system clock to 48 MHz */
	SystemClock_Config();

	 /* Initialize all port */
	Port_Init(); 
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

