/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void TIMER_Configure( void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;
TIMER2n_PERIODICCFG_Type TIMER2n_Config;
__IO	uint32_t count;
__IO	uint32_t flag;
const uint8_t menu[] =
"************************************************\n\r"
"Timer PWM demo \n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" Using Timer 2n Match Polling\n\r"
"************************************************\n\r";


/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Match Interrupt TEST Pin
	HAL_GPIO_ConfigOutput(PB,1,PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB,1,DISPUPD);
	HAL_GPIO_ClearPin(PB,_BIT(1));

	// T20O pin config set if need	
	HAL_GPIO_ConfigFunction(PB, 10, FUNC6);
	HAL_GPIO_ConfigOutput(PB, 10, ALTERN_FUNC);	
}


/**********************************************************************
 * @brief		TIMER_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_Configure(void)
{
	// Timer20 clock source select : MCCR2 / PCLK / EC20
	#if 1   //Timer20 clock source from MCCR2
	HAL_SCU_MiscClockConfig(2, TIMER20_TYPE, MCLK_MISC_CLOCK, 2);
	HAL_SCU_Timer20_ClockConfig(T20CLK_MCCR2); 
	TIMER2n_Config.CkSel = MCCR2PCLK;
	TIMER2n_Config.Prescaler = 24;
	#endif
	
	#if 0  //Timer20 clock source from PCLK
	HAL_SCU_Timer20_ClockConfig(T20CLK_PCLK); 
	TIMER2n_Config.CkSel = MCCR2PCLK;
	TIMER2n_Config.Prescaler = 24;
	#endif
	
	#if 0   //Timer20 clock source from EC2n (PA0, PA5)
	// EC20 pin config set
	HAL_GPIO_ConfigFunction(PA, 5, FUNC4);
	HAL_GPIO_ConfigOutput(PA, 5, ALTERN_FUNC);

	TIMER2n_Config.CkSel = EC2n;  
	TIMER2n_Config.ECE = 0; // 0:falling edge, 1:rising edge	
	TIMER2n_Config.Prescaler = 1;  // not used in ECn mode  
	#endif 
	
	TIMER2n_Config.T2nADR = (1000); 			// 2msec PERIOD	= ADATA
	TIMER2n_Config.StartLevel = START_HIGH;	// START_LOW, START_HIGH

	if (HAL_TIMER2n_Init(TIMER20, PERIODIC_MODE, &TIMER2n_Config) != HAL_OK )
	{
		Error_Handler();
	}
	HAL_TIMER2n_ConfigInterrupt(TIMER20, TIMER2n_CR_MATCH_INTR, DISABLE);

	HAL_TIMER2n_Cmd(TIMER20, ENABLE); // timer start
	
	count=0;
	while(1)
	{
		
		while(!HAL_TIMER2n_GetMatchInterrupt(TIMER20));
		HAL_TIMER2n_ClearMatchInterrupt(TIMER20);
		
		count++;
		if(count > 5)
		{
			if(flag==0)
			{
				HAL_GPIO_SetPin(PB,_BIT(1));
				flag=1;
			}
			else
			{
				HAL_GPIO_ClearPin(PB,_BIT(1));
				flag=0;
			}
			count=0;
		}	
	}	
}


/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void mainloop(void)
{	
	/*Initialize menu prinf*/
	DEBUG_Print_menu();
	
	/*Initialize port peripheral*/
	GPIO_Configure();
	
	/*Initialize timer peripheral*/
	TIMER_Configure();

	 /*Enable IRQ Interrupts */
	 __enable_irq(); 
	 
      /* Infinite Loop */
	while(1)
	{
	}
}


/**********************************************************************
 * @brief		main
 * @param[in]	None
 * @return 	None
 **********************************************************************/
int main(void)
{
	/* Configure the system clock to 48 MHz */
	SystemClock_Config();

	 /* Initialize all port */
	Port_Init(); 
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	return (0);
}
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
