/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void TIMER_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void TIMER_Configure( void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;
uint32_t fflag;
const uint8_t menu[] =
"************************************************\n\r"
"Timer30 Capture\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
"************************************************\n\r";

/**********************************************************************
 * @brief		TIMER_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TIMER_IRQHandler_IT (void)
{
	if ((HAL_TIMER3n_GetStatus_IT(TIMER30) & T30INT_CAPEN) == T30INT_CAPEN)
	{
		HAL_TIMER3n_ClearStatus_IT(TIMER30, T30INT_CAPEN);
		fflag++;
	}	
}


/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Test Pin setting PB2
	HAL_GPIO_ConfigOutput(PB, 2, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 2, DISPUPD);	

	// T30C pin config set if need
	HAL_GPIO_ConfigOutput(PA, 12, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 12, FUNC4);
}


/**********************************************************************
 * @brief		TIMER_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_Configure(void)
{
	TIMER3n_CFG_Type TIMER30_Config;
	uint32_t capture_count;
	uint32_t i;	

// Timer30 clock source select : MCCR2 / PCLK / ECn
#if	1	// Timer30 clock source from MCCR2
	HAL_SCU_Timer30_ClockConfig(T30CLK_MCCR2);	// MCCR2
	HAL_SCU_MiscClockConfig(2, TIMER30_TYPE, CLKSRC_MCLK, 1);
	TIMER30_Config.CLK = T30_CLKINT;
	TIMER30_Config.Prescaler = 48;
	
#elif	0	// Timer30 clock source from PCLK
	HAL_SCU_Timer30_ClockConfig(T30CLK_PCLK);		// PCLK
	TIMER30_Config.CLK = T30_CLKINT;
	TIMER30_Config.Prescaler = 48;
	
#else	//Timer30 clock source from EC2n (PA12)
	// EC30 pin config set
	HAL_GPIO_ConfigOutput(PA, 12, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 12, FUNC3);
	
	TIMER30_Config.CLK = T30_CLKEXT;
	TIMER30_Config.ECE = T30_FEDGE;	
	TIMER30_Config.Prescaler = 48;
#endif

	TIMER30_Config.MS = T30_CAPM;
	TIMER30_Config.FORCA = T30_6CHMOD;
	TIMER30_Config.UPDT = T30_UPMATCH;
	TIMER30_Config.PMOC = T30_E1PERIOD;
	TIMER30_Config.CPOL = T30_CAPRISE;
	
	NVIC_SetPriority(TIMER30_IRQn, 3);
	NVIC_EnableIRQ(TIMER30_IRQn);

	if (HAL_TIMER3n_Init(TIMER30, &TIMER30_Config) != HAL_OK)
	{
		Error_Handler();
	}
	
	HAL_TIMER3n_ConfigInterrupt(TIMER30, ENABLE, T30INT_CAPEN);
	HAL_TIMER3n_ClearStatus_IT(TIMER30, T30INT_CAPEN);
	
	HAL_TIMER3n_SetPeriod(TIMER30, 0xFFFF);
	HAL_TIMER3n_SetADuty(TIMER30, 0xFFFF); 
	
	HAL_TIMER3n_Start(TIMER30, ENABLE);
	
	HAL_GPIO_ClearPin(PB, _BIT(2));
	for (i=0;i<1000;i++);
	for (i=0;i<1000;i++);

	HAL_GPIO_SetPin(PB, _BIT(2));
	for (i=0;i<1000;i++);	
	for (i=0;i<1000;i++);	
	for (i=0;i<1000;i++);	
	for (i=0;i<1000;i++);	
	for (i=0;i<1000;i++);	
	for (i=0;i<1000;i++);		
	for (i=0;i<1000;i++);		
	HAL_GPIO_ClearPin(PB, _BIT(2));
	for (i=0;i<1000;i++);
	HAL_GPIO_SetPin(PB, _BIT(2));

	_DBG("Timer30 Capturing... Wait!!!\n\r");
	
	capture_count=0;
	while(1)
	{
		switch(fflag)
		{
			case 0:
				break;
			case 1:
				break;
			case 2:
				capture_count = T30_GetCapData(); 
				_DBG("Capture DATA :\t"); _DBD16(capture_count);
				fflag=3;
				break;
			default:
				break;
		}
	}	
}


/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void mainloop(void)
{
	
	 /*Enable IRQ Interrupts */
	 __enable_irq(); 
	 
	/*Initialize menu prinf*/
	DEBUG_Print_menu();
	
	/*Initialize port peripheral*/
	GPIO_Configure();
	
	/*Initialize timer peripheral*/
	TIMER_Configure();
	
      /* Infinite Loop */
	while(1)
	{
	}
}


/**********************************************************************
 * @brief		main
 * @param[in]	None
 * @return 	None
 **********************************************************************/
int main(void)
{
	/* Configure the system clock to 48 MHz */
	SystemClock_Config();

	/* Initialize all port */
	Port_Init(); 
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	return (0);
}
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
