/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void TIMER_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void TIMER_Configure( void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;
uint8_t Period_count;
uint8_t A_count;
uint8_t B_count;
uint8_t C_count;

const uint8_t menu[] =
"************************************************\n\r"
"Timer30 Match Interrupt\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
"************************************************\n\r";


/**********************************************************************
 * @brief		TIMER_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TIMER_IRQHandler_IT (void)
{
	// TIMER 30 PERIOD MATCH FLAG
	if ((HAL_TIMER3n_GetStatus_IT(TIMER30) & T30INT_PMATEN) == T30INT_PMATEN)
	{
		HAL_TIMER3n_ClearStatus_IT(TIMER30, T30INT_PMATEN);	
		if(Period_count)
		{
			HAL_GPIO_ClearPin(PB, _BIT(4));
			Period_count=0;
		}
		else
		{
			HAL_GPIO_SetPin(PB, _BIT(4));
			Period_count=1;
		}
	}
	
	// TIMER 30 A MATCH INTERRUPT FLAG
	if ((HAL_TIMER3n_GetStatus_IT(TIMER30) & T30INT_AMATEN) == T30INT_AMATEN)
	{
		HAL_TIMER3n_ClearStatus_IT(TIMER30, T30INT_AMATEN);		
		if(A_count)
		{
			HAL_GPIO_ClearPin(PC, _BIT(3));
			A_count=0;
		}
		else
		{
			HAL_GPIO_SetPin(PC, _BIT(3));
			A_count=1;
		}
	}	
	
	// TIMER 30 B MATCH INTERRUPT FLAG
	if ((HAL_TIMER3n_GetStatus_IT(TIMER30) & T30INT_BMATEN) == T30INT_BMATEN)
	{
		HAL_TIMER3n_ClearStatus_IT(TIMER30, T30INT_BMATEN);	
		if(B_count)
		{
			HAL_GPIO_ClearPin(PC, _BIT(7));
			B_count=0;
		}
		else
		{
			HAL_GPIO_SetPin(PC, _BIT(7));
			B_count=1;
		}		
	}
	
	// TIMER 30 C MATCH INTERRUPT FLAG
	if ((HAL_TIMER3n_GetStatus_IT(TIMER30) & T30INT_CMATEN) == T30INT_CMATEN)
	{
		HAL_TIMER3n_ClearStatus_IT(TIMER30, T30INT_CMATEN);		
		if(C_count)
		{
			HAL_GPIO_ClearPin(PC, _BIT(9));
			C_count=0;
		}
		else
		{
			HAL_GPIO_SetPin(PC, _BIT(9));
			C_count=1;
		}
	}
}


/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Period-Match Test Pin setting PB4
	HAL_GPIO_ConfigOutput(PB, 4, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 4, DISPUPD);	
	HAL_GPIO_ClearPin(PB, _BIT(4));

	// A-Match Test Pin setting PC3
	HAL_GPIO_ConfigOutput(PC, 3, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PC, 3, DISPUPD);	
	HAL_GPIO_ClearPin(PC, _BIT(3));

	// B-Match Test Pin setting PC7
	HAL_GPIO_ConfigOutput(PC, 7, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PC, 7, DISPUPD);	
	HAL_GPIO_ClearPin(PC, _BIT(7));

	// C-Match Test Pin setting PC9
	HAL_GPIO_ConfigOutput(PC, 9, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PC, 9, DISPUPD);	
	HAL_GPIO_ClearPin(PC, _BIT(9));	
}


/**********************************************************************
 * @brief		TIMER_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_Configure(void)
{
	TIMER3n_CFG_Type TIMER30_Config;
	
// Timer30 clock source select : MCCR2 / PCLK / ECn
#if	1	// Timer30 clock source from MCCR2
	HAL_SCU_Timer30_ClockConfig(T30CLK_MCCR2);	// MCCR2
	HAL_SCU_MiscClockConfig(2, TIMER30_TYPE, CLKSRC_MCLK, 1);
	TIMER30_Config.CLK = T30_CLKINT;
	TIMER30_Config.Prescaler = 48;
	
#elif	0	// Timer30 clock source from PCLK
	HAL_SCU_Timer30_ClockConfig(T30CLK_PCLK);		// PCLK
	TIMER30_Config.CLK = T30_CLKINT;
	TIMER30_Config.Prescaler = 48;
	
#else	//Timer30 clock source from EC2n (PA12)
	// EC30 pin config set
	HAL_GPIO_ConfigOutput(PA, 12, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 12, FUNC3);
	
	TIMER30_Config.CLK = T30_CLKEXT;
	TIMER30_Config.ECE = T30_FEDGE;	
	TIMER30_Config.Prescaler = 48;
#endif
	
	TIMER30_Config.MS = T30_INVM;
	TIMER30_Config.FORCA = T30_6CHMOD;
	TIMER30_Config.UPDT = T30_UPMATCH;
	TIMER30_Config.PMOC = T30_E1PERIOD;

	NVIC_SetPriority(TIMER30_IRQn, 3);
	NVIC_EnableIRQ(TIMER30_IRQn);

	if (HAL_TIMER3n_Init(TIMER30, &TIMER30_Config) != HAL_OK)
	{
		Error_Handler();
	}
	
	HAL_TIMER3n_ConfigInterrupt(TIMER30, ENABLE, T30INT_PMATEN | T30INT_AMATEN | T30INT_BMATEN | T30INT_CMATEN);
	HAL_TIMER3n_ClearStatus_IT(TIMER30, T30INT_PMATEN | T30INT_AMATEN | T30INT_BMATEN | T30INT_CMATEN);

	HAL_TIMER3n_SetPeriod(TIMER30, 10000);
	HAL_TIMER3n_SetADuty(TIMER30, 7000); 
	HAL_TIMER3n_SetBDuty(TIMER30, 4000);
	HAL_TIMER3n_SetCDuty(TIMER30, 1000);
	
	HAL_TIMER3n_Start(TIMER30, ENABLE);

	while(1)
	{
	}
}


/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void mainloop(void)
{		
	 /*Enable IRQ Interrupts */
	 __enable_irq(); 
	 	
	/*Initialize menu prinf*/
	DEBUG_Print_menu();
	
	/*Initialize port peripheral*/
	GPIO_Configure();
	
	/*Initialize timer peripheral*/
	TIMER_Configure();

	/* Infinite Loop */
	while(1)
	{
	}
}


/**********************************************************************
 * @brief		main
 * @param[in]	None
 * @return 	None
 **********************************************************************/
int main(void)
{
	/* Configure the system clock to 48 MHz */
	SystemClock_Config();

	/* Initialize all port */
	Port_Init(); 
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	return (0);
}


/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
