/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void TIMER_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void TIMER_Configure( void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;

const uint8_t menu[] =
"************************************************\n\r"
"Timer30 PWM\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
"************************************************\n\r";
/*
void mainloop(void)
{
		
	TIMER30->OUTCR = 0 
		|(0xe06cUL<<16)
		|(1<<15)   //PWM30xB Output Polarity Selection bit. 0:low start 1:high start
		|(1<<14)   //PWM30xA Output Polarity Selection bit. 0:low start 1:high start
		|(1<<13)   //PABOE
		|(1<<12)   //PBBOE
		|(1<<11)   //PCBOE
		|(1<<10)   //PAAOE		
		|(1<<9)   //PBAOE	
		|(1<<8)   //PCAOE			
		|(1<<6)   //LVLAB  Configure PWM30AB output When Disable
		|(1<<5)   //LVLBB
		|(1<<4)   //LVLCB
		|(1<<2)   //LVLAA		
		|(1<<1)   //LVLBA	
		|(1<<0)   //LVLCA			
	;
	
	TIMER30->HIZCR = 0
		|(1<<8)	  //HIZCSSEL 0:BLNK, 1:T40_OUT
		|(1<<7)   //HIZEN
		|(0<<4)   //HIZSW		
		|(1<<2)   //HEDGE 0:falling BLNK, 1:rising BLNK
		|(0<<0)   //HIZCLR		
	;	

}
*/

/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// PWM30AA(PA8)
	HAL_GPIO_ConfigOutput(PA, 8, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 8, FUNC3);
	// PWM30AB(PA7)
	HAL_GPIO_ConfigOutput(PA, 7, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 7, FUNC3);
	// PWM30BA(PA9)
	HAL_GPIO_ConfigOutput(PA, 9, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 9, FUNC3);
	// PWM30BB(PB0)
	HAL_GPIO_ConfigOutput(PB, 0, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 0, FUNC3);
	// PWM30CA(PA10)
	HAL_GPIO_ConfigOutput(PA, 10, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 10, FUNC3);
	// PWM30CB(PB1)
	HAL_GPIO_ConfigOutput(PB, 1, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 1, FUNC3);
}

/**********************************************************************
 * @brief		TIMER_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_Configure(void)
{
	TIMER3n_CFG_Type TIMER30_Config;
	
// Timer30 clock source select : MCCR2 / PCLK / ECn
#if	1	// Timer30 clock source from MCCR2
	HAL_SCU_Timer30_ClockConfig(T30CLK_MCCR2);	// MCCR2
	HAL_SCU_MiscClockConfig(2, TIMER30_TYPE, CLKSRC_MCLK, 1);
	TIMER30_Config.CLK = T30_CLKINT;
	TIMER30_Config.Prescaler = 48;
	
#elif	0	// Timer30 clock source from PCLK
	HAL_SCU_Timer30_ClockConfig(T30CLK_PCLK);		// PCLK
	TIMER30_Config.CLK = T30_CLKINT;
	TIMER30_Config.Prescaler = 48;
	
#else	//Timer30 clock source from EC2n (PA12)
	// EC30 pin config set
	HAL_GPIO_ConfigOutput(PA, 12, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 12, FUNC3);
	
	TIMER30_Config.CLK = T30_CLKEXT;
	TIMER30_Config.ECE = T30_FEDGE;	
	TIMER30_Config.Prescaler = 48;
#endif
	
	TIMER30_Config.MS = T30_INVM;
	TIMER30_Config.FORCA = T30_6CHMOD;
	TIMER30_Config.UPDT = T30_UPWRITE;
	TIMER30_Config.PMOC = T30_E1PERIOD;

	if (HAL_TIMER3n_Init(TIMER30, &TIMER30_Config) != HAL_OK)
	{
		Error_Handler();
	}

	HAL_TIMER3n_OutputCtrl(TIMER30, ENABLE, T30O_APOHIGH, T30O_BPOLOW);	
	
	HAL_TIMER3n_SetPeriod(TIMER30, 10000);
	HAL_TIMER3n_SetADuty(TIMER30, 7000); 
	HAL_TIMER3n_SetBDuty(TIMER30, 4000);
	HAL_TIMER3n_SetCDuty(TIMER30, 1000);
	
	HAL_TIMER3n_Start(TIMER30, ENABLE);

	while(1)
	{
	}
}

/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void mainloop(void)
{		
	 /*Enable IRQ Interrupts */
	 __enable_irq(); 
	 	
	/*Initialize menu prinf*/
	DEBUG_Print_menu();
	
	/*Initialize port peripheral*/
	GPIO_Configure();
	
	/*Initialize timer peripheral*/
	TIMER_Configure();

	/* Infinite Loop */
	while(1)
	{
	}
}


/**********************************************************************
 * @brief		main
 * @param[in]	None
 * @return 	None
 **********************************************************************/
int main(void)
{
	/* Configure the system clock to 48 MHz */
	SystemClock_Config();

	/* Initialize all port */
	Port_Init(); 
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	return (0);
}


/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif



