/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void TIMER_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void TIMER_Configure( void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
TIMER40_CAPTURECFG_Type TIMER4n_Config;
__IO	uint32_t count;
__IO	uint32_t fflag;
__IO uint32_t i;
const uint8_t menu[] =
"************************************************\n\r"
"TIMER40 Capture Mode\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
"************************************************\n\r";


/**********************************************************************
 * @brief		TIMER_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TIMER_IRQHandler_IT (void)
{
	uint32_t status;

	status = HAL_TIMER4n_GetCaptureInterrupt(TIMER40);
	if (status == 1){
		HAL_TIMER4n_ClearCaptureInterrupt(TIMER40);
		if (fflag == 0)
		{
			fflag=1;
			HAL_GPIO_SetPin(PA,_BIT(10));
		}
		else if (fflag == 1)
		{
			fflag=2;
			HAL_GPIO_ClearPin(PA,_BIT(10));
		}		
	}
}


/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{	
	//Test Pin Input capture signal
	HAL_GPIO_ConfigOutput(PA,0,PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PA,0,DISPUPD);
	HAL_GPIO_ClearPin(PA,_BIT(0));
		
	// TEST Flag Toggle
	HAL_GPIO_ConfigOutput(PA,10,PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PA,10,DISPUPD);
	HAL_GPIO_ClearPin(PA,_BIT(10));
	
	// Test Pin setting T40CAP_CH1
	HAL_GPIO_ConfigFunction(PB, 4, FUNC7);		// T40 OUT Alternative Function 7
	HAL_GPIO_ConfigOutput(PB, 4, ALTERN_FUNC);	// T40CAP_CH1	
}


/**********************************************************************
 * @brief		TIMER_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_Configure(void)
{
	
#if	1	/**Timer40 clock source from MCCR6**/
	HAL_SCU_MiscClockConfig(6, TIMER40_TYPE, CLKSRC_MCLK, 1);
	HAL_SCU_Timer40_ClockConfig(T40CLK_MCCR6);
	TIMER4n_Config.Prescaler = 48;    
	TIMER4n_Config.Prescaler_capture = 1;
	
#elif	0	/**Timer40 clock source from PCLK**/
	HAL_SCU_Timer40_ClockConfig(T40CLK_PCLK);	
	TIMER4n_Config.CkSel = MCCR6PCLK;    
	TIMER4n_Config.Prescaler = 48;    
	TIMER4n_Config.Prescaler_capture = 1;
	
#else	/**Timer40 clock source from ECn**/
	// EC40 pin config set
	HAL_SCU_Timer40_ClockConfig(T40CLK_PCLK); // Use Capture Clock
	
	HAL_GPIO_ConfigOutput(PF, 2, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PF, 2, FUNC7);
	
	TIMER4n_Config.CkSel = EC4n;  
	TIMER4n_Config.ECE = 0;				// 0:falling edge, 1:rising edge	
	TIMER4n_Config.Prescaler = 1;		// not used in ECn mode  
	TIMER4n_Config.Prescaler_capture = 1;	
#endif
	
	
	TIMER4n_Config.TnADR = (0xFFFF);			//	Fclk/((TnADR+1) *2)
	TIMER4n_Config.Channel = TIMER40_CH1;	//	TIMER40_CAPTURE CHANNEL
	TIMER4n_Config.ClrMode = RISING_EGDE;	// 	0 : falling, 1: rising, 2: both
			
	if(HAL_TIMER4n_Init(TIMER40, CAPTURE_MODE, &TIMER4n_Config) != HAL_OK)
	{
		Error_Handler();
	}
	HAL_TIMER4n_ConfigInterrupt(TIMER40, TIMER40_INTCFG_CIE, ENABLE);	// Interrupt Enable
	HAL_TIMER4n_ClearCaptureInterrupt(TIMER40);
	
	NVIC_SetPriority(TIMER40_IRQn,3);
	NVIC_EnableIRQ(TIMER40_IRQn);
	
	HAL_TIMER4n_ClearCounter(TIMER40);		// timer count clear
	HAL_TIMER4n_Cmd(TIMER40, ENABLE);	// timer start & clear
		
	__NOP();
	__NOP();
	
	_DBG("\n\r Wait Pulse! \n\r");
	
	for(i=0;i<100000;i++);
	HAL_GPIO_SetPin(PA,_BIT(0));
	for(i=0;i<1000;i++);
	for(i=0;i<1000;i++);
	for(i=0;i<1000;i++);
	HAL_GPIO_ClearPin(PA,_BIT(0));
	for(i=0;i<1000;i++);	
	for(i=0;i<1000;i++);
	HAL_GPIO_SetPin(PA,_BIT(0));
	for(i=0;i<1000;i++);
	for(i=0;i<1000;i++);
	HAL_GPIO_ClearPin(PA,_BIT(0));
	for(i=0;i<1000;i++);
	
	while(1)
	{
		if(fflag==2)
		{
			count=HAL_TIMER4n_GetCaptureData(TIMER40);
			_DBG("TIMER40_Capture_Data :\t"); _DBD16(HAL_TIMER4n_GetCaptureData(TIMER40));
			fflag=3;
		}
		__NOP();				
	}	
}


/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void mainloop(void)
{
	 /*Enable IRQ Interrupts */
	 __enable_irq(); 
	
	/*Initialize menu prinf*/
	DEBUG_Print_menu();
	
	/*Initialize port peripheral*/
	GPIO_Configure();
	
	/*Initialize timer peripheral*/
	TIMER_Configure();
	 
     /* Infinite Loop */
	while(1)
	{
	}
}


/**********************************************************************
 * @brief		main
 * @param[in]	None
 * @return 	None
 **********************************************************************/
int main(void)
{
	/* Configure the system clock to 48 MHz */
	SystemClock_Config();

	/* Initialize all port */
	Port_Init(); 
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	return (0);
}
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
