/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void TIMER_Configure( void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
TIMER40_PERIODICCFG_Type TIMER4n_Config;
__IO	uint32_t count;
__IO	uint32_t fflag;
const uint8_t menu[] =
"************************************************\n\r"
" Using Timer40 Match Polling \n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
"************************************************\n\r";


/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	/**Match Interrupt Test Pin (PB3) **/
	HAL_GPIO_ConfigOutput(PB,3,PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB,3,DISPUPD);
	HAL_GPIO_ClearPin(PB,_BIT(3));

	/**Timer40 OUT (PB4) **/
	HAL_GPIO_ConfigOutput(PB, 4, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 4, FUNC8);	
}


/**********************************************************************
 * @brief		TIMER_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void TIMER_Configure(void)
{
	TIMER40_PERIODICCFG_Type TIMER4n_Config;
	
#if	0	/**Timer40 clock source from MCCR6**/
	HAL_SCU_MiscClockConfig(6, TIMER40_TYPE, CLKSRC_MCLK, 1);
	HAL_SCU_Timer40_ClockConfig(T40CLK_MCCR6);
	TIMER4n_Config.Prescaler = 48;    // 48Mhz / 48 = 1Mhz ->1us
	
#elif	1	/**Timer40 clock source from PCLK**/
	HAL_SCU_Timer40_ClockConfig(T40CLK_PCLK);		
	TIMER4n_Config.CkSel = MCCR6PCLK;    
	TIMER4n_Config.Prescaler = 48;    // 48Mhz / 48 = 1Mhz ->1us
	
#else	/**Timer40 clock source from ECn**/
	// EC40 pin config set
	HAL_GPIO_ConfigOutput(PF, 2, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PF, 2, FUNC7);
	TIMER4n_Config.CkSel = EC4n;  
	TIMER4n_Config.ECE = 0; // 0:falling edge, 1:rising edge	
	TIMER4n_Config.Prescaler = 1;  // not used in ECn mode  
#endif
		
	TIMER4n_Config.TnADR = (999);	// 1us*(999+1)=1ms
	TIMER4n_Config.StartLevel = START_HIGH;
	
	if (HAL_TIMER4n_Init(TIMER40, PERIODIC_MODE, &TIMER4n_Config) != HAL_OK)
	{
		Error_Handler();
	}

	HAL_TIMER4n_ClearCounter(TIMER40);	// timer count clear
	HAL_TIMER4n_Cmd(TIMER40, ENABLE);	// timer start & clear
	
	while(1)
	{
		while(HAL_TIMER4n_GetMatchInterrupt(TIMER40)!=1);
		HAL_TIMER4n_ClearMatchInterrupt(TIMER40);
		count++;
		if(count>3)
		{
			if(fflag==0)
			{
				HAL_GPIO_SetPin(PB,_BIT(3));
				fflag=1;
			}
			else if(fflag==1)
			{
				HAL_GPIO_ClearPin(PB,_BIT(3));
				fflag=0;
			}
			count=0;
		}			
	}	
}


/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void mainloop(void)
{		
	/*Initialize menu prinf*/
	DEBUG_Print_menu();
	
	/*Initialize port peripheral*/
	GPIO_Configure();
	
	/*Initialize timer peripheral*/
	TIMER_Configure();

	 /*Enable IRQ Interrupts */
	 __enable_irq(); 
	 
      /* Infinite Loop */	
	while(1)
	{
	}
}


/**********************************************************************
 * @brief		main
 * @param[in]	None
 * @return 	None
 **********************************************************************/
int main(void)
{
	/* Configure the system clock to 48 MHz */
	SystemClock_Config();

	/* Initialize all port */
	Port_Init(); 
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	return (0);
}
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif


