/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
//USART10 <-> USART11
#define USART10_M_USART11_S	0
#define USART10_S_USART11_M	0
//USART12 <-> USART13
#define USART12_M_USART13_S	0
#define USART12_S_USART13_M	1

uint8_t _usart10_rx_buffer[255], _usart10_buffer_tail, _usart10_buffer_head;
uint8_t _usart11_rx_buffer[255], _usart11_buffer_tail, _usart11_buffer_head;
uint8_t _usart12_rx_buffer[255], _usart12_buffer_tail, _usart12_buffer_head;
uint8_t _usart13_rx_buffer[255], _usart13_buffer_tail, _usart13_buffer_head;

uint8_t _usart10_tx_flag, _usart10_rx_flag;
uint8_t _usart11_tx_flag, _usart11_rx_flag;
uint8_t _usart12_tx_flag, _usart12_rx_flag;
uint8_t _usart13_tx_flag, _usart13_rx_flag;

/* Private function prototypes -----------------------------------------------*/
void USART10_IRQHandler_IT(void);
void USART11_IRQHandler_IT(void);
void USART13_IRQHandler_IT(void);
void USART_SPI_SS_control(USART_Type *USART1n, char ch);
void USARTSend(USART_Type *USARTx, uint8_t *txbuf, uint8_t buflen);
void USARTReceive(USART_Type *USARTx, uint8_t *rxbuf, uint8_t buflen);
void GPIO_Configure(void);
void SPI_Configure(void);
void SPI_Receive_loop(void);
void USART_SPI_Receive_loop(void);
void UART_DebugMessage(void);
int main (void);
void check_failed(uint8_t* file, uint32_t line);


/* Private variables ---------------------------------------------------------*/
uint8_t tx_ch=0, rx_ch=0;

const uint8_t menu[] =
"************************************************\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" Using SPI Interrupt test \n\r"
"************************************************\n\r";

 USART_CFG_Type USARTConfigStruct;



/*********************************************************************//**
 * @brief		USART10 interrupt handler sub-routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART10_IRQHandler_IT(void)
{
    uint32_t intsrc, tmp;

	/* Determine the interrupt source */
	intsrc = HAL_USART_GetStatus(USART10);
	tmp = intsrc & UST_SR_BITMASK;

	// Receive Data Available or Character time-out
	if ((tmp & UST_SR_RXC) == UST_SR_RXC){
		_usart10_rx_buffer[_usart10_buffer_tail++] = HAL_USART_ReceiveByte(USART10);
		_usart10_rx_flag = 1;
	}

	// Transmit Holding Empty
	if ((tmp & UST_SR_TXC) == UST_SR_TXC){
        HAL_USART_ClearStatus(USART10, USART_STATUS_TXC);
		_usart10_tx_flag = 1;
	}
}

/*********************************************************************//**
 * @brief		USART11 interrupt handler sub-routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART11_IRQHandler_IT(void)
{
	uint32_t intsrc, tmp;

	/* Determine the interrupt source */
	intsrc = HAL_USART_GetStatus(USART11);
	tmp = intsrc & UST_SR_BITMASK;

	// Receive Data Available or Character time-out
	if ((tmp & UST_SR_RXC) == UST_SR_RXC){
        _usart11_rx_buffer[_usart11_buffer_tail++] = HAL_USART_ReceiveByte(USART11);
		_usart11_rx_flag = 1;
	}

	// Transmit Holding Empty
	if ((tmp & UST_SR_TXC) == UST_SR_TXC){
        HAL_USART_ClearStatus(USART11, USART_STATUS_TXC);
		_usart11_tx_flag = 1;
	}
}

/*********************************************************************//**
 * @brief		USART12 interrupt handler sub-routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART12_IRQHandler_IT(void)
{
	uint32_t intsrc, tmp;

	/* Determine the USART12 interrupt source */
	intsrc = HAL_USART_GetStatus(USART12);
	tmp = intsrc & UST_SR_BITMASK;

	// Receive Data Available or Character time-out
	if ((tmp & UST_SR_RXC) == UST_SR_RXC){
		_usart12_rx_flag = 1;
        _usart12_rx_buffer[_usart12_buffer_tail++] = HAL_USART_ReceiveByte(USART12);
	}

	// Transmit Holding Empty
	if ((tmp & UST_SR_TXC) == UST_SR_TXC){
		_usart12_tx_flag = 1;
        HAL_USART_ClearStatus(USART12, USART_STATUS_TXC);
	}
}

/*********************************************************************//**
 * @brief		USART13 interrupt handler sub-routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART13_IRQHandler_IT(void)
{
	uint32_t intsrc, tmp;

	/* Determine the USART13 interrupt source */
	intsrc = HAL_USART_GetStatus(USART13);
	tmp = intsrc & UST_SR_BITMASK;

	// Receive Data Available or Character time-out
	if ((tmp & UST_SR_RXC) == UST_SR_RXC){
		_usart13_rx_flag = 1;
        _usart13_rx_buffer[_usart13_buffer_tail++] = HAL_USART_ReceiveByte(USART13);
	}

	// Transmit Holding Empty
	if ((tmp & UST_SR_TXC) == UST_SR_TXC){
		_usart13_tx_flag = 1;
        HAL_USART_ClearStatus(USART13, USART_STATUS_TXC);
	}
}

/**********************************************************************
 * @brief       Print menu
 * @param[in]   None
 * @return      None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
    #ifdef _DEBUG_MSG
    _DBG(menu);
    #endif
}


/**********************************************************************
 * @brief       DEBUG_Init
 * @param[in]   None
 * @return      None
 **********************************************************************/
void DEBUG_Init(void)
{
    #ifdef _DEBUG_MSG
    debug_frmwrk_init();
    #endif
}


/**********************************************************************
 * @brief		USART_SPI_SS_control
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART_SPI_SS_control(USART_Type *USART1n, char ch)
{
	if (USART1n == USART10){
		if (ch==0) 
			PA->OUTDR &= ~(1<<12);
		else 
			PA->OUTDR |= (1<<12);
	}
	else if (USART1n == USART11){
		if (ch==0) 
			PA->OUTDR &= ~(1<<1);
		else 
			PA->OUTDR |= (1<<1);		
	}
	if (USART1n == USART12){
		if (ch==0) 
			PB->OUTDR &= ~(1<<14);
		else 
			PB->OUTDR |= (1<<14);
	}
	else if (USART1n == USART13){
		if (ch==0) 
			PA->OUTDR &= ~(1<<15);
		else 
			PA->OUTDR |= (1<<15);		
	}
}


/**********************************************************************
 * @brief		USARTSend
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USARTSend(USART_Type *USARTx, uint8_t *txbuf, uint8_t buflen)
{
    uint32_t i, timeout;
    uint8_t *tx_flag;
    
    if(USARTx == USART10){
        tx_flag = &_usart10_tx_flag;
    }else if(USARTx == USART11){
        tx_flag = &_usart11_tx_flag;
    }else if(USARTx == USART12){
        tx_flag = &_usart12_tx_flag;
    }else if(USARTx == USART13){
        tx_flag = &_usart13_tx_flag;
    }
        
    for(i=0;i<buflen;i++){
        timeout = 10000;
        HAL_USART_TransmitByte(USARTx, (txbuf[i]));
        while(timeout--){
            if(*tx_flag) break;
        }
		*tx_flag = 0;
    }
}

/**********************************************************************
 * @brief		USARTReceive
 * @param[in]	None
 * @return	None
 **********************************************************************/
void USARTReceive(USART_Type *USARTx, uint8_t *rxbuf, uint8_t buflen)
{
    uint32_t tmp, i;
    uint8_t *buf, *head, *tail, *flag;
    if(USARTx == USART10){
        buf = _usart10_rx_buffer;
        head = &_usart10_buffer_head;
        tail = &_usart10_buffer_tail;
		flag = &_usart10_rx_flag;
    }else if(USARTx == USART11){
        buf = _usart11_rx_buffer;
        head = &_usart11_buffer_head;
        tail = &_usart11_buffer_tail;
		flag = &_usart11_rx_flag;
    }else if(USARTx == USART12){
        buf = _usart12_rx_buffer;
        head = &_usart12_buffer_head;
        tail = &_usart12_buffer_tail;
		flag = &_usart12_rx_flag;
    }
		else if(USARTx == USART13){
        buf = _usart13_rx_buffer;
        head = &_usart13_buffer_head;
        tail = &_usart13_buffer_tail;
				flag = &_usart13_rx_flag;
    }
		while(*flag != 1){};
    *flag = 0;
    
    if(*head > *tail){
        tmp = *tail+256;
    }else{
        tmp = *tail;
    }
    
    for(i=0;(i<buflen && *head<tmp);i++, (*head)++){
        rxbuf[i] = buf[*head];
    }
}



/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
    // Initialize USART12 pin connect
	
	// MOSI12 (PB10)	 
	HAL_GPIO_ConfigOutput(PB, 10, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 10, FUNC1);
	// MISO12 (PB11)	
	HAL_GPIO_ConfigOutput(PB, 11, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 11, FUNC1);
	//SCK12 (PB12) 
	HAL_GPIO_ConfigOutput(PB, 12, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 12, FUNC1);
	
	#if USART12_M_USART13_S
		//SS12 (PB14)   : master control as GPIO 
		HAL_GPIO_ConfigOutput(PB, 14, PUSH_PULL_OUTPUT);
	#endif
	#if USART12_S_USART13_M
		//SS12 (PB14)   : slave set as SSx 
		HAL_GPIO_ConfigOutput(PB, 14, ALTERN_FUNC);
		HAL_GPIO_ConfigFunction(PB, 14, FUNC1);
	#endif

	// Initialize USART13
	 
	// MOSI12 (PC10)	 
	HAL_GPIO_ConfigOutput(PC, 10, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PC, 10, FUNC1);
	// MISO12 (PC11)	
	HAL_GPIO_ConfigOutput(PC, 11, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PC, 11, FUNC1);
	//SCK12 (PC12) 
	HAL_GPIO_ConfigOutput(PC, 12, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PC, 12, FUNC1);

	#if USART12_M_USART13_S
		//SS13 (PA15)   : slave set as SSx 
		HAL_GPIO_ConfigOutput(PA, 15, ALTERN_FUNC);
		HAL_GPIO_ConfigFunction(PA, 15, FUNC1);
	#endif
	#if USART12_S_USART13_M
		//SS13 (PA15)   : master control as GPIO 
		HAL_GPIO_ConfigOutput(PA, 15, PUSH_PULL_OUTPUT);
	#endif

}

/**********************************************************************
 * @brief       USART SPI_Configure
 * @param[in]   None
 * @return  None
 **********************************************************************/
void SPI_Configure(void)
{
 
 
		HAL_USART_SPI_Mode_Config(&USARTConfigStruct);

		if ( HAL_USART_Init(USART12, &USARTConfigStruct) != HAL_OK)
		{
		   Error_Handler();
		}

		#if USART12_M_USART13_S
		// SPI Master
		HAL_USART_DataControlConfig(USART12, USART_CONTROL_MASTER, ENABLE);
		USART_SPI_SS_control(USART12, 1); // SS init
		#endif
		#if USART12_S_USART13_M
		// SPI slave
		HAL_USART_DataControlConfig(USART12, USART_CONTROL_MASTER, DISABLE);
		// Slave SS Function Enable
		HAL_USART_DataControlConfig(USART12, USART_CONTROL_USTSSEN, ENABLE);
		#endif
		//interrupt
		NVIC_SetPriority(USART12_IRQn, 3);
		NVIC_EnableIRQ(USART12_IRQn);

		HAL_USART_ConfigInterrupt(USART12, USART_INTCFG_TXC, ENABLE);
		HAL_USART_ConfigInterrupt(USART12, USART_INTCFG_RXC, ENABLE);

		// USART13 initialize
		if (HAL_USART_Init(USART13, &USARTConfigStruct) != HAL_OK)
		{
		   Error_Handler();
		}
		#if USART12_M_USART13_S
		// SPI slave
		HAL_USART_DataControlConfig(USART13, USART_CONTROL_MASTER, DISABLE);
		// Slave SS Function Enable
		HAL_USART_DataControlConfig(USART13, USART_CONTROL_USTSSEN, ENABLE);
		#endif
		#if USART12_S_USART13_M
		// SPI Master
		HAL_USART_DataControlConfig(USART13, USART_CONTROL_MASTER, ENABLE);
		USART_SPI_SS_control(USART13, 1); // SS init
		#endif
		//interrupt
		NVIC_SetPriority(USART13_IRQn, 3);
		NVIC_EnableIRQ(USART13_IRQn);

		HAL_USART_ConfigInterrupt(USART13, USART_INTCFG_TXC, ENABLE);
		HAL_USART_ConfigInterrupt(USART13, USART_INTCFG_RXC, ENABLE);
	  
}

/**********************************************************************
 * @brief       SPI_test_loop
 * @param[in]   None
 * @return  None
 **********************************************************************/
void SPI_test_loop(void)
{
		
		_DBG("MASTER SEND DATA ");
		tx_ch = _DG;
		_DBG(" = "); _DBC(tx_ch);_DBG(" : ");_DBH(tx_ch); _DBG_(" ");

		rx_ch = tx_ch-1;
		_DBG("SLAVE SEND DATA = "); _DBC(rx_ch);_DBG(" = ");_DBH(rx_ch); _DBG_(" ");
		#if USART10_M_USART11_S
			USART11->DR = rx_ch;
			USART_SPI_SS_control(USART10, 0);
			USARTSend(USART10, &tx_ch,1);
			USART_SPI_SS_control(USART10, 1);
			USARTReceive(USART10,&tx_ch,1);
			USARTReceive(USART11,&rx_ch,1);
		#endif

		#if USART10_S_USART11_M
			USART10->DR = rx_ch;
			USART_SPI_SS_control(USART11, 0);
			USARTSend(USART11, &tx_ch,1);
			USART_SPI_SS_control(USART11, 1);
			USARTReceive(USART11,&tx_ch,1);
			USARTReceive(USART10,&rx_ch,1);	
		#endif
			
		#if USART12_M_USART13_S
			USART13->DR = rx_ch;
			USART_SPI_SS_control(USART12, 0);
			USARTSend(USART12, &tx_ch,1);
			USART_SPI_SS_control(USART12, 1);
			USARTReceive(USART12,&tx_ch,1);
			USARTReceive(USART13,&rx_ch,1);
		#endif
			
		#if USART12_S_USART13_M
			USART12->DR = rx_ch;
			USART_SPI_SS_control(USART13, 0);
			USARTSend(USART13, &tx_ch,1);
			USART_SPI_SS_control(USART13, 1);
			USARTReceive(USART13,&tx_ch,1);
			USARTReceive(USART12,&rx_ch,1);
		#endif

		_DBG("MASTER GET DATA = "); _DBC(tx_ch);_DBG(" = ");_DBH(tx_ch); _DBG_(" ");
		_DBG("SLAVE GET DATA = "); _DBC(rx_ch);_DBG(" = ");_DBH(rx_ch); _DBG_(" ");_DBG_(" ");
}

/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{

    /*Initialize menu prinf*/
    DEBUG_Print_menu();
    
    /*Initialize port peripheral*/
    GPIO_Configure();
    
    /*Initialize ADC peripheral*/
    SPI_Configure();

    /*Enable IRQ Interrupts */
	 __enable_irq(); 
	 
   /* Infinite Loop */
    while(1)
    {
        SPI_test_loop();
    }
}


/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{

	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	 DEBUG_Init();  
	
	/* Infinite loop */
	mainloop();  
	

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif



