/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
//USART10 <-> USART11
#define USART10_M_USART11_S	0
#define USART10_S_USART11_M	0
//USART12 <-> USART13
#define USART12_M_USART13_S	0
#define USART12_S_USART13_M	1


/* Private function prototypes -----------------------------------------------*/
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void USART_SPI_SS_control(USART_Type *USART1n, char ch);
void GPIO_Configure(void);
void SPI_Configure(void);
void UART_DebugMessage(void);
void SPI_Receive_loop(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint8_t tx_ch=0, rx_ch=0;

const uint8_t menu[] =
"************************************************\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" Using SPI Polling test \n\r"
"************************************************\n\r";
USART_CFG_Type USARTConfigStruct;


/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		USART_SPI_SS_control
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART_SPI_SS_control(USART_Type *USART1n, char ch)
{
	if (USART1n == USART10){
		if (ch==0) 
			PA->OUTDR &= ~(1<<12);
		else 
			PA->OUTDR |= (1<<12);
	}
	else if (USART1n == USART11){
		if (ch==0) 
			PA->OUTDR &= ~(1<<1);
		else 
			PA->OUTDR |= (1<<1);		
	}
	else if (USART1n == USART12){
		if (ch==0) 
			PB->OUTDR &= ~(1<<14);
		else 
			PB->OUTDR |= (1<<14);
	}
	else if (USART1n == USART13){
		if (ch==0) 
			PA->OUTDR &= ~(1<<15);
		else 
			PA->OUTDR |= (1<<15);		
	}
}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Initialize USART12 pin connect
	
	// MOSI12 (PB10)	 
	HAL_GPIO_ConfigOutput(PB, 10, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 10, FUNC1);
	// MISO12 (PB11)	
	HAL_GPIO_ConfigOutput(PB, 11, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 11, FUNC1);
	//SCK12 (PB12) 
	HAL_GPIO_ConfigOutput(PB, 12, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 12, FUNC1);
	
	#if USART12_M_USART13_S
  //SS12 (PB14)   : master control as GPIO 
	HAL_GPIO_ConfigOutput(PB, 14, PUSH_PULL_OUTPUT);
	#endif
	#if USART12_S_USART13_M
	//SS12 (PB14)   : slave set as SSx 
	HAL_GPIO_ConfigOutput(PB, 14, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 14, FUNC1);
	#endif
	

	// Initialize USART13
	 
	// MOSI13 (PC10)	 
	HAL_GPIO_ConfigOutput(PC, 10, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PC, 10, FUNC1);
	// MISO13 (PC11)	
	HAL_GPIO_ConfigOutput(PC, 11, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PC, 11, FUNC1);
	//SCK13 (PC12) 
	HAL_GPIO_ConfigOutput(PC, 12, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PC, 12, FUNC1);

	#if USART12_M_USART13_S
	//SS13 (PA15)   : slave set as SSx 
	HAL_GPIO_ConfigOutput(PA, 15, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 15, FUNC1);
	#endif
	#if USART12_S_USART13_M
  //SS13 (PA15)   : master control as GPIO 
	HAL_GPIO_ConfigOutput(PA, 15, PUSH_PULL_OUTPUT);
	#endif
	

}


/**********************************************************************
 * @brief		SPI_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SPI_Configure(void)
{
     HAL_USART_SPI_Mode_Config(&USARTConfigStruct);
	
  		// USART12 initialize
	if ( HAL_USART_Init(USART12, &USARTConfigStruct) != HAL_OK )
	{
		 Error_Handler();
	}
	   	
	#if USART12_M_USART13_S
	// SPI Master
	HAL_USART_DataControlConfig(USART12, USART_CONTROL_MASTER, ENABLE);
	USART_SPI_SS_control(USART12, 1); // SS init
	#endif
	#if USART12_S_USART13_M
	// SPI slave
	HAL_USART_DataControlConfig(USART12, USART_CONTROL_MASTER, DISABLE);
	// Slave SS Function Enable
	HAL_USART_DataControlConfig(USART12, USART_CONTROL_USTSSEN, ENABLE);
	#endif

	// USART13 initialize
	if ( HAL_USART_Init(USART13, &USARTConfigStruct) != HAL_OK )
	{
		 Error_Handler();
	}
	
	#if USART12_M_USART13_S
	// SPI slave
	HAL_USART_DataControlConfig(USART13, USART_CONTROL_MASTER, DISABLE);
	// Slave SS Function Enable
	HAL_USART_DataControlConfig(USART13, USART_CONTROL_USTSSEN, ENABLE);
	#endif
	#if USART12_S_USART13_M
	// SPI Master
	HAL_USART_DataControlConfig(USART13, USART_CONTROL_MASTER, ENABLE);
	USART_SPI_SS_control(USART13, 1); // SS init
	#endif
}


/**********************************************************************
 * @brief		UART_DebugMessage
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_DebugMessage(void)
{
	#ifdef _DEBUG_MSG
	_DBG_(">> USART SPI TEST.............. ");
	#endif
}


/**********************************************************************
 * @brief		SPI_Receive_loop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SPI_Receive_loop(void)
{

		_DBG("MASTER SEND DATA ");
		tx_ch = _DG;
		_DBG(" = "); _DBC(tx_ch);_DBG(" : ");_DBH(tx_ch); _DBG_(" ");

		rx_ch = tx_ch-1;
		_DBG("SLAVE SEND DATA = "); _DBC(rx_ch);_DBG(" = ");_DBH(rx_ch); _DBG_(" ");

		#if USART10_M_USART11_S
		HAL_USART_TransmitByte(USART11, rx_ch); 
		USART_SPI_SS_control(USART10, 0);
		HAL_USART_Transmit(USART10, &tx_ch,1);
		while(!(HAL_USART_GetStatus(USART10)&UST_SR_TXC)){} // wait 
		//while(!(HAL_USART_GetStatus(USART10) & UST_SR_RXC)){} // wait 			
		tx_ch = HAL_USART_ReceiveByte(UART10);
		HAL_USART_ClearStatus(USART10, USART_STATUS_TXC);			
		USART_SPI_SS_control(USART10, 1);
		HAL_USART_Receive(USART11,&rx_ch,1);
		#endif

		#if USART10_S_USART11_M
		HAL_USART_TransmitByte(USART10, rx_ch); 
		USART_SPI_SS_control(USART11, 0);
		HAL_USART_Transmit(USART11, &tx_ch,1);
		while(!(HAL_USART_GetStatus(USART11)&UST_SR_TXC)){} // wait
		//while(!(HAL_USART_GetStatus(USART11) & UST_SR_RXC)){} // wait 			
		tx_ch = HAL_USART_ReceiveByte(UART11);
		HAL_USART_ClearStatus(USART11, USART_STATUS_TXC);		
		USART_SPI_SS_control(USART11, 1);
		HAL_USART_Receive(USART10,&rx_ch,1);
		#endif
			
		#if USART12_M_USART13_S
		HAL_USART_TransmitByte(USART13, rx_ch); 
		USART_SPI_SS_control(USART12, 0);			
		HAL_USART_Transmit(USART12, &tx_ch,1);
		while(!(HAL_USART_GetStatus(USART12) & UST_SR_TXC)){} // wait 
		//while(!(HAL_USART_GetStatus(USART12) & UST_SR_RXC)){} // wait 
		tx_ch = HAL_USART_ReceiveByte(UART12);
		HAL_USART_ClearStatus(USART12, USART_STATUS_TXC);		
		USART_SPI_SS_control(USART12, 1);
		HAL_USART_Receive(USART13,&rx_ch,1);

		#endif
			
		#if USART12_S_USART13_M
		HAL_USART_TransmitByte(USART12, rx_ch); 
		USART_SPI_SS_control(USART13, 0);						
		HAL_USART_Transmit(USART13, &tx_ch,1);
		while(!(HAL_USART_GetStatus(USART13) & UST_SR_TXC)){} // wait 
		//while(!(HAL_USART_GetStatus(USART13) & UST_SR_RXC)){} // wait 			
		tx_ch = HAL_USART_ReceiveByte(USART13);
		HAL_USART_ClearStatus(USART13, USART_STATUS_TXC);		
		USART_SPI_SS_control(USART13, 1);
		HAL_USART_Receive(USART12,&rx_ch,1);
		#endif

		_DBG("MASTER GET DATA = "); _DBC(tx_ch);_DBG(" = ");_DBH(tx_ch); _DBG_(" ");
		_DBG("SLAVE GET DATA = "); _DBC(rx_ch);_DBG(" = ");_DBH(rx_ch); _DBG_(" ");_DBG_(" ");

}

/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void mainloop(void)
{
    /*Initialize menu prinf*/
    DEBUG_Print_menu();
    
    /*Initialize port peripheral*/
    GPIO_Configure();
    
    /*Initialize ADC peripheral*/
    SPI_Configure();

   /*Configure UART DEBUG peripheral*/
   UART_DebugMessage();

   /*Enable IRQ Interrupts */
   __enable_irq(); 
	 
   /* Infinite Loop */
    while(1)
    {
        SPI_Receive_loop();
    }
}

/**********************************************************************
 * @brief		main
 * @param[in]	None
 * @return 	None
 **********************************************************************/
int main(void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	 DEBUG_Init();  
	
	/* Infinite loop */
	mainloop();  
	

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
