/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define USART1x USART10
//#define USART1x USART11
//#define USART1x USART12
//#define USART1x USART13

#define USART1x_IRQn	USART10_IRQn
//#define USART1x_IRQn	USART11_IRQn
//#define USART1x_IRQn	USART12_IRQn
//#define USART1x_IRQn	USART13_IRQn

//-------------------------
// USART SELECTION
//-------------------------
#define DMACx	DMAC0
//#define DMACx	DMAC1
//#define DMACx	DMAC2
//#define DMACx	DMAC3

#define usart_size	10
/* Private function prototypes -----------------------------------------------*/
void USART10_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void UART_Configure(void);
void DMAC_Configure(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" Uart DMA test \n\r"
"************************************************\n\r";

uint32_t usart_data[usart_size] = {'\n', 'D', 'M', 'A', '_', 'D', 'A', 'T', 'A', '\n'};

USART_CFG_Type USARTConfigStruct;	
DMAC_Ch_CFG_Type DMACU1Tx;
/**********************************************************************
 * @brief		USART10_IRQHandler
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART10_IRQHandler_IT(void)
{
	uint32_t status, temp;
	status = HAL_USART_GetStatus(USART10);
	temp = status & UST_SR_BITMASK;
	
	if((temp & UST_SR_DMARX) == UST_SR_DMARX){
		HAL_USART_ClearStatus(USART10, USART_STATUS_DMARX);
	}
	else if((temp & UST_SR_DMATX) == UST_SR_DMATX){
			HAL_USART_ClearStatus(USART10, USART_STATUS_DMATX);
	}
}

/**********************************************************************
 * @brief		USART11_IRQHandler
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART11_IRQHandler_IT(void)
{
	uint32_t status, temp;
	status = HAL_USART_GetStatus(USART11);
	temp = status & UST_SR_BITMASK;
	
	if((temp & UST_SR_DMARX) == UST_SR_DMARX){
		HAL_USART_ClearStatus(USART11, USART_STATUS_DMARX);
	}
	else if((temp & UST_SR_DMATX) == UST_SR_DMATX){
		HAL_USART_ClearStatus(USART11, USART_STATUS_DMATX);
	}
}

/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART12_IRQHandler_IT(void)
{
	uint32_t status, temp;
	status = HAL_USART_GetStatus(USART12);
	temp = status & UST_SR_BITMASK;
	
	if((temp & UST_SR_DMARX) == UST_SR_DMARX){
		HAL_USART_ClearStatus(USART12, USART_STATUS_DMARX);
	}
	else if((temp & UST_SR_DMATX) == UST_SR_DMATX){
		HAL_USART_ClearStatus(USART12, USART_STATUS_DMATX);
	}
}

/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART13_IRQHandler_IT(void)
{
	uint32_t status, temp;
	status = HAL_USART_GetStatus(USART13);
	temp = status & UST_SR_BITMASK;
	
	if((temp & UST_SR_DMARX) == UST_SR_DMARX){
		HAL_USART_ClearStatus(USART13, USART_STATUS_DMARX);
	}
	else if((temp & UST_SR_DMATX) == UST_SR_DMATX){
		HAL_USART_ClearStatus(USART13, USART_STATUS_DMATX);
	}
}

/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}



/**********************************************************************
 * @brief		UART_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_Configure(void)
{
// default: 38400-8-N-1	
	USARTConfigStruct.Mode = USART_UART_MODE;
	USARTConfigStruct.Baud_rate = 38400;
	USARTConfigStruct.Databits = USART_DATABIT_8;
	USARTConfigStruct.Parity = USART_PARITY_NONE;
	USARTConfigStruct.Stopbits = USART_STOPBIT_1;
		
	if ( HAL_USART_Init(USART1x, &USARTConfigStruct)!= HAL_OK)
	{
	   Error_Handler();
	}
	
	NVIC_SetPriority(USART1x_IRQn, 3);
	NVIC_EnableIRQ(USART1x_IRQn);		

	HAL_USART_Enable(USART1x, ENABLE);
	
	HAL_USART_ConfigInterrupt(USART1x, USART_INTCFG_DMATX, ENABLE);
}

/**********************************************************************
 * @brief		DMAC_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void DMAC_Configure(void)
{
	if ( HAL_DMAC_Init()!= HAL_OK )
	{
		Error_Handler();
	}
	DMACU1Tx.TransferCnt = usart_size;
	DMACU1Tx.PeriSel = DMAC_PERISEL_USART10TX;
	DMACU1Tx.TransferWidth = DMAC_WIDTH_WORD;
	DMACU1Tx.Dir = DMAC_DIR_TX;	
	DMACU1Tx.PeriAddr = USART10_BASE+0x14;
	DMACU1Tx.MemAddr = (uint32_t)(&usart_data[0]);	
	
	HAL_DMAC_Setup(DMACx, &DMACU1Tx);
}

/**********************************************************************
 * @brief		UART_DMA_test
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_DMA_test(void)
{
	uint8_t ch = 0;

	while(1)
	{
		_DBG("\r\nPress Enter UART DMA test\r\n");
		
		HAL_USART_Receive(USART1x, &ch, 1);
		if(ch == 0x0d || ch == 0x20)			// ENTER or SPACE
		{
				DMACU1Tx.TransferCnt = usart_size;
				DMACU1Tx.PeriSel = DMAC_PERISEL_USART10TX;
				DMACU1Tx.TransferWidth = DMAC_WIDTH_WORD;
				DMACU1Tx.Dir = DMAC_DIR_TX;	
				DMACU1Tx.PeriAddr = USART10_BASE+20;
				DMACU1Tx.MemAddr = (uint32_t)(&usart_data[0]);	
				
				HAL_DMAC_Setup(DMACx, &DMACU1Tx);
			
				HAL_DMAC_ChCmd(DMACx, ENABLE);
				
				while (HAL_DMAC_GetState(DMACx) == RESET) ;

				HAL_DMAC_ChCmd(DMACx, DISABLE);
		}
	}
}

/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void mainloop(void)
{
    /*Initialize menu prinf*/
    DEBUG_Print_menu();
    
    /*Initialize UART peripheral*/
    UART_Configure();

   /*Initialize DMAC peripheral*/
    DMAC_Configure();

     /*Enable IRQ Interrupts */
     __enable_irq(); 
   /* Infinite Loop */
    while(1)
    {
			UART_DMA_test();
    }
}
  
/**********************************************************************
 * @brief		main
 * @param[in]	None
 * @return 		None
 **********************************************************************/
int main(void)
{

     /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

