/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define DMA_USART
//-------------------------
// USART SELECTION
//-------------------------
#define USART1x	USART10
//#define USART1x	USART11
//#define USART1x	USART12
//#define USART1x	USART13

#define USART1x_IRQn	USART10_IRQn
//#define USART1x_IRQn	USART11_IRQn
//#define USART1x_IRQn	USART12_IRQn
//#define USART1x_IRQn	USART13_IRQn

/* Private macro -------------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void USART10_IRQHandler_IT(void);
void USART11_IRQHandler_IT(void);
void USART12_IRQHandler_IT(void);
void USART13_IRQHandler_IT(void);
void DEBUG_Init(void);
void UART_DebugMessage(void);
void USARTSend(USART_Type *USARTx, uint8_t *txbuf, uint8_t buflen);
void USARTReceive(USART_Type *USARTx, uint8_t *rxbuf, uint8_t buflen);
void UART_Configure(void);
void USART_ReceiveSend_loop (void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
// Current Tx Interrupt enable state
__IO FlagStatus TxIntStat;

uint8_t _usart10_rx_buffer[255], _usart10_buffer_tail, _usart10_buffer_head;
uint8_t _usart11_rx_buffer[255], _usart11_buffer_tail, _usart11_buffer_head;
uint8_t _usart12_rx_buffer[255], _usart12_buffer_tail, _usart12_buffer_head;
uint8_t _usart13_rx_buffer[255], _usart13_buffer_tail, _usart13_buffer_head;

uint8_t _usart10_tx_flag, _usart10_rx_flag;
uint8_t _usart11_tx_flag, _usart11_rx_flag;
uint8_t _usart12_tx_flag, _usart12_rx_flag;
uint8_t _usart13_tx_flag, _usart13_rx_flag;

const uint8_t menu[] =
"************************************************\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" USART Uart Interrupt test \n\r"
"************************************************\n\r";

USART_CFG_Type USARTConfigStruct;
/**********************************************************************
 * @brief		USART10_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART10_IRQHandler_IT(void)
{
	uint32_t intsrc, tmp;

	/* Determine the interrupt source */
	intsrc = HAL_USART_GetStatus(USART10);
	tmp = intsrc & UST_SR_BITMASK;

	// Receive Data Available or Character time-out
	if ((tmp & UST_SR_RXC) == UST_SR_RXC){
        _usart10_rx_buffer[_usart10_buffer_tail++] = HAL_USART_ReceiveByte(USART10);
		_usart10_rx_flag = 1;
	}

	// Transmit Holding Empty
	if ((tmp & UST_SR_TXC) == UST_SR_TXC){
        HAL_USART_ClearStatus(USART10, USART_STATUS_TXC);
		_usart10_tx_flag = 1;
	}
}
	
/*********************************************************************//**
 * @brief		USART11 interrupt handler sub-routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART11_IRQHandler_IT(void)
{
	uint32_t intsrc, tmp;

	/* Determine the interrupt source */
	intsrc = HAL_USART_GetStatus(USART11);
	tmp = intsrc & UST_SR_BITMASK;

	// Receive Data Available or Character time-out
	if ((tmp & UST_SR_RXC) == UST_SR_RXC){
        _usart11_rx_buffer[_usart11_buffer_tail++] = HAL_USART_ReceiveByte(USART11);
		_usart11_rx_flag = 1;
	}

	// Transmit Holding Empty
	if ((tmp & UST_SR_TXC) == UST_SR_TXC){
        HAL_USART_ClearStatus(USART11, USART_STATUS_TXC);
		_usart11_tx_flag = 1;
	}
}

/*********************************************************************//**
 * @brief		USART12 interrupt handler sub-routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART12_IRQHandler_IT(void)
{
	uint32_t intsrc, tmp;

	/* Determine the interrupt source */
	intsrc = HAL_USART_GetStatus(USART12);
	tmp = intsrc & UST_SR_BITMASK;

	// Receive Data Available or Character time-out
	if ((tmp & UST_SR_RXC) == UST_SR_RXC){
        _usart12_rx_buffer[_usart12_buffer_tail++] = HAL_USART_ReceiveByte(USART12);
		_usart12_rx_flag = 1;
	}

	// Transmit Holding Empty
	if ((tmp & UST_SR_TXC) == UST_SR_TXC){
        HAL_USART_ClearStatus(USART12, USART_STATUS_TXC);
		_usart12_tx_flag = 1;
	}
}

/*********************************************************************//**
 * @brief		USART13 interrupt handler sub-routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART13_IRQHandler_IT(void)
{
	uint32_t intsrc, tmp;

	/* Determine the interrupt source */
	intsrc = HAL_USART_GetStatus(USART13);
	tmp = intsrc & UST_SR_BITMASK;

	// Receive Data Available or Character time-out
	if ((tmp & UST_SR_RXC) == UST_SR_RXC){
        _usart13_rx_buffer[_usart13_buffer_tail++] = HAL_USART_ReceiveByte(USART13);
		_usart13_rx_flag = 1;
	}

	// Transmit Holding Empty
	if ((tmp & UST_SR_TXC) == UST_SR_TXC){
        HAL_USART_ClearStatus(USART13, USART_STATUS_TXC);
		_usart13_tx_flag = 1;
	}
}


/**********************************************************************
 * @brief       DEBUG_Print_menu
 * @param[in]   None
 * @return      None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
    #ifdef _DEBUG_MSG
    _DBG(menu);
    #endif
}


/**********************************************************************
 * @brief       DEBUG_Init
 * @param[in]   None
 * @return      None
 **********************************************************************/
void DEBUG_Init(void)
{
    #ifdef _DEBUG_MSG
    debug_frmwrk_init();
    #endif
}
/**********************************************************************
 * @brief		USARTSend
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USARTSend(USART_Type *USARTx, uint8_t *txbuf, uint8_t buflen)
{
    uint32_t i, timeout;
    uint8_t *tx_flag;
    
    if(USARTx == USART10){
        tx_flag = &_usart10_tx_flag;
    }else if(USARTx == USART11){
        tx_flag = &_usart11_tx_flag;
	}else if(USARTx == USART12){
        tx_flag = &_usart12_tx_flag;
    }else if(USARTx == USART13){
        tx_flag = &_usart13_tx_flag;
    }
 		while(!(USARTx->ST & UST_SR_DRE)){ }
    
    for(i=0;i<buflen;i++){
        timeout = 10000;
        HAL_USART_TransmitByte(USARTx, (txbuf[i]));
        while(timeout--){
            if(*tx_flag) break;
        }
		*tx_flag = 0;
    }
}

/**********************************************************************
 * @brief		USARTReceive
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USARTReceive(USART_Type *USARTx, uint8_t *rxbuf, uint8_t buflen)
{
    uint32_t tmp, i;
    uint8_t *buf, *head, *tail, *flag;
    if(USARTx == USART10){
        buf = _usart10_rx_buffer;
        head = &_usart10_buffer_head;
        tail = &_usart10_buffer_tail;
		flag = &_usart10_rx_flag;
    }else if(USARTx == USART11){
        buf = _usart11_rx_buffer;
        head = &_usart11_buffer_head;
        tail = &_usart11_buffer_tail;
		flag = &_usart11_rx_flag;
    }else if(USARTx == USART12){
        buf = _usart12_rx_buffer;
        head = &_usart12_buffer_head;
        tail = &_usart12_buffer_tail;
		flag = &_usart12_rx_flag;
    }else if(USARTx == USART13){
        buf = _usart13_rx_buffer;
        head = &_usart13_buffer_head;
        tail = &_usart13_buffer_tail;
		flag = &_usart13_rx_flag;
    }
	while(*flag != 1){}
    *flag = 0;
	
    if(*head > *tail){
        tmp = *tail+256;
    }else{
        tmp = *tail;
    }
    
    for(i=0;(i<buflen && *head<tmp);i++, (*head)++){
        rxbuf[i] = buf[*head];
    }
}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_Configure(void)
{
	uint8_t ch;
   // default: 38400-8-N-1	
	HAL_USART_UART_Mode_Config(&USARTConfigStruct);	

	USARTConfigStruct.Mode = USART_UART_MODE;
	USARTConfigStruct.Baud_rate = 38400;
	USARTConfigStruct.Databits = USART_DATABIT_8;
	USARTConfigStruct.Parity = USART_PARITY_NONE;
	USARTConfigStruct.Stopbits = USART_STOPBIT_1;
		
	if (HAL_USART_Init(USART1x, &USARTConfigStruct) != HAL_OK)
	{
		Error_Handler();
	}

  /* Enable USART interrupt */
	HAL_USART_ConfigInterrupt((USART_Type *)USART1x, USART_INTCFG_RXC, ENABLE);
	HAL_USART_ConfigInterrupt((USART_Type *)USART1x, USART_INTCFG_TXC, ENABLE);
	
	NVIC_SetPriority(USART1x_IRQn, 3);
	NVIC_EnableIRQ(USART1x_IRQn);
	
#if 1
	__enable_irq();
	_DBG_("\r\n>> USART UART TEST..............START : ENTER...");
    //	USARTSend(USART1x,(uint8_t *)"Interrupt test start\n\r",22);
	USARTReceive(USART1x, &ch, 1);
	while(!(ch == 0x0d))  
	{
		USARTReceive(USART1x, &ch, 1);
	}
	USARTSend(USART1x,(uint8_t *)"Interrupt test start\n\r",22);
#endif
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void USART_ReceiveSend_loop (void)
{
	uint8_t ch;
	
	USARTReceive(USART1x, &ch, 1);
	if(ch == 0x0d || ch == 0x20)			// ENTER or SPACE
	{
		USARTSend(USART1x, (uint8_t *)"Hello A31G324\n\r", 16);
	}
	else
	{	
		USARTSend(USART1x, &ch, 1);
	}

}
/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void mainloop(void)
{
	
	/*Initialize menu prinf*/
	DEBUG_Print_menu();
	
	/*Initialize UART peripheral*/
	UART_Configure();

	 /*Enable IRQ Interrupts */
	 __enable_irq(); 
		 
      /* Infinite Loop */
	while(1)
	{
		USART_ReceiveSend_loop();
	}
 
}

/**********************************************************************
 * @brief		main
 * @param[in]	None
 * @return 		None
 **********************************************************************/
int main(void)
{

	/* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();

	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();

	/* Infinite loop */
	mainloop();   


	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

