/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define DMA_USART
#define USART1x USART10
//#define USART1x USART11
//#define USART1x USART12
//#define USART1x USART13

#define USART1x_IRQn	USART10_IRQn
//#define USART1x_IRQn	USART11_IRQn
//#define USART1x_IRQn	USART12_IRQn
//#define USART1x_IRQn	USART13_IRQn
/* Private macro -------------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void USART10_IRQHandler_IT(void);
void USART11_IRQHandler_IT(void);
void USART12_IRQHandler_IT(void);
void USART13_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void UART_Configure(void);
void UART_ReceiveSend_loop(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" Uart RTO test \n\r"
"************************************************\n\r";

USART_CFG_Type USARTConfigStruct;
/**********************************************************************
 * @brief		USART10_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART10_IRQHandler_IT(void)
{
	uint32_t intsrc, tmp;

	/* Determine the interrupt source */
	intsrc = HAL_USART_GetStatus(USART10);
	tmp = intsrc & UST_SR_BITMASK;

	if ((tmp & UST_SR_RTO) == UST_SR_RTO){
		_DBG("\r\n RTO interrupt EVENT!!\r\n");
		HAL_USART_ClearStatus(USART10, USART_STATUS_RTO);
	}
}

/*********************************************************************//**
 * @brief		USART11 interrupt handler sub-routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART11_IRQHandler_IT(void)
{
	uint32_t intsrc, tmp;

	/* Determine the interrupt source */
	intsrc = HAL_USART_GetStatus(USART11);
	tmp = intsrc & UST_SR_BITMASK;

	if ((tmp & UST_SR_RTO) == UST_SR_RTO){
		HAL_USART_ClearStatus(USART11, USART_STATUS_RTO);
	}
}

/*********************************************************************//**
 * @brief		USART12 interrupt handler sub-routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART12_IRQHandler_IT(void)
{
	uint32_t intsrc, tmp;

	/* Determine the interrupt source */
	intsrc = HAL_USART_GetStatus(USART12);
	tmp = intsrc & UST_SR_BITMASK;

	if ((tmp & UST_SR_RTO) == UST_SR_RTO){
		HAL_USART_ClearStatus(USART12, USART_STATUS_RTO);
	}
}


/*********************************************************************//**
 * @brief		USART13 interrupt handler sub-routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART13_IRQHandler_IT(void)
{
	uint32_t intsrc, tmp;

	/* Determine the interrupt source */
	intsrc = HAL_USART_GetStatus(USART13);
	tmp = intsrc & UST_SR_BITMASK;

	if ((tmp & UST_SR_RTO) == UST_SR_RTO){
		HAL_USART_ClearStatus(USART13, USART_STATUS_RTO);
	}
}

/**********************************************************************
 * @brief		DEBUG_Print_menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		USART_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_Configure(void)
{
// default: 38400-8-N-1	
	USARTConfigStruct.Mode = USART_UART_MODE;
	USARTConfigStruct.Baud_rate = 38400;
	USARTConfigStruct.Databits = USART_DATABIT_8;
	USARTConfigStruct.Parity = USART_PARITY_NONE;
	USARTConfigStruct.Stopbits = USART_STOPBIT_1;
		
	if ( HAL_USART_Init(USART1x, &USARTConfigStruct)!= HAL_OK )
	{
		Error_Handler();
	}

	
	NVIC_SetPriority(USART1x_IRQn, 3);
	NVIC_EnableIRQ(USART1x_IRQn);	

	HAL_USART_Enable(USART1x, ENABLE);

	HAL_USART_Transmit(USART1x,(uint8_t *)"\r\nPolling test\n\r",15);
	HAL_USART_RTOEnable(USART1x, ENABLE);
	HAL_USART_ConfigInterrupt(USART1x, USART_INTCFG_RTO, ENABLE);
}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_ReceiveSend_loop(void)
{
	uint8_t ch;
	
	HAL_USART_Receive(USART1x, &ch, 1);
	if(ch == 0x0d || ch == 0x20)			// ENTER or SPACE
	{
		HAL_USART_Transmit(USART1x, (uint8_t *)"Hello A31G324\n\r", 16);
	}
	else
	{	
		HAL_USART_Transmit(USART1x, &ch, 1);
	}
}


/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void mainloop(void)
{

    /*Initialize menu prinf*/
    DEBUG_Print_menu();
    
    /*Initialize UART peripheral*/
    UART_Configure();
   
	/*Enable IRQ Interrupts */
	 __enable_irq(); 
	 
   /* Infinite Loop */
    while(1)
    {
      UART_ReceiveSend_loop();
    }
}

/**********************************************************************
 * @brief		main
 * @param[in]	None
 * @return 		None
 **********************************************************************/
int main(void)
{

     /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

