/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
//USART10 <-> USART11
#define USART10_M_USART11_S	0
#define USART10_S_USART11_M	0
//USART12 <-> USART13
#define USART12_M_USART13_S	0
#define USART12_S_USART13_M	1
/* Private macro -------------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void USART10_IRQHandler_IT(void);
void USART11_IRQHandler_IT(void);
void USART12_IRQHandler_IT(void);
void USART13_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void USARTSend(USART_Type *USARTx, uint8_t *txbuf, uint8_t buflen);
void USARTReceive(USART_Type *USARTx, uint8_t *rxbuf, uint8_t buflen);
void USART_Configure(void);
void UART_DebugMessage(void);
void USART_ReceiveSend_loop(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
" USART interrupt demo \n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" USART Interrupt test \n\r"
"************************************************\n\r";
uint8_t _usart10_rx_buffer[255], _usart10_buffer_tail, _usart10_buffer_head;
uint8_t _usart11_rx_buffer[255], _usart11_buffer_tail, _usart11_buffer_head;
uint8_t _usart12_rx_buffer[255], _usart12_buffer_tail, _usart12_buffer_head;
uint8_t _usart13_rx_buffer[255], _usart13_buffer_tail, _usart13_buffer_head;

uint8_t _usart10_tx_flag, _usart10_rx_flag;
uint8_t _usart11_tx_flag, _usart11_rx_flag;
uint8_t _usart12_tx_flag, _usart12_rx_flag;
uint8_t _usart13_tx_flag, _usart13_rx_flag;
uint8_t tx_ch=0, rx_ch=0;
USART_CFG_Type USARTConfigStruct;

/**********************************************************************
 * @brief		USART10_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART10_IRQHandler_IT(void)
{
    uint32_t intsrc, tmp;

	/* Determine the interrupt source */
	intsrc = HAL_USART_GetStatus(USART10);
	tmp = intsrc & UST_SR_BITMASK;

	// Receive Data Available or Character time-out
	if ((tmp & UST_SR_RXC) == UST_SR_RXC){
		_usart10_rx_buffer[_usart10_buffer_tail++] = HAL_USART_ReceiveByte(USART10);
		_usart10_rx_flag = 1;
	}

	// Transmit Holding Empty
	if ((tmp & UST_SR_TXC) == UST_SR_TXC){
        HAL_USART_ClearStatus(USART10, USART_STATUS_TXC);
		_usart10_tx_flag = 1;
	}
}

/**********************************************************************
 * @brief		USART11_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART11_IRQHandler_IT(void)
{
	uint32_t intsrc, tmp;

	/* Determine the interrupt source */
	intsrc = HAL_USART_GetStatus(USART11);
	tmp = intsrc & UST_SR_BITMASK;

	// Receive Data Available or Character time-out
	if ((tmp & UST_SR_RXC) == UST_SR_RXC){
        _usart11_rx_buffer[_usart11_buffer_tail++] = HAL_USART_ReceiveByte(USART11);
		_usart11_rx_flag = 1;
	}

	// Transmit Holding Empty
	if ((tmp & UST_SR_TXC) == UST_SR_TXC){
        HAL_USART_ClearStatus(USART11, USART_STATUS_TXC);
		_usart11_tx_flag = 1;
	}
}

/*********************************************************************//**
 * @brief		USART12 interrupt handler sub-routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART12_IRQHandler_IT(void)
{
	uint32_t intsrc, tmp;

	/* Determine the UST12 interrupt source */
	intsrc = HAL_USART_GetStatus(USART12);
	tmp = intsrc & UST_SR_BITMASK;

	// Receive Data Available or Character time-out
	if ((tmp & UST_SR_RXC) == UST_SR_RXC)
	{
		_usart12_rx_buffer[_usart12_buffer_tail++] = HAL_USART_ReceiveByte(USART12);
		_usart12_rx_flag = 1;
	}

	// Transmit Holding Empty
	if ((tmp & UST_SR_TXC) == UST_SR_TXC)
	{
		HAL_USART_ClearStatus(USART12, USART_STATUS_TXC);
		_usart12_tx_flag = 1;
	}
    
}

/*********************************************************************//**
 * @brief		USART13 interrupt handler sub-routine
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART13_IRQHandler_IT(void)
{
	uint32_t intsrc, tmp;

    /* Determine the UST13 interrupt source */
	intsrc = HAL_USART_GetStatus(USART13);
	tmp = intsrc & UST_SR_BITMASK;

	// Receive Data Available or Character time-out
	if ((tmp & UST_SR_RXC) == UST_SR_RXC){
        _usart13_rx_buffer[_usart13_buffer_tail++] = HAL_USART_ReceiveByte(USART13);
		_usart13_rx_flag = 1;
	}

	// Transmit Holding Empty
	if ((tmp & UST_SR_TXC) == UST_SR_TXC){
        HAL_USART_ClearStatus(USART13, USART_STATUS_TXC);
		_usart13_tx_flag = 1;
	}
}
/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}
/**********************************************************************
 * @brief		USARTSend
 * @param    	None
 * @return 		None
 **********************************************************************/
void USARTSend(USART_Type *USARTx, uint8_t *txbuf, uint8_t buflen)
{
	uint32_t i, timeout;
	uint8_t *tx_flag;
	
	if(USARTx == USART10){
			tx_flag = &_usart10_tx_flag;
	}else if(USARTx == USART11){
			tx_flag = &_usart11_tx_flag;
	}else if(USARTx == USART12){
			tx_flag = &_usart12_tx_flag;
	}else if(USARTx == USART13){
			tx_flag = &_usart13_tx_flag;
	}
			
	for(i=0;i<buflen;i++){
			timeout = 10000;
			HAL_USART_TransmitByte(USARTx, (txbuf[i]));
			while(timeout--){
					if(*tx_flag) break;
			}
	}
}

/**********************************************************************
 * @brief		USARTReceive
 * @param    	None
 * @return 		None
 **********************************************************************/
void USARTReceive(USART_Type *USARTx, uint8_t *rxbuf, uint8_t buflen)
{
	uint32_t tmp, i;
	uint8_t *buf, *head, *tail, *flag;
	if(USARTx == USART10){
			buf = _usart10_rx_buffer;
			head = &_usart10_buffer_head;
			tail = &_usart10_buffer_tail;
	flag = &_usart10_rx_flag;
	}else if(USARTx == USART11){
			buf = _usart11_rx_buffer;
			head = &_usart11_buffer_head;
			tail = &_usart11_buffer_tail;
	flag = &_usart11_rx_flag;
	}else if(USARTx == USART12){
			buf = _usart12_rx_buffer;
			head = &_usart12_buffer_head;
			tail = &_usart12_buffer_tail;
	flag = &_usart12_rx_flag;
	}else if(USARTx == USART13){
			buf = _usart13_rx_buffer;
			head = &_usart13_buffer_head;
			tail = &_usart13_buffer_tail;
	flag = &_usart13_rx_flag;
}
	while(*flag != 1);
	*flag = 0;

	if(*head > *tail){
			tmp = *tail+256;
	}else{
			tmp = *tail;
	}
	
	for(i=0;(i<buflen && *head<tmp);i++, (*head)++){
			rxbuf[i] = buf[*head];
	}
}


/**********************************************************************
 * @brief		USART_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
		/*
	 * Initialize USART12 pin connect
	 */
  HAL_GPIO_ConfigOutput(PB, 10, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 10, FUNC0);    // USART12 TXD
	HAL_GPIO_ConfigPullup(PB, 10, ENPU);   // enable Pull up
	
	HAL_GPIO_ConfigOutput(PB, 11, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 11, FUNC0);    // USART12 RXD
	HAL_GPIO_ConfigPullup(PB, 11, ENPU);   // enable Pull up

 	//SCK12 (PB12)   
	HAL_GPIO_ConfigOutput(PB, 12, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 12, FUNC1);		// USART12 SCK\
	
   	/*
	 * Initialize USART13
	 */
	//USART13
	// Initialize USART13 (UART) pin connection
  HAL_GPIO_ConfigOutput(PA, 0, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 0, FUNC0);    // USART13 TXD
	HAL_GPIO_ConfigPullup(PA, 0, ENPU);   // enable Pull up
	
	HAL_GPIO_ConfigOutput(PA, 1, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 1, FUNC0);    // USART13 RXD
	HAL_GPIO_ConfigPullup(PA, 1, ENPU);   // enable Pull up

	//SCK13 (PC12) 
	HAL_GPIO_ConfigOutput(PC, 12, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PC, 12, FUNC1);
}
/**********************************************************************
 * @brief		USART_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void USART_Configure(void)
{
    // default: 38400-8-N-1	
	HAL_USART_USRT_Mode_Config(&USARTConfigStruct);

	if ( HAL_USART_Init(USART12, &USARTConfigStruct)!= HAL_OK)
	{
		Error_Handler();
	}
	
#if USART12_M_USART13_S
	// USRT Marster
	HAL_USART_DataControlConfig(USART12, USART_CONTROL_MASTER, ENABLE);
	HAL_USART_DataControlConfig(USART12, USART_CONTROL_DISSCK, ENABLE);
#endif
#if USART12_S_USART13_M
// USRT Slave
	HAL_USART_DataControlConfig(USART12, USART_CONTROL_MASTER, DISABLE);
	#endif

	//interrupt
	NVIC_SetPriority(USART12_IRQn, 3);
	NVIC_EnableIRQ(USART12_IRQn);
	
	HAL_USART_ConfigInterrupt(USART12, USART_INTCFG_TXC, ENABLE);
	HAL_USART_ConfigInterrupt(USART12, USART_INTCFG_RXC, ENABLE);
	
	
	if ( HAL_USART_Init(USART13, &USARTConfigStruct) != HAL_OK )
	{
		Error_Handler();
	}
	
#if USART12_M_USART13_S
// USRT Slave
	HAL_USART_DataControlConfig(USART13, USART_CONTROL_MASTER, DISABLE);
	#endif
#if USART12_S_USART13_M
	// USRT Marster
	HAL_USART_DataControlConfig(USART13, USART_CONTROL_MASTER, ENABLE);
	HAL_USART_DataControlConfig(USART13, USART_CONTROL_DISSCK, ENABLE);
#endif
	//interrupt
	NVIC_SetPriority(USART13_IRQn, 3);
	NVIC_EnableIRQ(USART13_IRQn);

	HAL_USART_ConfigInterrupt(USART13, USART_INTCFG_TXC, ENABLE);
	HAL_USART_ConfigInterrupt(USART13, USART_INTCFG_RXC, ENABLE);
}

/**********************************************************************
 * @brief		UART_DebugMessage
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_DebugMessage(void)
{
	 _DBG_(">> USART USRT TEST.............. ");
}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void USART_ReceiveSend_loop(void)
{
	_DBG("MASTER SEND DATA ");
	tx_ch = _DG;
	_DBG(" = "); _DBC(tx_ch);_DBG(" : ");_DBH(tx_ch); _DBG_(" ");

	rx_ch = tx_ch-1;
	_DBG("SLAVE SEND DATA = "); _DBC(rx_ch);_DBG(" = ");_DBH(rx_ch); _DBG_(" ");        

	#if USART10_M_USART11_S
	USART11->DR = rx_ch;
	USARTSend(USART10, &tx_ch,1);
	USARTReceive(USART11,&rx_ch,1);
	USARTReceive(USART10,&tx_ch,1);
	#endif

	#if USART10_S_USART11_M
	USART10->DR = rx_ch;
	USARTSend(USART11, &tx_ch,1);
	USARTReceive(USART10,&rx_ch,1);
	USARTReceive(USART11,&tx_ch,1);
	#endif

	#if USART12_M_USART13_S
	USART13->DR = rx_ch;
	USARTSend(USART12, &tx_ch,1);
	USARTReceive(USART13,&rx_ch,1);
	USARTReceive(USART12,&tx_ch,1);
	#endif

	#if USART12_S_USART13_M
	USART12->DR = rx_ch;
	USARTSend(USART13, &tx_ch,1);
	USARTReceive(USART12,&rx_ch,1);
	USARTReceive(USART13,&tx_ch,1);
	#endif

	_DBG("MASTER GET DATA = "); _DBC(tx_ch);_DBG(" = ");_DBH(tx_ch); _DBG_(" ");
	_DBG("SLAVE GET DATA = "); _DBC(rx_ch);_DBG(" = ");_DBH(rx_ch); _DBG_(" ");_DBG_(" ");
}

/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void mainloop(void)
{
	/*Initialize menu prinf*/
	DEBUG_Print_menu();

	/*Initialize UART peripheral*/
	GPIO_Configure();

	/*Initialize UART peripheral*/
	USART_Configure();

	/*Initialize UART debug*/
	UART_DebugMessage();

   	/*Enable IRQ Interrupts */
	 __enable_irq(); 
	
	/* Infinite Loop */
	while(1)
	{
		USART_ReceiveSend_loop();
	}
}

/**********************************************************************
 * @brief		main
 * @param[in]	None
 * @return 		None
 **********************************************************************/
int main(void)
{

     /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif


