/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define DMA_USART
//USART10 <-> USART11
#define USART10_M_USART11_S	0
#define USART10_S_USART11_M	0
//USART12 <-> USART13
#define USART12_M_USART13_S	0
#define USART12_S_USART13_M	1
/* Private macro -------------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void USART_Configure(void);
void UART_DebugMessage(void);
void USART_ReceiveSend_loop(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
const uint8_t menu[] =
"************************************************\n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
" Usrt Polling test \n\r"
"************************************************\n\r";
uint8_t tx_ch=0, rx_ch=0;
USART_CFG_Type USARTConfigStruct;	

/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
		/*
	 * Initialize USART12 pin connect
	 */
  HAL_GPIO_ConfigOutput(PB, 10, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 10, FUNC0);    // USART12 TXD
	HAL_GPIO_ConfigPullup(PB, 10, ENPU);   // enable Pull up
	
	HAL_GPIO_ConfigOutput(PB, 11, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 11, FUNC0);    // USART12 RXD
	HAL_GPIO_ConfigPullup(PB, 11, ENPU);   // enable Pull up

 	//SCK12 (PB12)   
	HAL_GPIO_ConfigOutput(PB, 12, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 12, FUNC1);		// USART12 SCK\
	
   	/*
	 * Initialize USART13
	 */
	//USART13
	// Initialize USART13 (UART) pin connection
  HAL_GPIO_ConfigOutput(PA, 0, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 0, FUNC0);    // USART13 TXD
	HAL_GPIO_ConfigPullup(PA, 0, ENPU);   // enable Pull up
	
	HAL_GPIO_ConfigOutput(PA, 1, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 1, FUNC0);    // USART13 RXD
	HAL_GPIO_ConfigPullup(PA, 1, ENPU);   // enable Pull up

	//SCK13 (PC12) 
	HAL_GPIO_ConfigOutput(PC, 12, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PC, 12, FUNC1);
}



/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void USART_Configure(void)
{
    // default: 38400-8-N-1	
	HAL_USART_USRT_Mode_Config(&USARTConfigStruct);

	if ( HAL_USART_Init(USART12, &USARTConfigStruct)!= HAL_OK)
	{
		Error_Handler();
	}
	
#if USART12_M_USART13_S
	// USRT Marster
	HAL_USART_DataControlConfig(USART12, USART_CONTROL_MASTER, ENABLE);
	HAL_USART_DataControlConfig(USART12, USART_CONTROL_DISSCK, ENABLE);
#endif
#if USART12_S_USART13_M
// USRT Slave
	HAL_USART_DataControlConfig(USART12, USART_CONTROL_MASTER, DISABLE);
	#endif
	
	
	if ( HAL_USART_Init(USART13, &USARTConfigStruct) != HAL_OK )
	{
		Error_Handler();
	}
	
#if USART12_M_USART13_S
// USRT Slave
	HAL_USART_DataControlConfig(USART13, USART_CONTROL_MASTER, DISABLE);
	#endif
#if USART12_S_USART13_M
	// USRT Marster
	HAL_USART_DataControlConfig(USART13, USART_CONTROL_MASTER, ENABLE);
	HAL_USART_DataControlConfig(USART13, USART_CONTROL_DISSCK, ENABLE);
#endif

}

/**********************************************************************
 * @brief		UART_DebugMessage
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_DebugMessage(void)
{
	_DBG_(">> USART USRT TEST.............. ");

}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void USART_ReceiveSend_loop(void)
{

	_DBG("MASTER SEND DATA ");
	tx_ch = _DG;
	_DBG(" = "); _DBC(tx_ch);_DBG(" : ");_DBH(tx_ch); _DBG_(" ");

	rx_ch = tx_ch-1;
	_DBG("SLAVE SEND DATA = "); _DBC(rx_ch);_DBG(" = ");_DBH(rx_ch); _DBG_(" ");        

	//USRT Communication Start
	#if USART10_M_USART11_S
	USART11->DR = rx_ch; 
	HAL_USART_Transmit(USART10, &tx_ch,1);
	while(!(HAL_USART_GetStatus(USART10) & UST_SR_TXC)){} // wait 
	while(!(HAL_USART_GetStatus(USART10) & UST_SR_RXC)){} // wait 
	tx_ch = USART10->DR;
	HAL_USART_ClearStatus(USART10, USART_STATUS_TXC);
	HAL_USART_Receive(USART11,&rx_ch,1);
	#endif
	#if USART10_S_USART11_M
	USART10->DR = rx_ch; 
	HAL_USART_Transmit(USART11, &tx_ch,1);
	while(!(HAL_USART_GetStatus(USART11) & UST_SR_TXC)){} // wait 
	while(!(HAL_USART_GetStatus(USART11) & UST_SR_RXC)){} // wait 
	tx_ch = USART11->DR;
	HAL_USART_ClearStatus(USART11, USART_STATUS_TXC);
	HAL_USART_Receive(USART10,&rx_ch,1);
	#endif
	#if USART12_M_USART13_S
	USART13->DR = rx_ch; 
	HAL_USART_Transmit(USART12, &tx_ch,1);
	while(!(HAL_USART_GetStatus(USART12) & UST_SR_TXC)){} // wait 
	tx_ch = USART12->DR;
	HAL_USART_ClearStatus(USART12, USART_STATUS_TXC);
	HAL_USART_Receive(USART13,&rx_ch,1);
	#endif
	#if USART12_S_USART13_M
	USART12->DR = rx_ch; 
	HAL_USART_Transmit(USART13, &tx_ch,1);
	while(!(HAL_USART_GetStatus(USART13) & UST_SR_TXC)){} // wait 
	tx_ch = USART13->DR;
	HAL_USART_ClearStatus(USART13, USART_STATUS_TXC);
	HAL_USART_Receive(USART12,&rx_ch,1);
	#endif

	_DBG("MASTER GET DATA = "); _DBC(tx_ch);_DBG(" = ");_DBH(tx_ch); _DBG_(" ");
	_DBG("SLAVE GET DATA = "); _DBC(rx_ch);_DBG(" = ");_DBH(rx_ch); _DBG_(" ");_DBG_(" ");

	}

/**********************************************************************
* @brief		mainloop
* @param[in]	None
* @return 		None
**********************************************************************/
void mainloop(void)
{

	/*Initialize menu prinf*/
	DEBUG_Print_menu();

	/*Initialize UART peripheral*/
	GPIO_Configure();

	/*Initialize UART peripheral*/
	USART_Configure();

	/*Initialize UART debug*/
	UART_DebugMessage();

      /*Enable IRQ Interrupts */
	 __enable_irq(); 
	/* Infinite Loop */
	while(1)
	{
	  USART_ReceiveSend_loop();
	}
}

/**********************************************************************
 * @brief		main
 * @param[in]	None
 * @return 		None
 **********************************************************************/
int main(void)
{

     /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif

