/**********************************************************************
* @file		A31G32x_SystemClock.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
//#define USED_CLKO
//------------------
//#define USED_LSI		
//#define USED_HSI		
//#define USED_HSE		
//#define USED_LSE		
//#define USED_HSIPLL
#define USED_HSEPLL
//----------------------
/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config (void);
/* Private variables ---------------------------------------------------------*/



/**********************************************************************
 * @brief		Initialize default clock for A31G324
 * @param[in]	None
 * @return	None
 **********************************************************************/
void SystemClock_Config(void)
{
	uint32_t i;

	/*******************************************
	 CLKO function setting. check PORT setting (PF0).
	 CLKO = MCLK (CLKODIV = 0)
	 CLKO = MCLK/(2*(CLKDIV+1)) (CLKODIV > 0)
	*******************************************/
#ifdef USED_CLKO
	HAL_SCU_ClockOutput(4,ENABLE);	
#else
	HAL_SCU_ClockOutput(4,DISABLE);
#endif

	SCU->CMR&=~(1<<7);		//mclk monitoring disable
	
#ifdef USED_LSI	//750khz
	HAL_SCU_LSI_ClockConfig(LSIOSC_EN);	//LSIOSC_EN_DIV2, LSIOSC_EN_DIV4
	SystemCoreClock=750000;	//750khz
	SystemPeriClock=750000;	//750khz	
	
	for (i=0;i<10;i++);	
	
	HAL_SCU_SystemClockChange(SC_LSIOSC);
#endif 
	
#ifdef USED_LSE	//32.768khz
	HAL_SCU_LSE_ClockConfig(EXSOSC_EN);
	SystemCoreClock=32768;		//32.768khz
	SystemPeriClock=32768;		//32.768khz	

	for (i=0;i<100;i++);	

	// wait for SOSC stable	
	HAL_SCU_LSE_ClockMonitoring();
	HAL_SCU_SystemClockChange(SC_SOSC);
#endif 	

#ifdef USED_HSI		//48Mhz
	HAL_SCU_HSI_ClockConfig(HSIOSC_EN);
	SystemCoreClock=48000000; //48Mhz
	SystemPeriClock=48000000; //48Mhz	
	
	for (i=0;i<10;i++);	

	HAL_SCU_SystemClockChange(SC_HSIOSC);
#endif 	

#ifdef USED_HSE	//xMhz
	HAL_SCU_HSE_ClockConfig(EXOSC_EN);
	SystemCoreClock=8000000; //xMhz
	SystemPeriClock=8000000; //xMhz	

	for (i=0;i<100;i++);	

	HAL_SCU_HSE_ClockMonitoring();
	HAL_SCU_SystemClockChange(SC_EXOSC);
#endif


#ifdef USED_HSEPLL

	HAL_SCU_HSE_ClockConfig(EXOSC_EN);
	
	/*wait for stabilizing*/
	for (i=0;i<300;i++);	
	
	HAL_SCU_SystemClockFinClock(SC_FIN_HSE);	// Setting PLL Clock is HSE
	
	/**
	************************************************************************************************
	* PLL setting 
	* FIN=PLLINCLK/(R+1)                                 			// R: Pre Divider   
	* FOUT=(FIN*(N1+1)*(D+1))  / ((N2+1)*(P+1))          			// N1: Post Divider1,N2:Post Div2, P:Out Div,      
	*             = FVCO *(D+1)                         		 	// D:Frequency Doubler
	*
	*ex)    FIN=PLLINCLK/(R+1) = 8M/(3+1) = 2M            		// R:3, PLLINCLK:8MHz(HSE)
	*       FOUT=(2M*(47+1)*(0+1)) / ((1+1)*(0+1) = 48MHz   // N1:47, D:0, N2:1, P:0
	*
	************************************************************************************************
*/
	if (HAL_SCU_PLL_ClockConfig(ENABLE,  
		PLLCON_BYPASS_PLL,      //PLLCON_BYPASS_FIN:0, PLLCON_BYPASS_PLL:1
		0,                      //0:FOUT==VCO, 1:FOUT==2xVCO,  D=0
		3,                      //PREDIV, R=3
		47,                     //POSTDIV1, N1=47  
		1,                      //POSTDIV2, N2=1
		0)==ERROR)		//OUTDIV P=0
	{
		Error_Handler();
	}
	
  /* HSE -->  HSEPLL : PLL output clock*/
	HAL_SCU_SystemClockChange(SC_EXOSCPLL);
	
	SystemCoreClock=48000000; 
	SystemPeriClock=48000000; 
#endif

	
#ifdef USED_HSIPLL
/*SCU_CSCR= Enable High speed internal oscillator control*/	
	HAL_SCU_HSI_ClockConfig(HSIOSC_EN_DIV8);   /* 48MHz/8 = 6MHz */	
	
	/*wait for stabilizing*/
	for (i=0;i<10;i++);	

	/*HSE clock is used as FIN clock*/
	HAL_SCU_SystemClockFinClock(SC_FIN_HSI);	// Setting PLL Clock is HSI
	
	/************************************************************************************************
	* PLL setting 
	* FIN=PLLINCLK/(R+1)                                 		// R: Pre Divider   
	* FOUT=(FIN*(N1+1)*(D+1))  / ((N2+1)*(P+1))          		// N1: Post Divider1,N2:Post Div2, P:Out Div,      
	*             = FVCO *(D+1)                         		// D:Frequency Doubler
	*
	*ex)    FIN=PLLINCLK/(R+1) = 6M/(2+1) = 2M            	        // R:2, PLLINCLK:6MHz(HSI)
	*       FOUT=(2M*(47+1)*(0+1)) / ((1+1)*(0+1)) = 48MHz          // N1:47, D:0, N2:1, P:0
	*
	************************************************************************************************/
	
	if (HAL_SCU_PLL_ClockConfig(ENABLE,  
		PLLCON_BYPASS_PLL,      //PLLCON_BYPASS_FIN:0, PLLCON_BYPASS_PLL:1
		0,                      //0:FOUT==VCO, 1:FOUT==2xVCO,  D=0
		2,                      //PREDIV, R=2
		47,                     //POSTDIV1, N1=47  
		1,                      //POSTDIV2, N2=1
		0)==ERROR)      //OUTDIV P=0
	{
		Error_Handler();
	}

	/*HSI -->  HSIPLL : PLL output clock*/
	HAL_SCU_SystemClockChange(SC_HSIOSCPLL);
	
	SystemCoreClock=48000000; 
	SystemPeriClock=48000000; 
#endif
	
	SCU->CMR|=(1<<7); //mclk monitoring enable

// wait setting order 1. default wait setting -> 2. clock change -> 3. adjust wait setting
//// flash memory controller
	FMC->MR = 0x81;       // after changing 0x81 -> 0x28 in FMMR reg, flash access timing will be able to be set.
	FMC->MR = 0x28;       // enter flash access timing changing mode
//	FMC->FMCFG = (0x7858<<16) | (0<<8);  //flash access cycles 	20
//	FMC->FMCFG = (0x7858<<16) | (1<<8);  //flash access cycles 	40
	FMC->CFG = (0x7858<<16)   | (2<<8);  //flash access cycles 	60		
//	FMC->FMCFG = (0x7858<<16) | (3<<8);  //flash access cycles 		
	                              // flash access time cannot overflow 20Mhz.
	                              // ex) if MCLK=48Mhz, 
	                              //       48/1 = 48 (can't set no wait)
	                              //       48/2 = 24 (1 wait is not ok)
	                              //       48/3 = 16 (2 wait is ok)								  
	                              // so, 2 wait is possible.
	FMC->MR = 0;	      // exit flash access timing --> normal mode				
}



