#include "A31G32x_usb_user.h"
#include "stdio.h"
#include "a31g32x.h"


volatile uint32_t   USB_DOEP1_DMA_DESC_RX[2];
volatile uint32_t   USB_DOEP1_DMA_BUF_RX[64];
volatile uint32_t   USB_DIEP1_DMA_DESC_TX[2];
volatile uint32_t   USB_DIEP1_DMA_BUF_TX[64];

volatile uint32_t   USB_EP1_status = 0;
//uint8_t cb[31];

uint32_t Offset;                  /* R/W Offset */
uint32_t Length;                  /* R/W Length */

extern  uint8_t  Memory[MSC_MemorySize];  /* MSC RAM */
uint8_t  MemOK;                   /* Memory OK */


MSC_CBW CBW;
MSC_CSW CSW;


extern volatile uint8_t BulkStage;
extern uint8_t  BulkBuf[64]; /* Bulk In/Out Buffer */
uint8_t  BulkLen;                 /* Bulk In/Out Length */


#define SOFCTMR_TEST
#ifdef SOFCTMR_TEST

#endif

void rx_USB_UART(uint8_t *ch, uint32_t length);
void Process_line_coding(void);


void MSC_MemoryWrite (void) {
  uint32_t n;

  if ((Offset + BulkLen) > MSC_MemorySize) {
    BulkLen = MSC_MemorySize - Offset;
    BulkStage = MSC_BS_CSW;
		
		USB->DOEPCTL1 |= (0x1   << 21); //Stall
		USB->DOEPCTL1 |= (0x1UL << 31); //EPEna
    //USB_SetStallEP(MSC_EP_OUT);
  }

  for (n = 0; n < BulkLen; n++) {
    Memory[Offset + n] = BulkBuf[n];
  }

  Offset += BulkLen;
  Length -= BulkLen;

  CSW.dDataResidue -= BulkLen;

  if ((Length == 0) || (BulkStage == MSC_BS_CSW)) {
    //GPIOB->ODR &= ~LED_WR;      /* Turn Off Write LED */
    CSW.bStatus = CSW_CMD_PASSED;
    MSC_SetCSW();
  }
}


/*
 *  MSC Memory Verify Callback
 *   Called automatically on Memory Verify Event
 *    Parameters:      None (global variables)
 *    Return Value:    None
 */

void MSC_MemoryVerify (void) {
  uint32_t n;

  if ((Offset + BulkLen) > MSC_MemorySize) {
    BulkLen = MSC_MemorySize - Offset;
    BulkStage = MSC_BS_CSW;

		USB->DOEPCTL1 |= (0x1   << 21); //Stall
		USB->DOEPCTL1 |= (0x1UL << 31); //EPEna
    //USB_SetStallEP(MSC_EP_OUT);
  }

  for (n = 0; n < BulkLen; n++) {
    if (Memory[Offset + n] != BulkBuf[n]) {
      MemOK = FALSE;
      break;
    }
  }

  Offset += BulkLen;
  Length -= BulkLen;

  CSW.dDataResidue -= BulkLen;

  if ((Length == 0) || (BulkStage == MSC_BS_CSW)) {
    CSW.bStatus = (MemOK) ? CSW_CMD_PASSED : CSW_CMD_FAILED;
    MSC_SetCSW();
  }
}



void MSC_MemoryRead (void) {
  uint32_t n;

  if (Length > 64) {
    n = 64;
  } else {
    n = Length;
  }

  if ((Offset + n) > MSC_MemorySize) {
    n = MSC_MemorySize - Offset;
    BulkStage = MSC_BS_DATA_IN_LAST_STALL;
  }

	USB_Send_DATA_EP1(&Memory[Offset], n);
  //USB_WriteEP(MSC_EP_IN, &Memory[Offset], n);
	
  Offset += n;
  Length -= n;

  CSW.dDataResidue -= n;

  if (Length == 0) {
    BulkStage = MSC_BS_DATA_IN_LAST;
  }

  if (BulkStage != MSC_BS_DATA_IN) {
    //GPIOB->ODR &= ~LED_RD;      /* Turn Off Read LED */
    CSW.bStatus = CSW_CMD_PASSED;
  }
}


uint8_t MSC_RWSetup (void) {
  uint32_t n;

  /* Logical Block Address of First Block */
  n = (CBW.CB[2] << 24) |
      (CBW.CB[3] << 16) |
      (CBW.CB[4] <<  8) |
      (CBW.CB[5] <<  0);

  Offset = n * MSC_BlockSize;

  /* Number of Blocks to transfer */
  n = (CBW.CB[7] <<  8) |
      (CBW.CB[8] <<  0);

  Length = n * MSC_BlockSize;

  if (CBW.dDataLength != Length) {
//    USB_SetStallEP(MSC_EP_IN);
//    USB_SetStallEP(MSC_EP_OUT);
		USB->DIEPCTL1 |= (0x1   << 21); //Stall
		USB->DIEPCTL1 |= (0x1UL << 31); //EPEna	
		USB->DOEPCTL1 |= (0x1   << 21); //Stall
		USB->DOEPCTL1 |= (0x1UL << 31); //EPEna
		
    CSW.bStatus = CSW_PHASE_ERROR;
    MSC_SetCSW();
    return (FALSE);
  }

  return (TRUE);
}
/*
 *  MSC SCSI Inquiry Callback
 *    Parameters:      None (global variables)
 *    Return Value:    None
 */

void MSC_Inquiry (void) {

  //if (!DataInFormat()) return;

  BulkBuf[ 0] = 0x00;          /* Direct Access Device */
  BulkBuf[ 1] = 0x80;          /* RMB = 1: Removable Medium */
  BulkBuf[ 2] = 0x00;          /* Version: No conformance claim to standard */
  BulkBuf[ 3] = 0x01;

  BulkBuf[ 4] = 36-4;          /* Additional Length */
  BulkBuf[ 5] = 0x80;          /* SCCS = 1: Storage Controller Component */
  BulkBuf[ 6] = 0x00;
  BulkBuf[ 7] = 0x00;

  BulkBuf[ 8] = 'K';           /* Vendor Identification */
  BulkBuf[ 9] = 'e';
  BulkBuf[10] = 'i';
  BulkBuf[11] = 'l';
  BulkBuf[12] = ' ';
  BulkBuf[13] = ' ';
  BulkBuf[14] = ' ';
  BulkBuf[15] = ' ';

  BulkBuf[16] = 'M';           /* Product Identification */
  BulkBuf[17] = 'C';
  BulkBuf[18] = 'B';
  BulkBuf[19] = 'S';
  BulkBuf[20] = 'T';
  BulkBuf[21] = 'M';
  BulkBuf[22] = '3';
  BulkBuf[23] = '2';
  BulkBuf[24] = ' ';
  BulkBuf[25] = 'D';
  BulkBuf[26] = 'I';
  BulkBuf[27] = 'S';
  BulkBuf[28] = 'K';
  BulkBuf[29] = ' ';
  BulkBuf[30] = ' ';
  BulkBuf[31] = ' ';

  BulkBuf[32] = '1';           /* Product Revision Level */
  BulkBuf[33] = '.';
  BulkBuf[34] = '0';
  BulkBuf[35] = ' ';

  BulkLen = 36;
  //DataInTransfer();
	
	if (BulkLen > CBW.dDataLength) {
    BulkLen = CBW.dDataLength;
  }

	USB_Send_DATA_EP1(BulkBuf, BulkLen);
  //USB_WriteEP(MSC_EP_IN, BulkBuf, BulkLen);
	
  BulkStage = MSC_BS_DATA_IN_LAST;

  CSW.dDataResidue -= BulkLen;
  CSW.bStatus = CSW_CMD_PASSED;
}

void MSC_ReadFormatCapacity (void) {

 // if (!DataInFormat()) return;

  BulkBuf[ 0] = 0x00;
  BulkBuf[ 1] = 0x00;
  BulkBuf[ 2] = 0x00;
  BulkBuf[ 3] = 0x08;          /* Capacity List Length */

  /* Block Count */
  BulkBuf[ 4] = (MSC_BlockCount >> 24) & 0xFF;
  BulkBuf[ 5] = (MSC_BlockCount >> 16) & 0xFF;
  BulkBuf[ 6] = (MSC_BlockCount >>  8) & 0xFF;
  BulkBuf[ 7] = (MSC_BlockCount >>  0) & 0xFF;

  /* Block Length */
  BulkBuf[ 8] = 0x02;          /* Descriptor Code: Formatted Media */
  BulkBuf[ 9] = (MSC_BlockSize >> 16) & 0xFF;
  BulkBuf[10] = (MSC_BlockSize >>  8) & 0xFF;
  BulkBuf[11] = (MSC_BlockSize >>  0) & 0xFF;

  BulkLen = 12;
	
	if (BulkLen > CBW.dDataLength) {
    BulkLen = CBW.dDataLength;
  }

	USB_Send_DATA_EP1(BulkBuf, BulkLen);
  //USB_WriteEP(MSC_EP_IN, BulkBuf, BulkLen);
	
  BulkStage = MSC_BS_DATA_IN_LAST;

  CSW.dDataResidue -= BulkLen;
  CSW.bStatus = CSW_CMD_PASSED;
}

void MSC_ReadCapacity (void) {

  //if (!DataInFormat()) return;

  /* Last Logical Block */
  BulkBuf[ 0] = ((MSC_BlockCount - 1) >> 24) & 0xFF;
  BulkBuf[ 1] = ((MSC_BlockCount - 1) >> 16) & 0xFF;
  BulkBuf[ 2] = ((MSC_BlockCount - 1) >>  8) & 0xFF;
  BulkBuf[ 3] = ((MSC_BlockCount - 1) >>  0) & 0xFF;

  /* Block Length */
  BulkBuf[ 4] = (MSC_BlockSize >> 24) & 0xFF;
  BulkBuf[ 5] = (MSC_BlockSize >> 16) & 0xFF;
  BulkBuf[ 6] = (MSC_BlockSize >>  8) & 0xFF;
  BulkBuf[ 7] = (MSC_BlockSize >>  0) & 0xFF;

  BulkLen = 8;
	
	if (BulkLen > CBW.dDataLength) {
    BulkLen = CBW.dDataLength;
  }

	USB_Send_DATA_EP1(BulkBuf, BulkLen);
  //USB_WriteEP(MSC_EP_IN, BulkBuf, BulkLen);
	
  BulkStage = MSC_BS_DATA_IN_LAST;

  CSW.dDataResidue -= BulkLen;
  CSW.bStatus = CSW_CMD_PASSED;;
}

void MSC_ModeSense6 (void) {

 // if (!DataInFormat()) return;

  BulkBuf[ 0] = 0x03;
  BulkBuf[ 1] = 0x00;
  BulkBuf[ 2] = 0x00;
  BulkBuf[ 3] = 0x00;

  BulkLen = 4;
	
	if (BulkLen > CBW.dDataLength) {
    BulkLen = CBW.dDataLength;
  }

	USB_Send_DATA_EP1(BulkBuf, BulkLen);
  //USB_WriteEP(MSC_EP_IN, BulkBuf, BulkLen);
	
  BulkStage = MSC_BS_DATA_IN_LAST;

  CSW.dDataResidue -= BulkLen;
  CSW.bStatus = CSW_CMD_PASSED;;
}
/*
 *  MSC Get Command Block Wrapper Callback
 *    Parameters:      None (global variables)
 *    Return Value:    None
 */

void MSC_GetCBW (void) {
  uint32_t n;

//  for (n = 0; n < BulkLen; n++) {
//    *((uint8_t *)&cb + n) = BulkBuf[n];
//  }
	for(n = 0; n < BulkLen; n++)
	{
			*((uint8_t *)&CBW + n) = BulkBuf[n];
	}
	
  //if ((sizeof(CBW) == 31) && (CBW.dSignature == MSC_CBW_Signature)) {
	if (CBW.dSignature == MSC_CBW_Signature) 
	{
    /* Valid CBW */
    CSW.dTag = CBW.dTag;
    CSW.dDataResidue = CBW.dDataLength;
    if ((CBW.bLUN != 0) || (CBW.bCBLength < 1) || CBW.bCBLength > 16) {
fail: CSW.bStatus = CSW_CMD_FAILED;
      MSC_SetCSW();
    } else {
      switch (CBW.CB[0]) {
        case SCSI_TEST_UNIT_READY:
           if (CBW.dDataLength != 0) {
							if ((CBW.bmFlags & 0x80) != 0) {
								USB->DIEPCTL1 |= (0x1   << 21); //Stall
								USB->DIEPCTL1 |= (0x1UL << 31); //EPEna
								//USB_SetStallEP(MSC_EP_IN);
							} else {
								USB->DOEPCTL1 |= (0x1   << 21); //Stall
								USB->DOEPCTL1 |= (0x1UL << 31); //EPEna
								//USB_SetStallEP(MSC_EP_OUT);
							}
					}

					CSW.bStatus = CSW_CMD_PASSED;
					MSC_SetCSW();
          break;
        case SCSI_REQUEST_SENSE:
          //MSC_RequestSense();
				  BulkBuf[ 0] = 0x70;          /* Response Code */
					BulkBuf[ 1] = 0x00;
					BulkBuf[ 2] = 0x02;          /* Sense Key */
					BulkBuf[ 3] = 0x00;
					BulkBuf[ 4] = 0x00;
					BulkBuf[ 5] = 0x00;
					BulkBuf[ 6] = 0x00;
					BulkBuf[ 7] = 0x0A;          /* Additional Length */
					BulkBuf[ 8] = 0x00;
					BulkBuf[ 9] = 0x00;
					BulkBuf[10] = 0x00;
					BulkBuf[11] = 0x00;
					BulkBuf[12] = 0x30;          /* ASC */
					BulkBuf[13] = 0x01;          /* ASCQ */
					BulkBuf[14] = 0x00;
					BulkBuf[15] = 0x00;
					BulkBuf[16] = 0x00;
					BulkBuf[17] = 0x00;

					BulkLen = 18;
					//DataInTransfer();
					
					if (BulkLen > CBW.dDataLength) {
						BulkLen = CBW.dDataLength;
					}

					USB_Send_DATA_EP1(BulkBuf, BulkLen);
					//USB_WriteEP(MSC_EP_IN, BulkBuf, BulkLen);
					
					BulkStage = MSC_BS_DATA_IN_LAST;

					CSW.dDataResidue -= BulkLen;
					CSW.bStatus = CSW_CMD_PASSED;
          break;
        case SCSI_FORMAT_UNIT:
          goto fail;
        case SCSI_INQUIRY:
          MSC_Inquiry();
          break;
        case SCSI_START_STOP_UNIT:
          goto fail;
        case SCSI_MEDIA_REMOVAL:
          goto fail;
        case SCSI_MODE_SELECT6:
          goto fail;
        case SCSI_MODE_SENSE6:
          MSC_ModeSense6();
          break;
        case SCSI_MODE_SELECT10:
          goto fail;
        case SCSI_MODE_SENSE10:
					BulkBuf[ 0] = 0x00;
					BulkBuf[ 1] = 0x06;
					BulkBuf[ 2] = 0x00;
					BulkBuf[ 3] = 0x00;
					BulkBuf[ 4] = 0x00;
					BulkBuf[ 5] = 0x00;
					BulkBuf[ 6] = 0x00;
					BulkBuf[ 7] = 0x00;

					BulkLen = 8;
					//DataInTransfer();
				
					if (BulkLen > CBW.dDataLength) {
						BulkLen = CBW.dDataLength;
					}

					USB_Send_DATA_EP1(BulkBuf, BulkLen);
					//USB_WriteEP(MSC_EP_IN, BulkBuf, BulkLen);
					
					BulkStage = MSC_BS_DATA_IN_LAST;

					CSW.dDataResidue -= BulkLen;
					CSW.bStatus = CSW_CMD_PASSED;
          //MSC_ModeSense10();
          break;
        case SCSI_READ_FORMAT_CAPACITIES:
          MSC_ReadFormatCapacity();
          break;
        case SCSI_READ_CAPACITY:
          MSC_ReadCapacity();
          break;
        case SCSI_READ10:
          if (MSC_RWSetup()) {
            if ((CBW.bmFlags & 0x80) != 0) {
              //GPIOB->ODR |= LED_RD;       /* Turn On Read LED */
              BulkStage = MSC_BS_DATA_IN;
              MSC_MemoryRead();
            } else {
              //USB_SetStallEP(MSC_EP_OUT);
							USB->DOEPCTL1 |= (0x1   << 21); //Stall
							USB->DOEPCTL1 |= (0x1UL << 31); //EPEna
              CSW.bStatus = CSW_PHASE_ERROR;
              MSC_SetCSW();
            }
          }
          break;
        case SCSI_WRITE10:
          if (MSC_RWSetup()) {
            if ((CBW.bmFlags & 0x80) == 0) {
              //GPIOB->ODR |= LED_WR;       /* Turn On Write LED */
              BulkStage = MSC_BS_DATA_OUT;
            } else {
							USB->DIEPCTL1 |= (0x1   << 21); //Stall
							USB->DIEPCTL1 |= (0x1UL << 31); //EPEna
              //USB_SetStallEP(MSC_EP_IN);
              CSW.bStatus = CSW_PHASE_ERROR;
              MSC_SetCSW();
            }
          }
          break;
        case SCSI_VERIFY10:
          if (MSC_RWSetup()) {
            if ((CBW.bmFlags & 0x80) == 0) {
              BulkStage = MSC_BS_DATA_OUT;
              MemOK = TRUE;
            } else {
							USB->DIEPCTL1 |= (0x1   << 21); //Stall
							USB->DIEPCTL1 |= (0x1UL << 31); //EPEna
             // USB_SetStallEP(MSC_EP_IN);
              CSW.bStatus = CSW_PHASE_ERROR;
              MSC_SetCSW();
            }
          }
          break;
        default:
          goto fail;
      }
    }
  } else {
    /* Invalid CBW */
    //USB_SetStallEP(MSC_EP_IN);
    //USB_SetStallEP(MSC_EP_OUT);
    BulkStage = MSC_BS_ERROR;
  }
}


/*
 *  MSC Set Command Status Wrapper Callback
 *    Parameters:      None (global variables)
 *    Return Value:    None
 */

void MSC_SetCSW (void) {

  CSW.dSignature = MSC_CSW_Signature;
	
  //USB_WriteEP(MSC_EP_IN, (BYTE *)&CSW, sizeof(CSW));
	USB_Send_CSW_EP1((uint8_t *)&CSW);
  BulkStage = MSC_BS_CSW;
}


void USB_Send_CSW_EP1(uint8_t *ch){
	volatile uint32_t timeout;
	USB_DIEP1_DMA_DESC_TX[0] = 0x0A000000 | 0x0D;	//13Byte
	USB_DIEP1_DMA_DESC_TX[1] = (uint32_t)ch;
	
	BulkLen = 13;
	
	USB->DIEPDMA1 = (uint32_t)USB_DIEP1_DMA_DESC_TX;
	
	USB->DIEPCTL1 |= (0x0D   <<  0); //13Byte
	USB->DIEPCTL1 |= (0x1   << 26); //CNAK
	USB->DIEPCTL1 |= (0x1UL << 31); //EPEna
//	timeout = 10000;
//	while(!(((USB_DIEP1_DMA_DESC_TX[0] >> 30) & 0x3) == 0x3
//		|| ((USB_DIEP1_DMA_DESC_TX[0] >> 30) & 0x3) == 0x2)){if(!(timeout--)) break;}
}

void USB_Receive_DATA_EP1(uint8_t *ch){
	USB_DOEP1_DMA_DESC_RX[0] = 0x0A000000 | 0x40;	//1Byte
	USB_DOEP1_DMA_DESC_RX[1] = (uint32_t)ch;
	
	BulkLen = 64;//CDC_DATA_FS_MAX_PACKET_SIZE - (USB_DOEP1_DMA_DESC_RX[0] & 0xffff);
		
	USB->DOEPDMA1 = (uint32_t)USB_DOEP1_DMA_DESC_RX;
	
	USB->DOEPCTL1 |= (0x40  <<  0); //1Byte
	USB->DOEPCTL1 |= (0x1   << 26); //CNAK
	USB->DOEPCTL1 |= (0x1UL << 31); //EPEna
}

void USB_Send_DATA_EP1(uint8_t *ch, uint8_t len){
	volatile uint32_t timeout;
	USB_DIEP1_DMA_DESC_TX[0] = 0x0A000000 | len;	//13Byte
	USB_DIEP1_DMA_DESC_TX[1] = (uint32_t)ch;
	
	USB->DIEPDMA1 = (uint32_t)USB_DIEP1_DMA_DESC_TX;
	
	USB->DIEPCTL1 |= (len   <<  0); //13Byte
	USB->DIEPCTL1 |= (0x1   << 26); //CNAK
	USB->DIEPCTL1 |= (0x1UL << 31); //EPEna
//	timeout = 10000;
//	while(!(((USB_DIEP1_DMA_DESC_TX[0] >> 30) & 0x3) == 0x3
//		|| ((USB_DIEP1_DMA_DESC_TX[0] >> 30) & 0x3) == 0x2)){if(!(timeout--)) break;}
}

void MSC_BulkIn (void) {

  switch (BulkStage) {
    case MSC_BS_DATA_IN:
      switch (CBW.CB[0]) {
        case SCSI_READ10:
          MSC_MemoryRead();
          break;
      }
      break;
    case MSC_BS_DATA_IN_LAST:
      MSC_SetCSW();
      break;
    case MSC_BS_DATA_IN_LAST_STALL:
			USB->DIEPCTL1 |= (0x1   << 21); //Stall
			USB->DIEPCTL1 |= (0x1UL << 31); //EPEna	
      //USB_SetStallEP(MSC_EP_IN);
      MSC_SetCSW();
      break;
    case MSC_BS_CSW:
      BulkStage = MSC_BS_CBW;
      break;
  }
}


/*
 *  MSC Bulk Out Callback
 *    Parameters:      None (global variables)
 *    Return Value:    None
 */

void MSC_BulkOut (void) {

	USB_Receive_DATA_EP1(BulkBuf);
	
  //BulkLen = USB_ReadEP(MSC_EP_OUT, BulkBuf);
  switch (BulkStage) {
    case MSC_BS_CBW:
      MSC_GetCBW();
      break;
    case MSC_BS_DATA_OUT:
      switch (CBW.CB[0]) {
        case SCSI_WRITE10:
          MSC_MemoryWrite();
          break;
        case SCSI_VERIFY10:
          MSC_MemoryVerify();
          break;
      }
      break;
    default:
			USB->DOEPCTL1 |= (0x1   << 21); //Stall
			USB->DOEPCTL1 |= (0x1UL << 31); //EPEna	
//      USB_SetStallEP(MSC_EP_OUT);
      CSW.bStatus = CSW_PHASE_ERROR;
      MSC_SetCSW();
      break;
  }
}


