#include "A31G32x_hal_pcu.h"
#include "A31G32x_hal_scu.h"
//#include "A31G32x_hal_gpio.h"
#include "A31G32x_usb_user.h"


uint8_t device_desc[] = {
	0x12,                       // bLength
	eUSB_DT_DEVICE,             // bDescriptorType		: DEVICE
	LSB(USB_20_SPEC_VER),       // bcdUSB				: USB 2.0 Spec
	MSB(USB_20_SPEC_VER),       
	0x00,                       // bDeviceClass			: Device Class
	0x00,                       // bDeviceSubClass		: Device Subclass
	0x00,                       // bDeviceProtocol		: Device Protocol
	USB_EP0_MAX_PKT_SZ,         // bMaxPacketSize0		: MaxPacketSize
	LSB(USBD_VID),              // idVendor				: Vendor ID
	MSB(USBD_VID),              
	LSB(USBD_PID),              // idProduct			: Product ID
	MSB(USBD_PID),              
	0x00,                       // bcdDevice			: Device Release Number
	0x01,                       
	0x01,                       // iManufacturer		: Manufacturer String Index
	0x02,                       // iProduct				: Product String Index
	0x03,                       // iSerialNumber		: Serial Number String Index
	0x01                        // bNumConfigurations	: One Configuration
};

uint8_t device_qulifier_desc[] = {
	0x0A,			                // bLength
	eUSB_DT_DEVICE_QUALIFIER,		// bDescriptorType
	LSB(USB_20_SPEC_VER), 			// bcdUSB				: USB 2.0 Spec
	MSB(USB_20_SPEC_VER),
	0x00,							// bDeviceClass			: Device Class
	0x00,							// bDeviceSubClass		: Device Subclass
	0x00,							// bDeviceProtocol		: Device Protocol
	USB_EP0_MAX_PKT_SZ,				// bMaxPacketSize0		: MaxPacketSize
	0x01,							// bNumConfigurations	: One Configuration
	0x00
};

uint8_t other_speed_desc[] = {
    /****************************/
	/* Congifuration Descriptor */
	/****************************/
	eUSB_DESC_CONFIG_LEN,			// bLength				: 0x09
	eUSB_DT_OTHER_SPEED_CONFIGURATION,			// bDescriptorType		: eUSB_DT_OTHER_SPEED_CONFIGURATION
	0x22,							// wTotalLength			: assigned by process
	0x00,
	0x01,				            // bNumInterface		: 1
	0x01,							// bConfigurationValue	: 0x01 
	0x00,							// iConfiguration		: N/A 
	(0x3<<6),							// bmAttribute			: Self-powered
	0x32,							// iMaxPower 			: 
	
	/******************************************/
	/* HID Interface Descriptor (Interface 0) */
	/******************************************/
	// Standard HID Interface Descriptor
	eUSB_DESC_INTERFACE_LEN,	// bLength
	eUSB_DT_INTERFACE,			// bDescriptorType		: INTERFACE
	0x00,                       // bInterfaceNumber		: Interface ID 0
	0x00,						// bAlternateSetting	: Alternate 0
	0x01,						// bNumEndpoints		: One Endpoint 
	eUSB_CLASS_HID,				// bInterfaceClass		: HID
	0x01,						// bInterfaceSubclass	: Unused
	0x01,						// bInterfaceProtocol	: Unused
	0x00,						// iInterface			: Unused

	//HID Class-Specific Descriptor
	eUSB_DESC_HID_LEN,			// bLength
	eUSB_DT_HID,				// bDescriptorType		: HID DESCRIPTOR
	LSB(USB_11_SPEC_VER),		// bcdHID				: HID Spec 1.0
	MSB(USB_11_SPEC_VER),
	0x00,						// bCountryCode			: None 
	0x01,						// bNumDescriptors		: One Class Descriptor
	eUSB_DT_REPORT,				// bClassDescriptorType	: Report Descriptor
	0x3F,						// HID descriptor size (LSB)
	0x00,						// HID descriptor size (MSB)

	0x07,
	eUSB_DT_ENDPOINT,
	0x01 | 0x80,
	eUSB_XferINTR,
	0x08, 0x00,
	0x01,
};

uint8_t config_desc[] = {
    /****************************/
	/* Congifuration Descriptor */
	/****************************/
	eUSB_DESC_CONFIG_LEN,			// bLength				: 0x09
	eUSB_DT_CONFIGURATION,			// bDescriptorType		: CONFIGURATION
	0x20,							// wTotalLength			: assigned by process
	0x00,
	0x01,				            // bNumInterface		: 1
	0x01,							// bConfigurationValue	: 0x01 
	0x00,							// iConfiguration		: N/A 
	(1<<7|1<<6),							// bmAttribute			: Self-powered
	50,							// iMaxPower 			: 
   	
    /**************************************************/
	/* MSC IN/OUT Interface Descriptor (Interface 1) */
	/**************************************************/
	// Data class interface descriptor
	0x09,   					// bLength: Interface Descriptor size
	eUSB_DT_INTERFACE,    		// bDescriptorType		:
	0x00,   					// bInterfaceNumber		: Number of Interface
	0x00,   					// bAlternateSetting	: Alternate setting
	0x02,   					// bNumEndpoints		: Two endpoints used
	0x08,   					// bInterfaceClass		: CDC
	0x06,   					// bInterfaceSubClass	:
	0x50,						// bInterfaceProtocol	:
	0x00,						// iInterface			:
	
	// Endpoint OUT Descriptor
	0x07,						// bLength				: Endpoint Descriptor size
	eUSB_DT_ENDPOINT,			// bDescriptorType		: Endpoint
	0x81,			// bEndpointAddress
	0x02,						// bmAttributes			: Bulk
	LSB(64),	// wMaxPacketSize:
	MSB(64),
	0x01,						// bInterval			: ignore for Bulk transfer

	// Endpoint IN Descriptor
	0x07,   /* bLength: Endpoint Descriptor size */
	eUSB_DT_ENDPOINT,      /* bDescriptorType: Endpoint */
	0x01,                         /* bEndpointAddress */
	0x02,                              /* bmAttributes: Bulk */
	LSB(64),  /* wMaxPacketSize: */
	MSB(64),
	0x01  
};

uint8_t lang_desc[] = { //code
	0x04,				// bLength			: Size of this descriptor
	eUSB_DT_STRING,		// bDescriptorType	: String
	0x09, 0x04,			// bLang			: Language ID, 0409: English, 0404: Chinese Taiwan
};


uint8_t __manufact_desc[] = {
	34,	// bLength			: Size of this descriptor
	eUSB_DT_STRING,				// bDescriptorType	: String
	'A', 0,
	'B', 0,
	'O', 0,
	'V', 0,
	' ', 0,
	' ', 0,
	' ', 0,
	' ', 0,
	' ', 0,
	' ', 0,
	' ', 0,
	' ', 0,
	' ', 0,
	' ', 0,
	' ', 0,
	' ', 0,
};

uint8_t __product_desc[] = {
	34,	// bLength			: Size of this descriptor
	eUSB_DT_STRING,			// bDescriptorType	: String
	'M', 0,
	'S', 0,
	'C', 0,
	' ', 0,
	'E', 0,
	'x', 0,
	'a', 0,
	'm', 0,
	'p', 0,
	'l', 0,
	'e', 0,
	' ', 0,
	' ', 0,
	' ', 0,
	' ', 0,
	' ', 0,  
};

uint8_t __serial_desc[] = {
	12,		// bLength			: Size of this descriptor
	eUSB_DT_STRING,				// bDescriptorType	: String
	'1', '2',
	'3', '4',
	'5', '6',
	'7', '8',
	'9', '0',
	'9', '9'
};


uint8_t hid_desc[] = {
    /****************************/
	/* Congifuration Descriptor */
	/****************************/
	0x09,			// bLength				: 0x09
	0x21,		// bDescriptorType
	LSB(USB_20_SPEC_VER), 			// bcdUSB				: USB 1.1 Spec
	MSB(USB_20_SPEC_VER),
	0x00,							
	0x01,						
	0x22,							
  0x3F,
};

