/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void SysTick_Handler_IT(void);

#define __regw(x)       (*(volatile unsigned int *)(x))
#define USBCON			__regw(0x40001540)

uint8_t _uart_data[10] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9'};

#ifdef  __CC_ARM
  struct __FILE { int handle; /* Add whatever needed */ };
  FILE __stdout;
  FILE __stdin;
#elif	__ICCARM__
  struct FILE { int handle; /* Add whatever needed */ };
  FILE __stdout;
  FILE __stdin;
#endif

int fputc(int ch, FILE *f) 
{
    while(HAL_USART_CheckBusy(USART10));
    HAL_USART_TransmitByte(USART10, ch);
    return(ch);
}

/* Private variables ---------------------------------------------------------*/
uint32_t msec;


const uint8_t menu[] =
"************************************************\n\r"
" USB demo \n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART10 - 38400 bps \n\r"
" Using USB HID test \n\r"
"************************************************\n\r";

/**********************************************************************
 * @brief		SysTick handler sub-routine (1ms)
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler_IT(void) 					// SysTick Interrupt Handler @ 1000Hz
{
	if(msec)msec--;
}


/**********************************************************************
 * @brief       Print menu
 * @param[in]   None
 * @return      None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
    #ifdef _DEBUG_MSG
    _DBG(menu);
    #endif
}


/**********************************************************************
 * @brief       DEBUG_Init
 * @param[in]   None
 * @return      None
 **********************************************************************/
void DEBUG_Init(void)
{
    #ifdef _DEBUG_MSG
    debug_frmwrk_init();
    #endif
}


void delay(uint32_t dly)
{
	uint32_t i;
	
	for (i=0;i<dly;i++)
	{
		__NOP();
	}
}

void init_USB_Phy(){
	
	HAL_GPIO_ConfigOutput(PA, 11, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 11, FUNC10);
	HAL_GPIO_ConfigPullup(PA, 11, DISPUPD);
	
	HAL_GPIO_ConfigOutput(PA, 12, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 12, FUNC10);
	HAL_GPIO_ConfigPullup(PA, 12, DISPUPD);
	
	SCU->VDCCON = 0xA3000000; //VDC33 RUN mode
	
	SCU->MCCR5 = (0x7) << 24
				| (0x1) << 16
				;
	
	USBCON = (0x7 << 28)		//Drive Strengthen
			| (0x7 << 24)		//Drive Strengthen
			| (0x1 << 23)		//CONTROL_EN
			| (0x0 << 22)		//PHY_SUSPEND
			| (0x1 << 21)		//VREF_SEL
			| (0x0 << 19)		//PD_ENDN
			| (0x0 << 18)		//PD_ENDP
			| (0x0 << 17)		//PU_ENDN
			| (0x1 << 16)		//PU_ENDP
			| (0x0 << 12)		//VIH_CON
			| (0x0 << 8)		//VIL_CON
			| (0x7 << 4)		//PD_CON
			| (0x7 << 0)		//PU_CON
		;
	
}

void mainloop(void)
{
		__enable_irq();
	
		SysTick_Config(SystemCoreClock/1000);  //1msec interrupt 

    while(1)
		{
			 __NOP();
		}
		
}


int main(void)
{ 	
	 /* Initialize all port */
	Port_Init(); 
	
	HAL_GPIO_ConfigOutput(PF, 0, ALTERN_FUNC);		// MOSC XOUT
	HAL_GPIO_ConfigFunction(PF, 0, FUNC10);
	
	HAL_GPIO_ConfigOutput(PF, 1, ALTERN_FUNC);		// MOSC_XIN
	HAL_GPIO_ConfigFunction(PF, 1, FUNC10);

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();  

	init_USB_Phy();
	
  USB_Init();

  mainloop();


    return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
