/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void SysTick_Handler_IT(void);

#define __regw(x)       (*(volatile unsigned int *)(x))
#define USBCON			__regw(0x40001540)
#define USBBUFSIZE		(64)

volatile uint8_t uart_buffer[USBBUFSIZE];
volatile uint16_t head=0, tail=0;

// system default I/O setting
void PCU_Init(void);
void SCU_ClockInit(void);

// Defined __FILE struct by Compiler (IAR compiler includes ystdio.h library __FILE struct)
uint32_t msec;

const uint8_t menu[] =
"************************************************\n\r"
" USB demo \n\r"
"\t - MCU: A31G324 \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART0 - 38400 bps \n\r"
" Using USB CDC/USART test \n\r"
"************************************************\n\r";

/**********************************************************************
 * USB UART Rx & Buffer command 
 */

void  usb_printf (const  char *format, ...)
{
    char  buffer[64 + 1];
		char *s;
		va_list     vArgs;
    va_start(vArgs, format);
    vsprintf((char *)buffer, (char const *)format, vArgs);
    va_end(vArgs);
		
		s = (char *) buffer;

		while (*s)
		{
			USB_Send_DATA_EP1(*s++);
		}
}

void rx_USB_UART(uint8_t *ch, uint32_t length){
	uint32_t i;
	
	for(i=0;i<length;i++){
		tail++;
		if(tail >= USBBUFSIZE){
			tail = 0;
		}
		uart_buffer[tail] = ch[i];
	}
}

void UART_Tx(unsigned char tx_data) {
	USB_Send_DATA_EP1(tx_data);
}
uint8_t check_UART_Rx(void){
	if(tail == head)
		return 0;
	else
		return 1;
}

unsigned char UART_Rx(void) {
	uint8_t ret=0;
	if(check_UART_Rx()){
		
		head++;
		if(head >= USBBUFSIZE){
			head = 0;
		}
		ret = uart_buffer[head];
	}
	return ret;
}


/**********************************************************************
 * @brief		SysTick_Handler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler_IT (void) 					// SysTick Interrupt Handler @ 1000Hz
{
	if(msec)msec--;
}


/**********************************************************************
 * @brief       Print menu
 * @param[in]   None
 * @return      None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
    #ifdef _DEBUG_MSG
    _DBG(menu);
    #endif
}


/**********************************************************************
 * @brief       DEBUG_Init
 * @param[in]   None
 * @return      None
 **********************************************************************/
void DEBUG_Init(void)
{
    #ifdef _DEBUG_MSG
    debug_frmwrk_init();
    #endif
}

void delay(uint32_t dly)
{
	uint32_t i;
	
	for (i=0;i<dly;i++)
	{
		__NOP();
	}
}


void mainloop(void)
{
	uint8_t rx_ch, tx_ch;
	uint8_t i;
	
	__enable_irq();

	SysTick_Config(SystemCoreClock/1000);  //1msec interrupt 
	
  while(1){
		rx_ch = UART_Rx();
		if(rx_ch){	//USB
			HAL_USART_Transmit(USART10, &rx_ch,1);
		}
		
		if(USART10->ST & UST_SR_RXC){
			tx_ch = HAL_USART_ReceiveByte(USART10);
			UART_Tx(tx_ch);
		}	
		
		while(msec);
		msec = 100;
		
		i++;
		//usb_printf("OK %d\r\n", i);
	}
	
}

void init_USB_Phy(){
	
	HAL_GPIO_ConfigOutput(PA, 11, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 11, FUNC10);
	HAL_GPIO_ConfigPullup(PA, 11, DISPUPD);
	
	HAL_GPIO_ConfigOutput(PA, 12, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PA, 12, FUNC10);
	HAL_GPIO_ConfigPullup(PA, 12, DISPUPD);
	
	SCU->VDCCON = 0xA3000000; //VDC33 RUN mode
	
	SCU->MCCR5 = (0x7) << 24
				| (0x1) << 16
				;
	
	USBCON = (0x7 << 28)		//Drive Strengthen
			| (0x7 << 24)		//Drive Strengthen
			| (0x1 << 23)		//CONTROL_EN
			| (0x0 << 22)		//PHY_SUSPEND
			| (0x1 << 21)		//VREF_SEL
			| (0x0 << 19)		//PD_ENDN
			| (0x0 << 18)		//PD_ENDP
			| (0x0 << 17)		//PU_ENDN
			| (0x1 << 16)		//PU_ENDP
			| (0x0 << 12)		//VIH_CON
			| (0x0 << 8)		//VIL_CON
			| (0x7 << 4)		//PD_CON
			| (0x7 << 0)		//PU_CON
		;
	
	__NOP();__NOP();__NOP();__NOP();__NOP();__NOP();__NOP();__NOP();__NOP();__NOP();
}

void init_USART10(void){
	USART_CFG_Type USARTConfigStruct;	

  // default: 38400-8-N-1	
	HAL_USART_UART_Mode_Config(&USARTConfigStruct);	
	
	/*
	 * Initialize USART10 pin connect
	 */
	HAL_GPIO_ConfigOutput(PB, 6, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 6, FUNC0);    // USART10 TXD
	HAL_GPIO_ConfigPullup(PB, 6, ENPU);   // enable Pull up
	
	HAL_GPIO_ConfigOutput(PB, 7, ALTERN_FUNC);
	HAL_GPIO_ConfigFunction(PB, 7, FUNC0);    // USART10 RXD
	

	HAL_USART_Init(USART10, &USARTConfigStruct);
	
	HAL_USART_Enable(USART10, ENABLE);
}

int main(void)
{
  __disable_irq();
	 /* Initialize all port */
	Port_Init(); 
		
	HAL_GPIO_ConfigOutput(PF, 0, ALTERN_FUNC);		// MOSC XOUT
	HAL_GPIO_ConfigFunction(PF, 0, FUNC10);
	
	HAL_GPIO_ConfigOutput(PF, 1, ALTERN_FUNC);		// MOSC_XIN
	HAL_GPIO_ConfigFunction(PF, 1, FUNC10);

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();  

	//init_USART10();
	
	init_USB_Phy();
	
	USB_Init();

	mainloop();
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
