/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"



/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void WDT_IRQHandler_IT(void);
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void WDT_ConfigureStart(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;
uint32_t status;

const uint8_t menu[] =
"************************************************\n\r"
" Watch Dog Timer Interrupt Test \n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
"************************************************\n\r";
uint8_t wdtf_uf;  // underflow flag
uint8_t wdtf_wmf; // window match flag 
WDT_CFG_Type wdtCfg;

/**********************************************************************
 * @brief		WDT_IRQHandler IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void WDT_IRQHandler_IT(void)
{
	uint32_t status;
	
	status = HAL_WDT_GetStatus();
	
	if (status & WDTSR_UNFIFLAG)
	{
		if (wdtf_uf == 0)
		{
			wdtf_uf=1;
			HAL_GPIO_ClearPin(PB, _BIT(2));
		}
		else 
		{
			wdtf_uf=0;
			HAL_GPIO_SetPin(PB, _BIT(2));
		}
		HAL_WDT_ClearStatus(WDTSR_UNFIFLAG);	// When Underflow Flag Occurs flag must be cleared twice to Clear
	}


	if (status & WDTSR_WINMIFLAG)
	{
		if (wdtf_wmf == 0)
		{
			wdtf_wmf=1;
			HAL_GPIO_ClearPin(PB, _BIT(3));
		}
		else 
		{
			wdtf_wmf=0;
			HAL_GPIO_SetPin(PB, _BIT(3));
		}
		HAL_WDT_ClearStatus(WDTSR_WINMIFLAG);	
	}
	
}

/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Test Pin setting PB2 _ WDTSR_UNFIFLAG
	HAL_GPIO_ConfigOutput(PB, 2, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 2, DISPUPD);	
	HAL_GPIO_ClearPin(PB, _BIT(2));

	// Test Pin setting PB3 _ WDTSR_WINMIFLAG
	HAL_GPIO_ConfigOutput(PB, 3, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 3, DISPUPD);
	HAL_GPIO_ClearPin(PB, _BIT(3));
}


/**********************************************************************
 * @brief		WDT Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void WDT_ConfigureStart(void)
{
	// select WDT clock src: WTCLK_WDTRC, WTCLK_MCCR3 		
	#if 1 //WDT clock source from MCCR3
		HAL_SCU_MiscClockConfig(3, WDT_TYPE, MCLK_MISC_CLOCK, 48);
		HAL_SCU_WDT_ClockConfig(WDTCLK_MCCR3); 
			
		wdtCfg.wdtResetEn = DISABLE;
		wdtCfg.wdtClkDiv = WDT_DIV_4;
		wdtCfg.wdtTmrConst = 1000-1;
		wdtCfg.wdtWTmrConst = 1000/4-1;
	#else
		//WDT clock source from LSI40kHz
		HAL_SCU_WDTLSI40K_ClockConfig(ENABLE);
		HAL_SCU_WDT_ClockConfig(WDTCLK_LSI40kHz);
		
		wdtCfg.wdtResetEn = DISABLE;
		wdtCfg.wdtClkDiv = WDT_DIV_4; 
		wdtCfg.wdtTmrConst = 1000-1;
		wdtCfg.wdtWTmrConst = 1000/4-1;
	#endif 

	if (HAL_WDT_Init(wdtCfg) != HAL_OK)
	{
		Error_Handler();
	}
	HAL_WDT_ConfigInterrupt(WDT_INTCFG_UNFIEN, ENABLE);
	HAL_WDT_ConfigInterrupt(WDT_INTCFG_WINMIEN, ENABLE);	
	NVIC_SetPriority(WDT_IRQn, 3);
	NVIC_EnableIRQ(WDT_IRQn);

	HAL_WDT_Start(ENABLE);	
}

/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Enable IRQ Interrupts */
	 __enable_irq(); 
	
	/*Initialize menu prinf*/
	DEBUG_Print_menu();

	/*Initialize PCU*/
	GPIO_Configure();
	
	/*Initialize port peripheral*/
	WDT_ConfigureStart();	

	/* Infinite Loop */
	while(1)
	{
	}
}


/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main(void)
{
  /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	
	return (0);
}
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
