/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2017, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"


/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_Print_menu(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void WT_Configure(void);
void WT_Polling(void);
void mainloop(void);
int main (void);

/* Private variables ---------------------------------------------------------*/
uint32_t status;
const uint8_t menu[] =
"************************************************\n\r"
" Watch Timer Polling TEST \n\r"
"\t - MCU: A31G32x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: USART10 - 38400 bps \n\r"
"************************************************\n\r";
uint8_t wtif;  // flag

WT_CFG_Type wtCfg;

/**********************************************************************
 * @brief		Print menu
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Print_menu(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Test Pin setting PB2
	HAL_GPIO_ConfigOutput(PB, 2, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PB, 2, DISPUPD);	
	HAL_GPIO_ClearPin(PB, _BIT(2));
}
	

/**********************************************************************
 * @brief		WT_Configure
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void WT_Configure(void)
{	
	// select WT clock src: WTCLK_MCCR3, WTCLK_SOSC, WTCLK_WDTRC		
	#if 1
		//WT clock source from MCCR3
		HAL_SCU_MiscClockConfig(3, WT_TYPE, HSI_MISC_CLOCK, 48);
		HAL_SCU_WT_ClockConfig(WTCLK_MCCR3); 
		wtCfg.wtClkDiv = WT_DIV_2_14;
		wtCfg.wtTmrConst = 4095; 	// when WTINTV only 11b, can use this bit
	#else
		//WT clock source from WDTRC. WDTRC must set LSI clock enable!!
		HAL_SCU_LSE_ClockConfig(LSIOSC_EN);
		HAL_SCU_WT_ClockConfig(WTCLK_WDTRC);		
		wtCfg.wtClkDiv = WT_DIV_2_14_MUL_WTDR;
		wtCfg.wtTmrConst = 1;
	#endif 

	if (HAL_WT_Init(wtCfg) != HAL_OK )
	{
		Error_Handler();
	}
	
	HAL_WT_Start(ENABLE);
			
}

/**********************************************************************
 * @brief		WT_Polling
 * @param[in]	None
 * @return	None
 **********************************************************************/
void WT_Polling(void)
{
	do {
		status = (HAL_WT_GetStatus() & WTCR_WTIFLAG);
	} while(status==0);
	
	if (status & WTCR_WTIFLAG)
	{
		if (wtif == 0)
		{
			wtif=1;
			HAL_GPIO_ClearPin(PB, _BIT(3));
		}
		else
		{
			wtif=0;
			HAL_GPIO_SetPin(PB, _BIT(3));
		}
		HAL_WT_ClearStatus();
	}	
}

/**********************************************************************
 * @brief		mainloop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{

	/*Initialize menu prinf*/
	DEBUG_Print_menu();
	
	/*Initialize port peripheral*/
	GPIO_Configure();
	
	/*WT Init*/
	WT_Configure();

	/*Enable IRQ Interrupts */
	 __enable_irq(); 
		
	/* Infinite Loop */
	while(1) 
	{
		WT_Polling();
	}
	
}


/**********************************************************************
 * @brief		main
 * @param[in]	None
 * @return 		None
 **********************************************************************/
int main(void)
{
	 /* Initialize all port */
	Port_Init(); 
	
	/* Configure the system clock to 48 MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();
	
	/* Infinite loop */
	mainloop();   
	
	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
  while (1)
  {
		// User Code
  }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
